/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver - K7 Analog Demod Demodulator
 *  VXIpnp Style Instrument Driver
 *  Original Release: March 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 *****************************************************************************/

#include "rszvl.h"

/*****************************************************************************
 *- RSZVL K14 - Spectrogram Measurement -------------------------------------*
 *****************************************************************************/

//static ViReal64 rateArr[] = {15.625E3, 31.25E3, 62.5E3, 125E3, 250E3, 500E3, 1E6, 2E6, 4E6, 8E6, 16E6, 32E6};

/*****************************************************************************
 * Function: Spectrogram Mode
 * Purpose:  This function activates or deactivates the Spectrogram 
 *           Measurement option.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SpectrogramMode (ViSession instrSession,
                                            ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_SPEM_STATE,state),
                                            2, "State");
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Sweep
 * Purpose:  This function starts a continuous sweep measurement or stops it 
 *           and switches to single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramSweep (ViSession instrSession,
                                                      ViBoolean continuous)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_SWEEP_MODE_CONTINUOUS, continuous),
                                            2, "Continuous");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Display Parameters
 * Purpose:  This function configures Spectogram display parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramDisplayParameters (ViSession instrSession,
                                                                  ViInt32 color,
                                                                  ViInt32 size)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_SPEM_COLOR, color),
                                            2, "Color");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_SPEM_SIZE, size),
                                            3, "Size");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Measurement Parameters
 * Purpose:  This function configures Spectogram measurement parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramMeasurementParameters (ViSession instrSession,
                                                                      ViInt32 historyDepth,
                                                                      ViBoolean timeStamp,
                                                                      ViInt32 frameCount,
                                                                      ViBoolean continueFrame)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_SPEM_HDEP, historyDepth),
                                            2, "History Depth");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_SPEM_TIME_STAMP_STATE, timeStamp),
                                            3, "Time Stamp");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_SPEM_FRAME_COUNT, frameCount),
                                            4, "Frame Count");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_SPEM_CONTINUOUS, continueFrame),
                                            5, "Continue Frame");
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Spectrogram Clear
 * Purpose:  This function deletes the spectrogram result display and the history buffer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SpectrogramClear (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_SPEM_CLEAR, NULL));

    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Marker Frame Number
 * Purpose:  This function defines the frame number of the specified marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramMarkerFrameNumber (ViSession instrSession,
                                                                  ViInt32 markerNumber,
                                                                  ViInt32 frameNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, markerNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Number");
    }
    sprintf (buffer, "M%ld", markerNumber);  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARK_FRAME_NUMBER,
                                                frameNumber), 3, "Frame Number");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Spectrogram Marker Frame Number
 * Purpose:  This function returns the frame number of the specified marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSpectrogramMarkerFrameNumber (ViSession instrSession,
                                                            ViInt32 markerNumber,
                                                            ViInt32 *frameNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, markerNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Number");
    }
    sprintf (buffer, "M%ld", markerNumber);  
    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARK_FRAME_NUMBER,
                                                frameNumber), 3, "Frame Number");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Marker Search Area
 * Purpose:  This function defines the search area of the specified marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramMarkerSearchArea (ViSession instrSession,
                                                                 ViInt32 markerNumber,
                                                                 ViInt32 searchArea)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, markerNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Number");
    }
    sprintf (buffer, "M%ld", markerNumber);  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_AREA,
                                                searchArea), 3, "Search Area");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Marker Search
 * Purpose:  This function positions the specified marker on the specified place.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SpectrogramMarkerSearch (ViSession instrSession,
                                                    ViInt32 markerNumber,
                                                    ViInt32 searchMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, markerNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Number");
    }
    if (rszvl_invalidViInt32Range (instrSession, searchMode, 0, 17) == VI_TRUE)
    {
       viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Search Mode");
    }
    sprintf (buffer, "M%ld", markerNumber);  

    switch (searchMode){
        case RSZVL_VAL_SPECM_MARK_MAX_PEAK:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_PEAK, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MAX_NEXT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_PEAK_NEXT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MAX_LEFT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_PEAK_LEFT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MAX_RIGHT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_PEAK_RIGHT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_PEAK:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_MIN, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_NEXT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_MIN_NEXT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_LEFT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_MIN_LEFT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_RIGHT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_SEARCH_MIN_RIGHT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_PEAK:
            sprintf (buffer, "M%ld,Max,Peak", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_NEXT:
            sprintf (buffer, "M%ld,Max,Next", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_ABOVE:
            sprintf (buffer, "M%ld,Max,Above", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_BELOW:
            sprintf (buffer, "M%ld,Max,Below", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_PEAK:
            sprintf (buffer, "M%ld,Min,Peak", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_NEXT:
            sprintf (buffer, "M%ld,Min,Next", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_ABOVE:
            sprintf (buffer, "M%ld,Min,Above", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_BELOW:
            sprintf (buffer, "M%ld,Min,Below", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_XY_MAX:
            sprintf (buffer, "M%ld,Max", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_XY, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_XY_MIN:
            sprintf (buffer, "M%ld,Min", markerNumber);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_MARKER_SEARCH_XY, NULL));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Delta Marker Frame Number
 * Purpose:  This function defines the frame number of the specified delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramDeltaMarkerFrameNumber (ViSession instrSession,
                                                                       ViInt32 deltaMarkerNumber,
                                                                       ViInt32 frameNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, deltaMarkerNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Number");
    }
    sprintf (buffer, "DM%ld", deltaMarkerNumber);  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_FRAME_NUMBER,
                                                frameNumber), 3, "Frame Number");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: Get Spectrogram Delta Marker Frame Number
 * Purpose:  This function returns the frame number of the specified delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSpectrogramDeltaMarkerFrameNumber (ViSession instrSession,
                                                                 ViInt32 deltaMarker,
                                                                 ViInt32 *frameNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");
    }
    sprintf (buffer, "DM%ld", deltaMarker);  
    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_FRAME_NUMBER,
                                                frameNumber), 3, "Frame Number");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: Configure Spectrogram Delta Marker Search Area
 * Purpose:  This function defines the search area of the specified delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSpectrogramDeltaMarkerSearchArea (ViSession instrSession,
                                                                      ViInt32 deltaMarker,
                                                                      ViInt32 searchArea)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");
    }
    sprintf (buffer, "DM%ld", deltaMarker);  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_AREA,
                                                searchArea), 3, "Search Area");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Delta Marker Search
 * Purpose:  This function positions the specified delta marker on the specified place.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SpectrogramDeltaMarkerSearch (ViSession instrSession,
                                                         ViInt32 deltaMarker,
                                                         ViInt32 searchMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");
    }
    if (rszvl_invalidViInt32Range (instrSession, searchMode, 0, 17) == VI_TRUE)
    {
       viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Search Mode");
    }
    sprintf (buffer, "DM%ld", deltaMarker);  

    switch (searchMode){
        case RSZVL_VAL_SPECM_MARK_MAX_PEAK:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_PEAK, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MAX_NEXT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_PEAK_NEXT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MAX_LEFT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MAX_RIGHT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_PEAK:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_MIN, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_NEXT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_MIN_NEXT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_LEFT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_MIN_RIGHT:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_PEAK:
            sprintf (buffer, "DM%ld,Max,Peak", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_NEXT:
            sprintf (buffer, "DM%ld,Max,Next", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_ABOVE:
            sprintf (buffer, "DM%ld,Max,Above", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MAX_BELOW:
            sprintf (buffer, "DM%ld,Max,Below", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_PEAK:
            sprintf (buffer, "DM%ld,Min,Peak", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_NEXT:
            sprintf (buffer, "DM%ld,Min,Next", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_ABOVE:
            sprintf (buffer, "DM%ld,Min,Above", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_Y_MIN_BELOW:
            sprintf (buffer, "DM%ld,Min,Below", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_Y, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_XY_MAX:
            sprintf (buffer, "DM%ld,Max", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_XY, NULL));
        break;
        case RSZVL_VAL_SPECM_MARK_XY_MIN:
            sprintf (buffer, "DM%ld,Min", deltaMarker);
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                                                RSZVL_ATTR_SPEM_DELT_SEARCH_XY, NULL));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Spectrogram Time Stamp
 * Purpose:  This function queries the time stamp of the frames.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSpectrogramTimeStamp (ViSession instrSession,
                                              ViInt32 frame, ViInt32 arraySize,
                                              ViInt32 part1[], ViInt32 part2[],
                                              ViInt32 part3[], ViInt32 part4[],
                                              ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;  
    ViChar*     pbuffer = NULL;
    ViInt32     i;
    ViChar*     pstring_value;
    ViChar      buffer [RSZVL_IO_BUFFER_SIZE] = "";     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
	checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));
    if (strstr (buffer, "K14") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
    
    if (rszvl_invalidViInt32Range (instrSession, frame, 0, 1) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Frame");
    }
    
    viCheckErr (viPrintf (instrSession, ":CALC:SPEC:TST:DATA? %s\n", (frame==0)?"CURR":"ALL"));
    viCheckErr( Rs_ReadDataUnknownLength(instrSession, &pbuffer, VI_NULL));

    pstring_value = strtok (pbuffer, ",");              
    i = 0;
    while (pstring_value)
    {
        if (i < arraySize){
            part1[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
            part2[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
            part3[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
            part4[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
        }   
        else
        {
             pstring_value = strtok (NULL, ",");
             pstring_value = strtok (NULL, ",");
             pstring_value = strtok (NULL, ",");
             pstring_value = strtok (NULL, ",");
        }
        i++;
    };
    if (returnedValues) *returnedValues = i;
    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    if (pbuffer) free (pbuffer); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Read Spectrogram Trace Data
 * Purpose:  This function ReadSpectrogramTraceData
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadSpectrogramTraceData (ViSession instrSession,
                                           ViInt32 arraySize, 
										   ViReal64 results[], ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
	checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));
    if (strstr (buffer, "K14") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
        
    sprintf (buffer, ":TRAC? SPEC");
    viCheckErr (rszvl_dataReadTrace (instrSession, buffer, arraySize, 
                    results, returnedValues));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *- Callback Definition   ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSZVL K14 - Spectrogram Measurement ------------------------------*
 *****************************************************************************/
