/*****************************************************************************
 *- RSZVL K30 - Noise Figure and Gain Measurement ------------------------*
 *****************************************************************************/

#include "rszvl.h"
#include "rszvl_utility.h"

static ViString measTypeArr[] = {"","UPP","LOW", VI_NULL};
static ViString fetchTypeArr[]  = {"FIG","TEMP","GAIN", VI_NULL}; 

/*****************************************************************************
 * Function: Noise Mode
 * Purpose:  This function selects the Noise Figure and Gain Measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_NoiseMode(
    ViSession   instrSession
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                               RSZVL_ATTR_NOISE_MODE, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Frequency Settings
 * Purpose:  This function configures the Noise Frequency Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseFrequencySettings(
    ViSession   instrSession,
    ViReal64    startFrequency,
    ViReal64    stopFrequency,
    ViReal64    stepFrequency,
    ViInt32     mode,
    ViReal64    fixedLO,
    ViReal64    imageRejection,
    ViReal64    fixedFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((startFrequency<0.0)||(startFrequency>8.0e+9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Start Frequency");
    
    if ((stopFrequency<0.0)||(stopFrequency>8.0e+9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Stop Frequency");

    if ((stepFrequency<1.0)||(stepFrequency>8.0e+9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Step Frequency");
    
    if ((mode<RSZVL_VAL_NOISE_DUT_MODE_DIRECT)||
        (mode>RSZVL_VAL_NOISE_DUT_MODE_FIXED_LO_ABS))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Measurement Mode");
    
    if ((fixedLO<0.0)||(fixedLO>999.99e+9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Fixed LO");

    if ((imageRejection<0.0)||(imageRejection>999.99e+9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 7, "Image Rejection");
    
    if ((fixedFrequency<0.0)||(fixedFrequency>8.0e+9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 8, "Fixed Frequency");
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_FREQUENCY_START,
                                                startFrequency), 2, "Start Frequency");
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_FREQUENCY_STOP,
                                                stopFrequency), 3, "Stop Frequency");
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_FREQ_STEP,
                                                stepFrequency), 4, "Step Frequency");
    switch (mode)
    {
        case RSZVL_VAL_NOISE_DUT_MODE_DIRECT:
            {
                viCheckErr (rszvl_SetAttributeViInt32 (instrSession, "",
                                                          RSZVL_ATTR_NOISE_CONF_MODE_DUT,
                                                          RSZVL_VAL_NOISE_CONF_DUT_AMPLIFIER));
            }
            break;
        case RSZVL_VAL_NOISE_DUT_MODE_FIXED_LO_PLUS:
            {
                viCheckErr (rszvl_SetAttributeViInt32 (instrSession, "",
                                                          RSZVL_ATTR_NOISE_CONF_MODE_DUT,
                                                          RSZVL_VAL_NOISE_CONF_DUT_UPCONV));
                                                          
                viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_CONF_SYST_LOSC_FREQ,
                                                fixedLO), 6, "Fixed LO");
    
                viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_CORR_EREJ,
                                                imageRejection), 7, "Image Rejection");
                                              
            }
            break;
        case RSZVL_VAL_NOISE_DUT_MODE_FIXED_LO_ABS:
            {
                viCheckErr (rszvl_SetAttributeViInt32 (instrSession, "",
                                                          RSZVL_ATTR_NOISE_CONF_MODE_DUT,
                                                          RSZVL_VAL_NOISE_CONF_DUT_DOWNCONV));
                                                          
                viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_CONF_SYST_LOSC_FREQ,
                                                fixedLO), 6, "Fixed LO");
    
                viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_CORR_EREJ,
                                                imageRejection), 7, "Image Rejection");
                                              
            }
            break;
        default:viCheckErrElab(RS_ERROR_INVALID_VALUE, "The value of Mode is invalid.");
            break;
    }
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_NOISE_FREQ_FIXED,
                                                fixedFrequency), 8, "Fixed Frequency");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Frequency Table
 * Purpose:  This function configures the Noise Frequency Table.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseFrequencyTable(
    ViSession   instrSession,
    ViInt32     numberOfValues,
    ViReal64    receiverFrequencyValues[],
    ViReal64    localOscilatorFrequencyVal[],
    ViReal64    intermediateFrequencyValues[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     i = 0;
    ViChar      buffer[4*RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buffer = VI_NULL;
    ViUInt32    retCnt    = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((numberOfValues<1)||(numberOfValues>100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Number Of Values");
        
    p2buffer = buffer;

    p2buffer += sprintf (p2buffer, "SENS:FREQ:LIST:DATA ");
    
    if (numberOfValues > 0)
    {
        for (i = 0; i < numberOfValues; i++)
        {
            p2buffer += sprintf (p2buffer, "%.12f,",receiverFrequencyValues[i]);
            p2buffer += sprintf (p2buffer, "%.12f,",localOscilatorFrequencyVal[i]);
            p2buffer += sprintf (p2buffer, "%.12f,",intermediateFrequencyValues[i]);
        }
    
        *p2buffer = '\0';
        *--p2buffer = '\n';
        
        viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) (p2buffer - buffer), &retCnt));
    }
    
    //free(p2buffer);

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Measurement Mode
 * Purpose:  This function configures the Noise Measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseMeasurementMode(
    ViSession   instrSession,
    ViInt32     measurementMode
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attr[] = {RSZVL_ATTR_NOISE_CONF_CORR,
                          RSZVL_ATTR_NOISE_CONF_LIST_CONT,
                          RSZVL_ATTR_NOISE_CONF_LIST_SING,
                          RSZVL_ATTR_NOISE_CONF_SING};

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((measurementMode<RSZVL_VAL_NOISE_SECOND_STAGE_CORR_MEAS)||
        (measurementMode>RSZVL_VAL_NOISE_SINGLE_FREQ_MEAS))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Measurement Mode");                          
    
    viCheckParm (rszvl_SetAttributeViString(instrSession,"",
                                                attr[measurementMode],
                                                VI_NULL), 2,"Measurement Type");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Calibration
 * Purpose:  This function configures the Noise Calibration.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseCalibration(
    ViSession   instrSession,
    ViBoolean   _2ndStageCorrection
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                 RSZVL_ATTR_NOISE_CORR_STAT,
                                                 _2ndStageCorrection), 2, "2nd Stage Correction");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Analyzer Settings
 * Purpose:  This function configures the Noise Analyzer Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseAnalyzerSettings(
    ViSession   instrSession,
    ViReal64    resolutionBandwidth,
    ViReal64    sweepTime,
    ViReal64    settlingTime,
    ViInt32     average,
    ViReal64    RFAttenuation,
    ViReal64    range,
    ViBoolean   automaticRefLevel,
    ViReal64    refLevel,
    ViBoolean   preamplifier
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((resolutionBandwidth<10.0)||(resolutionBandwidth>50.0e+6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Resolution Bandwidth");
#if 0    
    if ((sweepTime<1.0e-6)||(sweepTime>16.0e+3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "SweepTime");
#endif
    if ((settlingTime<0.0)||(settlingTime>20.0))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Settling Time");
    
    if ((average<1)||(average>32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Average");
    
    if ((RFAttenuation<0.0)||(RFAttenuation>75.0))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "RF Attenuation");
    
    if ((range<10.0)||(range>999.99))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 7, "Range");
    
    if ((refLevel<-130)||(refLevel>5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 9, "Ref Level");
    
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_RESOLUTION_BANDWIDTH,
                                                resolutionBandwidth), 2, "Resolution Bandwidth");
    
#if 0    
	viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_SWEEP_TIME, 
                                                sweepTime), 3, "Sweep Time");
#endif
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_CONF_DUT_SETT_TIME,
                                                settlingTime), 4, "Settling Time");
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "",
                                               RSZVL_ATTR_NUMBER_OF_SWEEPS, 
                                               average), 5, "Average");
    
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "TR1",
                                                 RSZVL_ATTR_NOISE_DISP_TRAC_REF_LEV_STAT,
                                                 automaticRefLevel), 8, "Automatic Ref Level");
    
    if (automaticRefLevel == VI_FALSE)
    {
        viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                    RSZVL_ATTR_REFERENCE_LEVEL,
                                                    refLevel), 9, "Ref Level")
    }                                                    
    else
    {
        viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                    RSZVL_ATTR_NOISE_CONF_DUT_GAIN,
                                                    range), 7, "Range");
    }
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_ATTENUATION,
                                                RFAttenuation), 6, "RF Attenuation");

    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

	if (strstr (buffer, "B22,") != VI_NULL)
    {
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "",
                                                 RSZVL_ATTR_NOISE_INP_GAIN_STAT,
                                                 preamplifier), 10, "Pre-amplifier");
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise ENR Settings
 * Purpose:  This function configures the Noise ENR Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseENRSettings(
    ViSession   instrSession,
    ViInt32     selectionMode,
    ViReal64    ENRConstant,
    ViReal64    roomTemperature
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((selectionMode<RSZVL_VAL_NOISE_ERN_MODE_SEL_TABLE)||
        (selectionMode>RSZVL_VAL_NOISE_ERN_MODE_SEL_SPOT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Selection Mode");
        
    if ((ENRConstant<-999.99)||(ENRConstant>999.99))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "ENR Constant");
    
    if ((roomTemperature<10.0)||(roomTemperature>1000.0))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Room Temperature");
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "",
                                               RSZVL_ATTR_NOISE_CORR_ENR_MODE,
                                               selectionMode), 2, "Selection Mode");
    
    if( selectionMode == RSZVL_VAL_NOISE_ERN_MODE_SEL_SPOT)
    {
        viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                    RSZVL_ATTR_NOISE_CORR_ENR_SPOT,
                                                    ENRConstant), 3, "ENR Constant");
    }

    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_NOISE_CORR_TEMP,
                                                roomTemperature), 4, "Room Temperature");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise ENR Table
 * Purpose:  This function configures the Noise ENR Table.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseENRTable(
    ViSession   instrSession,
    ViInt32     numberOfValues,
    ViReal64    receiverFrequencyValues[],
    ViReal64    ENRValues[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     i = 0;
    ViChar      buffer[4*RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buffer = VI_NULL;
    ViUInt32    retCnt          = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((numberOfValues<1)||(numberOfValues>100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Number Of Values");

    p2buffer = buffer;

    p2buffer += sprintf (p2buffer, "SENS:CORR:ENR:TABL:DATA ");

    if (numberOfValues > 0)
    {
        for (i = 0; i < numberOfValues; i++)
        {
            p2buffer += sprintf (p2buffer, "%.12f,",receiverFrequencyValues[i]);
            p2buffer += sprintf (p2buffer, "%.12f,",ENRValues[i]);
        }

        *p2buffer = '\0';
        *--p2buffer = '\n';
    
        viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) (p2buffer - buffer), &retCnt));
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Loss Input Settings
 * Purpose:  This function configures the Noise Loss Input Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseLossInputSettings(
    ViSession   instrSession,
    ViInt32     selectionMode,
    ViReal64    lossInputConstant
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((selectionMode<RSZVL_VAL_NOISE_LOSS_MODE_TABLE)||
        (selectionMode>RSZVL_VAL_NOISE_LOSS_MODE_SPOT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Selection Mode");
        
    if ((lossInputConstant<-999.99)||(lossInputConstant>999.99))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Loss Input Constant");    
        
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "Inp",
                                               RSZVL_ATTR_NOISE_CORR_LOSS_MODE,
                                               selectionMode), 2, "Selection Mode");
    
    if( selectionMode == RSZVL_VAL_NOISE_LOSS_MODE_SPOT)
    {
        viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "Inp",
                                                    RSZVL_ATTR_NOISE_CORR_LOSS_SPOT,
                                                    lossInputConstant), 3, "Loss Input Constant");
    }

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Loss Input Table
 * Purpose:  This function configures the Noise Loss Input Table.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseLossInputTable(
    ViSession   instrSession,
    ViInt32     numberOfValues,
    ViReal64    receiverFrequencyValues[],
    ViReal64    lossInputValues[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     i = 0;
    ViChar      buffer[4*RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buffer = VI_NULL;
    ViUInt32    retCnt          = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((numberOfValues<1)||(numberOfValues>100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Number Of Values");

    p2buffer = buffer;

    p2buffer += sprintf (p2buffer, "SENS:CORR:LOSS:INP:TABL ");

    if (numberOfValues > 0)
    {
        for (i = 0; i < numberOfValues; i++)
        {
            p2buffer += sprintf (p2buffer, "%.12f,",receiverFrequencyValues[i]);
            p2buffer += sprintf (p2buffer, "%.12f,",lossInputValues[i]);
        }

        *p2buffer = '\0';
        *--p2buffer = '\n';
    
        viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) (p2buffer - buffer), &retCnt));
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Loss Output Settings
 * Purpose:  This function configures the Noise Loss Output Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseLossOutputSettings(
    ViSession   instrSession,
    ViInt32     selectionMode,
    ViReal64    lossOutputConstant
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((selectionMode<RSZVL_VAL_NOISE_LOSS_MODE_TABLE)||
        (selectionMode>RSZVL_VAL_NOISE_LOSS_MODE_SPOT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Selection Mode");
        
    if ((lossOutputConstant<-999.99)||(lossOutputConstant>999.99))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Loss Output Constant");    
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "Out",
                                               RSZVL_ATTR_NOISE_CORR_LOSS_MODE,
                                               selectionMode), 2, "Selection Mode");
    
    if( selectionMode == RSZVL_VAL_NOISE_LOSS_MODE_SPOT)
    {
        viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "Out",
                                                    RSZVL_ATTR_NOISE_CORR_LOSS_SPOT,
                                                    lossOutputConstant), 3, "Loss Output Constant");
    }

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Loss Output Table
 * Purpose:  This function configures the Noise Loss Output Table.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseLossOutputTable(
    ViSession   instrSession,
    ViInt32     numberOfValues,
    ViReal64    receiverFrequencyValues[],
    ViReal64    lossOutputValues[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     i = 0;
    ViChar      buffer[4*RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buffer = VI_NULL;
    ViUInt32    retCnt          = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((numberOfValues<1)||(numberOfValues>100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Number Of Values");

    p2buffer = buffer;

    p2buffer += sprintf (p2buffer, "SENS:CORR:LOSS:OUTP:TABL ");

    if (numberOfValues > 0)
    {
        for (i = 0; i < numberOfValues; i++)
        {
            p2buffer += sprintf (p2buffer, "%le,%le,",receiverFrequencyValues[i], lossOutputValues[i]);
        }

        *p2buffer = '\0';
        *--p2buffer = '\n';
    
        viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) (p2buffer - buffer), &retCnt));
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Result Settings
 * Purpose:  This function configures the Noise Display Result Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseResultSettings(
    ViSession   instrSession,
    ViInt32     combinedTraceDisplay,
    ViBoolean   tableFormResults
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((combinedTraceDisplay<RSZVL_VAL_DISP_FORMAT_SINGLE)||
        (combinedTraceDisplay>RSZVL_VAL_DISP_FORMAT_SPLIT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Combined Trace Display");
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_NOISE_DISP_FORMAT,
                                                combinedTraceDisplay), 2, "Combined Trace Display");
   
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "",
                                                 RSZVL_ATTR_NOISE_DISP_TABLE,
                                                 tableFormResults), 3, "Table Form Results");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Trace Settings
 * Purpose:  This function configures the Noise Trace Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseTraceSettings(
    ViSession   instrSession,
    ViInt32     YAxis,
    ViBoolean   symbols,
    ViBoolean   automaticScaling,
    ViReal64    minYAxis,
    ViReal64    maxYAxis
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      trace_no[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, YAxis, RSZVL_VAL_NOISE_DISP_DATA_OFF, RSZVL_VAL_NOISE_DISP_DATA_NFIGURE))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Y-Axis");
    }
    
    sprintf(trace_no,"TR1");   
    switch (YAxis){
        case RSZVL_VAL_NOISE_DISP_DATA_NFIGURE:
			if (rszvl_invalidViReal64Range (instrSession, maxYAxis, -75.0, 75.0))
            {
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Max Y-Axis");
            }
            if (rszvl_invalidViReal64Range (instrSession, minYAxis, -75.0, 75.0))
            {
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Min Y-Axis");
            }
            
		break;
        case RSZVL_VAL_NOISE_DISP_DATA_TEFF:
            if (rszvl_invalidViReal64Range (instrSession, maxYAxis, -999990000.0, 999990000.0))
            {
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Max Y-Axis");
            }
            if (rszvl_invalidViReal64Range (instrSession, minYAxis, -999990000.0, 999990000.0))
            {
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Min Y-Axis");
            }
		break;
            
    } 
	
	if (YAxis == RSZVL_VAL_NOISE_DISP_DATA_OFF)
	{
		viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                     RSZVL_ATTR_NOISE_DISP_TRAC_STAT,
                                                     VI_FALSE), 2, "Y-Axis");
	}
	else
	{
		viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                     RSZVL_ATTR_NOISE_DISP_TRAC_STAT,
                                                     VI_TRUE), 2, "Y-Axis");
        
        viCheckParm (rszvl_SetAttributeViInt32 (instrSession, trace_no,
                                                    RSZVL_ATTR_NOISE_DISP_DATA_TRAC,
                                                    YAxis), 2, "Y-Axis");

        viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                     RSZVL_ATTR_NOISE_DISP_SYMB,
                                                     symbols), 3, "Symbols");

        viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                     RSZVL_ATTR_NOISE_DISP_AUTO_SCAL,
                                                     automaticScaling), 4, "Automatic Scaling");

        if (automaticScaling == VI_FALSE)
        {
            viCheckParm (rszvl_SetAttributeViReal64 (instrSession, trace_no,
                                                        RSZVL_ATTR_NOISE_DISP_TRAC_BOTTOM,
                                                        minYAxis), 5, "Min Y-Axis");

            viCheckParm (rszvl_SetAttributeViReal64 (instrSession, trace_no,
                                                        RSZVL_ATTR_NOISE_DISP_TRAC_TOP,
                                                        maxYAxis), 6, "Max Y-Axis");
        }	
	}
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Gain Trace Settings
 * Purpose:  This function configures the Gain Trace Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseGainTraceSettings(
    ViSession   instrSession,
    ViBoolean   YAxis,
    ViBoolean   symbols,
    ViBoolean   automaticScaling,
    ViReal64    minYAxis,
    ViReal64    maxYAxis
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      trace_no[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((maxYAxis<-75)||(maxYAxis>75))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Max Y-Axis");
    if ((minYAxis<-75)||(minYAxis>75))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Min Y-Axis");

    sprintf(trace_no,"TR2");

    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                 RSZVL_ATTR_NOISE_DISP_TRAC_STAT,
                                                 YAxis), 2, "Y-Axis");
    
    if (YAxis == VI_TRUE)
    {
        viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                     RSZVL_ATTR_NOISE_DISP_SYMB,
                                                     symbols), 3, "Symbols");
    
        viCheckParm (rszvl_SetAttributeViBoolean (instrSession, trace_no,
                                                     RSZVL_ATTR_NOISE_DISP_AUTO_SCAL,
                                                     automaticScaling), 4, "Automatic Scaling");
    
        if (automaticScaling == VI_FALSE)
        {
            viCheckParm (rszvl_SetAttributeViReal64 (instrSession, trace_no,
                                                        RSZVL_ATTR_NOISE_DISP_TRAC_BOTTOM,
                                                        minYAxis), 5, "Min Y-Axis");
    
            viCheckParm (rszvl_SetAttributeViReal64 (instrSession, trace_no,
                                                        RSZVL_ATTR_NOISE_DISP_TRAC_TOP,
                                                        maxYAxis), 6, "Max Y-Axis");
        }
    }

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Trace Memory Display
 * Purpose:  This function switches the display of trace memory <n> on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseTraceMemoryDisplay (ViSession instrSession,
                                                             ViInt32 memory,
                                                             ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, memory, 1, 3))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Memory");
    }
    
    sprintf (buffer, "MEM%ld", memory);
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer,
                                                 RSZVL_ATTR_NOISE_DISP_TRACE_MEM,
                                                 state), 3, "State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Current Results Display
 * Purpose:  This function switches the display of the current measurement 
 *           results on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseCurrentResultsDisplay (ViSession instrSession,
                                                                ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                 RSZVL_ATTR_NOISE_DISPL_CURR_RESULTS,
                                                 state), 2, "State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*****************************************************************************
 * Function: Configure Noise X-Axis Frequency Display
 * Purpose:  This function selects the frequency to be displayed on the x-axis 
 *           if the DUT is not an amplifier, i.e. in a frequency-converting 
 *           measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureXAxisFrequencyDisplay (ViSession instrSession,
                                                           ViInt32 measurementResults,
                                                           ViInt32 frequency)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, measurementResults, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement Results");
    }
    
    sprintf (buffer, "TR%ld", measurementResults);
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                 RSZVL_ATTR_NOISE_DISP_X_AXIS,
                                                 frequency), 3, "Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Create Noise Limit Line
 * Purpose:  This function creates new limit or selects existing one and 
 *           configures basic parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CreateNoiseLimitLine(
    ViSession   instrSession,
    ViInt32     limit,
    ViString    name,
    ViString    comment
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");

    sprintf(lim_line_no,"L%d",limit);
    
    viCheckParm (rszvl_SetAttributeViString (instrSession, lim_line_no,
                                                RSZVL_LIMIT_NAME, name), 3, "Limit Line Name");
    
    viCheckParm (rszvl_SetAttributeViString (instrSession, lim_line_no,
                                                RSZVL_LIMIT_COMMENT, comment), 4, "Limit Line Comment");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Define Noise Limit Line
 * Purpose:  This function defines specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DefineNoiseLimitLine(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     resultType,
    ViInt32     type,
    ViInt32     count,
    ViReal64    xAxis[],
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     i = 0;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer= VI_NULL;
    ViChar      *p2buf;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");

    if ((resultType<RSZVL_VAL_NOISE_CALC_LIM_LINE_NFIGURE)||
        (resultType>RSZVL_VAL_NOISE_CALC_LIM_LINE_GAIN))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Result Type");

    if ((type<RSZVL_VAL_LIMIT_UPPER)||
        (type>RSZVL_VAL_LIMIT_LOWER))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Type");
    
    if (count<1)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Count");
    
    if (!xAxis)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "X Axis");
    }
    if (!amplitude)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Amplitude");
    }
    
    sprintf(lim_line_no,"L%d",limit);
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, lim_line_no,
                                               RSZVL_ATTR_NOISE_CALC_LIM_TRAC,
                                               resultType), 3, "Result Type");
    
    viCheckAlloc (pbuffer = (ViChar*) malloc (count * 20 + 200));
    
    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM%ld:CONT ", limit);
    
        for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", xAxis[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));
    
    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM%ld:%s ", limit, measTypeArr [type]);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", amplitude[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (pbuffer) free (pbuffer);

    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Enable Noise Limit Check
 * Purpose:  This function switches on or off the limit check and/or limit 
 *           line for the selected limit line in the selected measurement 
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_EnableNoiseLimitCheck(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     type,
    ViBoolean   limitEnabled,
    ViBoolean   checkEnabled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = ""; 
    ViAttr      attr[] = {0,
						  RSZVL_LIMIT_UPPER_STATE,
						  RSZVL_LIMIT_LOWER_STATE};

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");
        
    if ((type<RSZVL_VAL_LIMIT_UPPER)||
        (type>RSZVL_VAL_LIMIT_LOWER))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Type");    

    sprintf(lim_line_no,"L%d",limit);
    
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, lim_line_no, RSZVL_LIMIT_STATE, limitEnabled), 4, "Limit Enabled");
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, lim_line_no, attr[type], checkEnabled), 5, "Check Enabled");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Move Noise Limit Line
 * Purpose:  This function moves specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_MoveNoiseLimitLine(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     type,
    ViReal64    value
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = "";
    ViAttr      attr[] = {0,
						  RSZVL_LIMIT_UPPER_SHIFT,
						  RSZVL_LIMIT_LOWER_SHIFT,
                          RSZVL_LIMIT_CONTROL_SHIFT};    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");

    if ((type<RSZVL_VAL_LIMIT_UPPER)||
        (type>RSZVL_VAL_LIMIT_CONTROL))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Type");

    if (type == RSZVL_VAL_LIMIT_CONTROL)
    {
        if ((value<-300.0e9)||(value>300.0e9))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value");
    }
    else
    {
        if ((value<-200.0)||(value>200.0))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value");
    }
    
    sprintf(lim_line_no,"L%d",limit);
    
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, lim_line_no,
                                              attr[type],
                                              value),
                                              4, "Value");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Copy Noise Limit Line
 * Purpose:  This function copies specified limit line into another one.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CopyNoiseLimitLine(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     copyTo
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");

    sprintf(lim_line_no,"L%d",limit);
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, lim_line_no,
                                               RSZVL_LIMIT_COPY, copyTo),
                                               3, "Copy To");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Clear Noise Limit Line Results
 * Purpose:  This function deletes the result of the current limit check.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ClearNoiseLimitLineResults(
    ViSession   instrSession,
    ViInt32     limit
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");
    
    sprintf(lim_line_no,"L%d",limit);

    viCheckErr(rszvl_SetAttributeViString (instrSession, lim_line_no,
                                               RSZVL_LIMIT_CLEAR, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete Noise Limit Line
 * Purpose:  This function removes specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteNoiseLimitLine(
    ViSession   instrSession,
    ViInt32     limit
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");

    sprintf(lim_line_no,"L%d",limit);
    
    viCheckErr (rszvl_SetAttributeViString (instrSession, lim_line_no,
                                               RSZVL_LIMIT_DELETE, VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Noise Limit Check Result
 * Purpose:  This function queries the result of the limit check of the limit 
 *           line indicated in the selected measurement window. It should be 
 *           noted that a complete sweep must have been performed for obtaining 
 *           a valid result. A synchronization with  *OPC, *OPC? or *WAI should 
 *           therefore be provided.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetNoiseLimitCheckResult(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32*    state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((limit<1)||(limit>6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");

    sprintf(lim_line_no,"L%d",limit);
    
    viCheckParm (rszvl_GetAttributeViInt32 (instrSession, lim_line_no,
                                               RSZVL_LIMIT_CHECK_RESULT,
                                               state),
                                               3, "State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Noise Marker Enabled
 * Purpose:  This function assigns the marker1 to the indicated trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNoiseMarkerEnabled (ViSession instrSession,
                                                     ViBoolean markerEnabled,
                                                     ViInt32 trace)
{
	ViStatus	error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "M1", RSZVL_ATTR_MARKER_ENABLED, 
                markerEnabled), 2, "Marker Enabled");
    
	if (markerEnabled == VI_TRUE)                
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NOISE_MARKER_TRACE, 
                trace), 3, "Trace");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Noise Move Marker
 * Purpose:  This function moves the active marker to the specified horizontal 
 *           position.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_NoiseMoveMarker (ViSession instrSession,
                                         ViReal64 markerPosition)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "M1", RSZVL_ATTR_MARKER_POSITION, 
                markerPosition), 2, "Marker Position");
    
	checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Noise Marker Peak
 * Purpose:  This function set the marker to the min/max peak in the current 
 *           trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetNoiseMarkerPeak (ViSession instrSession,
                                            ViInt32 position)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	switch (position){
        case RSZVL_VAL_MARKER_SEARCH_MINIMUM:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "M1", RSZVL_ATTR_MARKER_SEARCH_MIN, ""));
        break;    
        case RSZVL_VAL_MARKER_SEARCH_HIGHEST:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "M1", RSZVL_ATTR_MARKER_SEARCH_PEAK, ""));
        break;    
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Position");
		break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Noise Marker
 * Purpose:  This function returns the marker amplitude level.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryNoiseMarker (ViSession instrSession,
                                          ViReal64 *markerAmplitude)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "M1", RSZVL_ATTR_MARKER_AMPLITUDE, 
                markerAmplitude), 2, "Marker Amplitude");
    
	checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Noise Disable All Markers
 * Purpose:  This function switches off the marker 1.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_NoiseDisableAllMarkers (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_MARKER_AOFF, 
                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Noise Trace Data To Memory
 * Purpose:  This function saves the current trace results (noise figure, noise 
 *           temperature and noise gain traces) to trace memory <n>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveNoiseTraceDataToMemory (ViSession instrSession,
                                                       ViInt32 memory)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, memory, 1, 3))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Memory");
    }
    
    sprintf (buffer, "MEM%ld", memory);
    viCheckErr( rszvl_SetAttributeViString (instrSession, buffer,
                                                 RSZVL_ATTR_NOISE_ARRAY_MEMORY,
                                                 NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Fetch Noise Array
 * Purpose:  This function returns the noise measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_FetchNoiseArray(
    ViSession   instrSession,
    ViInt32     modifier,
    ViInt32     arraySize,
    ViReal64    values[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL; 
    ViChar*     pstring_value;
    ViInt32     cnt     = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((modifier<RSZVL_VAL_NOISE_FETC_FIGURE)||
        (modifier>RSZVL_VAL_NOISE_FETC_GAIN))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Modifier");
    
    if ((arraySize<1)||
        (arraySize>100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Array Size");
    
    viCheckErr (viPrintf (instrSession, "FETC:ARR:NOIS:%s?\n", fetchTypeArr[modifier]));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 
   
    pstring_value = strtok(pbuffer, ",");
    while (pstring_value && cnt<arraySize)
    {
        values[cnt++] = atof (pstring_value);
        pstring_value = strtok(NULL, ",");
    }
    
    if (pbuffer) free(pbuffer);
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Fetch Noise Scalar
 * Purpose:  This function returns the noise measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_FetchNoiseScalar(
    ViSession   instrSession,
    ViInt32     modifier,
    ViReal64*   value
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((modifier<RSZVL_VAL_NOISE_FETC_FIGURE)||
        (modifier>RSZVL_VAL_NOISE_FETC_GAIN))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Modifier");
    
    viCheckErr (viPrintf (instrSession, "FETC:SCAL:NOIS:%s?\n", fetchTypeArr[modifier]));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 

    sscanf(pbuffer,"%le",value);

    if (pbuffer) free(pbuffer);
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Fetch Noise Memory Array
 * Purpose:  This function returns the measurement results of the selected 
 *           memory.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_FetchNoiseMemoryArray (ViSession instrSession,
                                                  ViInt32 modifier,
                                                  ViInt32 memory,
                                                  ViInt32 arraySize,
                                                  ViReal64 measResult[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL; 
    ViChar*     pstring_value;
    ViInt32     cnt     = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, modifier, RSZVL_VAL_NOISE_FETC_FIGURE, RSZVL_VAL_NOISE_FETC_GAIN))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Modifier");
    }
    
    if (rszvl_invalidViInt32Range (instrSession, memory, 1, 3))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Memory");
    }
    if (rszvl_invalidViInt32Range (instrSession, arraySize, 1, 100))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Array Size");
    }
  
    viCheckErr (viPrintf (instrSession, "FETC:ARR:MEM%ld:NOIS:%s?\n", memory, fetchTypeArr[modifier]));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 
   
    pstring_value = strtok(pbuffer, ",");
    while (pstring_value && cnt<arraySize)
    {
        measResult[cnt++] = atof (pstring_value);
        pstring_value = strtok(NULL, ",");
    }
    
    if (pbuffer) free(pbuffer);
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *- End Of RSZVL K30 - Noise Figure and Gain Measurement --------------------*
 *****************************************************************************/
