/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver - K7 Analog Demodulator
 *  VXIpnp Style Instrument Driver
 *  Original Release: March 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 *****************************************************************************/

#include "rszvl.h"

/*****************************************************************************
 *- RSZVL K7 - Analog Demodulator ---------------------------------------------*
 *****************************************************************************/

//static ViReal64 rateArr[] = {15.625E3, 31.25E3, 62.5E3, 125E3, 250E3, 500E3, 1E6, 2E6, 4E6, 8E6, 16E6, 32E6};
static ViString ADEMTypeArr[] = {"AM","FM","PM","FM:AFSP","PM:AFSP","SPEC","AM:REL","AM:REL:AFSP", VI_NULL};
static ViString resultTypeArr[] = {"OFF", "WRIT", "AVER", "MAXH", "MINH", "VIEW", VI_NULL};
static ViString detectorTypeArr[] = {"PosPeak","NegPeak","AverPeak","RMS"};
static ViString ADemMeasResArr[] = {"XTIM:AM:REL","XTIM:AM","XTIM:SPEC","XTIM:FM",
                                    "XTIM:PM","XTIM:AMS","XTIM:AMS","XTIM:FMS","XTIM:PMS",
                                    "XTIM:FM:AFSP","XTIM:PM:AFSP","XTIM:AM:REL:AFSP",
                                    VI_NULL};
/*****************************************************************************
 * Function: Analog Demod Mode
 * Purpose:  This function selects the Analog Demod Demodulator mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ADemodMode(ViSession   instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FMDEM_MODE, VI_NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*****************************************************************************
 * Function: Set Analog Demod State
 * Purpose:  This function activates the Analog Demod demodulator of the instrument. The 
 *           instrument will be set to timedomain measurement (span = 0) at the 
 *           current center frequency. The detector will be set to
 *           SAMPle, the demodulator itself will be set up according to the 
 *           parameters of command [SENSe:]ADEMod:SET.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetADemodState(ViSession   instrSession,
                                          ViBoolean   state)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FMDEM_STATE,state),
                                            2, "State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Select Analog Demod Result
 * Purpose:  This function selects Analog Demod demodulator measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelectADemodResult (ViSession instrSession,
                                                  ViInt32 operatingMode,
                                                  ViInt32 trace)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K7"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
	if (rszvl_invalidViInt32Range (instrSession, operatingMode, 0, 11))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operating Mode");
    }
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 4))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Trace");
    }
    switch (operatingMode)
    {
        case RSZVL_VAL_FM_FEED_AM_REL:
        case RSZVL_VAL_FM_FEED_AM:
        case RSZVL_VAL_FM_FEED_SPEC: 
        case RSZVL_VAL_FM_FEED_FM:
        case RSZVL_VAL_FM_FEED_PM:
             sprintf (buffer, ":CALC1:FEED '%s'\n", ADemMeasResArr[operatingMode]);
        break;
        case RSZVL_VAL_FM_FEED_AMS_REL: 
            sprintf (buffer, ":CALC1:FEED '%s%ld:REL'\n", ADemMeasResArr[operatingMode], trace);
        break;  
        case RSZVL_VAL_FM_FEED_AMS:
        case RSZVL_VAL_FM_FEED_FMS:
        case RSZVL_VAL_FM_FEED_PMS:
        case RSZVL_VAL_FM_FEED_FM_AFSP:
        case RSZVL_VAL_FM_FEED_PM_AFSP:
        case RSZVL_VAL_FM_FEED_AM_REL_AFSP:
            sprintf (buffer, ":CALC1:FEED '%s%ld'\n", ADemMeasResArr[operatingMode], trace);
        break;
    }
    viCheckErr (viPrintf (instrSession, "%s", buffer));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod Trigger
 * Purpose:  This function configures the Analog Demodulation Trigger
 *****************************************************************************/
 ViStatus _VI_FUNC rszvl_ConfigureADemodTrigger (ViSession instrSession,
                                               ViInt32 triggerSource,
                                               ViReal64 triggerOffset,
                                               ViInt32 triggerPolarity,
                                               ViReal64 triggerLevel)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_FMDEM_TRIGGER_SOURCE, triggerSource),
                                            2, "Trigger Source");
                                            
    switch (triggerSource){
        case RSZVL_VAL_TRG_IMM:
        case RSZVL_VAL_TRG_EXT:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_TRIGGER_DELAY, triggerOffset),
                                            3, "Trigger Offset");  
        break;
        case RSZVL_VAL_TRG_IFP:
           viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_TRIGGER_DELAY, triggerOffset),
                                            3, "Trigger Offset");
           viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_TRIGGER_IFP_LEVEL, triggerLevel),
                                            5, "Trigger Level");
        break;
        case RSZVL_VAL_TRG_FM:
                viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_ADEM_TRIGGER_FM_LEVEL, triggerLevel),
                                            5, "Trigger Level");
        break;
        case RSZVL_VAL_TRG_AM:
             viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_ADEM_TRIGGER_AM_LEVEL_ABS, triggerLevel),
                                            5, "Trigger Level");
        break;
        case RSZVL_VAL_TRG_PM:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_ADEM_TRIGGER_PM_LEVEL, triggerLevel),
                                            5, "Trigger Level");
        break;
        case RSZVL_VAL_TRG_AMR:
             viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_ADEM_TRIGGER_AM_LEVEL_REL, triggerLevel),
                                            5, "Trigger Level");
        break;
    }   
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_TRIGGER_SLOPE, triggerPolarity),
                                            4, "Trigger Polarity");                                            

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod
 * Purpose:  This function configures the Analog Demod demodulator.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodSet(
    ViSession   instrSession,
    ViReal64 sampleRate,
    ViInt32 recordLength,
    ViInt32 triggerSource,
    ViInt32 triggerSlope,
    ViInt32 pretriggerSamples,
    ViInt32 numberOfMeas
)
{
    ViStatus    error   = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K7"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);

    if (rszvl_invalidViReal64Range (instrSession, sampleRate, 122.0703125, 32.0e+6))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Sample Rate");
    }
    if (rszvl_invalidViInt32Range (instrSession, recordLength, 1, 480001))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Record Length");
    }
    if (rszvl_invalidViInt32Range (instrSession, triggerSource, 0, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Trigger Source");
    }
    if (rszvl_invalidViInt32Range (instrSession, triggerSlope, 0, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Trigger Slope");
    }
    if (rszvl_invalidViInt32Range (instrSession, pretriggerSamples, -65024, 130560))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Pretrigger Samples");
    }
    if (rszvl_invalidViInt32Range (instrSession, numberOfMeas, 0, 32767))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Number of Measurements");
    }

         
    sprintf (buffer, "SENS:ADEM:SET %.12fHZ,%ld,%s,%s,%ld,%ld\n",
        sampleRate, recordLength, rszvl_rngFmTriggerSource.rangeValues[triggerSource].cmdString,
        rszvl_rngPolarity.rangeValues[triggerSlope].cmdString, pretriggerSamples, numberOfMeas);

    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), &retCnt));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod Result Type
 * Purpose:  This function sets the result types to be measured 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodResultType(
    ViSession   instrSession,
    ViInt32 demodulationType,
    ViInt32 resultTypes[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));
    
    if (!strstr (buffer, "K7"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);

    if (rszvl_invalidViInt32Range (instrSession, demodulationType, 0, 7))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Demodulation Type");
    }

    sprintf (buffer, "SENS:ADEM:%s:TYPE %s,%s,%s",
            ADEMTypeArr[demodulationType], resultTypeArr[resultTypes[0]],
            resultTypeArr[resultTypes[1]], resultTypeArr[resultTypes[2]]);
    
	checkErr (viPrintf (instrSession, "%s\n", buffer));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod Filter
 * Purpose:  This function selects demodulation bandwidth AF filters of Analog Demod 
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodFilter(
    ViSession   instrSession,
    ViInt32     filterType,
    ViBoolean   filterState,
    ViReal64    filterFrequency,
    ViReal64    amfmDeemphasisValue
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (filterType)
    {
        case RSZVL_VAL_FMDEM_FILTER_HP:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FMDEM_FILT_HPAS_STAT, filterState),
                                                    3, "Filter State");
            if (filterState)
                viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_FILT_HPAS_FREQ, filterFrequency),
                                                        4, "Filter Frequency");
            break;
            
        case RSZVL_VAL_FMDEM_FILTER_LP:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FMDEM_FILT_LPAS_STAT, filterState),
                                                    3, "Filter State");
            if (filterState)
                viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_FILT_LPAS_FREQ, filterFrequency),
                                                        4, "Filter Frequency");
            break;

        case RSZVL_VAL_FMDEM_FILTER_DEEMP:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FMDEM_FILT_DEMP_STAT, filterState),
                                                    3, "Filter State");
            if (filterState)
                viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_FILT_DEMP_TCON, amfmDeemphasisValue),
                                                        5, "Deemphasis Value");
            break;
        case RSZVL_VAL_FMDEM_FILTER_LP_REL:
           
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FMDEM_FILT_LPAS_STAT, filterState),
                                                    3, "Filter State");
            if (filterState)
            {
                viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_ADEM_FILT_LPAS_FREQ_REL, filterFrequency), 
                            4, "Filter Frequency");
            }   
        break;

        default: viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Filter Type");
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod Zoom
 * Purpose:  This function sets analog demodulation parameters such as zoom 
 *           state, start time.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodZoom(
    ViSession   instrSession,
    ViBoolean   zoomState,
    ViReal64    startTime
)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FMDEM_ZOOM, zoomState),
                                            2, "Zoom State");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_ZOOM_START, startTime),
                                            3, "Start Time");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod PM Phase Wrap
 * Purpose:  This function sets PM phase wrap 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureAnalogDemodPMPhaseWrap (ViSession instrSession,
                                                    ViInt32 phaseWrap)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_ADEM_PHASE_WRAP, phaseWrap),
                                            2, "Phase Wrap");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod PM Phase Units
 * Purpose:  This function sets PM phase units 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureAnalogDemodPMPhaseUnits (ViSession instrSession,
                                                        ViInt32 PMUnits)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_ADEM_UNIT_ANGLE, PMUnits),
                                            2, "PM Units");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod PM Zero Phase Reference Point
 * Purpose:  This function sets PM zero phase reference point 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureAnalogDemodPMZeroPhaseReferencePoint (ViSession instrSession,
                                                                          ViReal64 referencePoint)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_ADEM_PM_RPO_X, referencePoint),
                                            2, "Reference Point");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod Bandwidth
 * Purpose:  This function configures analog demodulator bandwidth parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodBandwidth(ViSession   instrSession,
                                                    ViReal64    demodulationBandwidth)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_BAND_DEM, demodulationBandwidth),
                                            2, "Demodulation Bandwidth");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod AF Coupling
 * Purpose:  This function configures analog demodulator AF Coupling.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodAFCoupling (ViSession instrSession,
                                                      ViInt32 afCoupling)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_FMDEM_AF_COUP, afCoupling),
                                            2, "AF Coupling");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Configure Analog Demod Measuring Time
 * Purpose:  This function configures analog demodulator measuring time.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodMeasuringTime (ViSession instrSession,
                                                         ViReal64 measuringTime)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_MTIM, measuringTime),
                                            2, "Measuring Time");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod AF Frequency Start Stop
 * Purpose:  This function configures the frequency range of the result 
 *           display AF spectrum using start frequency and stop frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodAFFrequencyStartStop(
    ViSession   instrSession,
    ViReal64    startFrequency,
    ViReal64    stopFrequency
)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_AF_START, startFrequency),
                                            2, "Start Frequency");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_AF_STOP, stopFrequency),
                                            3, "Stop Frequency");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod AF Frequency Center Span
 * Purpose:  This function configures the frequency range of the result 
 *           display AF spectrum using the center frequency and the frequency 
 *           span. This function modifies the AF Frequency Start and AF 
 *           Frequency Stop attributes as follows:
 *           
 *               AF Frequency Start = CenterFrequency - span/2
 *               AF Frequency Stop = CenterFrequency + span/2
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodAFFrequencyCenterSpan(
    ViSession   instrSession,
    ViReal64    centerFrequency,
    ViReal64    span
)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_AF_CENTER, centerFrequency),
                                            2, "Center Frequency");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_AF_SPAN, span),
                                            3, "Span");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: Configure Analog Demod AF Full Span
 * Purpose:  This function sets the maximum span for the display of the AF spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodAFFullSpan(ViSession   instrSession)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FMDEM_AF_FULL_SPAN, ""));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod AF Param
 * Purpose:  This function configures bandwidth resolution and span for 
 *           display of the AF spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodAFParam(
    ViSession   instrSession,
    ViReal64    resolutionBandwidth)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_BAND_RES, resolutionBandwidth),
                                            2, "Resolution Bandwidth");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: Configure Analog Demod RF Param
 * Purpose:  This function configures bandwidth resolution and span for 
 *           display of the RF spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodRFParam(
    ViSession   instrSession,
    ViReal64    resolutionBandwidth,
    ViReal64    span
)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_BAND_RES, resolutionBandwidth),
                                            2, "Resolution Bandwidth");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_SPEC_SPAN, span),
                                            3, "Span");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod RF Zoom
 * Purpose:  This function configures the frequency range for result display
 *           of RF spectrum determined from Analog Demod demodulation data.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureADemodRFZoom(
    ViSession   instrSession,
    ViReal64    zoom
)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_SPEC_ZOOM, zoom),
                                            2, "Zoom");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Analog Demod Currently Set Value
 * Purpose:  This function reads selected value from instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetADemodCurrentlySetValue(
    ViSession   instrSession,
    ViInt32 value,
    ViReal64*   result
)
{
    ViStatus    error   = VI_SUCCESS;

    ViReal64    l_value = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (value)
    {
        case RSZVL_VAL_FMDEM_RLEN:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_RLEN, &l_value),
                                                    3, "Result");
            break;
            
        case RSZVL_VAL_FMDEM_SRATE:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FMDEM_SRATE, &l_value),
                                                    3, "Result");
        break;
        default: viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Value");
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

    if (result)
        *result = l_value;
    else
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result");

    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Analog Demod Result Values
 * Purpose:  This function returns analog demodulation result values.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetADemodResultValues(
    ViSession   instrSession,
    ViInt32 demodulationType,
    ViInt32 resultType,
    ViUInt32   timeout,
    ViInt32 arraySize,
    ViReal64  resultValues[],
    ViInt32*    returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;

    ViUInt32    len;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    ViChar*     pbuffer = VI_NULL;
    ViReal32*   p;
    ViInt32     i, num, points_to_output;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K7"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);

    if (rszvl_invalidViInt32Range (instrSession, demodulationType, 0, 7))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Demodulation Type");
    }
    if (rszvl_invalidViInt32Range (instrSession, resultType, 1, 4))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result Type");
    }
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Timeout"); 
    
    sprintf (buffer, ":FORM REAL,32;*CLS;:SENS:ADEM:%s:RES? %s;*OPC\n",
             ADEMTypeArr[demodulationType], resultTypeArr[resultType]);

    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
    checkErr (rszvl_WaitForOPC (instrSession, timeout));

    checkErr (rszvl_readBinaryDataBlock (instrSession, &pbuffer, &len));

    checkErr( rszvl_CheckStatus (instrSession));

    num = len/4;
    p = (ViReal32*)pbuffer;
    points_to_output = (arraySize > num) ? num : arraySize;
    
    for (i=0; i<points_to_output; i++)
        resultValues[i] = (ViReal64) p[i];

    if (returnedValues)
        *returnedValues = num;



Error:
    Rs_UnlockSession(instrSession, VI_NULL);

    if (pbuffer)
        free (pbuffer);
    
    return error;
}

/*****************************************************************************
 * Function: Get Analog Demod Offset
 * Purpose:  This function calculates the Analog Demod offset of the current measured 
 *           data set.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetADemodOffset(
    ViSession   instrSession,
    ViInt32 resultType,
    ViReal64*   value
)
{
    ViStatus    error   = VI_SUCCESS;
    ViString    fmOffsetArr[]={"Imm","Aver"};
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K7"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);

    if (rszvl_invalidViInt32Range (instrSession, resultType, 0, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Result Type");
    }
    /*
    switch (resultType)
    {
        case RSZVL_VAL_FMDEM_IMM:
            checkErr (viPrintf (instrSession, "*CLS;:SENS:ADEM:FM:OFFS? IMM\n"));
        break;
        case RSZVL_VAL_FMDEM_AVG:
            checkErr (viPrintf(instrSession, "*CLS;:SENS:ADEM:FM:OFFS? AVER\n"));
        break;
    }
    
    checkErr (viScanf (instrSession, "%le", value));*/
    sprintf (buffer,"FMOff%s", fmOffsetArr[resultType]);
        
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_ADEM_FM_OFFSET, value),
                                                    3, "Value");
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: Get Analog Demod Marker Modulation Value
 * Purpose:  This function returns analog modulation measurement summary.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetADemodMarkerModulationValue(
    ViSession   instrSession,
    ViInt32 demodulationType,
    ViInt32 detectorType,
    ViInt32 trace,
    ViReal64*   value
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K7"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);

    if (rszvl_invalidViInt32Range (instrSession, demodulationType, 0, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Demodulation Type");
    }
    if (rszvl_invalidViInt32Range (instrSession, detectorType, 0, 3))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Detector Type");
    }
	if (rszvl_invalidViInt32Range (instrSession, trace, 1, 4))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Trace");
    }


	sprintf (buffer,"Analog%s,TR%ld,ResDet%s",
        ADEMTypeArr[demodulationType], trace,detectorTypeArr[detectorType]);
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_ADEM_SUMM_RES, value),
                                                    5, "Value");
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: Get Analog Demod Marker Measurement Value
 * Purpose:  This function returns analog modulation measurement summary.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetADemodMarkerMeasValue(
    ViSession   instrSession,
    ViInt32 measurement,
    ViInt32 trace,
    ViReal64*   value
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSZVL_IO_BUFFER_SIZE];
    ViInt32     attribute;
    ViInt32     attrs[] = {RSZVL_ATTR_FMDEM_AFR_RES,RSZVL_ATTR_FMDEM_FERR_RES,
                           RSZVL_ATTR_FMDEM_SIN_RES, RSZVL_ATTR_FMDEM_THD_RES,
                           RSZVL_ATTR_FMDEM_CARR_RES};

    if (rszvl_invalidViInt32Range (instrSession, measurement, 0, 4))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 4))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Trace");
    }

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    attribute = attrs[measurement];
    

    sprintf (buf1,"TR%ld", trace);

    viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buf1, attribute, value), 
                    4, "Value"); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*****************************************************************************
 *- Callback Definition   ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSZVL K7 - Analog Demodulator ------------------------------------*
 *****************************************************************************/
