/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver - K72 - 3GPP FDD Base Station Test
 *  VXIpnp Style Instrument Driver
 *  Original Release: March 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 *****************************************************************************/

/*****************************************************************************
 *  RSZVL K72 - 3GPP FDD Base Station Test
 *****************************************************************************/

#include "rszvl.h"   

static ViString trace3GBSArr[]={":TRAC? TRACE1",":TRAC? CTAB",":TRAC? CWCD",":TRAC? PWCD"};   
static ViString result3GBSWCDPArr[]={"PTOT","FERR","TFR","MACC",
        "PCD","EVMR","EVMP","CERR","CSL","SRAT","CHAN","CDP","CDPR","IQOF",
        "IQIM","MTYP","RHO","TOFF","PSYM","ACH"}; 
static ViString result3GBSCDPArr[]={"ACP","CPOW","MCAC","OBAN"};        

/*===========================================================================*/
/* Function: Set 3GPP FDD BS Mode
/* Purpose:  This function switches the instrument into 3GPP FDD BS mode. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Set3GPPBSMode(ViSession  instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_BWCD_MODE, ""));

	checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set 3GPP FDD BS Measurement
/* Purpose:  This function configures the 3GPP FDD UE additional 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Set3GPPBSMeasurement(ViSession   instrSession,
                                                ViInt32 measurement)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_MEAS, 
                measurement), 2, "Measurement");
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure 3GPP FDD BS WCDP Measurement
/* Purpose:  This function configures the 3GPP FDD UE measurement
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Configure3GPPBSMeasurement(ViSession instrSession,
                                                        ViInt32 scramblingCode,
                                                        ViInt32 powerReference,
                                                        ViInt32 syncType,
                                                        ViBoolean   normalize,
                                                        ViBoolean   hsdpaupa,
                                                        ViReal64    inactiveChannelTreshold,
                                                        ViBoolean   invertQ,
                                                        ViInt32 antenna)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_SCODE, 
                scramblingCode), 2, "Scrambling Code");
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_WCDP_POW_REF, 
                powerReference), 3, "Power Reference");
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_WCDP_SYNC_TYPE, 
                syncType), 4, "Sync Type");
    
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_3GBS_WCDP_NORMALIZE, 
                normalize), 5, "Normalize");

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_3GBS_WCDP_HSDPAUPA_STATE, 
                hsdpaupa), 6, "HSDPA/UPA");

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_3GBS_WCDP_INACT_CHAN_THR, 
                inactiveChannelTreshold), 7, "Inactive Channel Threshold");             
    
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_3GBS_WCDP_INVERT_Q, 
                invertQ), 8, "Invert Q");
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_ANTENNA, 
                antenna), 9, "Antenna");  
    
	checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure 3GPP FDD BS WCDP Results
/* Purpose:  This function configures the 3GPP FDD BS results
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Configure3GPPBSResults(ViSession instrSession,
                                                    ViInt32 channel,
                                                    ViInt32 cpichSlot)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_RESULTS_SELECT_CHAN, 
                channel), 2, "Channel");                
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_RESULTS_SELECT_CPICH_SLOT, 
                cpichSlot), 3, "CPICH Slot");    
    
	checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure 3GPP FDD BS WCDP Result Type
/* Purpose:  This function configures the 3GPP FDD BS result type
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Configure3GPPBSResultType(ViSession instrSession,
                                                     ViInt32 resultType)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_RESULT, 
        resultType), 2, "Result Type"); 
                    
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure 3GPP FDD BS WCDP Result Summary View
/* Purpose:  This function sets the view of the Code Domain Result Summary.
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Configure3GPPBSResultSummaryView (ViSession instrSession,
                                                             ViInt32 view)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GBS_RESULT_SUM_VIEW, 
                 view), 2, "View");
                 
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure 3GPP FDD BS WCDP Y Scale Division
/* Purpose:  This function defines the scaling of the y-axis in dB.
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Configure3GPPBSYScaleDivision (ViSession instrSession,
                                                          ViReal64 yScaleDivision)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_3GPP_PDIV, 
                yScaleDivision), 2, " Y Scale Division");
                 
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: 3GPP FDD BS A Sequence
/* Purpose:  This function starts the following sequence:
/*           - Adjusts the reference level.
/*           - Searches automatically the scrambling code that leads to the
/*             highest signal power and stores it as new scrambling code.
/*           - Changes into the Result Summary result display.
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Set3GPPBSASequence (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rszvl_SetOPCTimeout (instrSession, 30000));    
    
    viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_3GPP_ASEQ, 
                NULL));
                 
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    if (old_timeout > 0) 
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Search 3GPP FDD BS Scrambling Code
/* Purpose:  This function searches automatically the scrambling code that 
/*           leads to the highest signal power and stores it as new scrambling 
/*           code for further measurements.
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Search3GPPBSScramblingCode (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_3GPP_SCODE_SEARCH, 
                NULL));
                 
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure 3GPP FDD BS Spectrum Emission Mask
/* Purpose:  This function configures the spectrum emission mask settings.
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Configure3GPPBSESP (ViSession instrSession,
                                            ViInt32 limitLineType,
                                            ViInt32 limitLineValue,
                                            ViReal64 _30kHz1MHzTransition)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GPP_ESP_LIMIT_LINE_SELECT, 
                limitLineType), 2, "Limit Line Type");
	
	if (limitLineType == RSZVL_VAL_3GPP_ESP_MODE_MAN)
	{
		viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_3GPP_ESP_PREDEFINED_LIMIT_LINE, 
                limitLineValue), 2, "Limit Line Value");	
	}
	
	viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_3GPP_ESP_TRANSITION, 
                _30kHz1MHzTransition), 2, "30kHz 1MHz Transition");
	
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Restore 3GPP FDD BS Spectrum Emission Mask Limit Lines
/* Purpose:  This function restores the standard limit lines for the spectrum 
/*           emission mask measurement. All modifications made to the standard 
/*           limit lines are lost and the factory  set values are restored.
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Restore3GPPBSESPLimitLines (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_3GPP_ESP_LIMIT_LINE_RESTORE, 
                NULL));
                 
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: 3GPP FDD BS Adjust Level
/* Purpose:  This function adjusts the reference level to the measured channel 
/*           power. This ensures that the settings of the RF attenuation and the
/*           reference level are optimally adjusted to the signal level
/*           without overloading the R&S ZVL or limiting the dynamic range by
/*           an S/N ratio that is too small..
/*===========================================================================*/   
ViStatus _VI_FUNC rszvl_Adjust3GPPBSLevel (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_3GPP_LEVEL_ADJUST, 
                NULL));
                 
    checkErr( rszvl_CheckStatus (instrSession));              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set 3GPP FDD BS Marker To Channel
/* Purpose:  This function sets the marker to selected channel
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Set3GPPBSMarkerToChannel(ViSession   instrSession,
                                                    ViInt32 channel)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	switch (channel){
        case RSZVL_VAL_3GPP_MARKER_CPICH:
            viCheckErr ( rszvl_SetAttributeViString (instrSession, "Cpich", 
                RSZVL_ATTR_3GBS_MARKER_TO_CHANNEL, VI_NULL)); 
        break;
        case RSZVL_VAL_3GPP_MARKER_PCCPCH:
            viCheckErr ( rszvl_SetAttributeViString (instrSession, "Pccpch", 
                RSZVL_ATTR_3GBS_MARKER_TO_CHANNEL, VI_NULL)); 
        break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }    
    checkErr( rszvl_CheckStatus (instrSession));       
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set 3GPP FDD BS Delta Marker To Channel
/* Purpose:  This function sets the delta marker to selected channel
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Set3GPPBSDeltaMarkerToChannel (ViSession instrSession,
                                                          ViInt32 channel)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    switch (channel){
        case RSZVL_VAL_3GPP_MARKER_CPICH:
            viCheckErr ( rszvl_SetAttributeViString (instrSession, "Cpich", 
                RSZVL_ATTR_3GBS_REFERENCE_MARKER_TO_CHANNEL, VI_NULL)); 
        break;
        case RSZVL_VAL_3GPP_MARKER_PCCPCH:
            viCheckErr ( rszvl_SetAttributeViString (instrSession, "Pccpch", 
                RSZVL_ATTR_3GBS_REFERENCE_MARKER_TO_CHANNEL, VI_NULL)); 
        break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }    
    checkErr( rszvl_CheckStatus (instrSession));       
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read  3GPP FDD BS Trace
/* Purpose:  This function initiates a signal acquisition based on the 
/*           present instrument configuration.  It then waits for the 
/*           acquisition to complete, and returns the trace as an array of 
/*           amplitude values. The amplitude array returns data that 
/*           represent the amplitude of the signals of the sweep from the 
/*           start frequency to the stop frequency (in frequency domain, in 
/*           time domain the amplitude array is ordered from beginning of 
/*           sweep to end).  The Amplitude Units attribute determines the 
/*           units of the points in the amplitude array.  This function 
/*           resets the sweep count.
/*           
/*           Notes:
/*           
/*           (1) If the spectrum analyzer did not complete the acquisition 
/*           within the time period the user specified with the maxTime 
/*           parameter, the function returns Max Time Exceeded error.
/*           
/*           (2) If the data was captured in an uncalibrated spectrum 
/*           analyzer mode, the function will generate a Measure Uncalibrated 
/*           warning flag. 
/*           
/*           Attribute(s):
/*           no attribute
/*           
/*           Remote-control command(s):
/*           INITiate[:IMMediate]
/*           TRACe<1|2>[:DATA] TRACe1 | CWCDp | CTABle | PWCDp
/*           :FORM REAL,32
/*           *CLS
/*           
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Get3GPPBSTrace(ViSession instrSession,
                                            ViInt32 trace,
                                            ViInt32 arrayLength,
                                            ViInt32*    actualPoints,
                                            ViReal64    values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *exBuf = VI_NULL;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

     if (!(strstr (buffer, "K72")))    
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (rszvl_invalidViInt32Range (instrSession, trace, RSZVL_VAL_3GBS_TRAC_1, RSZVL_VAL_3GBS_TRAC_PWCD))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    if (!actualPoints)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Actual Points");          
    
    viCheckErr (rszvl_dataReadTrace (instrSession, trace3GBSArr[trace], arrayLength, 
                    values, actualPoints));
                    
    checkErr( rszvl_CheckStatus (instrSession));           
    
Error:
    if (exBuf) free (exBuf);
    Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/*===========================================================================*/
/* Function: Get 3GPP FDD BS WCDP Result
/* Purpose:  This function returns the WCDP measurement Results
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Get3GPPBSResult(ViSession    instrSession,
                                            ViInt32     type,
                                            ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K72")))    
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_WCDP_RES_PTOT, RSZVL_VAL_WCDP_RES_TOFF))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
    if (!result)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result");        

    viCheckErr (viQueryf (instrSession, "CALC:MARK:FUNC:WCDP:RES? %s\n","%le", 
                result3GBSWCDPArr[type], result));
    checkErr( rszvl_CheckStatus (instrSession));    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get 3GPP FDD BS CDP Result
/* Purpose:  This function returns the Phase Spot Noise Y Position.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Get3GPPBSCDPResult(ViSession instrSession,
                                            ViInt32     type,
                                            ViInt32     arraySize,
                                            ViReal64    results[],
                                            ViInt32     *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf[RSZVL_IO_BUFFER_SIZE]; 
    ViInt32     i;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff = VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buf));

    if (!(strstr (buf, "K72")))    
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (rszvl_invalidViInt32Range (instrSession, type, 0, 3))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
    if (arraySize<0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Array Size"); 
    if ((arraySize>0) && (results == NULL))    
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Results");
    
    viCheckErr( viPrintf (instrSession, "CALC:MARK:FUNC:POW:RES? %s\n", 
        result3GBSCDPArr[type]));
    
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL));  
    
    pbuff = strtok (buffer, ",");
    i = 0;
    do{
        if (i<arraySize) results[i] = atof (pbuff);
        pbuff = strtok (NULL, ",");
        i++;
    }while(pbuff != NULL);   
    
    if (returnedValues) *returnedValues = i;
    
    checkErr( rszvl_CheckStatus (instrSession));    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get 3GPP FDD BS CDP Scrambling Code Results
/* Purpose:  This function returns result table of the automatic search sequence 
/*           containing the highest power values calculated and the corresponding
/*           scrambling codes.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Get3GPPBSCDPScramblingCodeResult (ViSession instrSession,
                                                             ViInt32 arraySize,
                                                             ViInt32 scrambingCode[],
                                                             ViReal64 power[],
                                                             ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pbuffer = NULL;
    ViChar*     pstring_value = NULL; 
    ViInt32     idx=0;
    ViUInt32    retCnt;
    ViChar      buf[RSZVL_IO_BUFFER_SIZE];  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));  
    
	checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buf));

    if (!(strstr (buf, "K72")))    
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);;
    }
    if ((arraySize>0)&&(!scrambingCode))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Scrambling Code");  
    if ((arraySize>0)&&(!power))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "WiPowerdow");  
    
    viCheckErr (viPrintf(instrSession, "*CLS;:CDP:LCODe:SEAR:LIST?\n"));   
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &pbuffer, &retCnt));
    
    if (strlen(pbuffer) > 1)
    {
        pstring_value = strtok(pbuffer, ",");

        while (pstring_value)
        {
            if (idx<arraySize)
            {
                sscanf (pstring_value, "%ld,%*[^,],%Le", &scrambingCode[idx], &power[idx]);
            }
            pstring_value = strtok(NULL, ",");   
            pstring_value = strtok(NULL, ",");   
            pstring_value = strtok(NULL, ",");          
            idx++;
        }
    }
    if (returnedValues)
    {
        *returnedValues=idx;
    }       
    
    checkErr( rszvl_CheckStatus (instrSession));     
    
Error:
    if (pbuffer) free(pbuffer);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get 3GPP FDD BS CDP List Evaluation Result
/* Purpose:  This function queries the list evaluation results. All results 
/*           are float values.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_Get3GPPBSCDPListEvaluationResult (ViSession instrSession,
                                                             ViInt32 noOfValues,
                                                             ViReal64 rangeNumber[],
                                                             ViReal64 startFrequency[],
                                                             ViReal64 stopFrequency[],
                                                             ViReal64 resolutionBandwidth[],
                                                             ViReal64 frequency[],
                                                             ViReal64 powerAbs[],
                                                             ViReal64 powerRel[],
                                                             ViReal64 delta[],
                                                             ViInt32 limitCheck[],
                                                             ViReal64 reserved1[],
                                                             ViReal64 reserved2[],
                                                             ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    //ViChar      *buffer = VI_NULL;
    //ViChar      *pbuf=VI_NULL;
    ViInt32     count=0;
    ViUInt32    retCnt;
    ViUInt32     i;
    ViChar      buf[RSZVL_IO_BUFFER_SIZE];  
    ViReal64    *values = NULL;
    ViInt32     arrayLength,
                actualPoints;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));  
    
	checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buf));

    if (!(strstr (buf, "K72")))    
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);;
    }

    if (rangeNumber == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Range number");
    if (startFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Start Frequency"); 
    if (stopFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Stop Frequency"); 
    if (resolutionBandwidth == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Resolution Bandwidth"); 
    if (frequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Frequency"); 
    if (powerAbs == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Power Abs");
    if (powerRel == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Power Rel"); 
    if (delta == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Delta"); 
    if (limitCheck == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Limit Check");
    if (reserved1 == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Reserved 1");
    if (reserved2 == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Reserved 2"); 
    
    arrayLength = noOfValues*11;
    values = malloc(arrayLength*sizeof (ViReal64));
    //viCheckErr (viPrintf (instrSession, "TRAC:DATA? LIST\n"));
    //viCheckErr( Rs_ReadDataUnknownLength(instrSession, &buffer, &retCnt));
    //values are returned in REAL,32 format
    viCheckErr (rszvl_dataReadTrace (instrSession, ":TRAC:DATA? LIST", arrayLength, 
                    values, &actualPoints));
    count=0;
    i=0;
    retCnt=(arrayLength<actualPoints)?arrayLength: actualPoints;
    
    do{
        rangeNumber[count] = values[i++];
        startFrequency[count] = values[i++];
        stopFrequency[count] = values[i++];
        resolutionBandwidth[count] = values[i++];
        frequency[count] = values[i++];
        powerAbs[count] = values[i++];
        powerRel[count] = values[i++];
        delta[count] = values[i++];
        limitCheck[count] = (ViInt32) values[i++];
        reserved1[count] = values[i++];
        reserved2[count] = values[i++];
        count++;
    }while (i<retCnt);  
 
    if (returnedValues)
        *returnedValues = actualPoints/11;
     
    checkErr( rszvl_CheckStatus (instrSession));     
Error:
    if (values) free(values);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *- User defined callbacks --------------------------------------------------*
 *****************************************************************************/
