/*===========================================================================*
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver       
 *  RSZVL K8 - Bluetooth Analyzer Group Functions
 *
 *  Original Release: March 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *

 *===========================================================================*/

#include "rszvl.h"    

/*****************************************************************************
 * Function:    Bluetooth Mode
 * Purpose:     This function selects the Bluetooth analyzer mode
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_BTOMode (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_BTO_MODE, ""));
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Select BTO Trace
 * Purpose:     This function selects the measurement curve for evaluation 
 *              of the modulation characteristics.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelectBTOTrace (ViSession instrSession,
                                        ViInt32 trace)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    sprintf (buffer, "TR%ld", trace);
    viCheckErr(rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_BTO_TRAC, ""));
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTOMeasurement
 * Purpose:     This function configures the common parameters of measurement 
 *              according Bluetooth standard.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOMeasurement (ViSession instrSession,
                                                    ViInt32 channel,
                                                    ViInt32 outputPowerClass,
                                                    ViInt32 samplesPerSymbols,
                                                    ViInt32 packetType,
                                                    ViReal64 anntennaGain)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_CHANNEL, 
                                                channel), 2, "Channel");  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_PCL, 
                                                outputPowerClass), 3, "Output Power Class");                                                  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_PRAT,  
                                  samplesPerSymbols), 4, "Samples Per Symbol");                                  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_PTYPE, 
                                  packetType), 5, "Packet Type");                                  
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_EGA_INP, 
                                  anntennaGain), 6, "Antenna Gain");                                   
                                  
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO AveragingMethod
 * Purpose:     This function configures the averaging area of the average 
 *              power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOAveragingMethod (ViSession instrSession,
                                                        ViReal64 startPosition,
                                                        ViReal64 stopPosition)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "Start", RSZVL_ATTR_BTO_POW_AVER,
                                  startPosition), 2, "Start Position");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "Stop", RSZVL_ATTR_BTO_POW_AVER,
                                  stopPosition), 2, "Stop Position");

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    ConfigureBTOSynchronization
 * Purpose:     This function configures the synchronization of the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOSynchronization (ViSession instrSession,
                                                        ViBoolean findSync,
                                                        ViInt32 syncOffset,
                                                        ViBoolean findBurst,
                                                        ViReal64 burstOffset,
                                                        ViBoolean searchLengthAuto,
                                                        ViReal64 searchLength)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_BTO_SEARCH_SYNC,  
                                  findSync), 2, "Find Sync");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_SYNC_OFFS,  
                                  syncOffset), 3, "Sync Offset");                                  
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_BTO_SEARCH_PULS,  
                                  findBurst), 4, "Find Burst");                                  
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_PULS_OFFS,  
                                  burstOffset), 5, "Burst Offset");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_BTO_SEARCH_TIME_AUTO,  
                                  searchLengthAuto), 6, "Search Length Auto");   
    if (!searchLengthAuto)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_SEARCH_TIME,  
                                  searchLength), 7, "Search Length");                                       
    }                                  
                                  

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    ConfigureBTOLAP
 * Purpose:     This function configures the LAP for synchronization of the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOLAP (ViSession instrSession,
                                                        ViInt32 lowest24Bits_ofDevAddress)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_LAP, 
                                  lowest24Bits_ofDevAddress), 2, 
                                  "Lowest 24 Bits of Dev. Address");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Measurement Mode
 * Purpose:     This function selects the current measurement according to the
 *              Bluetooth standard.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOMeasurementMode (ViSession instrSession,
                                                        ViInt32 measurementMode)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO,
                                               measurementMode), 2,"Measurement Mode");
                                               
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function:    Configure BTO Number Of Adjacent Channels
 * Purpose:     This function selects the number of adjacent channel pairs
 *              during the measurement of the adjacent channel power.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTONumberOfAdjacentChannels (ViSession instrSession,
                                                               ViInt32 number_ofAdjacentChannels)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_ACLR_ACP,
                                               number_ofAdjacentChannels), 2,"Number of Adjacent Channels");
    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Receive Filter
 * Purpose:     This function selects the receive filter for the signal to
 *              measure.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOReceiveFilter (ViSession instrSession,
                                                      ViInt32 filter)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_FILT_MEAS,
                                  filter), 2 , "Filter");                                                 

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Sweep Coupling Auto
 * Purpose:     This function activates auto coupling of selected coupling
 *              method.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOSweepCouplingAuto (ViSession instrSession,
                                                          ViInt32 sweepCoupling)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (sweepCoupling)
    {
        case  RSZVL_VAL_COUPLING_RBW:
            attribute = RSZVL_ATTR_BTO_BWID_AUTO;
            break;
            
        case RSZVL_VAL_COUPLING_VBW:
            attribute = RSZVL_ATTR_BTO_VID_AUTO;
            break;
            
        case RSZVL_VAL_COUPLING_SWEEP_TIME:
            attribute = RSZVL_ATTR_BTO_SWE_TIME_AUTO;
            break;
        default:
            viCheckParm (VI_ERROR_PARAMETER3, 2, "Sweep Coupling");
    }
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                attribute, VI_TRUE),
                                                1, "Dummy");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Sweep Coupling
 * Purpose:     This function configures the coupling value
 *              of the currently selected Bluetooth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOSweepCoupling (ViSession instrSession,
                                                      ViInt32 sweepCoupling,
                                                      ViReal64 couplingValue)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
    ViAttr      attribute2 = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (sweepCoupling)
    {
        case RSZVL_VAL_COUPLING_RBW:
            attribute = RSZVL_ATTR_BTO_BWID;
            attribute2 = RSZVL_ATTR_BTO_BWID_AUTO;
            break;
            
        case RSZVL_VAL_COUPLING_VBW:
            attribute = RSZVL_ATTR_BTO_VID;
            attribute2 = RSZVL_ATTR_BTO_VID_AUTO;      
            break;
            
        case RSZVL_VAL_COUPLING_SWEEP_TIME:
            attribute = RSZVL_ATTR_BTO_SWE_TIME;
            attribute2 = RSZVL_ATTR_BTO_SWE_TIME_AUTO;      
            break;
        default:
            viCheckParm (VI_ERROR_PARAMETER3, 2, "Sweep Coupling");
    }
    viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                attribute2, VI_FALSE));
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                            attribute, couplingValue),
                                            3, "Coupling Value");                                  
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Acquisition
 * Purpose:     This function configures the acquisition attributes of the
 *              currently selected Bluetooth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOAcquisition (ViSession instrSession,
                                                    ViBoolean sweepModeContinuous,
                                                    ViInt32 numberOfSweeps)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "Win1", 
                                                RSZVL_ATTR_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 2, "Sweep Mode Continuous");
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_BTO_SWE,
                                                numberOfSweeps), 3, "Number Of Sweeps");                                                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Trace Mode
 * Purpose:     This function configures the way of displaying and evaluating 
 *              the measurement curves for the selected Bluetooth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOTraceMode (ViSession instrSession,
                                              ViInt32 trace, 
                                              ViInt32 traceMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	sprintf (buffer, "TR%ld", trace);
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_BTO_TRAC_MOD,
                                              traceMode), 3,"Trace Mode");
    
	checkErr( rszvl_CheckStatus (instrSession));
                                               
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Trace Detector
 * Purpose:     This function configures the detector for the currently 
 *              selected Bluetooth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOTraceDetector (ViSession instrSession,
                                              ViInt32 trace,
                                              ViInt32 traceDetector)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 4))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    }
    sprintf (buffer, "TR%ld", trace);
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_BTO_TRAC_DET,
                                               traceDetector), 3,"Trace Detector");
    checkErr( rszvl_CheckStatus (instrSession));
                                               
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Packet Bytes SCO
 * Purpose:     This function selects the number of payload bytes that are 
 *              transmitted in a packet.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOPacketBytesSCO (ViSession instrSession,
                                                       ViInt32 packetBytesSCO)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_PBSC,
                                              packetBytesSCO), 2,"Packet Bytes SCO");
    checkErr( rszvl_CheckStatus (instrSession));
                                               
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Display Y-Axis Scaling
 * Purpose:     This function defines the scaling of the yaxis in 
 *              the currently selected unit.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTODisplayYAxisScaling (ViSession instrSession,
                                                            ViReal64 scaling)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_PDIV,
                                              scaling), 2,"Scaling");
    checkErr( rszvl_CheckStatus (instrSession));
                                               
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:   Configure BTO Analog Demodulation Zoom
 * Purpose:    This function sets analog demodulation parameters such as 
 *             zoom state, start time.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOAnalogDemodulationZoom (ViSession instrSession,
                                                               ViBoolean zoomState,
                                                               ViReal64 startTime)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_BTO_ZOOM_STATE,
                                              zoomState), 2,"Zoom State");
    if (zoomState == VI_TRUE)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_ZOOM_START,
                                              startTime), 3,"Start Time");
    }       
    checkErr( rszvl_CheckStatus (instrSession));
                                               
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *- Measurement -------------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function:    Get BTO Mesurement Status
 * Purpose:     This function returns the status of a preceeding measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOMesurementStatus (ViSession instrSession,
                                                   ViInt32 *status)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));     

    viCheckParm (rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_STAT, status), 2, "Status");
    
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/*****************************************************************************
 * Function:    Get BTO Packet Info
 * Purpose:     This function returns the packet info of of the packet 
 *              analyzed by the preceeding measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOPacketInfo (ViSession instrSession,
                                             ViInt32 *type, ViInt32 *length)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));  

    viCheckParm (rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_PTYP, type), 2, "Type");
    viCheckParm (rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_PLEN, length), 3, "Length");
    
    checkErr( rszvl_CheckStatus (instrSession));   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);   
    return error;
}



/*****************************************************************************
 * Function:    Get BTO Output Power
 * Purpose:     This function returns the output power results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOOutputPower (ViSession instrSession,
                                              ViInt32 type, 
                                              ViReal64 *power)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ( (type<RSZVL_VAL_MEASTYPE_MIN) || (type>RSZVL_VAL_MEASTYPE_PEAK) )
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Type");
    switch (type){
        case RSZVL_VAL_MEASTYPE_MIN:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Min", RSZVL_ATTR_BTO_OPOW_AVER, power),
                                                        3, "Power");
        break;   
        case RSZVL_VAL_MEASTYPE_MAX:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Max", RSZVL_ATTR_BTO_OPOW_AVER, power),
                                                        3, "Power");
        break;   
        case RSZVL_VAL_MEASTYPE_PEAK:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_OPOW_PEAK, power),
                                                        3, "Power");
        break;     
    }
    checkErr( rszvl_CheckStatus (instrSession));         
        
Error:
    Rs_UnlockSession(instrSession, VI_NULL);      
    return error;
}


/*****************************************************************************
 * Function:    Get BTO Number Of Exceptions
 * Purpose:     This function returns the number of exceptions which occured 
 *              during the adjacent channel power measurement according to 
 *              the Bluetooth specification.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTONumberOfExceptions (ViSession instrSession,
                                                     ViInt32 *numberOfExceptions)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_ACLR_EXC, numberOfExceptions),
                                                        2, "Number Of Exceptions");
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Power Of Channels
 * Purpose:     This function the power of selected adjacent channels of 
 *              the Bluetooth standard.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOPowerOfChannels (ViSession instrSession,
                                                  ViInt32 arraySize,
                                                  ViReal64 power[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     pstring_value;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K8"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (arraySize<0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Array Size");            
    if (!power)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Power List"); 
    
    viCheckErr( viPrintf (instrSession,":CALC:BTO:ACLR:LIST?\n")); 
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 
    pstring_value = strtok(pbuffer, ",");

    i=0;
    while (pstring_value)
    {
        if (i < arraySize)
        {
            sscanf(pstring_value, "%le", &power[i]);
        }
        pstring_value = strtok(NULL, ",");
        i++;
    }

    free(pbuffer);

    if (returnedValues)
        *returnedValues = i;
            
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Frequency Deviation
 * Purpose:     This function returns the value of frequency deviation for 
 *               varying bit patterns of the payload.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOFrequencyDeviation (ViSession instrSession,
                                                     ViInt32 frequencyDeviation,
                                                     ViInt32 type,
                                                     ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ( (type<RSZVL_VAL_MEASTYPE_MAXMIN) || (type>RSZVL_VAL_MEASTYPE_AVERMAX) )
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Type");
        
    
    switch (type){
        case RSZVL_VAL_MEASTYPE_MAXMIN:
            sprintf (buffer, "F%ld,Min", frequencyDeviation); 
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer,
                                                     RSZVL_ATTR_BTO_MCH_DF_MAX,
                                                     value), 4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_MAXMAX:
            sprintf (buffer, "F%ld,Max", frequencyDeviation);   
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer,
                                                     RSZVL_ATTR_BTO_MCH_DF_MAX,
                                                     value), 4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_MAXAVER:
            sprintf (buffer, "F%ld,Aver", frequencyDeviation);   
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer,
                                                     RSZVL_ATTR_BTO_MCH_DF_MAX,
                                                     value), 4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_AVERMIN:
            sprintf (buffer, "F%ld,Min", frequencyDeviation);
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer,
                                                     RSZVL_ATTR_BTO_MCH_DF_AVER,
                                                     value), 4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_AVERMAX:
            sprintf (buffer, "F%ld,Max", frequencyDeviation);
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer,
                                                     RSZVL_ATTR_BTO_MCH_DF_AVER,
                                                     value), 4, "Value");
        break;
        
    }
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Ratio Average Frequency Deviation
 * Purpose:     This function returns the value of ratio of the average 
 *              frequency deviations for varying bit patterns of the payload.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTORatioAverageFrequencyDeviation (ViSession instrSession,
                                                                 ViInt32 type,
                                                                 ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((type!=RSZVL_VAL_MEASTYPE_MIN)&&(type!=RSZVL_VAL_MEASTYPE_MAX)&&(type!=RSZVL_VAL_MEASTYPE_AVER))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Type");
   
    switch (type){
        case RSZVL_VAL_MEASTYPE_MIN:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Min", RSZVL_ATTR_BTO_MCH_RAT, value),
                                                    4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Max", RSZVL_ATTR_BTO_MCH_RAT, value), 
                                                    4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Aver", RSZVL_ATTR_BTO_MCH_RAT, value), 
                                                    4, "Value");
        break;                                                    
    }
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Init Carr Freq Tol
 * Purpose:     This function returns value of the Initial Carrier 
 *              Frequency Tolerance.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOInitCarrFreqTol (ViSession instrSession,
                                                  ViInt32 type, 
                                                  ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((type!=RSZVL_VAL_MEASTYPE_MIN)&&(type!=RSZVL_VAL_MEASTYPE_MAX)&&(type!=RSZVL_VAL_MEASTYPE_AVER))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Type");
   
    switch (type){
        case RSZVL_VAL_MEASTYPE_MIN:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Min", RSZVL_ATTR_BTO_ICFT,value),
                                                    4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Max", RSZVL_ATTR_BTO_ICFT, value), 
                                                    4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Aver", RSZVL_ATTR_BTO_ICFT, value), 
                                                    4, "Value");
        break;                                                    
    }
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Percentage Deviation
 * Purpose:     This function returns the percentage of measurements of 
 *              the frequency deviation.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOPercentageDeviation (ViSession instrSession,
                                                      ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));  
    
    viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_MCH_DF2_PERC,
                                                value), 2, "Value");
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Carr Freq Drift
 * Purpose:     This function returns the Carrier Frequency Drift.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOCarrFreqDrift (ViSession instrSession,
                                                ViInt32 type, 
                                                ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((type!=RSZVL_VAL_MEASTYPE_NONE)&&(type!=RSZVL_VAL_MEASTYPE_MAX))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Type");
   
    switch (type){
        case RSZVL_VAL_MEASTYPE_NONE:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_CFDR_RATE,value),
                                                    4, "Value");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_CFDR_MAX,value),
                                                    4, "Value");
        break;
    }
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Adjust BTO SE Gate Setting
 * Purpose:  This function adjusts the gate settings for the In-band 
 *           Spurious Emissions automatically.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_AdjustBTOSEGateSetting (ViSession instrSession)
{    
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_BTO_IBS_GATE_AUTO, NULL));
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/*****************************************************************************
 * Function: Configure BTO SE Num Adjacent Channels
 * Purpose:  This function selects the number of adjacent channel pairs during 
 *           the In-band Spurious Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOSENumAdjacentChannels (ViSession instrSession,
                                                              ViInt32 number)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_IBS_ACP,number),
                                                    2, "Number");
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Get BTO Channel Power of SE Measurement
 * Purpose:  This function returns the power of the TX channel in the In-band Spurious
 *           Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOChannelPowerOfSEMeasurement (ViSession instrSession,
                                                              ViInt32 parameter,
                                                              ViReal64 *value)
{
   ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    switch (parameter){
        case RSZVL_VAL_BTO_SE_TX:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_IBS_TXCHANNEL,value),
                                                    4, "Value");
        break;
        case RSZVL_VAL_BTO_SE_HADJ_LOW:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Low", RSZVL_ATTR_BTO_IBS_HADJ,value),
                                                    4, "Value");
        break;
        case RSZVL_VAL_BTO_SE_HADJ_UPP:
            viCheckParm (rszvl_GetAttributeViReal64 (instrSession, "Upp", RSZVL_ATTR_BTO_IBS_HADJ,value),
                                                    4, "Value");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Parameter");       
    }
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Get BTO SE Power Of Adjacent Channels
 * Purpose:  This function returns the power of all adjacent channels in the 
 *           Inband Spurious Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOSEPowerOfAdjacentChannels (ViSession instrSession,
                                                          ViInt32 arraySize,
                                                          ViReal64 power[],
                                                          ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     pstring_value;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K8"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (arraySize<0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Array Size");            
    if (!power)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Power List"); 
    
    viCheckErr( viPrintf (instrSession,":CALC:BTO:IBS?\n")); 
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 
    pstring_value = strtok(pbuffer, ",");

    i=0;
    while (pstring_value)
    {
        if (i < arraySize)
        {
            sscanf(pstring_value, "%le", &power[i]);
        }
        pstring_value = strtok(NULL, ",");
        i++;
    }

    free(pbuffer);

    if (returnedValues)
        *returnedValues = i;
            
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO SE Number of Exceptions
 * Purpose:     This function returns the number of results that exceeded 
 *              the specified limits in the In-band Spurious Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOSENumberOfExceptions (ViSession instrSession,
                                                       ViInt32 *exceptions)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_IBS_EXC,
                                  exceptions), 2 , "Exceptions");                                                 

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO CFS Block Count
 * Purpose:     This function sets the number of blocks to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTOCFSBlockCount (ViSession instrSession,
                                                      ViInt32 blockCount)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_CFST_BLOCK_COUNT,
                                  blockCount), 2 , "Block Count");                                                 

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Carrier Frequency Stability Deviation
 * Purpose:     This function returns value of the Carrier Frequency 
 *              Stability Frequency Error.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOCFSDeviation (ViSession instrSession,
                                                             ViInt32 parameter,
                                                             ViInt32 type,
                                                             ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    switch (type){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }       
    switch (parameter){ 
        case RSZVL_VAL_BTO_CFS_FERR_INITIAL: 
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_CFST_FERR_INITIAL,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_CFS_FERR_BLOCK:   
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_CFST_FERR_BLOCK,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_CFS_FERR_TOTAL:   
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_CFST_FERR_TOTAL,
                                  value), 4 , "Value");                                                 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Parameter"); 
    }       
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Carrier Frequency Stability Modul. Accuracy
 * Purpose:     This function returns value of the Carrier Frequency 
 *              Stability Modulation Accuracy.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTOCFSModulAccuracy (ViSession instrSession,
                                                                 ViInt32 parameter,
                                                                 ViInt32 type,
                                                                 ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";

    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    switch (type){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }       
    switch (parameter){ 
        case RSZVL_VAL_BTO_CFS_DEVM_RMS: 
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_CFST_DEVM_RMS,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_CFS_DEVM_99PCT:   
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_BTO_CFST_DEVM_99PCT,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_CFS_DEVM_PEAK:    
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_CFST_DEVM_PEAK,
                                  value), 4 , "Value");                                                 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Parameter"); 
    }       
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Differential Phase Encoding Values
 * Purpose:     This function returns the measured valeus of the differential 
 *              phase encoding.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTODifferentialPhaseEncodingValues (ViSession instrSession,
                                                                  ViInt32 parameter,
                                                                  ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    switch (parameter){ 
        case RSZVL_VAL_BTO_DPEN_TESTED:  
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_DPEN_PACKETS,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_DPEN_BER: 
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_DPEN_BER,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_DPEN_PASSED:  
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_BTO_DPEN_PACKETS_PASSED,
                                  value), 4 , "Value");                                                 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Parameter"); 
    }       
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/*****************************************************************************
 * Function:    Configure BTO RTP Power Measurement
 * Purpose:     This function configurest the start and stop time for power 
 *              measurement on slected packet sections.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBTORTPMeasurement (ViSession instrSession,
                                                            ViInt32 packetSection,
                                                            ViReal64 start,
                                                            ViReal64 stop)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    switch (packetSection){
        case RSZVL_VAL_BTO_RTP_GFSK:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "SECTION_TIME_GFSK,Start", RSZVL_ATTR_BTO_RTP_TIME,
                                  start), 3 , "Start");                                                 
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "SECTION_TIME_GFSK,Stop", RSZVL_ATTR_BTO_RTP_TIME,
                                  stop), 4 , "Stop");   
        break;
        case RSZVL_VAL_BTO_RTP_DPSK:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "SECTION_TIME_DPSK,Start", RSZVL_ATTR_BTO_RTP_TIME,
                                  start), 3 , "Start");                                                 
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "SECTION_TIME_DPSK,Stop", RSZVL_ATTR_BTO_RTP_TIME,
                                  stop), 4 , "Stop");   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packet Section");    
    }
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Relative Transmission Power
 * Purpose:     This function returns the transmission power of the Relative 
 *              Transmission Power measurement.

 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetBTORelativeTransmissionPower (ViSession instrSession,
                                                            ViInt32 parameter,
                                                            ViInt32 modificator,
                                                            ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]="";

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    switch (modificator){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Modificator");  
    }       
    switch (parameter){ 
        case RSZVL_VAL_BTO_RTP_DPSK:
            strcat(buffer,",SECTION_DPSK");
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_RTP,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_RTP_GFSK:
            strcat(buffer,",SECTION_GFSK");  
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_RTP,
                                  value), 4 , "Value");                                                 
        break;
        case RSZVL_VAL_BTO_RTP_RATIO:    
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_BTO_RTP_RATIO,
                                  value), 4 , "Value");                                                 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Parameter"); 
    }       
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/*****************************************************************************
 *- User defined callbacks --------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSZVL K8 - Bluetooth Analyzer Group Functions --------------------*
 *****************************************************************************/
