/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver - K9 Power Meter
 *  VXIpnp Style Instrument Driver
 *  Original Release: February 2009
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 *****************************************************************************/

#include "rszvl.h"

/*****************************************************************************
 *- RSZVL K9 -  Power Meter -------------------------------------------------*
 *****************************************************************************/

/*=========================================================================== 
 * Function: PWM Mode
 * Purpose:  This function switches measurements with a power sensor on or 
 *           off.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_PWMMode(ViSession  instrSession,
                                         ViBoolean  state)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_PMET_STATE, state), 2, "State");
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================* 
 * Function: Configure PWM Measurement
 * Purpose:  This function configures the power meter measurement 
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMMeasurement(ViSession   instrSession,
                                                    ViInt32     measurementTime, 
                                                    ViInt32     coupling,
                                                    ViReal64    frequency)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_PMET_MEAS_TIME, measurementTime), 2, "Measurement Time");
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_PMET_COUPLING, coupling), 3, "Coupling");
    
	if (coupling == RSZVL_VAL_PMET_COUP_OFF)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_PMET_FREQ, frequency), 4, "Frequency");    
    }    
    
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Configure PWM Relative
 * Purpose:  This function configures the relative measurement.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMRelative(ViSession   instrSession,
                                                ViReal64    referenceValue,
                                                ViInt32     unit)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	checkErr( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_PMET_RELATIVE, VI_TRUE));
    
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_PMET_REL, referenceValue), 2, "Reference Value");
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_PMET_UNIT_REL, unit), 3, "Unit");
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Configure PWM Absolute
 * Purpose:  This function configures the relative measurement.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMAbsolute(ViSession   instrSession,
                                                ViInt32 unit)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	checkErr( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_PMET_RELATIVE, VI_FALSE));
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_PMET_UNIT_ABS, unit), 2, "Unit");
    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Configure PWM Reference Level Offset State
 * Purpose:  This function defines whether the reference level offset set for
 *           the analyzer is taken into account for the measured power or not.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMReferenceLevelOffsetState (ViSession instrSession,
                                                               ViBoolean state)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_PMET_REF_LEVEL_OFFSET_STATE, 
                                              state), 2, "State");
    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Configure PWM Averaging
 * Purpose:  This function configures the averaging.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMAveraging (ViSession instrSession,
                                               ViBoolean manualAveraging,
                                               ViInt32 averagingCount)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_PMET_AVERAGE_AUTO, 
                                              manualAveraging), 2, "Manual Averaging");
    
    if (manualAveraging == VI_TRUE)
	{
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_PMET_AVERAGE_COUNT, 
                                              averagingCount), 3, "Averaging Count");
	}
	
	checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: ConfigurePWM Measure To Reference
 * Purpose:  This function accepts the current result as a reference value 
 *           for relative measurements.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMMeasToRef(ViSession   instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	checkErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_PMET_REL_AUTO, ""));
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Configure PWM Zero
 * Purpose:  This function zeroes the power sensor.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigurePWMZero(ViSession   instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_PMET_ZERO, ""));
    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Read PWM Result
 * Purpose:  This function triggers a measurement with the power sensor and 
 *           then outputs the result.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadPWMResult(ViSession  instrSession,
                                        ViUInt32    timeout, 
                                        ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout");
    
	viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rszvl_SetOPCTimeout (instrSession, timeout)); 
    
	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_PMET_READ, result), 3, "Result");
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 * Function: Fetch PWM Result
 * Purpose:  This function outputs the result of the power sensor.
 *===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchPWMResult(ViSession  instrSession,
                                        ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_PMET_FETCH, result), 2, "Result");
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

