/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver
 *  WLAN option K91
 *
 *  Original Release: March 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 *****************************************************************************/

#include "rszvl.h"   
/*  Array definitions   */
static ViString ACPMeas[] = {"CURR","MAXH"};

/*===========================================================================*/
/* Function: Set WLAN Mode
/* Purpose:  This function selects the WLAN analyzer mode.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WlanSetWlanMode(ViSession  instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WLAN_MODE, ""));
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Standard
/* Purpose:  This function configures the Wireless LAN standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanStandard(ViSession   instrSession,
                                                ViInt32 standard)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_STAN, standard), 2, "Standard");
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN  Trigger
/* Purpose:  This function configures the trigger parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanTrigger(ViSession   instrSession,
                                                ViInt32     mode,
                                                ViReal64    offset,
                                                ViBoolean   autoLevel,
                                                ViReal64    powerLevel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_TRIG_MODE, mode), 2, "Mode");
    switch (mode){
        case 0:
        break;
        case 1:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_TRIGGER_DELAY, offset), 3, "Offset");
        break;
        case 2:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_TRIGGER_DELAY, offset), 3, "Offset");
            viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_TRIG_POW_LEV_AUTO, autoLevel), 4, "Auto Level");
            if (!autoLevel)
                viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_TRIG_POW_LEV, powerLevel), 5, "Power Level");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Gate
/* Purpose:  This function configures the trigger parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanGate(ViSession  instrSession,
                                            ViBoolean   gating,
                                            ViInt32     mode,
                                            ViReal64    delay,
                                            ViReal64    length,
                                            ViBoolean   linkGateAndMarker)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_EGAT, gating), 2, "Gating");
    if (gating)
    {
        if (rszvl_invalidViInt32Range (instrSession, mode, 0, 1))
        {
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");
        }
        switch (mode){
            case RSZVL_VAL_WLAN_GATE_TIME:
                viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_EGAT_HOLD_TIME, delay), 4, "Delay");
                viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_EGAT_LENG_TIME, length), 5, "Length");
            break;
            case RSZVL_VAL_WLAN_GATE_SAMPLE:
                viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_EGAT_HOLD_SAMP, (ViInt32)delay), 4, "Delay");
                viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_EGAT_LENG_SAMP, (ViInt32)length), 5, "Length");
            break;
        }
        viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_EGAT_LINK, linkGateAndMarker), 6, "Link Gate Marker");
    }   
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN ACP Mode
/* Purpose:  This function sets the ACP measurement mode to either absolute 
/*           or relative.
/*           
/*           Note:
/*           
/*           (1) This function is available only for IEEE 802.11j standards.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanACPMode(ViSession instrSession,
                                       ViInt32  acpMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_ACH_MODE, acpMode), 2, "ACP Mode");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Gain Imbalance Unit
/* Purpose:  This function specifies the units for gain imbalance results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanGainImbalanceUnit(ViSession instrSession,
                                                 ViInt32 unit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_UNIT_GIMB, unit), 2, "Unit");
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Sweep
/* Purpose:  This function configures the acquisition.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWLANSweep (ViSession instrSession,
                                               ViBoolean sweepModeContinuous)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 3, "Sweep Mode Continuous");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Auto Level Once
/* Purpose:  This function performs the auto level routine immediately one 
/*           time regardless of the current state of the Auto Level State.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanAutoLevelOnce(ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_POW_AUTO_ONCE, ""));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Baseband Signal Level
/* Purpose:  This function configures the signal characteristics.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanBasebandSignalLevel (ViSession instrSession,
                                                  ViBoolean autoLevel,
                                                  ViReal64    autoLevelTime,
                                                  ViReal64 extAtt)
                                                 
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_POW_AUTO, autoLevel), 2, "Auto Level");
    
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_POW_AUTO_SWE_TIME, autoLevelTime), 3, "Auto Level Time");
    
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_REFERENCE_LEVEL_OFFSET, extAtt), 4, "Ext Att");

    checkErr( rszvl_CheckStatus (instrSession));    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Channel
/* Purpose:  This function configures the signal characteristics.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanRFChannelFrequency (ViSession instrSession,
                                                 ViInt32 channelNo,
                                                 ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (channelNo)
    {
        viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_CHAN, channelNo), 2, "Channel No")
    }       
    else
    {
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER, frequency), 3, "Frequency");
    }    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN RF Signal
/* Purpose:  This function configures the signal characteristics.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanRFSignalLevel(ViSession   instrSession,
                                                ViBoolean   autoLevel,
                                                ViReal64    autoLevelTime,
                                                ViReal64    signalLevel,
                                                ViReal64     rfAtt,
                                                ViReal64    extAtt)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_POW_AUTO, autoLevel), 2, "Auto Level");
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_POW_AUTO_SWE_TIME, autoLevelTime), 3, "Auto Level Time");
    if (!autoLevel)
    {
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_ATTENUATION, rfAtt), 5, "RF Att");
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_POW_EXP_RF, signalLevel), 4, "Signal Level");
    }    
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_REFERENCE_LEVEL_OFFSET, extAtt), 6, "Ext Att");
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure WLAN Marker Enabled
/* Purpose:  This function enables the active marker on the specified trace.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWLANMarkerEnabled(ViSession instrSession,
                                                        ViInt32     window,
                                                        ViInt32     marker,
                                                        ViBoolean   markerEnabled,
                                                        ViInt32     trace)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
     if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_WLAN_MARKER_ENABLED, 
                markerEnabled), 4, "Marker Enabled");
    if (markerEnabled == VI_TRUE)                
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_MARKER_TRACE, 
                trace), 5, "Trace");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Marker Position
/* Purpose:  This function configures the marker position.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanMarkPosition(ViSession   instrSession,
                                                    ViInt32 window,
                                                    ViInt32 marker,
                                                    ViInt32 position,
                                                    ViReal64 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    if (rszvl_invalidViInt32Range (instrSession, position, RSZVL_VAL_WLAN_POS_X, RSZVL_VAL_WLAN_POS_Y))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Position");
    }
    sprintf (buffer, "Win%ld,M%ld", window, marker);   
    switch (position){
        case RSZVL_VAL_WLAN_POS_X:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WLAN_MARKER_POSITION, value), 5, "Value");
        break;    
        case RSZVL_VAL_WLAN_POS_Y:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WLAN_MARK_Y, value), 5, "Value");
        break;    
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Marker Position for Constellation
/* Purpose:  This function positions the selected marker to the indicated 
/*           symbol/carrier.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanMarkerPositionConstellation(ViSession   instrSession,
                                                                    ViInt32 window,
                                                                    ViInt32 marker,
                                                                    ViInt32 type,
                                                                    ViInt32 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_WLAN_CONS_SYM, RSZVL_VAL_WLAN_CONS_CARR))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
    }
    sprintf (buffer, "Win%ld,M%ld", window, marker);    
    switch (type){
        case RSZVL_VAL_WLAN_CONS_SYM:
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WLAN_MARK_SYMB, position), 5, "Position");
        break;    
        case RSZVL_VAL_WLAN_CONS_CARR:
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WLAN_MARK_CARR, position), 5, "Position");
        break;    
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Marker Zoom
/* Purpose:  This function defines the ratio to be zoomed around the 
/*           selected marker in the selected measurement window.
/*           
/*           Note:
/*           (1) This function is only available for the following result 
/*           displays:
/*           
/*             Constellation versus Carrier
/*             Constellation versus Symbol
/*             PVT Full Burst
/*             PVT Rising / Falling
/*             Magnitude Capture Buffer
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanMarkerZoom(ViSession instrSession,
                                                    ViInt32 window,
                                                    ViInt32 marker,
                                                    ViReal64 zoomRatio)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    sprintf (buffer, "Win%ld,M%ld", window, marker);     
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WLAN_MARKER_ZOOM, zoomRatio), 4, "Zoom Ratio");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Marker Burst Symbol
/* Purpose:  This function positions the selected marker to the indicated 
/*           symbol in the indicated burst (Constellation vs Symbol and EVM 
/*           vs Symbol).
/*           
/*           Note:
/*           (1) This function is only valid for the following 802.11b 
/*           measurements:
/*           
/*                  Constellation vs Symbol 
/*                  EVM vs Symbol
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanMarkerBurstSymbol(ViSession   instrSession,
                                                            ViInt32 window,
                                                            ViInt32 marker,
                                                            ViInt32 burstNumber,
                                                            ViInt32 symbolNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (rszvl_invalidViInt32Range (instrSession, window, 2, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }   
    
    viCheckErr( viPrintf (instrSession, "CALC%ld:MARK%ld:BSYM %ld,%ld\n", window, marker, burstNumber, symbolNumber));;
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set WLAN Marker To Peak
/* Purpose:  This function set the selected marker to the min/max peak in 
/*           the current trace.
/*           
/*           Note:
/*           (1) This function is only available for the Spectrum Flatness 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_SetWlanMarkerPeak(ViSession   instrSession,
                                            ViInt32 window,
                                            ViInt32 marker,
                                            ViInt32 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    if (rszvl_invalidViInt32Range (instrSession, position, RSZVL_VAL_MARKER_SEARCH_HIGHEST, RSZVL_VAL_MARKER_SEARCH_MINIMUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Position");
    }
    sprintf (buffer, "Win%ld,M%ld", window, marker);       

    switch (position){
        case RSZVL_VAL_MARKER_SEARCH_MINIMUM:
            viCheckErr( rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_WLAN_MARKER_SEARCH_MIN, ""));
        break;    
        case RSZVL_VAL_MARKER_SEARCH_HIGHEST:
            viCheckErr( rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_WLAN_MARKER_SEARCH_MAX, ""));
        break;    
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WLAN Disable All Markers
/* Purpose:  This function turns off all the markers in selected measurement 
/*           window.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WlanDisableAllMarkers(ViSession  instrSession,
                                                    ViInt32 window)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");

    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_WLAN_MARKERS_AOFF, 
                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*===========================================================================*/
/* Function: Configure WLAN Demodulation
/* Purpose:  This function configures the demodulation parameters. 
/*           
/*           Note:
/*           (1) This function is not available for IEEE 802.11b standard.
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanDemodulation(ViSession   instrSession,
                                                    ViBoolean   useSignalFieldContent,
                                                    ViInt32     burstType,
                                                    ViBoolean   autoDemodulation,
                                                    ViInt32     demodulator,
                                                    ViBoolean   channelEstimation,
                                                    ViBoolean   equalBurstLength,
                                                    ViInt32 minNumberofDataSymbols,
                                                    ViInt32 maxNumberofDataSymbols)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      options[RSZVL_IO_BUFFER_SIZE] = "";
    ViInt32     is_k91 = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST,
                                             RSZVL_IO_BUFFER_SIZE, options));

    is_k91 = (strstr (options, "K91") != NULL);
    if (rszvl_invalidViInt32Range (instrSession, minNumberofDataSymbols, 1, 1366))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Min Number of Data Symbols");
    }
    if (rszvl_invalidViInt32Range (instrSession, maxNumberofDataSymbols, 1, 1366))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Max Number of Data Symbols");
    }
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_BTYP, burstType), 3, "Burst Type");
    
    if (is_k91)
    {
        viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_BCON_AUTO, autoDemodulation), 4, "Auto Demodulation");
    }
    
    if (!autoDemodulation)
    {
        if (is_k91)
        {
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_SIGS, useSignalFieldContent), 2, "Use Signal Field Content");
        }
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_BAN, demodulator), 5, "Demodulator");
    }
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_CEST, channelEstimation), 6, "Channel Estimation");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_SYM_EQU, equalBurstLength), 7, "Equal Burst Length");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Min", RSZVL_ATTR_WLAN_SYM, minNumberofDataSymbols), 8, "Min Number of Data Symbols");
    if (!equalBurstLength)
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Max", RSZVL_ATTR_WLAN_SYM, maxNumberofDataSymbols), 9, "Max Number of Data Symbols");
    
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN PLCP Demodulation
/* Purpose:  This function configures the demodulation parameters for PLCP 
/*            Burst Type demodulation.. 
/*           
/*           Note:
/*           (1) This function is available only for IEEE 802.11b/g(PLCP) standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanPLCPDemodulation(ViSession instrSession,
                                                        ViBoolean   usePLCPHeaderContent,
                                                        ViInt32     burstType,
                                                        ViBoolean   autoDemodulation, 
                                                        ViInt32     demodulator,
                                                        ViInt32     lengthType,
                                                        ViBoolean   equalBurstLength,
                                                        ViInt32     minPayloadLength,
                                                        ViInt32     maxPayloadLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_BTYP, burstType), 3, "Burst Type");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_BCON_AUTO, autoDemodulation), 4, "Auto Demodulation");

    if (!autoDemodulation)
    {
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_BAN, demodulator), 5, "Demodulator");
        viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_SIGS, usePLCPHeaderContent), 2, "Use Signal Field Content");
    }    
    
    if (rszvl_invalidViInt32Range (instrSession, lengthType, RSZVL_VAL_WLAN_DEM_DUR_MS, RSZVL_VAL_WLAN_DEM_DUR_BYTE))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Length Type");
    }
    
    switch (lengthType){
        case RSZVL_VAL_WLAN_DEM_DUR_MS:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_DUR_EQU, equalBurstLength), 7, "Equal Burst Length");
            viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Min", RSZVL_ATTR_WLAN_DUR, minPayloadLength), 8, "Min Payload Length");
            if (!equalBurstLength)
                viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Max", RSZVL_ATTR_WLAN_DUR, maxPayloadLength), 9, "Max Payload Length");
        break;
        case RSZVL_VAL_WLAN_DEM_DUR_BYTE:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_DBYT_EQU, equalBurstLength), 7, "Equal Burst Length");
            viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Min", RSZVL_ATTR_WLAN_DBYT, minPayloadLength), 8, "Min Payload Length");
            if (!equalBurstLength)
            {
                viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Max", RSZVL_ATTR_WLAN_DBYT, maxPayloadLength), 9, "Max Payload Length");
            }    
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Tracking
/* Purpose:  This function configures the tracking parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanTracking(ViSession  instrSession,
                                                ViBoolean   phase,
                                                ViBoolean   timing,
                                                ViBoolean   level)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_TRAC_PHAS, phase), 2, "Phase");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_TRAC_TIME, timing), 3, "Time");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WLAN_TRAC_LEV, level), 4, "Level");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Select Filter
/* Purpose:  This function selects the TX and RX filters. The names of the 
/*           filters correspond to the file names returned by 
/*           RSZVL_ATTR_WLAN_FILT_CAT attribute.
/*           
/*           Note:
/*           (1) This function is available only for IEEE 802.11b & g 
/*           standards
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanSelectFilter(ViSession   instrSession,
                                            ViString    txFilterName,
                                            ViString    rxFilterName)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    checkErr (viPrintf (instrSession, "DEM:FILT:MOD '%s','%s'\n", txFilterName, rxFilterName));
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN Filter Catalog
/* Purpose:  This function reads the names of all available filters.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanFilterCatalog(ViSession   instrSession,
                                            ViInt32 bufferSize,
                                            ViChar  filterList[],
                                            ViInt32*    returnedLength)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *buff;
    ViUInt32     count=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    checkErr (viPrintf (instrSession, "DEM:FILT:CAT?"));         
    checkErr (Rs_ReadDataUnknownLength(instrSession, &buff, &count));
    if (bufferSize != 0)
    {
        strncpy(filterList, buff, (bufferSize > (ViInt32)count) ? count : bufferSize);
    }
    if (returnedLength) *returnedLength = count;
    checkErr( rszvl_CheckStatus (instrSession));       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Display
/* Purpose:  This function configures the display parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanDisplay(ViSession   instrSession,
                                                ViInt32 combinedTraceDisplay,
                                                ViBoolean   displayTable)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_WLAN_DISP_FORMAT,
                                                combinedTraceDisplay), 2, "Combined Trace Display");
    
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_WLAN_DISP_TABL, 
                                                displayTable), 3, "Display Table");
    
	checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Select WLAN Screen
/* Purpose:  This function selects whether screen A or screen B is active. 
/*           SSELect means Screen SELect. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_SelectWlanScreen (ViSession instrSession, ViInt32 window)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	sprintf (buffer, "Win%ld", window);
    
	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_WLAN_SCREEN_SELECT, NULL));
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Limit
/* Purpose:  This function configures the limit.
/*      
/*           Note:
/*           (1) This function is not valid for standard IEEE 802.11b 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanLimit(ViSession     instrSession,
                                                ViInt32 window, 
												ViInt32     limit,
                                                ViInt32     limitType,
                                                ViInt32     mode,
                                                ViReal64    value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, limitType, RSZVL_VAL_WLAN_LIM_FERR, RSZVL_VAL_WLAN_LIM_EVM_PIL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Type");
    } 
    if ((mode!=RSZVL_VAL_MEASTYPE_MAX) && (mode!=RSZVL_VAL_MEASTYPE_AVER))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Mode");
    }
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MAX:
            strcat (buffer, ",Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            strcat (buffer, ",Aver");
        break;
    }
    
    switch (limitType){
        case RSZVL_VAL_WLAN_LIM_FERR:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FERR, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_SYMB:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_SYMB, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_IQOFF:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_IQOF, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM_ALL:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_ALL, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM_DATA:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_DATA, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM_PIL:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_PIL, 
            value), 6, "Value");
        break;
    }    

    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN All Limits
/* Purpose:  This function sets all limits.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWLANAllLimits (ViSession instrSession,
                                                   ViInt32 window, 
												   ViInt32 limit,
                                                   ViInt32 arraySize,
                                                   ViReal64 values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!strstr (buffer, "K91"))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (!values)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Values");    
    }
    pbuffer = buffer;
    
    pbuffer += sprintf (pbuffer, "CALC%ld:LIMit%ld:BURSt:ALL ", window, limit);
    
    for (i=0;i<arraySize;i++)
         pbuffer += sprintf (pbuffer,"%.12f,", values[i]);
         
    *--pbuffer = '\0';
    
    viCheckErr (viPrintf (instrSession, "%s\n", buffer));
    
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN PLCP Limit
/* Purpose:  This function configures the limit.
/*           
/*           Note:
/*           (1) This function is valid only if standard IEEE 802.11b or 
/*           IEEE 802.11g (PLCP) is selected.
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanPLCPLimit(ViSession   instrSession,
                                                    ViInt32 window, 
													ViInt32     limit,
                                                    ViInt32     limitType,
                                                    ViInt32     mode,
                                                    ViReal64    value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, limitType, RSZVL_VAL_WLAN_LIM_RTIME, RSZVL_VAL_WLAN_LIM_EVM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Type");
    } 
    if ((mode!=RSZVL_VAL_MEASTYPE_MAX) && (mode!=RSZVL_VAL_MEASTYPE_AVER))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Mode");
    }
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MAX:
            strcat (buffer, ",Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            strcat (buffer, ",Aver");
        break;
    }
    
    switch (limitType){
        case RSZVL_VAL_WLAN_LIM_RTIME:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_TRIS, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_FTIME:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_TFAL, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM, 
            value), 6, "Value");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Get ACP Channel Limit
/* Purpose:  This function returns the ACP channel limit.
/*           
/*           Note:
/*           (1) This function is available only for IEEE 802.11j standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanACPChannelLimit(ViSession     instrSession,
                                                    ViInt32 window, 
													ViInt32     limit,
                                                    ViInt32     channelType,
                                                    ViReal64*   value
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, channelType, RSZVL_VAL_WLAN_ACP_ADJ, RSZVL_VAL_WLAN_ACP_ALT))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel Type");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (channelType){
        case RSZVL_VAL_WLAN_ACP_ADJ:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_ACP_ADJ, 
                    value), 5, "Value");
        break;
        case RSZVL_VAL_WLAN_ACP_ALT:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_ACP_ALT, 
                    value), 5, "Value");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN Spectrum Max Overstepping
/* Purpose:  This function returns the alue at the maximum overstepping of 
/*           the spectrum mask limits.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanSpectrumMaxOverstepping(ViSession     instrSession,
                                                            ViInt32 window, 
															ViInt32     limit,
                                                            ViInt32     channelType,
                                                            ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, channelType, RSZVL_VAL_FREQ, RSZVL_VAL_POW))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel Type");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (channelType){
        case RSZVL_VAL_FREQ:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_SPEC_MASK_X, 
                    value), 5, "Value");
        break;
        case RSZVL_VAL_POW:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_SPEC_MASK_Y, 
                    value), 5, "Value");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN All Limits
/* Purpose:  This function returns all limits.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanAllLimits(ViSession   instrSession,
                                            ViInt32 window, 
											ViInt32     limit,
                                            ViReal64    values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     maxIdx = 12;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (!values)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Values");    
    }
        
    checkErr (viQueryf (instrSession, ":CALC%ld:LIM%ld:BURS:ALL?\n","%s", window, limit, buffer));
                        
    pbuffer = strtok(buffer, ",");
    while ((pbuffer) && (i < maxIdx)) 
    {
        values[i++] = atof (pbuffer);
        pbuffer = strtok(NULL, ",");        
    }            
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Measurement
/* Purpose:  This function configures the measurement parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanMeas(ViSession   instrSession,
                                            ViBoolean   overallBurstCount,
                                            ViInt32     noofBursttoAnalyze,
                                            ViReal64    captureTime,
                                            ViBoolean   swapIQ,
                                            ViInt32     sweepCount)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_BURS_COUN_STAT, overallBurstCount), 2, "Overall Burst Count");
    if (overallBurstCount)
        viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_BURS_COUN, noofBursttoAnalyze), 3, "No of Burst to Analyze");
#if 0
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_SWEEP_TIME , captureTime), 4, "Capture Time");
#endif
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WLAN_SWAP_IQ, swapIQ), 5, "Swap IQ");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_NUMBER_OF_SWEEPS, sweepCount), 6, "Sweep Count");

    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN PVT Measurement
/* Purpose:  This function sets measurement type to be Power Versus Time 
/*           and configures measurement parameters
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanPVTMeas(ViSession   instrSession,
                                                ViInt32 risingFalling)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_PVT_SEL, risingFalling), 2, "Rising/Falling");
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_PVT, ""));
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN PVT for IEEE 802.11b
/* Purpose:  This function sets measurement type to be Power Versus Time and
/*           configures measurement parameters for IEEE802.11b standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanPLCPPVT(ViSession   instrSession,
                                                ViInt32 referencePower,
                                                ViInt32 ramp,
                                                ViInt32 averageLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_PVT_RPOW, referencePower), 2, "Reference Power");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_PVT_SEL, ramp), 3, "Ramp");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_PVT_AVER, averageLength), 4, "Average Length");
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_PVT, ""));
    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN EVM Measurement
/* Purpose:  This function sets measurement type to be EVM.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanEVMMeasurement(ViSession   instrSession,
                                                        ViInt32     mode,
                                                        ViBoolean   autoScaling,
                                                        ViReal64    perDivision,
                                                        ViInt32     unit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_WLAN_EVM_MODE_SYMBOL, RSZVL_VAL_WLAN_EVM_MODE_FREQ))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Mode");
    }
     
    switch (mode){
        case RSZVL_VAL_WLAN_EVM_MODE_SYMBOL:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_EVM_ESYM, ""));
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_UNIT_EVM, unit), 5, "Unit");
        break;
        case RSZVL_VAL_WLAN_EVM_MODE_CARRIER:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_EVM_ECAR, ""));
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_UNIT_EVM, unit), 5, "Unit");
        break;
        case RSZVL_VAL_WLAN_EVM_MODE_PHAS:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_PRE, "")); 
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_PRE_SEL, RSZVL_VAL_PRE_PHAS));
        break;
        case RSZVL_VAL_WLAN_EVM_MODE_FREQ:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_PRE, "")); 
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_PRE_SEL, RSZVL_VAL_PRE_FREQ));
        break;
    }
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "Win2,TR1", RSZVL_ATTR_WLAN_DISP_AUTO_Y, autoScaling), 3, "Auto Scaling");
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "Win2,TR1", RSZVL_ATTR_WLAN_Y_PDIV, perDivision), 4, "Per Division");
   
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Spectrum Measurement
/* Purpose:  This function sets measurement type to be Spectrum.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanSpectrumMeasurement(ViSession   instrSession,
                                                            ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_WLAN_SPEC_MODE_IEEE, RSZVL_VAL_WLAN_SPEC_MODE_FLAT))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Mode");
    }
    switch (mode){
        case RSZVL_VAL_WLAN_SPEC_MODE_IEEE:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_SPEC_MASK_SEL,
                                                                            RSZVL_VAL_SPEC_IEEE));
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_SPEC_MASK, ""));
        break;
        case RSZVL_VAL_WLAN_SPEC_MODE_ETSI:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_SPEC_MASK_SEL,
                                                      RSZVL_VAL_SPEC_ETSI));
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_SPEC_MASK, ""));
        break;
        case RSZVL_VAL_WLAN_SPEC_MODE_FFT:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_SPEC_FFT, ""));
        break;
        case RSZVL_VAL_WLAN_SPEC_MODE_ACPR:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_SPEC_ACPR, ""));
        break;
        case RSZVL_VAL_WLAN_SPEC_MODE_FLAT:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_SPEC_FLAT, ""));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*===========================================================================*/
/* Function: Configure WLAN Constellation Measurement
/* Purpose:  This function configures the Constellation measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanConstellationMeasurement(ViSession   instrSession,
                                                    ViInt32 constelationMode,
                                                    ViInt32 carrierSelection,
                                                    ViInt32 carrierNumber)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, constelationMode, RSZVL_VAL_WLAN_CONS_SYM, RSZVL_VAL_WLAN_CONS_CARR))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Constelation Mode");
    }
    if (rszvl_invalidViInt32Range (instrSession, carrierSelection, RSZVL_VAL_SELECTION_ALL, RSZVL_VAL_SELECTION_NUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Carrier Selection");
    }
    switch (constelationMode){
        case RSZVL_VAL_WLAN_CONS_SYM:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_CONS_CSYM, ""));
            switch (carrierSelection){
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                               RSZVL_ATTR_WLAN_CONS_CARR_SEL,
                                                               RSZVL_VAL_SELECTION_ALL), 3, "Carrier Selection");
                break;
                case RSZVL_VAL_SELECTION_PIL:
                    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                               RSZVL_ATTR_WLAN_CONS_CARR_SEL,
                                                               RSZVL_VAL_SELECTION_PIL), 3, "Carrier Selection");
                break;
                case RSZVL_VAL_SELECTION_NUM:
                    if (rszvl_invalidViInt32Range (instrSession, carrierNumber, -26, 26) || (carrierNumber == 0))
                    {
                        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Carrier Number");
                    }
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WLAN_CONS_CARR_SEL_NR, carrierNumber), 4, "Carrier Number");                        
                break; 
             }   
        break;
        case RSZVL_VAL_WLAN_CONS_CARR:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_CONS_CCAR, ""));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Constellation Measurement
/* Purpose:  This function configures the Constellation measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanPLCPConstellation(ViSession   instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_CONS_CSYM, ""));
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN Statistics Measurement
/* Purpose:  This function configures the Statistics measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanStatisticsMeasurement(ViSession   instrSession,
                                                        ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_WLAN_STAT_CCDF, RSZVL_VAL_WLAN_STAT_SFI))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Mode");
    }
    switch (mode){
        case RSZVL_VAL_WLAN_STAT_CCDF:
            viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WLAN_STAT_CCDF, ""));
        break;
        case RSZVL_VAL_WLAN_STAT_BSTR:
            viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WLAN_STAT_BSTR, ""));
        break;
        case RSZVL_VAL_WLAN_STAT_SFI:
            viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WLAN_STAT_SFI, ""));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WLAN IQ Measurement Parameters
/* Purpose:  This function configures the IQ Trace parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWlanIQMeasurementParameters(ViSession   instrSession,
                                                                ViReal64    sampleRate)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WLAN_SRAT, sampleRate), 2, "Sample Rate");
    
	checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN ACP Measurements
/* Purpose:  This function returns the result values of the adjacent channel 
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanACPMeasurement(ViSession   instrSession,
                                                ViInt32     window,
                                                ViInt32     marker,
                                                ViInt32     mode,
                                                ViReal64    values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     maxIdx = 12;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    } 
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_ACP_CURR, RSZVL_VAL_ACP_MAX))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Mode");
    } 
    if (!values)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Values");    
    }
        
    checkErr (viQueryf (instrSession, ":CALC%ld:MARK%ld:FUNC:POW:RES:%s?\n","%s", window, marker, ACPMeas[mode], buffer));
                        
    pbuffer = strtok(buffer, ",");
    i=0;
    
    while ((pbuffer) && (i < maxIdx)) 
    {
        values[i++] = atof (pbuffer);
        pbuffer = strtok(NULL, ",");        
    }            
    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Load WLAN IQ Data
/* Purpose:  This function loads the IQ data from the specified .iqw file.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_LoadWlanIQData (ViSession instrSession,
                                           ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    if (!fileName)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "File Name");   
    }
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_IQ_LOAD, fileName));
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Store WLAN Store IQ Data
/* Purpose:  This function stores the IQ data to the specified .iqw file.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_StoreWlanIQData (ViSession instrSession,
                                            ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    if (!fileName)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "File Name");   
    }
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_IQ_SAVE, fileName));
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WLAN Burst Recalc
/* Purpose:  This function forces the IQ measurement results to be 
/*           recalculated according to the current settings.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WlanBurstImm (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WLAN_CALC_BURS, ""));
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read WLAN Trace Data
/* Purpose:  This function returns all the measured data that relates to the 
/*           currently selected measurement type. The returned data is 
/*           particular to the currently selected measurement type and is 
/*           specified in the Trace Data parameter description.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadWlanTraceData(ViSession   instrSession,
                                              ViInt32   sourceTrace,
                                              ViInt32   arraySize,
                                              ViReal64 _VI_FAR traceData[],
                                              ViPInt32  noofValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer=VI_NULL;
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (rszvl_invalidViInt32Range (instrSession, sourceTrace, 1, 6))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source Trace");
    }
    sprintf (buffer, ":TRAC? TRACE%ld", sourceTrace);
    
    viCheckErr (rszvl_dataReadTrace (instrSession, buffer, arraySize, 
                    traceData, noofValues));
      
    checkErr( rszvl_CheckStatus (instrSession));           
    
Error:
    if (pbuffer) free(pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/*===========================================================================*/
/* Function: Read WLAN Memory IQ Data
/* Purpose:  This function returns all the IQ data associated with the 
/*           measurement acquisition time. The result values are scaled 
/*           linear in unit Volt and correspond to the voltage at the RF 
/*           input of the instrument. The command returns a comma-separated 
/*           list of the measured voltage values in floating point format 
/*           (Comma Separated Values = CSV). The number of values returned is 
/*           2 * number of samples, the first half being the I values, the 
/*           second half the Q values.
/*           
/*           Notes:
/*           (1) If there are not I/Q data available in memory because the 
/*           corresponding measurement had not been started, the function 
/*           will cause a Query Error.
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadWlanMemoryIQData(ViSession   instrSession,
                                                ViInt32     offsetSamples,
                                                ViInt32     noofSamples,
                                                ViInt32     bufferSize,
                                                ViInt32*    noofPoints,
                                                ViReal64    realPartsI[],
                                                ViReal64    imaginaryPartsQ[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViInt32     num;
    ViReal64    *data=VI_NULL;
    ViInt32     data_num;
    ViInt32     i;
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    checkErr (viPrintf(instrSession, ":FORM REAL,32;*CLS;:TRAC:IQ:DATA:MEM? %ld,%ld\n",
                offsetSamples, noofSamples));
    data_num = noofSamples * 2;
    data = malloc (sizeof (ViReal64) * data_num); 
    /*viCheckErr (rszvl_dataReadTraceIQ (instrSession, bufferSize, noofPoints, 
                    realPartsI, imaginaryPartsQ));*/
    viCheckErr (rszvl_dataReadTraceOnly (instrSession, data_num, data, &num));
    
    if (noofPoints) *noofPoints=num/2;

    num = (noofSamples>bufferSize)?bufferSize:noofSamples;

    for (i=0;i<num;i++)
    {
        realPartsI[i]=data[2*i];
        imaginaryPartsQ[i]=data[2*i+1];
    }
    checkErr( rszvl_CheckStatus (instrSession));           
    
Error:
    if (data) free (data);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/*===========================================================================*/
/* Function: Read WLAN SEM Results
/* Purpose:  This function returns the Spectrum Emission Mask summary results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadWlanSEMResults (ViSession instrSession,
                                               ViInt32 noOfValues,
                                               ViReal64 index[],
                                               ViReal64 startFrequencyBand[],
                                               ViReal64 stopFrequencyBand[],
                                               ViReal64 resolutionBandwidth[],
                                               ViReal64 limitFailFrequency[],
                                               ViReal64 powerAbs[],
                                               ViReal64 powerRel[],
                                               ViReal64 limitDistance[],
                                               ViReal64 failureFlag[],
                                               ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuf=VI_NULL;
    ViInt32     count=0;
    ViUInt32    retCnt;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (index == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Index");
    if (startFrequencyBand == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Start Frequency Band"); 
    if (stopFrequencyBand == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Stop Frequency Band"); 
    if (resolutionBandwidth == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Resolution Bandwidth"); 
    if (limitFailFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Limit Fail Frequency"); 
    if (powerAbs == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Power Abs");
    if (powerRel == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Power Rel"); 
    if (limitDistance == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Limit Distance"); 
    if (failureFlag == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Failure Flag");
    
    viCheckErr (viPrintf (instrSession, "TRAC:DATA? LIST\n"));
    viCheckErr( Rs_ReadDataUnknownLength(instrSession, &buffer, &retCnt));
    
    pbuf=strtok(buffer, ",");
    if (pbuf)
        do{
            if (count<noOfValues)
            {
                sscanf (pbuf, "%le", &index[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &startFrequencyBand[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &stopFrequencyBand[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &resolutionBandwidth[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &limitFailFrequency[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &powerAbs[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &powerRel[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &limitDistance[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &failureFlag[count]);
                pbuf=strtok(NULL, ",");
            }               
            else
            {
                for (i=0;i<11;i++)
                    pbuf=strtok(NULL, ","); 
            }       
            count++;
        }while (pbuf);  

    if (returnedValues)
        *returnedValues = count;
     
    checkErr( rszvl_CheckStatus (instrSession));     
Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN Burst All
/* Purpose:  This function returns all the results.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanBurstAll(ViSession   instrSession,
                                            ViReal64    results[],
                                            ViInt32*    returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    checkErr (viQueryf (instrSession, "FETC:BURS:ALL?\n","%s", buffer));
                        
    pbuffer = strtok(buffer, ",");
    i=0;
    if (results)
    { 
        while (pbuffer) 
        {
            results[i++] = atof (pbuffer);
            pbuffer = strtok(NULL, ",");        
        }            
    }
    else
    {
        while (pbuffer)
        {
            i++;
            pbuffer = strtok(NULL, ",");
         }   
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN Burst Power
/* Purpose:  This function returns the power measurement value, measured 
/*           during the measurement.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanBurstPower(ViSession   instrSession,
                                                ViInt32     powerMeasurement,
                                                ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, powerMeasurement, RSZVL_VAL_WLAN_MEAS_PREA, RSZVL_VAL_WLAN_MEAS_CREST_AVER))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Power Measurement");
    }
    
    switch (powerMeasurement){
        case RSZVL_VAL_WLAN_MEAS_PREA:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_WLAN_FETC_PRE, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_MEAS_PAYL:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_WLAN_FETC_PAYL, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_MEAS_RMS_MAX:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "Max", RSZVL_ATTR_WLAN_FETC_RMS, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_MEAS_RMS_MIN:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "Min", RSZVL_ATTR_WLAN_FETC_RMS, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_MEAS_RMS_AVER:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "Aver", RSZVL_ATTR_WLAN_FETC_RMS, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_MEAS_PEAK:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_WLAN_FETC_PEAK, 
                    value), 3, "Value");
        break;            
        case RSZVL_VAL_WLAN_MEAS_CREST_MAX:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "Max", RSZVL_ATTR_WLAN_FETC_CRES, 
                    value), 3, "Value");                    
        break;
        case RSZVL_VAL_WLAN_MEAS_CREST_MIN:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "Min", RSZVL_ATTR_WLAN_FETC_CRES, 
                    value), 3, "Value");                    
        break;
        case RSZVL_VAL_WLAN_MEAS_CREST_AVER:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "Aver", RSZVL_ATTR_WLAN_FETC_CRES, 
                    value), 3, "Value");                    
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN Burst Time
/* Purpose:  This function returns the burst rise/falling time.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*           
/*           (2) This function is available only for IEEE 802.11b standard
/*          
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanBurstTime(ViSession     instrSession,
                                                ViInt32     time,
                                                ViInt32     mode,
                                                ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, time, RSZVL_VAL_WLAN_TIME_FALL, RSZVL_VAL_WLAN_TIME_RISE))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Time");
    }
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");    
    }
    
    switch (time){
        case RSZVL_VAL_WLAN_TIME_FALL:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_TFAL, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_TIME_RISE:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_TRIS, 
                    value), 3, "Value");
        break;
    }      
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN Burst Error
/* Purpose:  This function returns the burst errors.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanBurstError(ViSession    instrSession,
                                                ViInt32     errorType,
                                                ViInt32     mode,
                                                ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, errorType, RSZVL_VAL_WLAN_ERR_FREQ, RSZVL_VAL_WLAN_ERR_SYMB))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Error Type");
    }
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");    
    }
    
    switch (errorType){
        case RSZVL_VAL_WLAN_ERR_FREQ:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_FERR, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_ERR_SYMB:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_SYMB, 
                    value), 3, "Value");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN IQ Impairments
/* Purpose:  This function returns the burst errors.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanIQImp(ViSession     instrSession,
                                            ViInt32     iqImpairment,
                                            ViInt32     mode,
                                            ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, iqImpairment, RSZVL_VAL_WLAN_IQ_OFFS, RSZVL_VAL_WLAN_IQ_QUAD))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "IQ Impairment");
    }
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");    
    }
    switch (iqImpairment){
        case RSZVL_VAL_WLAN_IQ_OFFS:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_IQOF, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_IQ_IMBA:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_GIMB, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_IQ_QUAD:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_QUAD, 
                    value), 3, "Value");
        break;
     } 
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN EVM
/* Purpose:  This function returns the EVM measurement value.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanEVM(ViSession   instrSession,
                                        ViInt32     evm,
                                        ViInt32     mode,
                                        ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, evm, RSZVL_VAL_WLAN_EVM_ALL, RSZVL_VAL_WLAN_EVM_PEAK))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "EVM");
    }
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");    
    }
    switch (evm){
        case RSZVL_VAL_WLAN_EVM_ALL:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_EVM_ALL, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_EVM_DATA:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_EVM_DATA, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_EVM_PILOT:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_EVM_PILOT, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_EVM:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_EVM, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WLAN_EVM_PEAK:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WLAN_FETC_EVM_DIR, 
                    value), 3, "Value");
        break;
     }   
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WLAN Count
/* Purpose:  This function returns returns the count of analyzed burst or 
/*            number of symbols of analyzed burst.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWlanCount(ViSession   instrSession,
                                        ViInt32     type,
                                        ViInt32*   count)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_WLAN_BURS, RSZVL_VAL_WLAN_SYMB))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
    }
    
    switch (type){
        case RSZVL_VAL_WLAN_BURS:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_FETC_BURS_COUNT, 
                            count), 3, "Count");    
        break;
        case RSZVL_VAL_WLAN_SYMB:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WLAN_FETC_SYMB_COUNT, 
                            count), 3, "Count");    
        break;
   }     
   checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN Limit Check Result
/* Purpose:  This function returns the limit results
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanLimitCheckResult(ViSession   instrSession,
                                                ViInt32 window, 
												ViInt32     limit,
                                                ViInt32     limitType,
                                                ViInt32     mode,
                                                ViInt32*    result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, limitType, RSZVL_VAL_WLAN_LIM_FERR, RSZVL_VAL_WLAN_LIM_EVM_PIL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Type");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MAX:
            strcat (buffer, ",Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            strcat (buffer, ",Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Mode");
    }
    
    switch (limitType){
        case RSZVL_VAL_WLAN_LIM_FERR:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_FERR_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_SYMB:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_SYMB_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_IQOFF:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_IQOF_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM_ALL:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_ALL_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM_DATA:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_DATA_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM_PIL:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_PIL_RES, 
            result), 6, "Result");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN PLCP Limit Check Result
/* Purpose:  This function returns the limit result
/*           
/*           Note:
/*           (1) This function is valid only if standard IEEE 802.11b/d (PLCP) is 
/*           selected.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanPLCPLimitCheckResult(ViSession   instrSession,
                                                    ViInt32 window, 
													ViInt32     limit,
                                                    ViInt32     limitType,
                                                    ViInt32     mode,
                                                    ViInt32*    result)
{
    ViStatus    error = VI_SUCCESS;
     ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, limitType, RSZVL_VAL_WLAN_LIM_RTIME, RSZVL_VAL_WLAN_LIM_EVM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Type");
    } 
    sprintf (buffer, "Win%ld,L%ld", window, limit);  
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MAX:
            strcat (buffer, ",Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            strcat (buffer, ",Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Mode");
    }
    
    switch (limitType){
        case RSZVL_VAL_WLAN_LIM_RTIME:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_TRIS_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_FTIME:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_TFAL_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WLAN_LIM_EVM:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_EVM_RES, 
            result), 6, "Result");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN ACP Channel Limit Check Result
/* Purpose:  This function returns the ACP channel limit.
/*           
/*           Note:
/*           (1) This function is available only for IEEE 802.11j standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanACPChannLimitCheckResult(ViSession   instrSession,
                                                    ViInt32         window, 
													ViInt32         limit,
                                                    ViInt32         channelType,
                                                    ViInt32*        result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, channelType, RSZVL_VAL_WLAN_ACP_ADJ, RSZVL_VAL_WLAN_ACP_ALT))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel Type");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (channelType){
        case RSZVL_VAL_WLAN_ACP_ADJ:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_ACP_ADJ_RES, 
            result), 5, "Result");
        break;
        case RSZVL_VAL_WLAN_ACP_ALT:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_ACP_ALT_RES, 
            result), 5, "Result");
        break;
    }        
         
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN All Limit Check Results
/* Purpose:  This function returns all limit results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanAllLimitCheckResults(ViSession instrSession,
                                                    ViInt32 window, 
													ViInt32 limit,
                                                    ViInt32 results[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    ViInt32     maxIdx = 12;
    ViChar      *pbuffer;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K91")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    
    checkErr (viQueryf (instrSession, "CALC%ld:LIM%ld:BURS:ALL:RESULT?\n","%s", window, limit, buffer));
    
    i=0;                    
    pbuffer = strtok(buffer, ",");
    while ((pbuffer) && (i < maxIdx))   
    {
        results[i++] = atol (pbuffer);
        pbuffer = strtok(NULL, ",");    
    }   
    
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WLAN Limit Check
/* Purpose:  This function returns the result of the limit check of the 
/*           limit line indicated in the selected
/*           measurement window.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWlanLimitCheck(ViSession    instrSession,
                                                    ViInt32 window, 
													ViInt32     limit,
                                                    ViInt32     *result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WLAN_LIM_FAIL_RES, 
            result), 3, "Result");
    checkErr( rszvl_CheckStatus (instrSession));      
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
