/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver
 *  WiMAX option K93
 *
 *  Original Release: March 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 *****************************************************************************/

#include "rszvl.h"   
/*  Array definitions   */
static ViString ACPMeas[] = {"CURR","MAXH"}; 
static ViString switchArr[] = {"OFF","ON"};      
//static ViString WiMAXTDomPreTypeArr[] = {"PRE", "SUBF", "ZONE", VI_NULL};
/*===========================================================================*/
/* Function: Set WiMAX Mode
/* Purpose:  This function selects the WiMAX analyzer mode.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_SetWiMAXMode(ViSession   instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WIMAX_MODE, ""));
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Standard
/* Purpose:  This function configures the Wireless LAN standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXStandard(ViSession instrSession,
                                                  ViInt32   standard)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_STAN, standard), 2, "Standard");
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Trigger
/* Purpose:  This function configures the trigger parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXTrigger(ViSession  instrSession,
                                                ViInt32     mode,
                                                ViReal64    offset,
                                                ViBoolean   autoTriggerLevel,
                                                ViReal64    powerTriggerLevel,
												ViReal64    externalTriggerLevel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
        viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_TRIG_MODE, mode), 2, "Mode");
    switch (mode){
        case 0:
        break;
        case 1:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_TRIGGER_DELAY, offset), 3, "Offset");
			viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_WIMAX_TRIG_EXT_LEV, externalTriggerLevel), 6, "External Trigger Level");
        break;
        case 2:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_TRIGGER_DELAY, offset), 3, "Offset");
            viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_TRIG_POW_LEV_AUTO, autoTriggerLevel), 4, "Auto Level");
            if (!autoTriggerLevel)
                viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_TRIG_POW_LEV, powerTriggerLevel), 5, "Power Level");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Gate
/* Purpose:  This function configures the trigger parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXGate(ViSession instrSession,
                                            ViBoolean   gating,
                                            ViInt32     mode,
                                            ViReal64    delay,
                                            ViReal64    length,
                                            ViBoolean   linkGateAndMarker)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_EGAT, gating), 2, "Gating");
    if (gating)
    {
        if (rszvl_invalidViInt32Range (instrSession, mode, 0, 5))
        {
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");
        }
        switch (mode){
            case RSZVL_VAL_WLAN_GATE_TIME:
                viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_EGAT_HOLD_TIME, delay), 4, "Delay");
                viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_EGAT_LENG_TIME, length), 5, "Length");
            break;
            case RSZVL_VAL_WLAN_GATE_SAMPLE:
                viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_EGAT_HOLD_SAMP, (ViInt32)delay), 4, "Delay");
                viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_EGAT_LENG_SAMP, (ViInt32)length), 5, "Length");
            break;
        }
        viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_EGAT_LINK, linkGateAndMarker), 6, "Link Gate Marker");
    }   
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX ACP
/* Purpose:  This function configures the ACP measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXACP (ViSession instrSession,
                                              ViInt32 acpMode,
                                              ViInt32 numberOfChannels,
                                              ViBoolean noiseCorrection)
{
    ViStatus    error = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ACH_MODE, acpMode), 2, "ACP Mode");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ACH_ACP, numberOfChannels), 
                                            3, "Number Of Channels");
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_ACP_NOISE_CORR, noiseCorrection), 
                                            4, "Noise Correction");     
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX ACP Channel Bandwidth
/* Purpose:  This function configures the ACP channel bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXACPChannelBandwidth (ViSession instrSession,
                                                              ViInt32 channel,
                                                              ViInt32 channelNumber,
                                                              ViReal64 bandwidth)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";   
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    switch (channel){
        case RSZVL_VAL_ACP_TX:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_ACH_BW_CHAN, bandwidth), 
                                            4, "Bandwidth");        
        break;
        case RSZVL_VAL_ACP_ADJ:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_ACH_BW_ACH, bandwidth), 
                                            4, "Bandwidth");
        break;
        case RSZVL_VAL_ACP_ALT:
            if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 4) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");  
            sprintf (buffer, "A%ld", channelNumber); 
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_ACH_BW_ALT, bandwidth), 
                                            5, "Bandwidth");    
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");         
    }
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX ACP Channel Spacing
/* Purpose:  This function configures the ACP channel spacing.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXACPChannelSpacing (ViSession instrSession,
                                                            ViInt32 channel,
                                                            ViInt32 channelNumber,
                                                            ViReal64 spacing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";   
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    switch (channel){
        case RSZVL_VAL_ACP_ADJ:
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_ACH_SPAC_ACH, spacing), 
                                            4, "Spacing");
        break;
        case RSZVL_VAL_ACP_ALT:
            if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 4) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");  
            sprintf (buffer, "A%ld", channelNumber); 
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_ACH_SPAC_ALT, spacing), 
                                            4, "Spacing");  
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");         
    }
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Level
/* Purpose:  This function configures the reference level, and reference 
/*           level offset.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXLevel(ViSession    instrSession,
                                                ViBoolean   autoLevel,
                                                ViReal64    referenceLevel,
                                                ViReal64    externalAttenuation)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_POW_AUTO, autoLevel), 
                                                2, "Auto Level");
    if (!autoLevel)
    {
         viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_RLEV_RF, referenceLevel), 
                                                3, "Reference Level");  
    }       

	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_REFERENCE_LEVEL_OFFSET, externalAttenuation), 
                                                4, "External Attenuation");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*===========================================================================*/
/* Function: Configure WiMAX SEM Mode
/* Purpose:  This function sets the Spectrum Emission Mask (SEM) analysis to 
/*           be UL or DL (uplink or downlink).
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXSEMMode (ViSession instrSession,
                                                  ViInt32 linkMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ESP_ANALYSIS, linkMode),
            2, "Link Mode");
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX SEM TTA
/* Purpose:  This function sets the Spectrum Emission Mask (SEM) analysis 
/*           according to TTA standard or a user defined mask.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXSEMTTA (ViSession instrSession, ViInt32 TTA)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ESP_TTA, TTA),
            2, "TTA");
	
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Auto Level Once
/* Purpose:  This function performs the auto level routine immediately one 
/*           time regardless of the current state of the Auto Level State.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXAutoLevelOnce(ViSession    instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_POW_AUTO_ONCE, ""));
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Baseband Signal Level
/* Purpose:  This function configures the signal characteristics.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXBasebandSignalLevel (ViSession instrSession,
                                                  ViBoolean autoLevel,
                                                  ViReal64   autoLevelTime,
                                                  ViReal64 extAtt)
                                                 
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_POW_AUTO, autoLevel), 2, "Auto Level");
    
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_POW_AUTO_SWE_TIME, autoLevelTime), 3, "Auto Level Time");
    
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_REFERENCE_LEVEL_OFFSET, extAtt), 4, "Ext Att");

    checkErr( rszvl_CheckStatus (instrSession));    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX RF Channel Frequency
/* Purpose:  This function the channel frequency.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXRFChannelFrequency(ViSession   instrSession,
                                                ViInt32     channelNo,
                                                ViReal64    frequency)
                                            
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (channelNo)
    {
        viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_CHAN, channelNo), 2, "Channel No");
    }   
    else
    {
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER, frequency), 3, "Frequency");
    }   
    checkErr( rszvl_CheckStatus (instrSession));  


Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX RF Signal Level
/* Purpose:  This function configures the signal characteristics.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXRFSignalLevel(ViSession    instrSession,
                                                ViBoolean   autoLevel,
                                                ViReal64    autoLevelTime,
                                                ViReal64    signalLevel,
                                                ViReal64    rfAtt,
                                                ViReal64    extAtt,
												ViInt32     signalInput)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_POW_AUTO, autoLevel), 2, "Auto Level");
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_POW_AUTO_SWE_TIME, autoLevelTime), 3, "Auto Level Time");
    if (!autoLevel)
    {
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_ATTENUATION, rfAtt), 5, "RF Att");
        //viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_POW_EXP_RF, signalLevel), 4, "Signal Level");
    }    
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_REFERENCE_LEVEL_OFFSET, extAtt), 6, "Ext Att");
	
	viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_SIGNAL_INPUT, signalInput), 7, "Signal Input");
    if (!autoLevel)
    {
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_POW_EXP_RF, signalLevel), 4, "Signal Level");
    }    
    
	checkErr( rszvl_CheckStatus (instrSession));  


Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX RF Signal Level
/* Purpose:  This function configures the signal characteristics.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXReferenceLevel (ViSession instrSession,
                                                   ViReal64 *referenceLevel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_GetAttributeViReal64(instrSession, "", RSZVL_ATTR_REFERENCE_LEVEL, referenceLevel), 
                2, "Reference Level");
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Marker Enabled
/* Purpose:  This function enables the active marker on the specified trace.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMarkerEnabled(ViSession    instrSession,
                                                        ViInt32     window,
                                                        ViInt32     marker,
                                                        ViBoolean   markerEnabled,
                                                        ViInt32     trace)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
     if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_ENABLED, 
                markerEnabled), 4, "Marker Enabled");
    if (markerEnabled == VI_TRUE)                
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_TRACE, 
                trace), 5, "Trace");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Marker Position
/* Purpose:  This function configures the marker position.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMarkPosition(ViSession     instrSession,
                                                        ViInt32     window,
                                                        ViInt32     marker,
                                                        ViReal64    value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    
	sprintf (buffer, "Win%ld,M%ld", window, marker);   
    
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_POSITION, value), 4, "Value");
    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX Marker Amplitude
/* Purpose:  This function returns the marker amplitude.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXMarkAmplitude (ViSession instrSession,
                                               ViInt32 window, ViInt32 marker,
                                               ViReal64 *amplitude)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    
	sprintf (buffer, "Win%ld,M%ld", window, marker);   
    
	viCheckParm( rszvl_GetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_MARK_Y, amplitude), 4, "Amplitude");
    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Marker Position for Constellation
/* Purpose:  This function positions the selected marker to the indicated 
/*           symbol/carrier.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMarkerPositionConstellation(ViSession  instrSession,
                                                                    ViInt32 window,
                                                                    ViInt32 marker,
                                                                    ViInt32 type,
                                                                    ViInt32 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_WIMAX_CONS_SYM, RSZVL_VAL_WIMAX_CONS_BURST))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
    }
    sprintf (buffer, "Win%ld,M%ld", window, marker);    
    switch (type){
        case RSZVL_VAL_WIMAX_CONS_SYM:
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_MARK_SYMB, position), 5, "Position");
        break;    
        case RSZVL_VAL_WIMAX_CONS_CARR:
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_MARK_CARR, position), 5, "Position");
        break; 
        case RSZVL_VAL_WIMAX_CONS_BURST:
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_MARK_BURST, position), 5, "Position");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Marker Zoom
/* Purpose:  This function defines the ratio to be zoomed around the 
/*           selected marker in the selected measurement window.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMarkerZoom(ViSession   instrSession,
                                                    ViInt32     window,
                                                    ViReal64    zoomRatio)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
  
    sprintf (buffer, "Win%ld,M1", window);     
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_ZOOM, zoomRatio), 4, "Zoom Ratio");
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set WiMAX Marker To Peak
/* Purpose:  This function set the selected marker to the min/max peak in 
/*           the current trace.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_SetWiMAXMarkerPeak(ViSession instrSession,
                                                ViInt32 window,
                                                ViInt32 marker,
                                                ViInt32 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    }
    if (rszvl_invalidViInt32Range (instrSession, position, RSZVL_VAL_MARKER_SEARCH_HIGHEST, RSZVL_VAL_MARKER_SEARCH_MINIMUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Position");
    }
    sprintf (buffer, "Win%ld,M%ld", window, marker);    
    switch (position){                                       

        case RSZVL_VAL_MARKER_SEARCH_MINIMUM:
            viCheckErr( rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_SEARCH_MIN, ""));
        break;    
        case RSZVL_VAL_MARKER_SEARCH_HIGHEST:
            viCheckErr( rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_SEARCH_MAX, ""));
        break;    
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WiMax Disable All Markers
/* Purpose:  This function turns off all the markers in selected measurement 
/*           window.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WiMAXDisableAllMarkers(ViSession instrSession,
                                                    ViInt32 window)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");

    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_WIMAX_MARKER_AOFF, 
                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Demodulation
/* Purpose:  This function configures the WiMax demodulation parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXDemodulation(ViSession instrSession,
                                                    ViInt32     modulationDetectionMode,
                                                    ViInt32     demodulator,
                                                    ViInt32     linkMode,
                                                    ViBoolean   channelEstimation,
                                                    ViBoolean   equalBurstLength,
                                                    ViInt32     minNumberofDataSymbols,
                                                    ViInt32     maxNumberofDataSymbols)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, minNumberofDataSymbols, 1, 1366))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Min Number of Data Symbols");
    }
    if (rszvl_invalidViInt32Range (instrSession, maxNumberofDataSymbols, 1, 1366))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Max Number of Data Symbols");
    }
    
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_BCON_AUTO, modulationDetectionMode), 
                                                    2, "Modulation Detection Mode");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_BAN, demodulator), 3, "Demodulator");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_LMODE, linkMode), 4, "Link Mode");          
  
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_CEST, channelEstimation), 5, "Channel Estimation");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_SYM_EQU, equalBurstLength), 6, "Equal Burst Length");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Min", RSZVL_ATTR_WIMAX_SYM, minNumberofDataSymbols), 7, "Min Number of Data Symbols");
    if (!equalBurstLength)
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Max", RSZVL_ATTR_WIMAX_SYM, maxNumberofDataSymbols), 8, "Max Number of Data Symbols");
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Demodulation for OFDMA
/* Purpose:  This function configures the WiMAX OFDMA demodulation parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXDemodulationOFDMA (ViSession instrSession,
                                                            ViInt32 channelEstimation,
                                                            ViReal64 FFTOffset)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_CEST_OFDMA, channelEstimation),
                                                2, "Channel Estimation");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_WIMAX_FFT_OFFSET, FFTOffset),
                                                3, "FFT Offset");
    checkErr( rszvl_CheckStatus (instrSession)); 
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Subchannelization
/* Purpose:  This function configures the WiMax subchannelization 
/*           parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXSubchannelization(ViSession    instrSession,
                                                            ViBoolean   state,
                                                            ViInt32 subchannel,
                                                            ViInt32 upLinkPhysicalModifier )
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_SUBCHANNEL_STATE, state), 
                                                2, "State");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SUBCHANNEL, subchannel),
                                                3, "Subchannel");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SUBCHANNEL_ULPH, upLinkPhysicalModifier),
                                                4, "Uplink Physical Modifier");
    checkErr( rszvl_CheckStatus (instrSession)); 
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Tracking
/* Purpose:  This function configures the tracking parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXTracking(ViSession instrSession,
                                                    ViBoolean   phase,
                                                    ViBoolean   timing,
                                                    ViBoolean   level)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_TRAC_PHAS, phase), 2, "Phase");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_TRAC_TIME, timing), 3, "Time");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_TRAC_LEV, level), 4, "Level");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Tracking for OFDMA WiBro
/* Purpose:  This function configures the tracking parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXOFDMATracking (ViSession instrSession,
                                                        ViBoolean phase,
                                                        ViBoolean timing,
                                                        ViInt32 trackingPilot)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_TRAC_PHAS, phase), 2, "Phase");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_TRAC_TIME, timing), 3, "Time");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_TRACK_PILOTS, trackingPilot), 
                4, "Tracking Pilot");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Select Filter
/* Purpose:  This function selects the TX and RX filters. The names of the 
/*           filters correspond to the file names returned by 
/*           RSZVL_ATTR_WIMAX_FILT_CAT attribute.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXSelectFilter(ViSession instrSession,
                                            ViString    txFilterName,
                                            ViString    rxFilterName)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);

    checkErr (viPrintf (instrSession, "DEM:FILT:MOD '%s','%s'\n", txFilterName, rxFilterName));
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX Filter Catalog
/* Purpose:  This function reads the names of all available filters.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXFilterCatalog(ViSession  instrSession,
                                                ViInt32 bufferSize,
                                                ViChar  filterList[],
                                                ViInt32*    returnedLength)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *buff;
    ViUInt32    count=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    checkErr (viPrintf (instrSession, "DEM:FILT:CAT?"));         
    checkErr (Rs_ReadDataUnknownLength(instrSession, &buff, &count));
    if (bufferSize != 0)
    {
        strncpy(filterList, buff, ((ViUInt32)bufferSize > count) ? count : bufferSize);
    }
    if (returnedLength) *returnedLength = count;
    checkErr( rszvl_CheckStatus (instrSession));       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Display
/* Purpose:  This function configures the display parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXDisplay(ViSession  instrSession,
                                                ViInt32 combinedTraceDisplay,
                                                ViBoolean   displayTable)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_WIMAX_DISP_FORMAT,
                                                combinedTraceDisplay), 2, "Combined Trace Display");
    
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_WIMAX_DISP_TABL, displayTable), 3, "Display Table");
    
	checkErr( rszvl_CheckStatus (instrSession));   
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Select WiMAX Screen
/* Purpose:  This function selects whether screen A or screen B is active. 
/*           SSELect means Screen SELect. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_SelectWiMAXScreen (ViSession instrSession, ViInt32 window)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	sprintf (buffer, "Win%ld", window);
    
	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_WIMAX_SCREEN_SELECT, NULL));
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Limit
/* Purpose:  This function configures the limit.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXLimit(ViSession    instrSession,
                                                ViInt32     window,
                                                ViInt32     limit,
                                                ViInt32     limitType,
                                                ViInt32     mode,
                                                ViReal64    value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, limitType, RSZVL_VAL_WIMAX_LIM_FERR, RSZVL_VAL_WIMAX_LIM_BERP))
   	{
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Type");
    } 
       
    if ((mode!=RSZVL_VAL_MEASTYPE_MAX) && (mode!=RSZVL_VAL_MEASTYPE_AVER))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Mode");
    }
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MAX:
            strcat (buffer, ",Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            strcat (buffer, ",Aver");
        break;
    }
    
    switch (limitType){
        case RSZVL_VAL_WIMAX_LIM_FERR:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FERR, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WIMAX_LIM_SYMB:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_SYMB, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WIMAX_LIM_SST:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_SST, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WIMAX_LIM_EVM_ALL:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_EVM_ALL, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WIMAX_LIM_EVM_DATA:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_EVM_DATA, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WIMAX_LIM_IQOFF:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_IQOF, 
            value), 6, "Value");
        break;
        case RSZVL_VAL_WIMAX_LIM_BERP:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_BERP, 
            value), 6, "Value");
        break;
    }    

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX All Limits
/* Purpose:  This function sets all limits.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXAllLimits (ViSession instrSession,
                                                    ViInt32 window,
                                                    ViInt32 limit,
                                                    ViInt32 arraySize,
                                                    ViReal64 values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (!values)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Values");    
    }
    pbuffer = buffer;
    
    pbuffer += sprintf (pbuffer, "CALC%ld:LIM%ld:BURS:ALL ", window, limit);
    
    for (i=0;i<arraySize;i++)
         pbuffer += sprintf (pbuffer,"%f,", values[i]);
         
    *--pbuffer = '\0';
    
    viCheckErr (viPrintf (instrSession, "%s\n", buffer));
    
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Get WiMAX Spectrum Max Overstepping
/* Purpose:  This function returns the value at the maximum overstepping of 
/*           the spectrum mask limits.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXSpectrumMaxOverstepping(ViSession    instrSession,
                                                            ViInt32     window,
                                                            ViInt32     limit,
                                                            ViInt32     channelType,
                                                            ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, channelType, RSZVL_VAL_FREQ, RSZVL_VAL_POW))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel Type");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    switch (channelType){
        case RSZVL_VAL_FREQ:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_SPEC_MASK_X, 
                    value), 6, "Value");
        break;
        case RSZVL_VAL_POW:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_SPEC_MASK_Y, 
                    value), 6, "Value");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX All Limits
/* Purpose:  This function returns all limits.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXAllLimits(ViSession  instrSession,
                                            ViInt32     window,
                                            ViInt32     limit,
                                            ViReal64    values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     maxIdx = 12;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

     if (!(strstr (buffer, "K93"))) 
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (!values)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Values");    
    }
        
    checkErr (viQueryf (instrSession, ":CALC%ld:LIM%ld:BURS:ALL?\n","%s", window, limit, buffer));
                        
    pbuffer = strtok(buffer, ",");
    while ((pbuffer) && (i < maxIdx)) 
    {
        values[i++] = atof (pbuffer);
        pbuffer = strtok(NULL, ",");        
    }            
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Frame
/* Purpose:  This function configures the frame parameters
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXFrame (ViSession instrSession,
                                                ViInt32 IDCell,
                                                ViInt32 ULSubframeFrameNumber,
                                                ViReal64 TDDFrame)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_DL_IDCELL, IDCell),
                    2, "ID Cell");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_UL_FRAME, ULSubframeFrameNumber), 
                    3, "UL Subframe Frame Number");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_WIMAX_TDD_TTG, TDDFrame), 
                4, "TDD Frame");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX FrameOFDMA MAP
/* Purpose:  This function configures whether the DSP should perform a 
/*           pre-analysis to determine the Zone, Burst and Downlink
/*           subChannel Map for the current captured signal prior to
/*           calculating the results for OFDMA.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXFrameOFDMAMAP (ViSession instrSession,
                                                   ViInt32 useforAnalysis)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_BCON_AUTO, useforAnalysis),
                    2, "Use For Analysis");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure WiMAX Frame Downlink Preamble Index
/* Purpose:  This function specifies the preamble index calculation.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXDLFramePreambleIndex (ViSession instrSession,
                                                               ViInt32 mode,
                                                               ViInt32 preamblePattern)
{  

    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_DL_PREAMBLE_MODE, mode),
                    2, "Mode");
    if (mode == RSZVL_VAL_WIMAX_PRE_USER)
    {
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_DL_PREAMBLE_INDEX, preamblePattern), 
                    3, "Preamble Pattern");
    }   
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Frame Downlink Used Subchannel
/* Purpose:  This function specifies the allowable logical sub channel usage of 
/*           the transmission spectrum for one of the three downlink PUSC 
/*           segments.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXFrameDLUsedSubchannel (ViSession instrSession,
                                                                ViInt32 segment,
                                                                ViInt32 subchannelUsage)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, segment, 1, 3))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Segment");
    }
    sprintf (buffer, "WSEG%ld", segment);  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_DL_SUBCHANNEL, 
                    subchannelUsage), 3, "Subchannel Usage");
                        
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Load WiMAX Frame Data
/* Purpose:  This function loads a K93 zone frame setup from an xml file 
/*           created from a SMU signal generator, with support for 
/*           the IEEE 80216e/D11 standard.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_LoadWiMAXFrameData (ViSession instrSession,
                                               ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    if (!fileName)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "File Name");   
    }
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_FRAME_LOAD, fileName));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Store WiMAX Frame Data
/* Purpose:  This function sends a locally created and stored K93 zone frame 
/*           file name, with support for the IEEE 80216e/D11 standard, to the 
/*           SMU signal generator. If the command succeeds, the SMU will generate 
/*           the waveform associated with the file sent.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_StoreWiMAXFrameData (ViSession instrSession,
                                                ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    if (!fileName)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "File Name");   
    }
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_FRAME_SAVE, fileName));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Create WiMAX New Zone
/* Purpose:  This function defines a new zone. It accepts only seven arguments 
/*           which make up all the input settings to create a new zone. Up to 
/*           26 user zones can be defined.
/*           New zones can only be appended to the end of the existing defined 
/*           zones. For example if 4 zones are already defined, then the suffix 
/*           required to enter a new zone is 5.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_CreateWiMAXNewZone (ViSession instrSession,
                                               ViInt32 zone, ViBoolean analyze,
                                               ViInt32 type, ViInt32 segment,
                                               ViInt32 length, ViInt32 offset,
                                               ViInt32 permbase, ViInt32 PRBS_ID)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    if (rszvl_invalidViInt32Range (instrSession, zone, 1, 26))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Zone");
    }
    if (rszvl_invalidViBooleanRange (instrSession, analyze))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyze");
    }
    if (rszvl_invalidViInt32Range (instrSession, type, 0, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
    }
    if (rszvl_invalidViInt32Range (instrSession, segment, 0, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Segment");
    }
     
    sprintf (buffer, ":CONF:WIM:ZONE%ld:CONT %s,%s,%ld,%ld,%ld,%ld,%ld\n", zone, 
                        switchArr[analyze], rszvl_rngWiMAXZoneType.rangeValues[type].cmdString, 
                        segment, length, offset, permbase, PRBS_ID);
    viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;                        
}

/*===========================================================================*/
/* Function: Configure WiMAX Zone
/* Purpose:  This function configures zone parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXZone (ViSession instrSession,
                                               ViInt32 zone, 
                                               ViBoolean analyze,
                                               ViInt32 type, 
                                               ViInt32 segment,
                                               ViInt32 length, 
                                               ViInt32 offset,
                                               ViInt32 permbase, 
                                               ViInt32 PRBS_ID)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, zone, 1, 26))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Zone");
    }
    sprintf (buffer, "Z%ld", zone);
    viCheckParm(rszvl_SetAttributeViBoolean(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_ANALYZE_STATE, analyze), 
                3, "Analyze");
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_TYPE, type), 
                4, "Type");
                    
    switch (type){
        case RSZVL_VAL_WIMAX_ZONE_TYPE_DLPUSC:
            viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_SEGMENT_NUM, segment), 
                5, "Segment");
        break;
        default:
        break;
    }   
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_SYMBOL_COUNT, length), 
                6, "Length");

    //Not supported now
    //  viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_SYMBOL_OFFSET, offset), 
    //          7, "Offset");
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_PERM_BASE, permbase), 
                8, "Permbase");
    switch (type){
        case RSZVL_VAL_WIMAX_ZONE_TYPE_DLFUSC:
        case RSZVL_VAL_WIMAX_ZONE_TYPE_DLPUSC:
            viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_PRBS, PRBS_ID), 
                9, "PRBS ID");  
        break;
        default:
        break;
    }   
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Create WiMAX New Zone Burst
/* Purpose:  This function can be used to enter a burst definition that is to 
/*           be associated with a specific zone. It accepts eight arguments 
/*           which make up all the input settings to create a new burst.
/*           A zone may have up to 32 bursts defined within it.
/*           New bursts can only be appended to the end of the existing burst
/*           list. For example if 4 bursts are already defined, then the
/*           suffix required to enter a new burst is 5.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_CreateWiMAXNewZoneBurst (ViSession instrSession,
                                                    ViInt32 zone, 
                                                    ViInt32 burst,
                                                    ViInt32 modulation,
                                                    ViInt32 subchannels,
                                                    ViInt32 symbols,
                                                    ViInt32 slotDuration,
                                                    ViInt32 subchannelOffset,
                                                    ViInt32 symbolOffset,
                                                    ViReal64 burstPower,
                                                    ViInt32 burstType)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    if (rszvl_invalidViInt32Range (instrSession, zone, 1, 26))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Zone");
    }
    if (rszvl_invalidViInt32Range (instrSession, burst, 1, 32))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Burst");
    }
    if (rszvl_invalidViInt32Range (instrSession, modulation, 0, 6))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Modulation");
    }
    if (rszvl_invalidViInt32Range (instrSession, burstType, 0, 3))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Burst Type");
    }
    viCheckErr (viPrintf (instrSession, ":CONF:WIM:ZONE%ld:BURS%ld:CONT %s,%ld,%ld,%ld,%ld,%ld,%.12lf,%s\n", zone, burst,  
                        rszvl_rngWiMAXZoneBurstFormat.rangeValues[modulation].cmdString, subchannels, symbols, slotDuration, 
                        subchannelOffset, 
                        symbolOffset, burstPower, rszvl_rngWiMAXZoneBurstType.rangeValues[burstType].cmdString));
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;                        
}

/*===========================================================================*/
/* Function: Configure WiMAX Zone Burst
/* Purpose:  This function configures burst with a specific zone. A zone may 
/*           have up to 32 bursts defined within it.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXZoneBurst (ViSession instrSession,
                                                    ViInt32 zone, 
                                                    ViInt32 burst,
                                                    ViInt32 modulation,
                                                    ViInt32 subchannels,
                                                    ViInt32 symbols,
                                                    ViInt32 slotDuration,
                                                    ViReal64 burstPower,
                                                    ViInt32 burstType,
                                                    ViBoolean offsetAuto,
                                                    ViInt32 subchannelOffset,
                                                    ViInt32 symbolOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";   
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	sprintf (buffer, "Z%ld,BU%ld", zone, burst); 
    
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_FORMAT, modulation), 
                4, "Modulation");
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_SYMBOL_COUNT, symbols), 
                6, "Symbols");
    if (burstType == RSZVL_VAL_WIMAX_ZONE_BURS_TYPE_ULMAP)
    {
        viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_SLOT_DURATION, slotDuration), 
                7, "Slot Duration");
    } 
    else
    {
        viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_SUBC_COUNT, subchannels), 
                5, "Subchannels");
    }
    viCheckParm(rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_POWER, burstPower), 
                8, "Burst Power");
    viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_TYPE, burstType), 
                9, "Burst Type");
    viCheckParm(rszvl_SetAttributeViBoolean(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_OFFSET_AUTO, offsetAuto), 
                10, "Offset Auto");
    if (offsetAuto == VI_FALSE)
    {
        viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_SUBC_OFFSET, subchannelOffset), 
                11, "Subchannel Offset");
        viCheckParm(rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_SYMBOL_OFFSET, symbolOffset), 
                12, "Symbol Offset");
    }

    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Ext Frame Zone Generator Address
/* Purpose:  This function allows an external SMU signal generator to be connected 
/*           via TCPIP to K93, so that frame zone setup can be downloaded 
/*           directly to K93.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXExtFrameZoneGeneratorAddress (ViSession instrSession,
                                                                       ViString TCPIPAddress)
{
    ViStatus    error = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SMU_ADDRESS, TCPIPAddress), 
                2, "TCP/IP Address");
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX Zone Count
/* Purpose:  This function returns the current count of user defined zones.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXZoneCount (ViSession instrSession,
                                              ViInt32 *count)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm(rszvl_GetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ZONE_COUNT_QUERY, count), 
                3, "Count");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX Zone Burst Count
/* Purpose:  This function returns the current count of user defined bursts 
/*           within the specified zone.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXZoneBurstCount (ViSession instrSession,
                                                   ViInt32 zone, 
                                                   ViInt32 *count)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, zone, 1, 26))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Zone");
    }
    
    sprintf (buffer, "Z%ld", zone); 
    viCheckParm(rszvl_GetAttributeViInt32(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_COUNT_QUERY, count), 
                3, "Count");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WiMAX Zone Delete
/* Purpose:  This function deletes a specific zone. If the specified zone is 
/*           within a larger list of zones, then all following zones will be 
/*           shuffled down to take up the space occupied by the deleted zone.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WiMAXZoneDelete (ViSession instrSession,
                                               ViInt32 zone)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, zone, 1, 26))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Zone");
    }
    
    sprintf (buffer, "Z%ld", zone); 
    viCheckErr(rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_DELETE, VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: WiMAX Zone Burst Delete
/* Purpose:  This function deletes a specific burst from within the specified 
/*           zone.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WiMAXZoneBurstDelete (ViSession instrSession,
                                                 ViInt32 zone, 
                                                 ViInt32 burst)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, zone, 1, 26))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Zone");
    }
    if (rszvl_invalidViInt32Range (instrSession, burst, 1, 32))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Burst");
    }
    
    sprintf (buffer, "Z%ld,BU%ld", zone, burst); 
    viCheckErr(rszvl_SetAttributeViString(instrSession, buffer, RSZVL_ATTR_WIMAX_ZONE_BURST_DELETE, VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Measurement
/* Purpose:  This function configures the measurement parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMeas(ViSession     instrSession,
                                              ViBoolean overallBurstCount,
                                              ViInt32 noofBursttoAnalyze,
                                              ViReal64 captureTime,
                                              ViBoolean swapIQ,
                                              ViInt32 sweepCount,
                                              ViBoolean sweepTimeAutoMaskACP,
                                              ViReal64 sweepTimeMaskACP,
                                              ViInt32 frequencyBand,
                                              ViReal64 channelBandwidth,
                                              ViInt32 guardRatio)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_BURS_COUN_STAT, overallBurstCount), 2, "Overall Burst Count");
    if (overallBurstCount)
        viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_BURS_COUN, noofBursttoAnalyze), 3, "No of Burst to Analyze");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_FBAN, frequencyBand), 
                                                9, "Frequency Band");  
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_BAND_CHAN, channelBandwidth), 
                                                10, "Channel Bandwidth");  
#if 0
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_SWEEP_TIME , captureTime), 4, "Capture Time");
#endif
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_SWAP_IQ, swapIQ), 5, "Swap IQ");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_NUMBER_OF_SWEEPS, sweepCount), 6, "Sweep Count");
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_ACPR_TIME_AUTO, sweepTimeAutoMaskACP),
                7, "Sweep Time Auto (Mask/ACP)");
    if (sweepTimeAutoMaskACP == VI_FALSE)
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_ACPR_TIME, sweepTimeMaskACP), 
                                                8, "Sweep Time (Mask/ACP)");  
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_IGR, guardRatio), 
                                                11, "Guard Ratio"); 

    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Measurement for OFDMA
/* Purpose:  This function configures the measurement parameters for OFDMA Standard. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMeasOFDMA (ViSession instrSession,
                                                    ViReal64 captureTime,
                                                    ViBoolean swapIQ,
                                                    ViInt32 sweepCount,
                                                    ViBoolean sweepTimeAutoMaskACP,
                                                    ViReal64 sweepTimeMaskACP,
                                                    ViInt32 frequencyBand,
                                                    ViReal64 channelBandwidth,
                                                    ViInt32 guardRatio,
                                                    ViInt32 FFTSize,
                                                    ViInt32 numberofSubframestoAnalyze)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, frequencyBand, RSZVL_VAL_WIMAX_FBAN_UNSP, RSZVL_VAL_WIMAX_FBAN_WCS))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Frequency Band");
    }
#if 0
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_SWEEP_TIME , captureTime), 2, "Capture Time");
#endif
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_SWAP_IQ, swapIQ), 3, "Swap IQ");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_NUMBER_OF_SWEEPS, sweepCount), 4, "Sweep Count");
     viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_ACPR_TIME_AUTO, sweepTimeAutoMaskACP),
                5, "Sweep Time Auto (Mask/ACP)");
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_ACPR_TIME, sweepTimeMaskACP), 
                                                6, "Sweep Time (Mask/ACP)");  
        viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_FBAN, frequencyBand), 
                                                7, "Frequency Band");  
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_BAND_CHAN, channelBandwidth), 
                                                8, "Channel Bandwidth");                                                   
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_IGR, guardRatio), 
                                                9, "Guard Ratio"); 
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_NFFT, FFTSize), 
                                                10, "FFT Size"); 
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ZONE_COUNT, numberofSubframestoAnalyze), 
                                                11, "Number Of Frames to Analyze"); 
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure WiMAX Measurement for WiBro
/* Purpose:  This function configures the measurement parameters for WiBro Standard. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXMeasWiBro (ViSession instrSession,
                                                    ViReal64 captureTime,
                                                    ViBoolean swapIQ,
                                                    ViInt32 sweepCount,
                                                    ViBoolean sweepTimeAutoMaskACP,
                                                    ViReal64 sweepTimeMaskACP,
                                                    ViReal64 channelBandwidth,
                                                    ViInt32 numberofSubframestoAnalyze)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
#if 0 
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_SWEEP_TIME , captureTime), 2, "Capture Time");
#endif
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_SWAP_IQ, swapIQ), 3, "Swap IQ");
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_NUMBER_OF_SWEEPS, sweepCount), 4, "Sweep Count");
     viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", RSZVL_ATTR_WIMAX_ACPR_TIME_AUTO, sweepTimeAutoMaskACP),
                6, "Sweep Time Auto (Mask/ACP)");
    if (sweepTimeAutoMaskACP == VI_FALSE)
    {
        viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_ACPR_TIME, sweepTimeMaskACP), 
                                                    6, "Sweep Time (Mask/ACP)");  
    }   
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_BAND_CHAN, channelBandwidth), 
                                                7, "Channel Bandwidth");   
    
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_ZONE_COUNT, numberofSubframestoAnalyze), 
                                                8, "Number Of Frames to Analyze"); 

 
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX PVT Measurement
/* Purpose:  This function sets measurement type to be Power Versus Time and 
/*           configures measurement parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXPVTMeas(ViSession  instrSession,
                                                ViInt32 risingFalling,
                                                ViInt32 burstSelection)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_PVT_SEL, risingFalling), 2, "Rising/Falling");
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_PVT, ""));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_BURS_PVT_BURS, burstSelection),
                                                3, "Burst Selection");
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function:Configure WiMAX PVT Measurement for OFDMA WiBro
/* Purpose:  This function sets measurement type to be Power Versus Time and 
/*           configures measurement parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXPVTMeasOFDMA (ViSession instrSession,
                                                       ViInt32 risingFalling)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_PVT_SEL, risingFalling), 2, "Rising/Falling");
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_PVT, ""));
   
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX EVM Measurement
/* Purpose:  This function sets measurement type to be EVM.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXEVMMeasurement(ViSession   instrSession,
                                                        ViInt32     mode,
                                                        ViBoolean   autoScaling,
                                                        ViReal64    perDivision,
                                                        ViInt32     unit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_WIMAX_EVM_MODE_SYMBOL, RSZVL_VAL_WIMAX_EVM_MODE_FREQ))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Mode");
    }
     
    switch (mode){
        case RSZVL_VAL_WIMAX_EVM_MODE_SYMBOL:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_EVM_ESYM, ""));
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_UNIT_EVM, unit), 5, "Unit");
            viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "Win2,TR1", RSZVL_ATTR_WIMAX_DISP_AUTO_Y, autoScaling), 3, "Auto Scaling");
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "Win2,TR1", RSZVL_ATTR_WIMAX_Y_PDIV, perDivision), 4, "Per Division");
   		break;
        case RSZVL_VAL_WIMAX_EVM_MODE_CARRIER:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_EVM_ECAR, ""));
            viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_UNIT_EVM, unit), 5, "Unit");
			viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "Win2,TR1", RSZVL_ATTR_WIMAX_DISP_AUTO_Y, autoScaling), 3, "Auto Scaling");
            viCheckParm( rszvl_SetAttributeViReal64(instrSession, "Win2,TR1", RSZVL_ATTR_WIMAX_Y_PDIV, perDivision), 4, "Per Division");
        break;
        case RSZVL_VAL_WIMAX_EVM_MODE_PHAS:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_PRE, "")); 
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_PRE_SEL, RSZVL_VAL_PRE_PHAS));
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_UNIT_PRE, RSZVL_VAL_UNIT_DEG));
        break;
        case RSZVL_VAL_WIMAX_EVM_MODE_FREQ:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_PRE, "")); 
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_PRE_SEL, RSZVL_VAL_PRE_FREQ));
            viCheckErr( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_UNIT_PRE, RSZVL_VAL_UNIT_HZ));            
        break;
    }
    //viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "Win2,TR1", RSZVL_ATTR_WIMAX_DISP_AUTO_Y, autoScaling), 3, "Auto Scaling");
    //viCheckParm( rszvl_SetAttributeViReal64(instrSession, "Win2,TR1", RSZVL_ATTR_WIMAX_Y_PDIV, perDivision), 4, "Per Division");
   
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Spectrum Measurement
/* Purpose:  This function sets measurement type to be Spectrum.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXSpectrumMeasurement(ViSession  instrSession,
                                                             ViInt32    mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
        if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_WIMAX_MODE_IEEE, RSZVL_VAL_WIMAX_MODE_SFLAT))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Mode");
    }
    switch (mode){
        case RSZVL_VAL_WIMAX_MODE_IEEE:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_MASK_SEL,
                                                                            RSZVL_VAL_SPEC_IEEE));
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_MASK, ""));
        break;
        case RSZVL_VAL_WIMAX_MODE_ETSI:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_MASK_SEL,
                                                      RSZVL_VAL_SPEC_ETSI));
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_MASK, ""));
        break;
        case RSZVL_VAL_WIMAX_MODE_FFT:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_FFT, ""));
        break;
        case RSZVL_VAL_WIMAX_MODE_FLAT:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_FLAT_SEL,
                                                      RSZVL_VAL_WIMAX_FLAT_FLAT));
        break;
        case RSZVL_VAL_WIMAX_MODE_FLAT_GRDEL:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_FLAT_SEL,
                                                      RSZVL_VAL_WIMAX_FLAT_GRD));
        break;
        case RSZVL_VAL_WIMAX_MODE_FLAT_DIFF:
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_FLAT_SEL,
                                                      RSZVL_VAL_WIMAX_FLAT_DIF));
        break;
        case RSZVL_VAL_WIMAX_MODE_ACPR_ABS:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_ACPR, ""));
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_ACPR_SEL,
                                                      RSZVL_VAL_ABS));
        break;
        case RSZVL_VAL_WIMAX_MODE_ACPR_REL:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_ACPR, ""));
            viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_SPEC_ACPR_SEL,
                                                      RSZVL_VAL_REL));
        break;
        case RSZVL_VAL_WIMAX_MODE_MASK:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_MASK, ""));
        break;
		case RSZVL_VAL_WIMAX_MODE_SFLAT:
			viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_SPEC_FLAT, ""));
		break;
    }
    checkErr( rszvl_CheckStatus (instrSession));     
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Constellation Measurement
/* Purpose:  This function configures the Constellation measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXConstellationMeasurement(ViSession instrSession,
                                                                    ViInt32 constellationMode,
                                                                    ViInt32 carrierSelection,
                                                                    ViInt32 carrierNumber)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, constellationMode, RSZVL_VAL_WIMAX_CONS_SYM, RSZVL_VAL_WIMAX_CONS_CARR))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Constelation Mode");
    }
    if (rszvl_invalidViInt32Range (instrSession, carrierSelection, RSZVL_VAL_SELECTION_ALL, RSZVL_VAL_SELECTION_NUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Carrier Selection");
    }
    switch (constellationMode){
        case RSZVL_VAL_WIMAX_CONS_SYM:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_CONS_CSYM, ""));
            switch (carrierSelection){
                case RSZVL_VAL_SELECTION_NUM:
                    if (rszvl_invalidViInt32Range (instrSession, carrierNumber, -100, 100) || (carrierNumber == 0))
                    {
                        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Carrier Number");
                    }
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "Carrier", RSZVL_ATTR_WIMAX_CONS_SEL_NR, carrierNumber), 4, "Carrier Number");                        
                break; 
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Carrier",
                                                               RSZVL_ATTR_WIMAX_CONS_SEL,
                                                               RSZVL_VAL_SELECTION_ALL), 
                                                              3, "Carrier Selection");
                break;
                case RSZVL_VAL_SELECTION_PIL:
                    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Carrier",
                                                               RSZVL_ATTR_WIMAX_CONS_SEL,
                                                               RSZVL_VAL_SELECTION_PIL), 3, "Carrier Selection");
                break;
             }   
        break;
        case RSZVL_VAL_WLAN_CONS_CARR:
            viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_CONS_CCAR, ""));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Constellation Selection
/* Purpose:  This function configures the Constellation Selection for 
*            IEEE 802.16e-2005 OFDMA & WiBro.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXConstellationSelection (ViSession instrSession,
                                                                 ViInt32 modulation,
                                                                 ViInt32 burstSelection,
                                                                 ViInt32 burstNumber,
                                                                 ViInt32 carrierSelection,
                                                                 ViInt32 carrierNumber,
                                                                 ViInt32 symbolSelection,
                                                                 ViInt32 symbolNumber)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, burstSelection, RSZVL_VAL_SELECTION_ALL, RSZVL_VAL_SELECTION_NUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Burst Selection");
    }
    if (rszvl_invalidViInt32Range (instrSession, carrierSelection, RSZVL_VAL_SELECTION_ALL, RSZVL_VAL_SELECTION_NUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Carrier Selection");
    }
    if ((symbolSelection!=RSZVL_VAL_SELECTION_ALL)&&(symbolSelection!=RSZVL_VAL_SELECTION_NUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Symbol Selection");
    }
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Constellation", RSZVL_ATTR_WIMAX_FORMAT_SELECT,
                                                               modulation), 3, "Modulation");
                                                               
    switch (burstSelection){
                case RSZVL_VAL_SELECTION_NUM:
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "Burst", RSZVL_ATTR_WIMAX_CONS_SEL_NR, 
                    burstNumber), 4, "Burst Number");                        
                break; 
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Burst",
                                                               RSZVL_ATTR_WIMAX_CONS_SEL,
                                                               RSZVL_VAL_SELECTION_ALL));
                break;
                case RSZVL_VAL_SELECTION_PIL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Burst",
                                                               RSZVL_ATTR_WIMAX_CONS_SEL,
                                                               RSZVL_VAL_SELECTION_PIL));
                break;
                
    }   
    switch (carrierSelection){
                case RSZVL_VAL_SELECTION_NUM:
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "Carrier", RSZVL_ATTR_WIMAX_CONS_SEL_NR, 
                    carrierNumber), 6, "Carrier Number");                        
                break; 
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Carrier",
                                                               RSZVL_ATTR_WIMAX_CONS_SEL,
                                                               RSZVL_VAL_SELECTION_ALL));
                break;
                case RSZVL_VAL_SELECTION_PIL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Carrier",
                                                              RSZVL_ATTR_WIMAX_CONS_SEL,
                                                              RSZVL_VAL_SELECTION_PIL));
                break;
                
    }
    switch (symbolSelection){
                case RSZVL_VAL_SELECTION_NUM:
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "Symbol", RSZVL_ATTR_WIMAX_CONS_SEL_NR, 
                    symbolNumber), 8, "Symbol Number");                        
                break; 
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Symbol",
                                                              RSZVL_ATTR_WIMAX_CONS_SEL,
                                                              RSZVL_VAL_SELECTION_ALL));
                break;
    }  
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Bitstream Selection
/* Purpose:  This function configures the Bitstream Selection for 
/*           IEEE 802.16e-2005 OFDMA & WiBro.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXBitstreamSelection (ViSession instrSession,
                                                             ViInt32 modulation,
                                                             ViInt32 burstSelection,
                                                             ViInt32 burstNumber,
                                                             ViInt32 symbolSelection,
                                                             ViInt32 symbolNumber)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, burstSelection, RSZVL_VAL_SELECTION_NUM, RSZVL_VAL_SELECTION_PIL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Burst Selection");
    }
    if (rszvl_invalidViInt32Range (instrSession, symbolSelection, RSZVL_VAL_SELECTION_NUM, RSZVL_VAL_SELECTION_ALL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Symbol Selection");
    }
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "Bitstream", RSZVL_ATTR_WIMAX_FORMAT_SELECT,
                                                               modulation), 3, "Modulation");
                     
    switch (burstSelection){
                case RSZVL_VAL_SELECTION_NUM:
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "Burst", RSZVL_ATTR_WIMAX_BSTR_SEL_NR, 
                    burstNumber), 4, "Burst Number");                        
                break; 
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Burst",
                                                               RSZVL_ATTR_WIMAX_BSTR_SEL,
                                                               RSZVL_VAL_SELECTION_ALL));
                break;
                case RSZVL_VAL_SELECTION_PIL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Burst",
                                                               RSZVL_ATTR_WIMAX_BSTR_SEL,
                                                               RSZVL_VAL_SELECTION_PIL));
                break;
                
    }   
    switch (symbolSelection){
                case RSZVL_VAL_SELECTION_NUM:
                    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "Symbol", RSZVL_ATTR_WIMAX_BSTR_SEL_NR, 
                    symbolNumber), 8, "Symbol Number");                        
                break; 
                case RSZVL_VAL_SELECTION_ALL:
                    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "Symbol",
                                                              RSZVL_ATTR_WIMAX_BSTR_SEL,
                                                              RSZVL_VAL_SELECTION_ALL));
                break;
    }  
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX Statistics Measurement
/* Purpose:  This function configures the statistics measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXStatisticsMeasurement(ViSession    instrSession,
                                                                ViInt32     mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_WIMAX_STAT_CCDF, RSZVL_VAL_WIMAX_STAT_BSUM))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Mode");
    }
    switch (mode){
        case RSZVL_VAL_WIMAX_STAT_CCDF:
            viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WIMAX_STAT_CCDF, ""));
        break;
        case RSZVL_VAL_WIMAX_STAT_BSTR:
            viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WIMAX_STAT_BSTR, ""));
        break;
        case RSZVL_VAL_WIMAX_STAT_BSUM:
            viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_WIMAX_STAT_BSUM, ""));
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX IQ Measurement Parameters
/* Purpose:  This function configures the IQ Trace parameters. 
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXIQMeasurementParameters(ViSession  instrSession,
                                                                ViReal64    sampleRate)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_SRAT, sampleRate), 2, "Sample Rate");
    
	checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX SEM Measurement
/* Purpose:  This function configures the Spectrum Emission Mask measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXSEMMeasurement (ViSession instrSession,
                                                        ViInt32 powerClass)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_SEM_CLASS, powerClass), 2, "Power Class");          
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX ACP Measurements
/* Purpose:  This function returns the result values of the adjacent channel 
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXACPMeasurement(ViSession instrSession,
                                                ViInt32     window,
                                                ViInt32     marker,
                                                ViInt32     mode,
                                                ViReal64    values[])
{
    ViStatus    error   = VI_SUCCESS; 
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     maxIdx = 12;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 1))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
    } 
    if (rszvl_invalidViInt32Range (instrSession, mode, RSZVL_VAL_ACP_CURR, RSZVL_VAL_ACP_MAX))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Mode");
    } 
    if (!values)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Values");    
    }
        
    checkErr (viQueryf (instrSession, ":CALC%ld:MARK%ld:FUNC:POW:RES:%s?\n","%s", window, marker, ACPMeas[mode], buffer));
                        
    pbuffer = strtok(buffer, ",");
    i=0;
    
    while ((pbuffer) && (i < maxIdx)) 
    {
        values[i++] = atof (pbuffer);
        pbuffer = strtok(NULL, ",");        
    }            
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Load WiMAX IQ Data
/* Purpose:  This function loads the IQ data from the specified .iqw file.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_LoadWiMAXIQData(ViSession    instrSession,
                                            ViString    fileName)
{
    ViStatus    error = VI_SUCCESS;
    if (!fileName)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "File Name");   
    }
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_IQ_LOAD, fileName));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Store WiMAX IQ Data
/* Purpose:  This function stores the IQ data from/to the specified .iqw 
/*           file.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_StoreWiMAXIQData(ViSession   instrSession,
                                            ViString    fileName)
{
    ViStatus    error = VI_SUCCESS;
    if (!fileName)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "File Name");   
    }
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_IQ_SAVE, fileName));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WiMAX Burst Recalc
/* Purpose:  This function forces the IQ measurement results to be 
/*           recalculated according to the current settings.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WiMAXBurstImm (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_CALC_BURS, ""));
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WiMAX Refresh IQ Measurement
/* Purpose:  This function updates the current IQ measurement results to reflect 
/*           the current measurement settings. Note no new IQ data is captured.
/*           I.e. the measurement settings apply to the IQ data being currently 
/*           in the capture buffer. The command applies exclusively to IQ 
/*           measurements. It requires available IQ data.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WiMAXRefreshIQMeasurement (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_WIMAX_REFRESH, ""));
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: WiMAX Get Marker TTC
/* Purpose:  This function returns the time to the start of the first frame in 
/*           the capture buffer.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_WiMAXGetMarkerTTC (ViSession instrSession,
                                           ViInt32 window, ViInt32 markerNumber,
                                           ViReal64 *time)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    sprintf (buffer, "Win%ld,M%ld", window, markerNumber);
    
	viCheckParm( rszvl_GetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_WIMAX_QUERY_TTC, time), 4, "Time");
    
	checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read WiMAX Trace Data
/* Purpose:  This function returns all the measured data that relates to the 
/*           currently selected measurement type. The returned data is 
/*           particular to the currently selected measurement type and is 
/*           specified in the Trace Data parameter description.
/*           
/*           Attribute(s):
/*           no attribute
/*           
/*           Remote-control command(s):
/*           TRACE[:DATA]? TRACE1 | TRACE2 | TRACE3 | TRACE4 | TRACE5 | 
/*           TRACE6
/*           :FORM REAL,32
/*           *CLS
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadWiMAXTraceData(ViSession instrSession,
                                            ViInt32     sourceTrace,
                                            ViInt32     arraySize,
                                            ViReal64    traceData[],
                                            ViInt32*    noofValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))  
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (rszvl_invalidViInt32Range (instrSession, sourceTrace, 1, 6))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source Trace");
    }
    
	sprintf (buffer, ":TRAC? TRACE%ld", sourceTrace);
    viCheckErr (rszvl_dataReadTrace (instrSession, buffer, arraySize, 
                    traceData, noofValues));
    
	checkErr( rszvl_CheckStatus (instrSession));           

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read WiMAX Memory IQ Data
/* Purpose:  This function returns all the IQ data associated with the 
/*           measurement acquisition time. The result values are scaled 
/*           linear in unit Volt and correspond to the voltage at the RF 
/*           input of the instrument. The command returns a comma-separated 
/*           list of the measured voltage values in floating point format 
/*           (Comma Separated Values = CSV). The number of values returned is 
/*           2 * number of samples, the first half being the I values, the 
/*           second half the Q values.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadWiMAXMemoryIQData(ViSession  instrSession,
                                                ViInt32     offsetSamples,
                                                ViInt32     noofSamples,
                                                ViInt32     bufferSize,
                                                ViInt32*    noofPoints,
                                                ViReal64    realPartsI[],
                                                ViReal64    imaginaryPartsQ[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE]= "";
    ViInt32     num; 
    ViReal64    *data=VI_NULL;              
    ViInt32     data_num; 
    ViInt32     i;
    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))  
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    checkErr (viPrintf(instrSession, ":TRAC:IQ:DATA:MEM? %ld,%ld\n",
                offsetSamples, noofSamples));
    data_num = noofSamples * 2;
    data = malloc (sizeof (ViReal64) * data_num); 
    /*viCheckErr (rszvl_dataReadTraceIQ (instrSession, bufferSize, noofPoints, 
                    realPartsI, imaginaryPartsQ));*/
    viCheckErr (rszvl_dataReadTraceOnly (instrSession, data_num, data, &num));
    
    if (noofPoints) *noofPoints=num/2;

    num = (noofSamples>bufferSize)?bufferSize:noofSamples;

    for (i=0;i<num;i++)
    {
        realPartsI[i]=data[2*i];
        imaginaryPartsQ[i]=data[2*i+1];
    }
    /*
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &p_buffer, &ret_val));
    pbuffer = strtok(p_buffer, ",");
    i=0;
    while ((pbuffer) && (i < bufferSize)) 
    {
        realPartsI[i++] = atof (pbuffer);
        pbuffer = strtok(NULL, ",");        
    }
    i=0;
    while ((pbuffer) && (i < bufferSize)) 
    {
        imaginaryPartsQ[i++] = atof (pbuffer);
        pbuffer = strtok(NULL, ",");        
    }
    if (noofPoints) *noofPoints=2*noofSamples;
    */
    checkErr( rszvl_CheckStatus (instrSession));                      
Error:
    if (data) free (data); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read WiMAX SEM Results
/* Purpose:  This function returns the Spectrum Emission Mask summary results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ReadWiMAXSEMResults (ViSession instrSession,
                                                ViInt32 noOfValues,
                                                ViReal64 index[],
                                                ViReal64 startFrequencyBand[],
                                                ViReal64 stopFrequencyBand[],
                                                ViReal64 resolutionBandwidth[],
                                                ViReal64 limitFailFrequency[],
                                                ViReal64 powerAbs[],
                                                ViReal64 powerRel[],
                                                ViReal64 limitDistance[],
                                                ViReal64 failureFlag[],
                                                ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuf=VI_NULL;
    ViInt32     count=0;
    ViUInt32    retCnt;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (index == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Index");
    if (startFrequencyBand == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Start Frequency Band"); 
    if (stopFrequencyBand == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Stop Frequency Band"); 
    if (resolutionBandwidth == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Resolution Bandwidth"); 
    if (limitFailFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Limit Fail Frequency"); 
    if (powerAbs == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Power Abs");
    if (powerRel == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Power Rel"); 
    if (limitDistance == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Limit Distance"); 
    if (failureFlag == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Failure Flag");
    
    viCheckErr (viPrintf (instrSession, "TRAC:DATA? LIST\n"));
    viCheckErr( Rs_ReadDataUnknownLength(instrSession, &buffer, &retCnt));
    
    pbuf=strtok(buffer, ",");
    if (pbuf)
        do{
            if (count<noOfValues)
            {
                sscanf (pbuf, "%le", &index[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &startFrequencyBand[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &stopFrequencyBand[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &resolutionBandwidth[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &limitFailFrequency[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &powerAbs[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &powerRel[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &limitDistance[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &failureFlag[count]);
                pbuf=strtok(NULL, ",");
            }               
            else
            {
                for (i=0;i<11;i++)
                    pbuf=strtok(NULL, ","); 
            }       
            count++;
        }while (pbuf);  

    if (returnedValues)
        *returnedValues = count;
     
    checkErr( rszvl_CheckStatus (instrSession));     
Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX Burst All
/* Purpose:  This function returns all the results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXBurstAll(ViSession instrSession,
                                            ViReal64    results[],
                                            ViInt32*    returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    checkErr (viQueryf (instrSession, "FETC:BURS:ALL?\n","%s", buffer));
                        
    pbuffer = strtok(buffer, ",");
    i=0;
    if (results)
    { 
        while (pbuffer) 
        {
            results[i++] = atof (pbuffer);
            pbuffer = strtok(NULL, ",");        
        }            
    }
    else
    {
        while (pbuffer)
        {
            i++;
            pbuffer = strtok(NULL, ",");
         }   
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX Burst Power
/* Purpose:  This function returns the power measurement value, measured 
/*           during the measurement.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXBurstPower(ViSession   instrSession,
                                                ViInt32 powerMeasurement,
                                                ViInt32     mode,
                                                ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, powerMeasurement, RSZVL_VAL_WIMAX_MEAS_RMS, RSZVL_VAL_WIMAX_MEAS_RMS_DLPR))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Power Measurement");
    }
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");    
    }
    
    switch (powerMeasurement){
       case RSZVL_VAL_WIMAX_MEAS_RMS:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_RMS, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WIMAX_MEAS_CREST:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_CRES, 
                    value), 3, "Value");                    
        break;
        case RSZVL_VAL_WIMAX_MEAS_RMS_ALL:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_RMS_ALL, 
                    value), 3, "Value");                    
        break;
        case RSZVL_VAL_WIMAX_MEAS_RMS_PILOT:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_RMS_PILOT, 
                    value), 3, "Value");                    
        break;
        case RSZVL_VAL_WIMAX_MEAS_RMS_DATA:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_RMS_DATA, 
                    value), 3, "Value");                    
        break;
        case RSZVL_VAL_WIMAX_MEAS_RMS_DLPR:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_RMS_DLPR, 
                    value), 3, "Value");                    
        break;
        
    }
   
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX Burst Error
/* Purpose:  This function returns the burst errors.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXBurstError(ViSession   instrSession,
                                                ViInt32     errorType,
                                                ViInt32     mode,
                                                ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, errorType, RSZVL_VAL_WIMAX_ERR_FREQ, RSZVL_VAL_WIMAX_ERR_SYMB))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Error Type");
    }
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");
    }
   
    switch (errorType){
        case RSZVL_VAL_WIMAX_ERR_FREQ:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_FERR, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WIMAX_ERR_SYMB:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_SYMB, 
                    value), 3, "Value");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX IQ Impairments
/* Purpose:  This function returns the burst errors.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXIQImp(ViSession    instrSession,
                                            ViInt32     iqImpairment,
                                            ViInt32     mode,
                                            ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, iqImpairment, RSZVL_VAL_WIMAX_IQ_OFFS, RSZVL_VAL_WIMAX_IQ_QUAD))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "IQ Impairment");
    }

    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");
    }
    
    switch (iqImpairment){
        case RSZVL_VAL_WIMAX_IQ_OFFS:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_IQOF, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WIMAX_IQ_IMBA:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_GIMB, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WIMAX_IQ_QUAD:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_QUAD, 
                    value), 3, "Value");
        break;
     }   
 
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX EVM
/* Purpose:  This function returns the EVM measurement value.
/*           
/*           Note:
/*           (1) If no measurement has been performed yet, a query error 
/*           will occur.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXEVM(ViSession  instrSession,
                                        ViInt32     evm,
                                        ViInt32     mode,
                                        ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, evm, RSZVL_VAL_WIMAX_EVM_ALL, RSZVL_VAL_WIMAX_EVM_PILOT))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "EVM");
    }
    
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MIN:
            sprintf (buffer, "Min");
        break;
        case RSZVL_VAL_MEASTYPE_MAX:
            sprintf (buffer, "Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            sprintf (buffer, "Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode");
    }

    switch (evm){
        case RSZVL_VAL_WIMAX_EVM_ALL:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_EVM_ALL, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WIMAX_EVM_DATA:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_EVM_DATA, 
                    value), 3, "Value");
        break;
        case RSZVL_VAL_WIMAX_EVM_PILOT:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_WIMAX_FETC_EVM_PILOT, 
                    value), 3, "Value");
        break;
     }   

    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX CINR Values
/* Purpose:  This function returns the Carrier Interference Noise Ratio 
/*           values.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXCINRValues(ViSession   instrSession,
                                                ViReal64    values[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    checkErr (viQueryf (instrSession, "FETC:BURS:CINR?\n","%s", buffer));
                        
    pbuffer = strtok(buffer, ",");
    i=0;
    if (values)
    { 
        while (pbuffer) 
        {
            values[i++] = atof (pbuffer);
            pbuffer = strtok(NULL, ",");        
        }            
    }
    else
    {
        while (pbuffer)
        {
            i++;
            pbuffer = strtok(NULL, ",");
         }   
    }
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX RSSI Values
/* Purpose:  This function returns the Received Signal Strength Indication 
/*           values.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXRSSIValues(ViSession   instrSession,
                                                ViReal64    values[])
{
     ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *pbuffer;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    checkErr (viQueryf (instrSession, "FETC:BURS:RSSI?\n","%s", buffer));
                        
    pbuffer = strtok(buffer, ",");
    i=0;
    if (values)
    { 
        while (pbuffer) 
        {
            values[i++] = atof (pbuffer);
            pbuffer = strtok(NULL, ",");        
        }            
    }
    else
    {
        while (pbuffer)
        {
            i++;
            pbuffer = strtok(NULL, ",");
         }   
    }
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX Count
/* Purpose:  This function returns the count of analyzed burst or number of 
/*           symbols of analyzed burst.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXCount( ViSession   instrSession,
                                            ViInt32 type,
                                            ViInt32*    count)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_WIMAX_BURS, RSZVL_VAL_WIMAX_ZONE))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
    }
    
    switch (type){
        case RSZVL_VAL_WIMAX_BURS:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_FETC_BURS_COUNT, 
                            count), 3, "Count");    
        break;
        case RSZVL_VAL_WIMAX_SYMB:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_FETC_SYMB_COUNT, 
                            count), 3, "Count");    
        break;
        case RSZVL_VAL_WIMAX_ZONE:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_WIMAX_FETC_ZONE_COUNT, 
                            count), 3, "Count");    
        break;
   }     
   checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Fetch WiMAX Time Domain Preambles
/* Purpose:  This function returns the minimum, average and maximum time 
/*           domain values.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_FetchWiMAXTimeDomainPreamble (ViSession instrSession,
                                                         ViInt32 result,
                                                         ViReal64 *minimum,
                                                         ViReal64 *average,
                                                         ViReal64 *maximum)
{
    ViStatus    error = VI_SUCCESS;
    ViReal64    tmp_min = 0,
                tmp_max = 0,
                tmp_aver = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    switch (result){
        case RSZVL_VAL_WIMAX_PREA:
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Min", RSZVL_ATTR_WIMAX_FETC_TDOM_PREAMBLE, &tmp_min));
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Max", RSZVL_ATTR_WIMAX_FETC_TDOM_PREAMBLE, &tmp_max));
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Aver", RSZVL_ATTR_WIMAX_FETC_TDOM_PREAMBLE, &tmp_aver));
        break;
        case RSZVL_VAL_WIMAX_SUBF:
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Min", RSZVL_ATTR_WIMAX_FETC_TDOM_SUBFRAME, &tmp_min));
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Max", RSZVL_ATTR_WIMAX_FETC_TDOM_SUBFRAME, &tmp_max));
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Aver", RSZVL_ATTR_WIMAX_FETC_TDOM_SUBFRAME, &tmp_aver));
        break;
        case RSZVL_VAL_WIMAX_ZONE:
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Min", RSZVL_ATTR_WIMAX_FETC_TDOM_ZONE, &tmp_min));
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Max", RSZVL_ATTR_WIMAX_FETC_TDOM_ZONE, &tmp_max));
            viCheckErr(rszvl_GetAttributeViReal64(instrSession, "Aver", RSZVL_ATTR_WIMAX_FETC_TDOM_ZONE, &tmp_aver));
        break;
        default:
              viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Result");  
    }
    if (minimum)
       *minimum = tmp_min;  
    if (average)
        *average = tmp_aver;
    if (maximum)
        *maximum = tmp_max;
    
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure WiMAX List Result Units
/* Purpose:  This function specifies the units for table results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXListResultUnit(ViSession   instrSession,
                                                        ViInt32     unit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32(instrSession, "", RSZVL_ATTR_WIMAX_TABLE_UNIT, unit), 2, "Unit");     
    checkErr( rszvl_CheckStatus (instrSession));    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure WiMAX RSSI CINR Averaging
/* Purpose:  This function specifies the RSSI, CINR averaging parameter.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_ConfigureWiMAXRSSICINRAvg (ViSession instrSession,
                                                      ViReal64 averagingParameter)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViReal64(instrSession, "", RSZVL_ATTR_WIMAX_RSSI_AVG_PARAM, averagingParameter),
    2, "Averaging Parameter");     
    checkErr( rszvl_CheckStatus (instrSession));   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX Limit Check Result
/* Purpose:  This function returns the limit results
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXLimitCheckResult(ViSession   instrSession,
                                                ViInt32     window,
                                                ViInt32     limit,
                                                ViInt32     limitType,
                                                ViInt32     mode,
                                                ViInt32*    result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
		
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    if (rszvl_invalidViInt32Range (instrSession, limitType, RSZVL_VAL_WIMAX_LIM_FERR, RSZVL_VAL_WIMAX_LIM_BERP))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Type");
    } 
    
	sprintf (buffer, "Win%ld,L%ld", window, limit);  
    switch (mode){
        case RSZVL_VAL_MEASTYPE_MAX:
            strcat (buffer, ",Max");
        break;
        case RSZVL_VAL_MEASTYPE_AVER:
            strcat (buffer, ",Aver");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Mode");
    }
    
    switch (limitType){
        case RSZVL_VAL_WIMAX_LIM_FERR:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_FERR_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WIMAX_LIM_SYMB:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_SYMB_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WIMAX_LIM_SST:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_SST_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WIMAX_LIM_EVM_ALL:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_EVM_ALL_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WIMAX_LIM_EVM_DATA:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_EVM_DATA_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WIMAX_LIM_IQOFF:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_IQOF_RES, 
            result), 6, "Result");
        break;
        case RSZVL_VAL_WIMAX_LIM_BERP:
            viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_BERP_RES, 
            result), 6, "Result");
        break;
    }    
  
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX All Limit Check Results
/* Purpose:  This function returns all limit results.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXAllLimitCheckResults(ViSession   instrSession,
                                                    ViInt32     window,
                                                    ViInt32     limit,
                                                    ViInt32     results[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    ViInt32     maxIdx = 12;
    ViChar      *pbuffer;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rszvl_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSZVL_IO_BUFFER_SIZE, buffer));

    if (!(strstr (buffer, "K93")))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    
    checkErr (viQueryf (instrSession, "CALC%ld:LIM%ld:BURS:ALL:RESULT?\n","%s", window, limit, buffer));
    
    i=0;                    
    pbuffer = strtok(buffer, ",");
    while ((pbuffer) && (i < maxIdx))   
    {
        results[i++] = atol (pbuffer);
        pbuffer = strtok(NULL, ",");    
    }   
    
    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Get WiMAX Limit Check
/* Purpose:  This function returns the result of the limit check of the 
/*           limit line indicated in the selected measurement window.
/*===========================================================================*/
ViStatus _VI_FUNC rszvl_GetWiMAXLimitCheck(ViSession instrSession,
                                                    ViInt32     window,
                                                    ViInt32     limit,
                                                    ViInt32*    result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, window, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Window");
    }
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit");
    } 
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);
    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_WIMAX_LIM_FAIL_RES, 
            result), 4, "Result");
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

