/*****************************************************************************
 *  Rohde&Schwarz ZVL Network Analyzer instrument driver
 *  VXIpnp Style Instrument Driver
 *  Original Release: September 2007
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include <math.h>
#include <float.h>
#include <limits.h>
#include "rszvl.h"


/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString listFilterTypeArr[] = {"NORM","CFIL","RRC","FFT","PULS"};
static ViString switchArr[] = {"OFF","ON"};
static ViString measTypeArr[] = {"MEAN","PEAK","CFAC"};
static ViString resultsString = "PASSED,FAILED";
static ViString limTypeArr[] = {"", "UPP","LOW", VI_NULL};
static ViString analyzerPortArr[] = {"AREC","BREC", VI_NULL};
static ViString connectorGenderArr[] = {"MALE","FEM",VI_NULL};
static ViString propagationModeArr[] = {"TEM","WGU",VI_NULL};
static ViString connectorTypeArr[] = {"GEND","NGEN",VI_NULL};
static ViString sweepTypeArr[] = {"LIN","LOG","SEGM",VI_NULL};
static ViString calTypeArr[] = {"REFL","RSH","FOP","FTR","RTR","FRTR","OPTP","TOSM",VI_NULL};
static ViString generalStandardArr[] = {"THR","OPEN","SHOR","MATC","NET","ATT","REFL","SLID","LINE","LINE2",VI_NULL};
static ViString errorTermArr[] = {"'DIRECTIVITY'","'SRCMATCH'","'REFLTRACK'","'ISOLATION'",
                                  "'LOADMATCH'","'TRANSTRACK'",VI_NULL};
static ViString connectorKitArr[] = {"N50","N75","PC7","PC35","PC292","USER1","USER2","SMA",VI_NULL};
static ViString standardArr[] = {"MMTH","FFTH","MFTH","MMLI","FFLI","MFLI","MML2","FFL2","MFL2","OSH","MOSH",
                                    "FOSH","MMAT","FFAT","MFAT","MMSN","FFSN","MFSN","MOP",
                                    "FOP","MSH","FSH","MREF","FREF","MMTC","FMTC",VI_NULL};
static ViString approxArr[] = {"OPEN","SHOR","MATC",VI_NULL};
static ViString loadCalKitStandardArr[] = {"MMTH","FFTH","MFTH","MML","FFL","MFL","","","","","MOSH","FOSH",
										   "MMAT","FFAT","MFAT","MMSN","FFSN","MFSN","MOP","FOP",
										   "MSH","FSH","MREF","FREF","MMTC","FMTC","MSM","FSM",VI_NULL};
static ViString exportFormatArr[] = {"UNF","FORM",VI_NULL};
static ViString exportDataArr[] = {"COMP","LINP","LOGP",VI_NULL};
static ViString limitCheckArr[] = {"","UPP:","LOW:","",VI_NULL};
static ViString limitTypeArr[] = {"OFF","LMAX","LMIN",VI_NULL};
static ViString traceFormatArr[] = {"SDAT","FDAT","MDAT",VI_NULL};
static ViString statArr[] = {"MEAN","STDD","MAX","MIN","RMS","PTP","ELEN","PDEL","ALL",VI_NULL};
static ViString traceStyleArr[] = {"SOL","DASH","DOTT","DDOT","DDD",VI_NULL};
static ViString selStatusRegArr[] = {"",":LIM1",":LIM2",":ACPL",":POW",":SYNC",":LMAR1",":LMAR2",":FREQ",":CORR", VI_NULL};
static ViString statusRegArr[] = {"OPER:EVEN","OPER:COND","QUES:EVEN","QUES:COND","QUES:LIM1:EVEN","QUES:LIM1:COND",
								  "QUES:LIM2:EVEN","QUES:LIM2:COND","QUES:ACPL:EVEN","QUES:ACPL:COND",
								  "QUES:POW:EVEN","QUES:POW:COND","QUES:SYNC:EVEN","QUES:SYNC:COND",
								  "QUES:LMAR1:EVEN","QUES:LMAR1:COND","QUES:LMAR2:EVEN","QUES:LMAR2:COND",
								  "QUES:FREQ:EVEN","QUES:FREQ:COND","QUES:CORR:EVEN","QUES:CORR:COND",VI_NULL};
static ViString decimalSeparatorArr[] = {"POIN","COMM", VI_NULL};
static ViString fieldSeparatorArr[] = {"SEM","COMM","TAB","SPAC", VI_NULL};
static ViString calAutoTypeArr[] = {"FOP","OPTP","FNP","SFNP",VI_NULL};
//static ViString eventStartArr[] = {"SWE","POIN", VI_NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

ViStatus rszvl_CheckStatusCallback (ViSession io);
ViStatus rszvl_WaitForOPCCallback (ViSession io);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rszvl_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rszvl_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rszvl_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rszvl_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
	RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rszvl);
    checkErr( Rs_SpecificDriverNew (instrSession, "rszvl", newOptionString, attrList));

    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rszvl);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rszvl_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rszvl_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSZVL_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSZVL_ATTR_ID_QUERY_RESPONSE,
                                   0, RSZVL_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSZVL_VALID_ID_RESPONSE_STRING, strlen(RSZVL_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rszvl_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;

        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    if (error < VI_SUCCESS)
        (void) Rs_Dispose (*newInstrSession);

    return error;
}

/*****************************************************************************
 * Function: Configure Instrument Mode
 * Purpose:  This function switches between the measurement modes by means of 
 *           text parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureInstrumentMode(
	ViSession	instrSession,
	ViInt32	instrumentMode
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_INSTR_MODE, 
                                            instrumentMode), 2, "Instrument Mode");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Instrument Mode
 * Purpose:  This function returns the measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryInstrumentMode(
	ViSession	instrSession,
	ViInt32*	instrumentMode
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_INSTR_MODE, 
                                            instrumentMode), 2, "Instrument Mode");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Acquisition
 * Purpose:  This function configures the acquisition attributes of the 
 *           spectrum analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureAcquisition(
	ViSession	instrSession,
	ViBoolean	sweepModeContinuous,
	ViInt32	numberOfSweeps
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_SWEEP_MODE_CONTINUOUS, 
                                              sweepModeContinuous), 2, "Sweep Mode Continuous");

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NWA_NUMBER_OF_SWEEPS, 
                                            numberOfSweeps), 3, "Number Of Sweeps");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Average
 * Purpose:  This function configures the average.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureAverage(
	ViSession	instrSession,
	ViInt32	channel,
	ViBoolean	averageState,
	ViInt32	averageFactor
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_AVG_STATE, 
                                              averageState), 3, "Average State");
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_AVG_COUNT, 
                                            averageFactor), 4, "Average Factor");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Restart Average
 * Purpose:  This function starts a new average cycle, clearing all previous 
 *           results and thus eliminating their effect on the new cycle.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_RestartAverage(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_AVERAGE_CLEAR,
                                            VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Bandwidth
 * Purpose:  This function defines the resolution bandwidth of the analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureBandwidth(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64	measBandwidth
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_RESOLUTION_BANDWIDTH, 
                                             measBandwidth), 3, "Meas Bandwidth");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Points
 * Purpose:  This function defines the total number of measurement points per 
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepPoints(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	numberOfPoints
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_POINTS, 
                                            numberOfPoints), 3, "Number Of Points");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Active Channel
 * Purpose:  This function selects a channel <Ch> as the active channel. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureActiveChannel(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_STATE, 
                                             VI_TRUE));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Channel Add
 * Purpose:  This function creates channel and selects it as the active 
 *           channel. It also assigns a name to channel number.
 *           
 *           Note(s):
 *           
 *           A channel created with CONFigure:CHANnel<Ch>[:STATe] ON can be 
 *           configured but has no trace assigned so that no measurement can be 
 *           initiated. Use CALCulate<Ch>:PARameter:SDEFine 
 *           "<Trace_name>,"<Parameter>" to create a new channel and a new 
 *           trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ChannelAdd(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	channelName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_STATE, 
                                             VI_TRUE));

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_NAME, 
                channelName), 3, "Channel Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Channel Add Trace
 * Purpose:  This function creates a new channel and a new trace, which is 
 *           displayed in the selected existing diagram area. The new channel 
 *           settings (including a possible channel calibration) is preset to 
 *           the default settings; the trace is created with the default trace 
 *           settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ChannelAddTrace(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViInt32	channel,
	ViString	channelName,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_STATE, 
                                             VI_TRUE));

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_NAME, 
                                             channelName), 5, "Channel Name");

    viCheckErr (viPrintf (instrSession,"CALC%ld:PAR:SDEF '%s','S21'", channel, traceName));

	sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_TRUE));

	sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_ASSIGN_TRACE, 
                                             traceName), 6, "Trace Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Channel Add Trace Diagram Area
 * Purpose:  This function creates a new channel and a new trace, which is 
 *           displayed in a new diagram area. The new channel settings 
 *           (including a possible channel calibration) is preset to the 
 *           default settings; the trace is created with the default trace 
 *           settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ChannelAddTraceDiagramArea(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViInt32	channel,
	ViString	channelName,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (rszvl_ChannelAddTrace (instrSession, diagramArea, window_Trace, channel, channelName, traceName));
	
	sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_TRUE));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Channel Delete
 * Purpose:  This function deletes the current channel including all traces 
 *           assigned to the channel and removes all display elements related 
 *           to the channel from the diagram area.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ChannelDelete(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_STATE, 
                                             VI_FALSE));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Channel Rename
 * Purpose:  This function assigns a name to channel number. The channel must 
 *           be created before. Moreover it is not possible to assign the same 
 *           name to two different channels.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ChannelRename(
	ViSession	instrSession,
	ViInt32		channel,
	ViString	channelName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_NAME, 
                channelName), 3, "Channel Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Channel List
 * Purpose:  This function returns the numbers and names of all channels in 
 *           the current setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetChannelList(
	ViSession	instrSession,
	ViInt32	bufferSize,
	ViChar _VI_FAR catalog[]
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr( rszvl_QueryViString (instrSession, "CONF:CHAN:CAT?", bufferSize, catalog));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Channel Name
 * Purpose:  This function returns the channel <ch> name.
*****************************************************************************/
ViStatus _VI_FUNC rszvl_GetChannelName(
	ViSession	instrSession,
	ViInt32	channel,
	ViChar _VI_FAR channelName[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_GetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CHANNEL_NAME, 
                RSZVL_IO_BUFFER_SIZE, channelName), 3, "Channel Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Channel Number
 * Purpose:  This function returns the channel number (numeric suffix) of a 
 *           channel with known channel name. A channel name must be assigned 
 *           before.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetChannelNumber(
	ViSession	instrSession,
	ViString	channelName,
	ViInt32*	channelNumber
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (strlen(channelName)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Name");
   
	if (channelNumber==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    
	sprintf (buffer, "CONF:CHAN:NAME:ID? '%s'", channelName);        
    viCheckErr (rszvl_QueryViInt32 (instrSession, buffer, channelNumber));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Frequency Start Stop
 * Purpose:  This function configures the frequency range of the spectrum 
 *           analyzer using start frequency and stop frequency. If start 
 *           frequency is equal to the stop frequency, then the spectrum 
 *           analyzer is in time-domain mode.
 *           
 *           Notes:
 *           
 *           (1) In auto-coupled mode, resolution bandwidth (RBW), video 
 *           bandwidth (VBW), and sweep time may be affected by this function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFrequencyStartStop(
	ViSession	instrSession,
	ViReal64	startFrequency,
	ViReal64	stopFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    
	checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_START, 
                                             startFrequency), 2, "Start Frequency");

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_STOP, 
                                             stopFrequency), 3, "Stop Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Frequency Center
 * Purpose:  This function configures the center frequency 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFrequencyCenter(
	ViSession	instrSession,
	ViReal64	centerFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER, 
                                             centerFrequency), 2, "Center Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Frequency Center Span
 * Purpose:  This function configures the frequency range of the spectrum 
 *           analyzer using the center frequency and the frequency span.  If 
 *           span corresponds to zero hertz, then the spectrum analyzer is in 
 *           time-domain.  
 *           Notes:
 *           
 *           (1) In auto-coupled mode, RBW, VBW and Sweep time may be affected 
 *           by this function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFrequencyCenterSpan(
	ViSession	instrSession,
	ViReal64	centerFrequency,
	ViReal64	span
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER, 
                                             centerFrequency), 2, "Center Frequency");

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_SPAN, 
                                             span), 3, "Span");

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Frequency
 * Purpose:  This function queries the start frequency, stop frequency, center 
 *           frequency or frequecny span. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryFrequency(
	ViSession	instrSession,
	ViInt32	frequency,
	ViReal64*	value
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	switch (frequency)
	{
		case RSZVL_VAL_FREQ_START:
			attribute = RSZVL_ATTR_FREQUENCY_START;
		break;
		case RSZVL_VAL_FREQ_STOP:
			attribute = RSZVL_ATTR_FREQUENCY_STOP;
		break;
		case RSZVL_VAL_FREQ_CENTER:
			attribute = RSZVL_ATTR_FREQUENCY_CENTER;
		break;
		case RSZVL_VAL_FREQ_SPAN:
			attribute = RSZVL_ATTR_FREQUENCY_SPAN;
		break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Frequency");	
	}

	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", attribute, 
                                             value), 3, "Value");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Receiver Step Attenuators
 * Purpose:  This function sets an attenuation factor for the received waves 
 *           b1 and b2.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetReceiverStepAttenuators(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	analyzerPort,
	ViReal64	attenuationFactor
)
{
    ViStatus    error   = VI_SUCCESS;
    
	checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViInt32Range (instrSession, analyzerPort, RSZVL_VAL_ATTEN_AREC, RSZVL_VAL_ATTEN_BREC) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyzer Port");

	viCheckErr (viPrintf (instrSession, "SENS%ld:POW:ATT %s,%.12Lf", channel, 
                                                                    analyzerPortArr[analyzerPort],
                                                                    attenuationFactor));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Receiver Step Attenuators
 * Purpose:  This function returns an attenuation factor for the received 
 *           waves b1 and b2.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetReceiverStepAttenuators(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	analyzerPort,
	ViReal64*	attenuationFactor
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViInt32Range (instrSession, analyzerPort, RSZVL_VAL_ATTEN_AREC, RSZVL_VAL_ATTEN_BREC) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyzer Port");

	sprintf (buffer, "SENS%ld:POW:ATT? %s", channel, analyzerPortArr[analyzerPort]);
	
	viCheckErr (rszvl_QueryViReal64 (instrSession, buffer, attenuationFactor));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Source Port
 * Purpose:  This function selects a source port for the stimulus signal 
 *           (Drive Port). The setting acts on the active trace. The effect of 
 *           the drive port selection depends on the measurement parameter 
 *           associated to the active trace. 
 *           
 *           Note(s):
 *           
 *           (1) If an S-parameter S<out><in> is measured, the second port 
 *           number index <in> (input port of the DUT = drive port of the 
 *           analyzer) is set equal to the selected drive port: Drive port 
 *           selection affects the measured quantity.  
 *           
 *           (2) If a wave quantity or a ratio is measured, the drive port is 
 *           independent from the measured quantity.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSourcePort(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	sourcePort
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SOURCE_PORT, 
                sourcePort), 3, "Source Port");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Source Port
 * Purpose:  This function returns a source port of the stimulus signal (Drive 
 *           Port).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySourcePort(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32*	sourcePort
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SOURCE_PORT, 
                sourcePort), 3, "Source Port");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure RF Output 
 * Purpose:  This function turns the internal source power at all ports and 
 *           the power of all external generators on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRFOutput (ViSession instrSession,
                                           ViBoolean RFOutput)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_RF_OUTPUT, 
                                              RFOutput), 2, "RF Output");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Type
 * Purpose:  This function selects the sweep type, i.e. the sweep variable 
 *           (frequency/power/time) and the position of the sweep points across 
 *           the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepType(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	sweepType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_TYPE, 
                sweepType), 3, "Sweep Type");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Sweep Type
 * Purpose:  This function returns the sweep type, i.e. the sweep variable 
 *           (frequency/power/time) and the position of the sweep points across 
 *           the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySweepType(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32*	sweepType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_TYPE, 
                sweepType), 3, "Sweep Type");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Sweep Points
 * Purpose:  This function returns the total number of measurement points per 
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySweepPoints(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32*	numberOfPoints
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_POINTS, 
                numberOfPoints), 3, "Number Of Points");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Meas Delay
 * Purpose:  This function defines the Meas. Delay time for each partial 
 *           measurement.
 *           
 *           Note(s):
 *           
 *           The sweep duration is ignored for the sweep types Time and CW 
 *           Mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepMeasDelay(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64	measDelay
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_MEAS_DELAY, 
                measDelay), 3, "Meas Delay");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Sweep Meas Delay
 * Purpose:  This function returns the Meas. Delay time for each partial 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySweepMeasDelay(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64*	measDelay
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_MEAS_DELAY, 
                measDelay), 3, "Meas Delay");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Frequency Step Size
 * Purpose:  This function sets the distance between two consecutive sweep 
 *           points.
 *           
 *           Note(s):
 *           
 *           (1) This setting is valid for sweep types with equidistant sweep 
 *           points only. It does not apply to logarithmic and segmented 
 *           sweeps.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFrequencyStepSize(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64	stepSize
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_STEP, 
                stepSize), 3, "Step Size");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Frequency Step Size
 * Purpose:  This function returns the distance between two consecutive sweep 
 *           points.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryFrequencyStepSize(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64*	stepSize
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SWEEP_STEP, 
                stepSize), 3, "Step Size");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment
 * Purpose:  This function inserts a new sweep segment with specific channel 
 *           settings. The new segment must not overlap with any of the 
 *           existing segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegment(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViInt32	operation,
	ViReal64	startFrequency,
	ViReal64	stopFrequency,
	ViInt32	numberOfPoints,
	ViReal64	power,
	ViInt32	sweepTimeSelect,
	ViReal64	time,
	ViReal64	pointDelay,
	ViReal64	measBandwidth
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar*     p2buf = VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	if (rszvl_invalidViReal64Range (instrSession, startFrequency, 0.0, 6.0E9) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Start Frequency");
	if (rszvl_invalidViReal64Range (instrSession, stopFrequency, 0.0, 6.0E9) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Stop Frequency");
	if (rszvl_invalidViInt32Range (instrSession, numberOfPoints, 0, 4001) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Number Of Points");
	if (rszvl_invalidViReal64Range (instrSession, power, -40.0, 10.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Power");
    if (rszvl_invalidViReal64Range (instrSession, measBandwidth, 1.0e-6, 5.0e6))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Meas Bandwidth");

    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);

#if 0	 // command remowed
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_SELECT_SWEEP_TIME, 
                sweepTimeSelect), 9, "Sweep Time Select");
#endif
	
	switch (operation)
	{
		case RSZVL_VAL_SWE_SEG_OP_ADD:
			 viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_INSERT_NEW_SEGMENT, 
                VI_NULL));
		break;
		
		case RSZVL_VAL_SWE_SEG_OP_INS:
			p2buf = buffer + sprintf (buffer, "SENS%ld:SEGM%ld:INS %.12Lf,%.12Lf,%ld,%.12Lf,",
                                      channel,
                                      segment,
                                      startFrequency,
                                      stopFrequency,
                                      numberOfPoints,
                                      power);
    
#if 0 // sweepTimeSelect not used
			switch (sweepTimeSelect)
		    {
		        case RSZVL_VAL_SEG_TIME:
		            if (rszvl_invalidViReal64Range (instrSession, time, 0, 1000))
		                viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Time");
            
		            p2buf += sprintf (p2buf, "%.12Lf", time);
            
		            break;
		        case RSZVL_VAL_SEG_POINT:
		            if (rszvl_invalidViReal64Range (instrSession, pointDelay, 0, 2.5e+3))
		                viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Point Delay");
            
		            p2buf += sprintf (p2buf, "%.12Lf", pointDelay);
		            break;
		    }
#else
		            if (rszvl_invalidViReal64Range (instrSession, pointDelay, 0, 2.5e+3))
		                viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Point Delay");
            
		            p2buf += sprintf (p2buf, "%.12Lf", pointDelay);
#endif
    
		    p2buf += sprintf (p2buf, ",0,%.12Lf", measBandwidth);
			
			viCheckErr( viPrintf (instrSession, buffer));
		break;
		
		case RSZVL_VAL_SWE_SEG_OP_EDIT:
			p2buf = buffer + sprintf (buffer, "SENS%ld:SEGM%ld:DEF %.12Lf,%.12Lf,%ld,%.12Lf,",
                                      channel,
                                      segment,
                                      startFrequency,
                                      stopFrequency,
                                      numberOfPoints,
                                      power);
    
#if 0   // sweepTimeSelect not used  
			switch (sweepTimeSelect)
		    {
		        case RSZVL_VAL_SEG_TIME:
		            if (rszvl_invalidViReal64Range (instrSession, time, 0, 1000))
		                viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Time");
            
		            p2buf += sprintf (p2buf, "%.12Lf", time);
            
		            break;
		        case RSZVL_VAL_SEG_POINT:
		            if (rszvl_invalidViReal64Range (instrSession, pointDelay, 0, 2.5e+3))
		                viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Point Delay");
            
		            p2buf += sprintf (p2buf, "%.12Lf", pointDelay);
		            break;
		    }
#else
		            if (rszvl_invalidViReal64Range (instrSession, pointDelay, 0, 2.5e+3))
		                viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Point Delay");
            
		            p2buf += sprintf (p2buf, "%.12Lf", pointDelay);
#endif
		    p2buf += sprintf (p2buf, ",0,%.12Lf", measBandwidth);
			
			viCheckErr( viPrintf (instrSession, buffer));
		break;
		
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Operation");
		
	}
	
	checkErr( rszvl_CheckStatus (instrSession));
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment State
 * Purpose:  This function activates or deactivates the sweep segment 
 *           <Segment>. Sweep points belonging to inactive segments only are 
 *           not measured.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentState(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViBoolean	state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_STATE, 
                state), 4, "State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Start Frequency
 * Purpose:  This function defines the Start frequency of sweep segment no. 
 *           <Segment>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentStartFrequency(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64	startFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_FREQ_START, 
                startFrequency), 4, "Start Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Stop Frequency
 * Purpose:  This function defines the Stop frequency of sweep segment no. 
 *           <Segment>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentStopFrequency(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64	stopFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_FREQ_STOP, 
                stopFrequency), 4, "Stop Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Individual Settings
 * Purpose:  This function defines whether or not the individual settings can 
 *           be set independently for each sweep segment.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentIndividualSettings(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViBoolean	power,
	ViBoolean	measBandwidth,
	ViBoolean	selectivity,
	ViBoolean	time,
	ViBoolean	pointDelay
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_POWER_CONTROL, 
                power), 4, "Power");

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_BAND_CONTROL, 
                measBandwidth), 5, "Meas Bandwidth");

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_SELECT_CONTROL, 
                selectivity), 6, "Selectivity");

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_TIME_CONTROL, 
                time), 7, "Time");

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_MEAS_DELAY_CONTROL, 
                pointDelay), 8, "Point Delay");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Meas Bandwidth
 * Purpose:  This function defines the resolution bandwidth of the analyzer in 
 *           sweep segment no. <Segment>. At the same time, the command 
 *           activates separate bandwidth setting in all sweep segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentMeasBandwidth(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64	measBandwidth
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_RES_BAND, 
                measBandwidth), 4, "Meas Bandwidth");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Segment Attenuator Factors
 * Purpose:  This function defines the attenuation factors for the received 
 *           waves b1 and b2 in segment no. <Seg>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSegmentAttenuatorFactors(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViInt32	port,
	ViReal64	attenuationFactor
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	if (rszvl_invalidViInt32Range (instrSession, port, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Port");

    sprintf (buffer, "CH%ld,SEG%ld,P%ld", channel, segment, port);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_ATTEN, 
                attenuationFactor), 5, "Attenuation Factor");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Selectivity
 * Purpose:  This function defines the Selectivity in sweep segment no. <Seg>. 
 *           At the same time, the command activates separate selectivity 
 *           setting in all sweep segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentSelectivity(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViInt32	selectivity
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_SELECTIVITY, 
                selectivity), 4, "Selectivity");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Sweep Time
 * Purpose:  This function sets the duration of the sweep in sweep segment no. 
 *           <Segment>. At the same time, the command activates separate sweep 
 *           time setting in all sweep segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentSweepTime(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64	time
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_SWEEP_TIME, 
                time), 4, "Time");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Point Delay
 * Purpose:  This function defines the delay time for each partial measurement 
 *           in sweep segment no. <Segment>
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentPointDelay(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64	pointDelay
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_MEAS_DELAY, 
                pointDelay), 4, "Point Delay");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Points
 * Purpose:  This function defines the total number of measurement Points in 
 *           sweep segment no. <Segment>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentPoints(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViInt32	numberOfPoints
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_SWEEP_POINTS, 
                numberOfPoints), 4, "Number Of Points");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sweep Segment Power
 * Purpose:  This function defines the total number of measurement Points in 
 *           sweep segment no. <Segment>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSweepSegmentPower(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64	power
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_POWER, 
                power), 4, "Power");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Sweep Segments Count
 * Purpose:  This function queries the number of sweep segments in the channel 
 *           including all segments that are switched off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSweepSegmentsCount(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32*	count
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
    sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_GET_COUNT, 
                count), 3, "Count");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete Selected Segment
 * Purpose:  This function deletes the specified (single) sweep segment.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteSelectedSegment(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_DELETE, 
                VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete All Segments
 * Purpose:  This function deletes all sweep segments in the channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteAllSegments(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
    sprintf (buffer, "CH%ld", channel);
	
	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_DELETE_ALL, 
                VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Sweep Segment Center Frequency
 * Purpose:  This function queries the center frequency of sweep segment no. 
 *           <Segment>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSweepSegmentCenterFrequency(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64*	centerFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_GET_FREQ_CENTER, 
                centerFrequency), 4, "Center Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Sweep Segment Frequency Span
 * Purpose:  This function queries the width of the frequency range of sweep 
 *           segment no. <Segment>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSweepSegmentFrequencySpan(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	segment,
	ViReal64*	frequencySpan
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel, segment);
	
	viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SEGMENT_GET_FREQ_SPAN, 
                frequencySpan), 4, "Frequency Span");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Segment
 * Purpose:  This function saves the sweep segment definition of a specified 
 *           channel to a sweep segment file.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveSegment(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_STORE_SEGMENT, 
                fileName), 3, "File Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Segment
 * Purpose:  This function loads a sweep segment definition from a specified 
 *           file and assigns it to a specified channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_LoadSegment(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_LOAD_SEGMENT, 
                fileName), 3, "File Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Correction State
 * Purpose:  This function enables or disables the system error correction 
 *           for channel <Ch>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetCorrectionState (ViSession instrSession,
                                            ViInt32 channel,
                                            ViBoolean correctionState)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CORRECTION_STATE, 
                                              correctionState), 3, "Correction State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Correction Manager
 * Purpose:  This function provides operations which allows to store system 
 *           error correction data to the cal pool and to assign stored 
 *           correction data to channels:
 *           
 *             Copy
 *             Apply
 *             Resolve Pool Link
 *             Delete
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CorrectionManager(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	operationtobePerformed,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	switch (operationtobePerformed)
	{
		case RSZVL_VAL_CORR_MAN_COPY:
			attribute = RSZVL_ATTR_NWA_CORRECTION_STORE;
		break;
		case RSZVL_VAL_CORR_MAN_APPLY:
			attribute = RSZVL_ATTR_NWA_CORRECTION_LOAD;
		break;
		case RSZVL_VAL_CORR_MAN_RESOLVE:
			attribute = RSZVL_ATTR_NWA_CORRECTION_RESOLVE;
		break;
		case RSZVL_VAL_CORR_MAN_DELETE:
			attribute = RSZVL_ATTR_NWA_CORRECTION_DELETE;
		break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Operation to be Performed");	
	}

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, attribute, 
                                             fileName), 4, "File Name");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Calibration Data Current State
 * Purpose:  This function activates or deactivates the calibration mode where 
 *           the raw measurement data of the standards is stored after the 
 *           calibration is completed. The setting is valid for the current 
 *           calibration, where it overwrites the global setting 
 *           ([SENSe<Ch>:]CORRection:COLLect[:ACQuire]:RSAVe:DEFault). A new 
 *           calibration deletes the calibration data acquired in previous 
 *           calibrations.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationDataCurrentState(
	ViSession	instrSession,
	ViInt32	channel,
	ViBoolean	keepMeasData
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CORR_RSAV, 
                                             keepMeasData), 3, "Keep Meas Data");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Calibration Data Default State
 * Purpose:  This function activates or deactivates the calibration mode where 
 *           the raw measurement data of the standards is stored after the 
 *           calibration is completed. The setting remains valid for all 
 *           subsequent calibrations until it is changed explicitly. A new 
 *           calibration deletes the calibration data acquired in previous 
 *           calibrations.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationDataDefaultState(
	ViSession	instrSession,
	ViInt32	channel,
	ViBoolean	keepMeasDataDefault
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CORR_RSAV_DEFAULT, 
                                             keepMeasDataDefault), 3, "Keep Meas Data Default");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Export Calibration Data to Touchstone Files
 * Purpose:  This function exports the calibration unit's characterization 
 *           data as Touchstone files into the given directory.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ExpCalDataTouchstoneFile(
	ViSession	instrSession,
	ViString	directoryName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_CORR_EXP_DATA_TOUCH_FILE, 
                                             directoryName), 2, "Directory Name");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Port Connector
 * Purpose:  This function configures the port connector.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigurePortConnector(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	port,
	ViInt32	connector
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (port <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "port");
	
	sprintf (buffer, "CH%ld,P%ld", channel, port);
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_CORR_CONN_TYPE, 
                                             connector), 4, "Connector");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure User Port Connector
 * Purpose:  This function selects a user-defined connector type at a 
 *           specified port <Port> and its gender.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureUserPortConnector(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	port,
	ViString	connector,
	ViInt32	connectorGender
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (port <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "port");
	if (rszvl_invalidViInt32Range (instrSession, connectorGender, RSZVL_VAL_CONNECTOR_GENDER_MALE, RSZVL_VAL_CONNECTOR_GENDER_FEMALE) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Connector Gender");

	viCheckErr( viPrintf (instrSession, "SENS%ld:CORR:COLL:SCON%ld '%s',%s", channel, 
                                                               port, 
                                                               connector, 
                                                               connectorGenderArr[connectorGender]));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Independent Ports
 * Purpose:  This function qualifies whether the connector types at the 
 *           analyzer ports (but not their gender) are equal or independent.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureIndependentPorts(
	ViSession	instrSession,
	ViInt32	channel,
	ViBoolean	sameConnectorAtAllPorts
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_CORR_CONN_PORTS, 
                                             (ViInt32) sameConnectorAtAllPorts), 3, "Same Connector At All Ports");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure User Calibration Connector
 * Purpose:  This function configures the user-defined connector types.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureUserCalibrationConnector(
	ViSession	instrSession,
	ViInt32		channel,
	ViString	connectorName,
	ViInt32		propagationMode,
	ViInt32		connectorType,
	ViReal64	relativePermittivity,
	ViReal64	impedance
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViInt32Range (instrSession, propagationMode, RSZVL_VAL_CONNECTION_MODE_TEM, RSZVL_VAL_CONNECTION_MODE_WGUIDE) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Propagation Mode");
	if (rszvl_invalidViInt32Range (instrSession, connectorType, RSZVL_VAL_CONNECTION_CONNECTOR_GENDER, RSZVL_VAL_CONNECTION_CONNECTOR_NGENDER) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Connector Type");
	if (rszvl_invalidViReal64Range (instrSession, relativePermittivity, 0.0000000001, 1000) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Relative Permittivity");
	if (propagationMode == RSZVL_VAL_CONNECTION_MODE_TEM)
	{
		if (rszvl_invalidViReal64Range (instrSession, impedance, 1.0e-6, 1000.0e6) == VI_TRUE)
	        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Impedance");
	}
	else if (propagationMode == RSZVL_VAL_CONNECTION_MODE_WGUIDE)
	{
		if (rszvl_invalidViReal64Range (instrSession, impedance, 0.0, 1000.0e9) == VI_TRUE)
	        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Impedance");
	}
	
	viCheckErr( viPrintf (instrSession, "SENS%ld:CORR:CONN '%s',%s,%s,%.12Lf,%.12Lf", channel, 
                                                                                      connectorName, 
                                                                                      propagationModeArr[propagationMode], 
                                                                                      connectorTypeArr[connectorType],
                                                                                      relativePermittivity,
                                                                                      impedance));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get User Calibration Connector
 * Purpose:  This function returns the user-defined connector types.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetUserCalibrationConnector (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViString connectorName,
                                                     ViInt32 *propagationMode,
                                                     ViInt32 *connectorType,
                                                     ViReal64 *relativePermittivity,
                                                     ViReal64 *impedance)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar		buffer[RSZVL_IO_BUFFER_SIZE];
	ViChar		rd_buffer[RSZVL_IO_BUFFER_SIZE];
	ViChar		*p2buf;
	ViChar		connectorNameRet[RSZVL_IO_BUFFER_SIZE];
	ViChar		tmp_string_1[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar		tmp_string_2[RSZVL_IO_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf(buffer, "SENS%ld:CORR:CONN? '%s'", channel, connectorName);
	
	viCheckErr (rszvl_QueryViString (instrSession, 
		                           buffer, 
								   RSZVL_IO_BUFFER_SIZE, 
								   rd_buffer));

	p2buf = strtok (rd_buffer, ",");
	// first value returned is the connector name - skipping
	sscanf (p2buf, "'%[^']", connectorNameRet);
	
	p2buf = strtok (VI_NULL, ",");
	sscanf (p2buf, "%s", tmp_string_1);
	*propagationMode = rszvl_StringIndex (propagationModeArr, tmp_string_1);
	if (*propagationMode == -1)
		viCheckParm (RS_ERROR_INVALID_VALUE, 4, "Propagation Mode");
	
	p2buf = strtok (VI_NULL, ",");
	sscanf (p2buf, "%s", tmp_string_2);
	*connectorType = rszvl_StringIndex (connectorTypeArr, tmp_string_2);
	if (*connectorType == -1)
		viCheckParm (RS_ERROR_INVALID_VALUE, 5, "Connector Type");
	
	p2buf = strtok (VI_NULL, ",");
	sscanf (p2buf, "%Le", relativePermittivity);
	
	p2buf = strtok (VI_NULL, ",");
	sscanf (p2buf, "%Le", impedance);

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete User Calibration Connector
 * Purpose:  This function deletes a user-defined connector type named 
 *           '<conn_name>'.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteUserCalibrationConnector(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	connectorName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_DELETE_USER_CONN, 
                                             connectorName), 3, "Connector Name");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Calibration State
 * Purpose:  This function queries state of the channel's calibration.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCalibrationState (ViSession instrSession,
                                             ViInt32 channel,
                                             ViInt32 *calibrationState)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_CORR_GET_CAL_STATE, 
                                             calibrationState), 3, "Calibration State");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Calibration Data Parameters
 * Purpose:  This function queries the parameters of the channel's calibration
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCalibrationDataParameters(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64*	frequencyStart,
	ViReal64*	frequencyStop,
	ViInt32*	numberOfPoints,
	ViReal64*	internalSignalSourcePower,
	ViInt32*	sweepType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar      rd_buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar      tmp_string[RSZVL_IO_BUFFER_SIZE]="";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckErr( rszvl_GetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_GET_CAL_DATA_PARAMS, 
                                            RSZVL_IO_BUFFER_SIZE, rd_buffer));
	
	
	sscanf (rd_buffer, "%Le,%Le,%ld,%Le,%s", frequencyStart, frequencyStop, numberOfPoints, internalSignalSourcePower, tmp_string);
    
	*sweepType = rszvl_StringIndex (sweepTypeArr, tmp_string);
	if (*sweepType == -1)
		viCheckParm (RS_ERROR_INVALID_VALUE, 7, "Sweep Type");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Calibration Date
 * Purpose:  This function queries date and time of the channel's calibration.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCalibrationDate(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	bufferSize,
	ViChar _VI_FAR calibrationDate[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_GetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_GET_CAL_DATE, 
                                            bufferSize, calibrationDate), 4, "Calibration Date");
	
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Factory Calibration State
 * Purpose:  This function enables or disables the factory calibration for a 
 *           particular channel. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFactoryCalibrationState(
	ViSession	instrSession,
	ViInt32	channel,
	ViBoolean	factoryCalibration
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_CORR_FACT_CAL_STATE, 
                                            factoryCalibration), 3, "Factory Calibration");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Calibration Auto Simplified
 * Purpose:  This function selects and initiates a one-port or two-port
 *           automatic calibration at arbitrary analyzer ports.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CalibrationAutoSimplified (ViSession instrSession,
                                                   ViInt32 channel,
                                                   ViString calibrationKitName,
                                                   ViInt32 analyzerPort1,
                                                   ViInt32 analyzerPort2,
                                                   ViInt32 timeout)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (analyzerPort1 <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Analyzer Port 1");
	if (rszvl_invalidViInt32Range (instrSession, timeout, 0, 50000) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Timeout"); 
    
	checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));

    p2buf = buffer + sprintf (buffer, "SENS%ld:CORR:COLL:AUTO '%s',%ld",
                              channel,
                              calibrationKitName,
                              analyzerPort1);

    if (analyzerPort2)
    {
        p2buf += sprintf (p2buf, ",%ld", analyzerPort2);
    }
    
    viCheckErr (viPrintf (instrSession, "%s\n", buffer));

    checkErr (rszvl_CheckStatus (instrSession));

Error:
	if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Calibration Auto
 * Purpose:  This function selects and initiates a one-port or two-port
 *           automatic calibration at arbitrary analyzer and calibration unit
 *           ports. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CalibrationAuto (ViSession instrSession,
                                         ViInt32 channel,
                                         ViString calibrationKitName,
                                         ViInt32 analyzerPort1,
                                         ViInt32 calUnitPort1,
                                         ViInt32 analyzerPort2,
                                         ViInt32 calUnitPort2, ViInt32 timeout)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (analyzerPort1 <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Analyzer Port 1");
	if (rszvl_invalidViInt32Range (instrSession, timeout, 0, 50000) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Timeout"); 
    
	checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));

    p2buf = buffer + sprintf (buffer, "SENS%ld:CORR:COLL:AUTO:PORT '%s',%ld,%ld",
                              channel,
                              calibrationKitName,
                              analyzerPort1,
                              calUnitPort1);

    if (analyzerPort2 && calUnitPort2)
    {
        p2buf += sprintf (p2buf, ",%ld,%ld", analyzerPort2, calUnitPort2);
    }
    
	viCheckErr (viPrintf (instrSession, "%s\n", buffer));

    checkErr (rszvl_CheckStatus (instrSession));

Error:
	if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Generate Calibration Kit File
 * Purpose:  This function generates a cal kit file with the specified name
 *           containing the cal kit data of the active calibration unit. The
 *           cal kit file can be stored in the calibration unit or written to
 *           a directory on the analyzer. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GenerateCalibrationKitFile (ViSession instrSession,
                                                    ViString fileName)
{
	ViStatus	error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_GENERATE_CALIBRATION_KIT_FILE, 
                                             fileName), 2, "File Name");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/*****************************************************************************
 * Function: Get Calibration Port Assignment
 * Purpose:  This function returns the assignment between the network
 *           analyzer ports and the ports of the connected automatic
 *           calibration unit. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCalibrationPortAssignment (ViSession instrSession,
                                                      ViInt32 channel,
                                                      ViInt32 *analyzerPort1,
                                                      ViInt32 *analyzerPort2)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      *rd_buffer = NULL;
    ViChar      *p2buf;
    ViBoolean   data_returned = VI_FALSE;
    ViInt32     temp_port;
	ViUInt32	rd_count;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (viPrintf (instrSession, "SENS%ld:CORR:COLL:AUTO:PORT:CONN?\n", channel));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &rd_buffer, &rd_count));
    
    data_returned = (strcmp (rd_buffer, "") != 0);
    
    if (data_returned)
    {
        p2buf = strtok (rd_buffer, ",");
        sscanf (p2buf, "%ld", &temp_port); //not needed

        p2buf = strtok (VI_NULL, ",");
        sscanf (p2buf, "%ld", analyzerPort1);
    
        p2buf = strtok (VI_NULL, ",");
        sscanf (p2buf, "%ld", &temp_port); //not needed
    
        p2buf = strtok (VI_NULL, ",");
        sscanf (p2buf, "%ld", analyzerPort2);
    }
    
Error:
	(void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Select Active Calibration Unit
 * Purpose:  This function selects one of the USB-connected calibration units
 *           for calibration. This function is not necessary if only one cal
 *           unit is connected. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelectActiveCalibrationUnit (ViSession instrSession,
                                                  ViString calibrationUnit)
{
	ViStatus	error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_ACTIVE_CALIBRATION_UNIT, 
                                             calibrationUnit), 2, "Calibration Unit");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/*****************************************************************************
 * Function: Get All Calibration Units
 * Purpose:  This function returns the names (USB addresses) of all connected
 *           calibration units. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetAllCalibrationUnits (ViSession instrSession,
                                                ViInt32 bufferSize,
                                                ViChar calibrationUnit[])
{
	ViStatus	error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_GetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_GET_ALL_CALIBRATION_UNITS, 
                                             bufferSize, calibrationUnit), 3, "Calibration Unit");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/*****************************************************************************
 * Function: Configure Calibration Unit Standard
 * Purpose:  This function defines calibration unit standard. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationUnitStandard (ViSession instrSession,
                                                          ViInt32 standard,
                                                          ViInt32 port1,
                                                          ViInt32 port2)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, standard, RSZVL_VAL_CALCOLLSTDGEN_THR, RSZVL_VAL_CALCOLLSTDGEN_MATCH) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Standard")
    
    if (port1 <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Port 1");
	
	p2buf=buffer + sprintf (buffer, ":SYST:COMM:AKAL:CONN %s,%ld", 
                            generalStandardArr[standard], port1);
    
    if (port2)
    {
        p2buf += sprintf (p2buf,",%ld", port2);
    }
	
	viCheckErr (viPrintf (instrSession, "%s\n", buffer));

    checkErr (rszvl_CheckStatus (instrSession));

Error:
	(void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Export Calibration Data To Touchstone Format
 * Purpose:  This function converts the calibration data of the standards in
 *           the active calibration unit to Touchstone format and copies it
 *           to the specified directory. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ExportCalibrationDataToTouchstoneFormat (ViSession instrSession,
                                                                 ViString directoryName)
{
	ViStatus	error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViString (instrSession, "", 
                                             RSZVL_ATTR_NWA_EXPORT_CALIBRATION_DATA_TO_TOUCHSTONE_FORMAT, 
                                             directoryName), 2, "Directory Name");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/*****************************************************************************
 * Function: Configure Calibration Type
 * Purpose:  This function selects a one-port or two-port calibration type at 
 *           arbitrary analyzer ports.
 *           
 *           Note(s):
 *           
 *           (1) For an n-port calibration type (n = 1 to 2), n port numbers 
 *           must be specified. If more than n numbers are defined, the spare 
 *           numbers (the last in the list) are ignored.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationType(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	calibrationName,
	ViInt32	parameters,
	ViInt32	port1,
	ViInt32	port2
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar      *p2buf;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    if (strlen(calibrationName) == 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Calibration Name");
	if (rszvl_invalidViInt32Range (instrSession, parameters, RSZVL_VAL_CALTYPE_REFL, RSZVL_VAL_CALTYPE_TOSM) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Parameters");

	p2buf = buffer + sprintf (buffer, "SENS%ld:CORR:COLL:METH:DEF '%s',%s,%ld",channel,
             calibrationName, calTypeArr[parameters], port1);
    if (port2 != VI_NULL)
    {
        p2buf += sprintf (p2buf,",%ld", port2);
    }
    
	viCheckErr (viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Start Calibration
 * Purpose:  This function starts a calibration measurement in order to 
 *           acquire measurement data for the selected standards. The
 *           standards are reflection or transmission standards and can be 
 *           connected to arbitrary analyzer ports.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_StartCalibration(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	standard,
	ViInt32	port1,
	ViInt32	port2
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	switch (standard)
    {
        case RSZVL_VAL_CALCOLLSTDGEN_THR:
        case RSZVL_VAL_CALCOLLSTDGEN_NET:
        case RSZVL_VAL_CALCOLLSTDGEN_ATT:
        case RSZVL_VAL_CALCOLLSTDGEN_LINE:
		    sprintf (buffer, "SENS%ld:CORR:COLL:SEL %s,%ld,%ld", channel, generalStandardArr[standard],
                                                      port1, port2);
            break;
        case RSZVL_VAL_CALCOLLSTDGEN_OPEN:
        case RSZVL_VAL_CALCOLLSTDGEN_SHORT:
        case RSZVL_VAL_CALCOLLSTDGEN_MATCH:
        case RSZVL_VAL_CALCOLLSTDGEN_REFL:
        case RSZVL_VAL_CALCOLLSTDGEN_SLIDE:
            sprintf (buffer, "SENS%ld:CORR:COLL:SEL %s,%ld", channel, generalStandardArr[standard],
                                                      port1);
            break;
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Standard");
    }

	viCheckErr (viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Calibration Data
 * Purpose:  This function calculates the system error correction data from 
 *           the acquired one or two-port measurement results 
 *           (rszvb_StartCalibration), stores them and applies them to the 
 *           calibrated channel. To avoid incompatibilities, older system error 
 *           correction data is deleted unless it has been transferred into a 
 *           cal pool (MMEMory:STORe:CORRection <Ch>, '<file_name>').
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveCalibrationData(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_SAVE_CAL_DATA, 
                                            VI_NULL));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Generate Default Calibration Data
 * Purpose:  This function generates a set of default system error correction 
 *           data for the selected ports and calibration type. The default data 
 *           set corresponds to a test setup which does not introduce any 
 *           systematic errors.
 *           
 *           Note(s):
 *           
 *           (1) The main purpose of the default correction data set is to 
 *           provide a dummy system error correction which you can replace with 
 *           your own, external correction data. You may have acquired the 
 *           external data in a previous session or even on an other 
 *           instrument. If you want to use the external correction data on the 
 *           analyzer, simply generate the default data set corresponding to 
 *           your port configuration and calibration type and overwrite the 
 *           default data
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GenerateDefaultCalibrationData(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_SAVE_DEFAULT_DATA, 
                                            VI_NULL));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete Calibration Data
 * Purpose:  This function deletes system error correction data generated and 
 *           stored previously.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteCalibrationData(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	calibrationName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	sprintf (buffer, "CH%ld", channel);
	
	if (!calibrationName)
	{
		viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_DELETE_CAL_DATA_LAST, VI_NULL));
	}
	else
	{		
		viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_CORR_DELETE_CAL_DATA, 
                                             calibrationName), 3, "Calibration Name");
	}
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read Calibration Data
 * Purpose:  This function reads system error correction data for a specific 
 *           channel, calibration method, and port combination <Port 1>, <Port 
 *           2>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadCalibrationData(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	errorTermParameters,
	ViInt32	port1,
	ViInt32	port2,
	ViReal64 _VI_FAR calibrationData[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [RSZVL_IO_BUFFER_SIZE];
    ViInt32     noofPoints;

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViInt32Range (instrSession, errorTermParameters, RSZVL_VAL_CAL_DATA_DIRECTIVITY, RSZVL_VAL_CAL_DATA_TRANSTRACK) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Error Term Parameters");

    sprintf (buffer,":SENS%ld:CORR:CDAT? %s,%ld,%ld", channel,
                                                      errorTermArr[errorTermParameters],
                                                      port1, port2);
    
    viCheckErr (rszvl_dataReadTrace (instrSession, buffer, 10*RSZVL_IO_BUFFER_SIZE, calibrationData, &noofPoints));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Write Calibration Data
 * Purpose:  This function writes system error correction data for a specific 
 *           channel, calibration method, and port combination <Port 1>, <Port 
 *           2>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_WriteCalibrationData(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	errorTermParameters,
	ViInt32	port1,
	ViInt32	port2,
	ViReal64 _VI_FAR calibrationData[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViUInt16    remote_intfc    = VI_INTF_GPIB;
    ViChar*     pdynamic_buffer     = VI_NULL;
    ViChar*     pactual_position;
    ViInt32     digits, noofPoints;
    ViReal32    *pinstrument_data   = VI_NULL;
    ViReal64    *pdriver_data       = calibrationData;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViInt32Range (instrSession, errorTermParameters, RSZVL_VAL_CAL_DATA_DIRECTIVITY, RSZVL_VAL_CAL_DATA_TRANSTRACK) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Error Term Parameters");

    checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &remote_intfc));
    
    viCheckErr (rszvl_QuerySweepPoints (instrSession, channel, &noofPoints));
    
    noofPoints *= 2; /* Real and imaginary part */
    /* intfc specific operations */
    switch (remote_intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */

            viCheckErr (viPrintf (instrSession, ":FORMAT ASCII;:SENS%ld:CORR:CDAT %s,%ld,%ld,%,*Lf\n",
                                channel,
                                errorTermArr[errorTermParameters],
                                port1, port2,
                                calibrationData));
            break;

        case VI_INTF_GPIB: default: /* GPIB intfc. */

            /*
             * Alloc buffer. Size is calculated as:
             * max buffer for header + number of points * sizeof(ViReal32)
             *          64           +    noofPoints    * sizeof(ViReal32)
             */

            pdynamic_buffer  = (ViChar*)malloc(RSZVL_IO_BUFFER_SIZE + noofPoints * sizeof(ViReal32));
            pactual_position = pdynamic_buffer;

            digits = ((int)floor(log10(noofPoints*(double)sizeof(ViReal32)))) + 1;

            pactual_position += sprintf (pactual_position,
                                         ":FORM REAL,32;:SENS%ld:CORR:CDAT %s,%ld,%ld,#%ld%ld",
                                         channel,
                                         errorTermArr[errorTermParameters],
                                         port1, port2,
                                         digits,
                                         (ViInt32)(noofPoints*sizeof(ViReal32)));

            /* Pointer to begining of data */
            pinstrument_data = (ViReal32*)pactual_position;

            for (i = 0; i < noofPoints; i++)
                *pinstrument_data++ = (ViReal32)*pdriver_data++;
            /* Binary data byte order (MOTOROLA or INTEL). */
            rszvl_SWAP_32((unsigned char*) pactual_position, noofPoints);
            pactual_position = (ViChar*)pinstrument_data;

            if ((error = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:SELF:RTER EOI\n", 30, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (pdynamic_buffer);
                return error;
                }
            if ((error = viWrite (instrSession, (ViBuf) pdynamic_buffer,
                                          (ViUInt32) (pactual_position - pdynamic_buffer),
                                          VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (pdynamic_buffer);
                return error;
                }
            if ((error = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:SELF:RTER LFEOI\n", 32, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (pdynamic_buffer);
                return error;
                }

            /* Deallocate memory. */
            free (pdynamic_buffer);

            break;
    }

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Calibration Kit
 * Purpose:  This function selects the calibration kit to be used for a 
 *           specified connector type.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationKit(
	ViSession	instrSession,
	ViInt32	connector,
	ViString	calibrationKitName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rszvl_invalidViInt32Range (instrSession, connector, RSZVL_VAL_CONNECTOR_KIT_N50, RSZVL_VAL_CONNECTOR_KIT_SMA) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Connector");

    viCheckErr (viPrintf (instrSession, "SENS:CORR:CKIT:%s:SEL '%s'", connectorKitArr[connector], calibrationKitName));
    

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Calibration Kit
 * Purpose:  This function returns the calibration kit to be used for a 
 *           specified connector type.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCalibrationKit (ViSession instrSession,
                                           ViInt32 connector, ViInt32 bufferSize,
                                           ViChar calibrationKitName[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar      rd_buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rszvl_invalidViInt32Range (instrSession, connector, RSZVL_VAL_CONNECTOR_KIT_N50, RSZVL_VAL_CONNECTOR_KIT_SMA) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Connector");

	sprintf (buffer, "SENS:CORR:CKIT:%s:SEL?", connectorKitArr[connector]);
    
    
    viCheckErr (rszvl_QueryViString (instrSession, buffer, bufferSize, rd_buffer));
    
	sscanf (rd_buffer, "'%[^']", calibrationKitName);
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Calibration Kit User Connector Type
 * Purpose:  This function selects the calibration kit to be used for a 
 *           specified connector type.
 *           
 *           Note(s):
 *           
 *           (1) The function is suitable for connector types with arbitrary, 
 *           user-defined names. For standard connector types you can use the 
 *           command [SENSe<Ch>:]CORRection:CKIT:<conn_type>:SELect (function 
 *           rszvl_ConfigureCalibrationKit)
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationKitUserConnectorType(
	ViSession	instrSession,
	ViString	connector,
	ViString	calibrationKitName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (viPrintf (instrSession, "SENS:CORR:CKIT:SEL '%s','%s'", connector, calibrationKitName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Calibration Kit User Connector Type
 * Purpose:  This function returns the calibration kit to be used for a 
 *           specified connector type.
 *           
 *           Note(s):
 *           
 *           (1) The function is suitable for connector types with arbitrary, 
 *           user-defined names. For standard connector types you can use the 
 *           command [SENSe<Ch>:]CORRection:CKIT:<conn_type>:SELect (function 
 *           rszvl_ConfigureCalibrationKit)
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCalibrationKitUserConnectorType (ViSession instrSession,
                                                            ViString connector,
                                                            ViInt32 bufferSize,
                                                            ViChar calibrationKitName[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar      rd_buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buffer, "SENS:CORR:CKIT:SEL? '%s'", connector);

    viCheckErr (rszvl_QueryViString (instrSession, buffer, bufferSize, rd_buffer));
    
	sscanf (rd_buffer, "'%[^']", calibrationKitName);
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Calibration Kit
 * Purpose:  This function loads cal kit data from a specified NWA cal kit 
 *           file. 
 *           
 *           Notes:
 *           (1) The loaded file must be a NWA-specific cal kit file with the 
 *           extension *.calkit. ZVR cal kit files can be imported using the 
 *           [SENSe<Ch>:]CORRection:CKIT:INSTall command. Agilent cal kit files 
 *           can be imported manually and converted into *.calkit files.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_LoadCalibrationKit(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_LOAD_CAL_KIT, 
                                            fileName), 2, "File Name");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Calibration Kit from Touchstone File
 * Purpose:  This function loads cal kit data for a specific connector type, 
 *           cal kit, and calibration standard from a specified Touchstone 
 *           file, assigning a label for the cal data. A restriction on the 
 *           port assignment may be defined in addition.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_LoadCalibrationKitFromTouchstoneFile(
	ViSession	instrSession,
	ViString	connectorName,
	ViString	calibrationKitName,
	ViInt32		standard,
	ViString	calibrationKitLabel,
	ViString	fileName,
	ViInt32		portNumber1,
	ViInt32		portNumber2
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    if (!connectorName)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Connector Name");
	if (!calibrationKitName)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Calibration Kit Name");
	if (rszvl_invalidViInt32Range (instrSession, standard, RSZVL_VAL_CALSTD_MMTH, RSZVL_VAL_CALSTD_FSM) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Standard");
	if (!calibrationKitLabel)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Calibration Kit Label");
	if (!fileName)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "File Name");
	
	if (portNumber2 != 0)
    	sprintf (buffer, "MMEM:LOAD:CKIT:SDAT '%s','%s',%s,'%s','%s',%ld,%ld", 
			              connectorName, calibrationKitName, loadCalKitStandardArr[standard], 
						  calibrationKitLabel, fileName, portNumber1, portNumber2);
	else
    	sprintf (buffer, "MMEM:LOAD:CKIT:SDAT '%s','%s',%s,'%s','%s',%ld", 
			              connectorName, calibrationKitName, loadCalKitStandardArr[standard], 
						  calibrationKitLabel, fileName, portNumber1);
	
    
	viCheckErr (viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Calibration Kit
 * Purpose:  This function stores the data of a calibration kit to a specified 
 *           file. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveCalibrationKit(
	ViSession	instrSession,
	ViString	kitName,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (viPrintf (instrSession, "MMEM:STOR:CKIT '%s','%s'", kitName, fileName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Calibration Standard
 * Purpose:  This function defines the parameters of a non-ideal 1 port or 
 *           2-port calibration standard <std_type>. A particular physical 
 *           standard can be selected by specifying the name of the calibration 
 *           kit and its serial number. Depending on the standard type, only a 
 *           subset of the parameters may be used
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCalibrationStandard (ViSession instrSession,
                                                         ViInt32 connector,
                                                         ViInt32 standard,
                                                         ViString kit,
                                                         ViString serialNumber,
                                                         ViReal64 minFreqHz,
                                                         ViReal64 maxFreqHz,
                                                         ViReal64 lengthmm,
                                                         ViReal64 loss,
                                                         ViReal64 c0L0,
                                                         ViReal64 c1L1,
                                                         ViReal64 c2L2,
                                                         ViReal64 c3L3,
                                                         ViInt32 approximation)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    if (rszvl_invalidViInt32Range (instrSession, connector, RSZVL_VAL_CONNECTOR_KIT_N50, RSZVL_VAL_CONNECTOR_KIT_USER2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Connector");

	if (rszvl_invalidViInt32Range (instrSession, approximation, RSZVL_VAL_OPEN, RSZVL_VAL_SHORT) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 14, "Approximation");

	switch (standard)
    {
        case RSZVL_VAL_CALSTD_MMTH:
        case RSZVL_VAL_CALSTD_FFTH:
        case RSZVL_VAL_CALSTD_MFTH:
        case RSZVL_VAL_CALSTD_MMLI:
        case RSZVL_VAL_CALSTD_FFLI:
        case RSZVL_VAL_CALSTD_MFLI:
            sprintf (buffer, "SENS:CORR:CKIT:%s:%s '%s','%s',%Lf,%Lf,%Lf,%Lf", 
                         connectorKitArr[connector],
                         standardArr[standard],
                         kit,
                         serialNumber,
                         minFreqHz,
                         maxFreqHz,
                         lengthmm,
                         loss);
            break;
        case RSZVL_VAL_CALSTD_OSH:
        case RSZVL_VAL_CALSTD_MOSH:
        case RSZVL_VAL_CALSTD_FOSH:
        case RSZVL_VAL_CALSTD_MOP:
        case RSZVL_VAL_CALSTD_FOP:
        case RSZVL_VAL_CALSTD_MSH:
        case RSZVL_VAL_CALSTD_FSH:
            sprintf (buffer, "SENS:CORR:CKIT:%s:%s '%s','%s',%Lf,%Lf,%Lf,%Lf,%Lf,%Lf,%Lf,%Lf", 
                         connectorKitArr[connector],
                         standardArr[standard],
                         kit,
                         serialNumber,
                         minFreqHz,
                         maxFreqHz,
                         lengthmm,
                         loss,
                         c0L0,
                         c1L1,
                         c2L2,
                         c3L3);
            break;
        case RSZVL_VAL_CALSTD_MMAT:
        case RSZVL_VAL_CALSTD_FFAT:
        case RSZVL_VAL_CALSTD_MFAT:
        case RSZVL_VAL_CALSTD_MMTC:
        case RSZVL_VAL_CALSTD_FMTC:
            sprintf (buffer, "SENS:CORR:CKIT:%s:%s '%s','%s',%Lf,%Lf", 
                         connectorKitArr[connector],
                         standardArr[standard],
                         kit,
                         serialNumber,
                         minFreqHz,
                         maxFreqHz);
            break;
        case RSZVL_VAL_CALSTD_MMSN:
        case RSZVL_VAL_CALSTD_FFSN:
        case RSZVL_VAL_CALSTD_MFSN:
        case RSZVL_VAL_CALSTD_MREF:
        case RSZVL_VAL_CALSTD_FREF:
            sprintf (buffer, "SENS:CORR:CKIT:%s:%s '%s','%s',%Lf,%Lf,%Lf,%Lf,%Lf,%Lf,%Lf,%s", 
                         connectorKitArr[connector],
                         standardArr[standard],
                         kit,
                         serialNumber,
                         minFreqHz,
                         maxFreqHz,
                         lengthmm,
                         c0L0,
                         c1L1,
                         c2L2,
                         c3L3,
                         approxArr[approximation]);
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Standard");
    }
    
    viCheckErr (viPrintf (instrSession, buffer)); 
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Length
 * Purpose:  This function defines the offset parameter for test port <Port> 
 *           as an electrical length.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLength(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	port,
	ViReal64	electricalLength,
	ViReal64	mechanicalLength,
	ViReal64	permittivity,
	ViReal64	delay
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (port <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Port");

    sprintf (buffer, "CH%ld,P%ld", channel, port);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_ELENGTH, 
                                             electricalLength), 4, "Electrical Length");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_PERMITTIVITY, 
                                             permittivity), 6, "Permittivity");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_DISTANCE, 
                                             mechanicalLength), 5, "Mechanical Length");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_DELAY, 
                                             delay), 7, "Delay");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Auto Length
 * Purpose:  This function defines the offset parameter for the active test 
 *           port such that the residual delay of the active trace is minimized 
 *           across the entire sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_AutoLength(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	port
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (port <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Port");

    sprintf (buffer, "CH%ld,P%ld", channel, port);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_AUTO_LENGTH, 
                                             VI_NULL));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Loss
 * Purpose:  This function configures parameters of One-way Loss.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLoss(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	port,
	ViReal64	lossAtDC,
	ViReal64	lossAtFrequency,
	ViReal64	lossReferenceFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (port <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Port");

    sprintf (buffer, "CH%ld,P%ld", channel, port);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_LOSS, 
                                             lossAtDC), 4, "Loss at DC");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_LOSS_OFFSET, 
                                             lossAtFrequency), 5, "Loss At Frequency");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_LOSS_FREQUENCY, 
                                             lossReferenceFrequency), 6, "Loss Reference Frequency");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Auto Length And Loss
 * Purpose:  This function determines all offset parameters such that the 
 *           residual group delay of the active trace (defined as the negative 
 *           derivative of the phase response) is minimized and the measured 
 *           loss is minimized as far as possible across the entire sweep 
 *           range.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_AutoLengthAndLoss(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	port
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (port <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Port");

    sprintf (buffer, "CH%ld,P%ld", channel, port);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_AUTO_LENGTH_AND_LOSS, 
                                             VI_NULL));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Reset Offsets
 * Purpose:  This function resets the offset parameters for all test ports to 
 *           zero.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ResetOffsets(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_OFFSET_STATE, 
                                             VI_FALSE));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Source Power
 * Purpose:  This function defines the power of the internal signal source 
 *           (channel power). The setting is valid for all sweep types.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSourcePower(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64	sourcePower
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViReal64Range (instrSession, sourcePower, -300.0, 20.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Source Power");
	
	sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_SOURCE_POWER, 
                                             sourcePower), 3, "Source Power");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Select
 * Purpose:  This function selects an existing trace as the active trace of 
 *           the channel. All trace commands without explicit reference to the 
 *           trace name act on the active trace (e.g. CALCulate<Ch>:FORMat).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceSelect(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_SELECT, 
                                             traceName), 3, "Trace Name");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Add
 * Purpose:  This function creates a new trace in the current diagram area and 
 *           assigns it to the selected channel. The new trace is created with 
 *           the default trace and channel settings. Trace is not displayed. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceAdd(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	viCheckErr (viPrintf (instrSession, "CALC%ld:PAR:SDEF '%s','S21'", channel, traceName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Add Diagram Area
 * Purpose:  This function creates a new trace in a new diagram area and 
 *           assigns the trace to the selected channel. The new trace is 
 *           created with the default trace and channel settings and displayed 
 *           in the new diagram area.
 *           
 *           Note(s):
 *           
 *           A trace can be assigned to a diagram only once. If a attempt is 
 *           made to assign the same trace a second time an error message 
 *           -114,"Header suffix out of range" is generated.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceAddDiagramArea(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViInt32	channel,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel");

    viCheckErr (viPrintf (instrSession,"CALC%ld:PAR:SDEF '%s','S21'", channel, traceName));

	sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_TRUE));

	sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_ASSIGN_TRACE, 
                                             traceName), 5, "Trace Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Add Diagram Area By Name
 * Purpose:  This function creates a new trace in a new diagram area and 
 *           assigns the trace to the selected channel without using the trace 
 *           number - the trace is referenced by its name. The new trace is 
 *           created with the default trace and channel settings and displayed 
 *           in the new diagram area.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceAddDiagramAreaByName (ViSession instrSession,
                                                   ViInt32 diagramArea,
                                                   ViInt32 channel,
                                                   ViString traceName)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel");

    viCheckErr (viPrintf (instrSession,"CALC%ld:PAR:SDEF '%s','S21'", channel, traceName));

	sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_TRUE));

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_ASSIGN_TRACE_BY_NAME, 
                                             traceName), 5, "Trace Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Delete
 * Purpose:  This function deletes a trace with a specified trace name and 
 *           channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceDelete(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_DELETE, 
                                             traceName), 3, "Trace Name");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace List
 * Purpose:  This function returns the trace names and measurement parameters 
 *           of all traces assigned to a particular channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceList(
	ViSession	instrSession,
	ViInt32	channel,
	ViInt32	bufferSize,
	ViChar _VI_FAR catalog[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
    sprintf (buffer, "CALC%ld:PAR:CAT?", channel);

	viCheckErr( rszvl_QueryViString (instrSession, buffer, bufferSize, catalog));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Assign Diagram Area
 * Purpose:  This function assigns the active trace to another diagram area. 
 *           Selecting one of the existing area numbers assigns the active 
 *           trace to the existing diagram area: The active trace is removed 
 *           from the previous area and displayed in the new diagram area.
 *           
 *           Note(s):
 *           
 *           A trace can be assigned to a diagram only once. If a attempt is 
 *           made to assign the same trace a second time an error message 
 *           -114,"Header suffix out of range" is generated.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceAssignDiagramArea(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");

    sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_TRUE));

	sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_ASSIGN_TRACE, 
                                             traceName), 4, "Trace Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Unassign Diagram Area
 * Purpose:  This function releases the assignment between a trace and a 
 *           diagram area, as defined by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED <Trace_Name> and expressed 
 *           by the <WndTr> suffix. The trace itself is not deleted; this must 
 *           be done via CALCulate<Ch>:PARameter:DELete <Trace_Name>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceUnassignDiagramArea(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");

    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_UNASSIGN_DIAGRAM_AREA, 
                                             VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Shift Stimulus Value
 * Purpose:  This function shifts the active trace in horizontal direction, 
 *           leaving the positions of all markers unchanged. The positive or 
 *           negative offset value for the stimulus variable is entered into an 
 *           input field. The unit depends on the sweep type.
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON).
 *           
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceShiftStimulusValue(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViReal64	shiftStimulusValue
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");

    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_X_OFFSET, 
                                            shiftStimulusValue), 4, "Shift Stimulus Value");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Shift Response Value
 * Purpose:  This function modifies all points of the trace <WndTr> by means 
 *           of an added and/or a multiplied complex constant. The response 
 *           values M of the trace are transformed according to:
 *           
 *           M_new = M_old * 10^(<Magnitude> / 20dB) * a * e^(j <Phase> / 
 *           180Deg) + <Real> + j <Imaginary>
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON).
 *           
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceShiftResponseValue(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViReal64	magnitude,
	ViReal64	phase,
	ViReal64	real,
	ViReal64	imaginary
)
{
    ViStatus    error   = VI_SUCCESS;
    
	if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");
    if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");
    if (rszvl_invalidViReal64Range (instrSession, magnitude, -300.0, 300.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Magnitude");
	if (rszvl_invalidViReal64Range (instrSession, phase, -3.4e38, 3.4e38) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Phase");
	if (rszvl_invalidViReal64Range (instrSession, real, -3.4e38, 3.4e38) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Real");
	if (rszvl_invalidViReal64Range (instrSession, imaginary, -3.4e38, 3.4e38) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Imaginary");


    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (viPrintf (instrSession, "DISP:WIND%ld:TRAC%ld:Y:OFFS %.12Lf,%.12Lf,%.12Lf,%.12Lf",
		                                 diagramArea, 
										 window_Trace, 
										 magnitude, 
										 phase, 
										 real, 
										 imaginary));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Format
 * Purpose:  This function defines how the measured result at any sweep point 
 *           is post-processed and presented in the graphical display.
 *           
 *           Note(s):
 *           
 *           The analyzer allows arbitrary combinations of display formats and 
 *           measured quantities. Nevertheless, it is advisable to check which 
 *           display formats are generally appropriate for an analysis of a 
 *           particular measured quantity.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceFormat(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	format
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_FORMAT, 
                                            format), 3, "Format");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Aperture
 * Purpose:  This function defines the number of aperture steps for the group
 *           delay calculation.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceAperture (ViSession instrSession,
                                                ViInt32 channel_Trace,
                                                ViInt32 aperture)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_APERTURE, 
                                            aperture), 3, "Aperture");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Math Function
 * Purpose:  This function defines a simple mathematical relation between the 
 *           active trace and the active memory trace to calculate a new 
 *           mathematical trace and displays the mathematical trace.
 *           
 *           Notes:
 *           
 *           (1) This command places some restrictions on the mathematical 
 *           expression and the operands. Use 
 *           CALCulate<Chn>:MATH[:EXPRession]:SDEFine to define general 
 *           expressions. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceMathFunction(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	mathematicalFunction
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_MATH_FUNC, 
                                            mathematicalFunction), 3, "Mathematical Function");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Smoothing
 * Purpose:  This function activates the smoothing function for the active 
 *           trace, which may be a data or a memory trace. With active 
 *           smoothing function, each measurement point is replaced by the 
 *           arithmetic mean value of all measurement points located in a 
 *           symmetric interval centered on the stimulus value. The width of 
 *           the smoothing interval is referred to as the Smoothing Apertureand 
 *           can be adjusted according to the properties of the trace. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceSmoothing(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean	smoothing,
	ViReal64	aperture
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_SMO_STATE, 
                                              smoothing), 3, "Smoothing");

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_SMO_APER, 
                                             aperture), 4, "Aperture");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Max Hold
 * Purpose:  This function enables, disables, or restarts the max hold 
 *           function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceMaxHold(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	maxHoldState
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_MAX_HOLD, 
                                            maxHoldState), 3, "Max Hold State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Data To Memory
 * Purpose:  This function copies the current state of the active data trace 
 *           to a memory trace. If a mathematical trace is active, the data 
 *           trace associated with the mathematical trace is copied. The memory 
 *           trace is named Mem<n>[<Data_Trace>] where <n> counts all data and 
 *           memory traces in the active setup in chronological order, and 
 *           <Data_Trace> is the name of the associated (copied) data trace. 
 *           
 *           The exact function of the command depends on the number of memory 
 *           traces associated to the active data trace:
 *           
 *           - If no memory trace is associated to the active trace, a new 
 *           memory trace is generated.
 *           
 *           - If several memory traces are associated to the active trace, 
 *           the current measurement data overwrites the last generated or 
 *           changed memory trace. 
 *           
 *           Note(s):
 *           
 *           To copy a trace to the memory without overwriting an existing 
 *           memory trace or define a memory trace name, use TRACe:COPY 
 *           <memory_trc>,<data_trc>. To copy an active mathematical trace use 
 *           TRACe:COPY:MATH <memory_trc>,<data_trc>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceDataToMemory(
	ViSession	instrSession,
	ViInt32	channel_Trace
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckErr( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_MATH_MEMORIZE, 
                                            VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Data To Memory Trace
 * Purpose:  This function copies a data trace to a memory trace. The trace to 
 *           be copied is specified as a trace with a name (string variable). 
 *           The created memory trace is specified as a memory trace with an 
 *           arbitrary name (string variable). An existing memory trace with 
 *           the same name is overwritten.
 *           
 *           Note(s):
 *           
 *           The copied trace is the data trace which is not modified by any 
 *           mathematical operations. To copy a mathematical trace to a memory 
 *           trace, use TRACe:COPY:MATH. To copy the active trace to the memory 
 *           using an automatic memory trace name, use 
 *           CALCulate<Ch/Tr>:MATH:MEMorize.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceDataToMemoryTrace(
	ViSession	instrSession,
	ViString	memoryTrace,
	ViString	dataTrace
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (!memoryTrace)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Memory Trace");
	if (!dataTrace)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Data Trace");
	
	viCheckErr (viPrintf (instrSession, "TRAC:COPY '%s','%s'", memoryTrace, dataTrace));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Math To Memory Trace
 * Purpose:  This function copies a mathematical trace to a memory trace. The 
 *           trace to be copied is specified as a trace with a name (string 
 *           variable). The created memory trace is specified as a memory trace 
 *           with an arbitrary name (string variable). An existing memory trace 
 *           with the same name is overwritten.
 *           
 *           Note(s):
 *           
 *           To copy a data trace which is not modified by any mathematical 
 *           operations, use TRACe:COPY.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceMathToMemoryTrace(
	ViSession	instrSession,
	ViString	memoryTrace,
	ViString	dataTrace
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (!memoryTrace)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Memory Trace");
	if (!dataTrace)
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Data Trace");
	
	viCheckErr (viPrintf (instrSession, "TRAC:COPY:MATH '%s','%s'", memoryTrace, dataTrace));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete Memory Trace
 * Purpose:  This function deletes one of the memory traces Mem<n>[Trc<m>], 
 *           where n = 1, ... 8.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteMemoryTrace(
	ViSession	instrSession,
	ViInt32	memoryTrace
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NWA_TRACE_DELETE_MEMORY, 
                                            memoryTrace), 2, "Memory Trace");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace List Catalog
 * Purpose:  This function returns the numbers and names of all traces in the 
 *           current setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceListCatalog(
	ViSession	instrSession,
	ViInt32	bufferSize,
	ViChar _VI_FAR catalog[]
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rszvl_QueryViString (instrSession, "CONF:TRAC:CAT?", bufferSize, catalog));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Display State
 * Purpose:  This function displays or hides an existing trace, identified by 
 *           its trace name (CALCulate<Ch>:PARameter:SDEFine <Trace_Name>), 
 *           or a group of traces.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceDisplayState (ViSession instrSession,
                                           ViInt32 traceType,
                                           ViString singleTraceName,
                                           ViBoolean showTrace)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViBooleanRange (instrSession, showTrace) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Show Trace");
	
    switch (traceType)
	{
		case RSZVL_VAL_DISP_TRAC_DATA:
		{
			sprintf (buffer, "DISP:WIND:TRAC:SHOW DALL,%s\n", switchArr[showTrace]);
			break;
		}
		case RSZVL_VAL_DISP_TRAC_MEM:
		{
			sprintf (buffer, "DISP:WIND:TRAC:SHOW MALL,%s\n", switchArr[showTrace]);
			break;
		}
		case RSZVL_VAL_DISP_TRAC_SING:
		{
			sprintf (buffer, "DISP:WIND:TRAC:SHOW '%s',%s\n", singleTraceName, switchArr[showTrace]);
			break;
		}
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace Type");
	}
	
	viCheckErr (viPrintf (instrSession, buffer));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Trace Display State
 * Purpose:  This function returns whether the trace is shown or not.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryTraceDisplayState (ViSession instrSession,
                                                ViInt32 traceType,
                                                ViString singleTraceName,
                                                ViBoolean *showTrace)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (traceType)
	{
		case RSZVL_VAL_DISP_TRAC_DATA:
		{
			sprintf (buffer, "DISP:WIND:TRAC:SHOW? DALL\n");
			break;
		}
		case RSZVL_VAL_DISP_TRAC_MEM:
		{
			sprintf (buffer, "DISP:WIND:TRAC:SHOW? MALL\n");
			break;
		}
		case RSZVL_VAL_DISP_TRAC_SING:
		{
			sprintf (buffer, "DISP:WIND:TRAC:SHOW? '%s'\n", singleTraceName);
			break;
		}
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace Type");
	}
	
	viCheckErr (viQueryf (instrSession, buffer, "%d", showTrace));
		
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Autoscale
 * Purpose:  This function adjusts the Scale Divisions and the Ref. Value in 
 *           order to display the entire active trace in the diagram area, 
 *           leaving an appropriate display margin.
 *           
 *           In Cartesian diagrams, the analyzer re-calculates the values of 
 *           the vertical divisions so that the trace fits onto 80% of the 
 *           vertical grid. The reference value is chosen to center the trace 
 *           in the diagram.
 *           
 *           In circular diagrams (Polar, Smith, Inverted Smith), the analyzer 
 *           re-calculates the values of the radial divisions so that the 
 *           diagram is confined to approx. 80% of the outer circumference. The 
 *           reference value is set to the value of the outer circumference.
 *           
 *           Autoscale does not affect the stimulus values and the horizontal 
 *           axis.
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON). 
 *            
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *           
 *           (3) The trace is be referenced by its number <WndTr>. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceAutoscale(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");
	if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");
	
    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_AUTOSCALE, 
                                             VI_NULL));
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Autoscale By Name
 * Purpose:  This function adjusts the Scale Divisions and the Ref. Value in 
 *           order to display the entire active trace in the diagram area, 
 *           leaving an appropriate display margin.
 *           
 *           In Cartesian diagrams, the analyzer re-calculates the values of 
 *           the vertical divisions so that the trace fits onto 80% of the 
 *           vertical grid. The reference value is chosen to center the trace 
 *           in the diagram.
 *           
 *           In circular diagrams (Polar, Smith, Inverted Smith), the analyzer 
 *           re-calculates the values of the radial divisions so that the 
 *           diagram is confined to approx. 80% of the outer circumference. The 
 *           reference value is set to the value of the outer circumference.
 *           
 *           Autoscale does not affect the stimulus values and the horizontal 
 *           axis.
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON). 
 *            
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *           
 *           (3) The trace is referenced by its name <trace_name>. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceAutoscaleByName(
	ViSession	instrSession,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    
	checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!traceName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace Name");
	
    viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_TRACE_AUTOSCALE_BY_NAME, 
                                             traceName), 2, "Trace Name");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Scale
 * Purpose:  This function configures the diagram area <Wnd>.
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON). 
 *            
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceScale(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViReal64	bottom,
	ViReal64	top
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");
	if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");
	
    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_SCAL_BOTTOM, 
                                             bottom), 4, "Bottom");
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_SCAL_TOP, 
                                             top), 5, "Top");
   
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Scale Division
 * Purpose:  This function Sets the value between two grid graticules (value 
 *           per division) for the diagram area <Wnd>. When a new PDIVision 
 *           value is entered, the current RLEVel is kept the same, while the 
 *           top and bottom scaling is adjusted for the new PDIVision value.
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON). 
 *            
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceScaleDivision(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViReal64	scaleDivisions
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");
	if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");
	
    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_SCAL_PDIV, 
                                             scaleDivisions), 4, "Scale Divisions");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Reference Value
 * Purpose:  This function sets the reference level (or reference value) for a 
 *           particular displayed trace. Setting a new reference level does not 
 *           affect the value of PDIVision. 
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON). 
 *            
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceReferenceValue(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViReal64	referenceLevel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");
	if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");
	
    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_REF_LEVEL, 
                                             referenceLevel), 4, "Reference Level");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Reference Position
 * Purpose:  This function sets the point on the y-axis to be used as the 
 *           reference position as a percentage of the length of the y-axis. 
 *           The reference position is the point on the y-axis which should 
 *           equal the RLEVel.
 *           
 *           Note(s):
 *           
 *           (1) Window:
 *           Number of an existing diagram area (defined by means of 
 *           DISPlay:WINDow<Wnd>:STATe ON). 
 *            
 *           (2) Window (Trace):
 *           Existing trace number, assigned by means of 
 *           DISPlay:WINDow<Wnd>:TRACe<WndTr>:FEED.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceReferencePosition(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	window_Trace,
	ViReal64	referencePosition
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");
	if (window_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Window Trace");
	
    sprintf (buffer, "DA%ld,TR%ld", diagramArea, window_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_REF_POSITION, 
                                             referencePosition), 4, "Reference Position");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Evaluation Range
 * Purpose:  This function define the range for the statistical and phase 
 *           evaluation. The evaluation range is a continuous interval of the 
 *           sweep variable.
 *           
 *           It is possible to define and select up to ten different 
 *           evaluation ranges for each setup. Full Span means that the search 
 *           range is equal to the sweep range. The statistical and phase 
 *           evaluation takes into account all measurement points with stimulus 
 *           values xi between the Start and Stop value of the evaluation 
 *           range:
 *           
 *               Start  <= xi <= Stop
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceEvaluationRange(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	evaluationRange,
	ViReal64	start,
	ViReal64	stop
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_EVAL_RANGE, 
                                             evaluationRange), 3, "Evaluation Range");
   
    if (evaluationRange != RSZVL_VAL_TRAC_EVAL_FULL_SPAN) /* Full Range */
	{
		viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_EVAL_RANGE_START, 
                                             	start), 4, "Start");
   
    	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_TRACE_EVAL_RANGE_STOP, 
                                             	stop), 5, "Stop");
	}
		
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Display Results State
 * Purpose:  This function displays or hides the Phase Delay/El Length, 
 *           Min/Max/Peak-Peak, Mean/Std Dev or RMS results in the diagram area 
 *           of trace no. <Chn>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDisplayResultsState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	resultType,
	ViBoolean	displayResults
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	switch (resultType)
    {
        case  RSZVL_VAL_DISPLAY_RESULTS_ALL:
            attribute = RSZVL_ATTR_NWA_TRACE_EVAL_ALL_STATE;
            break;
            
        case RSZVL_VAL_DISPLAY_RESULTS_EPD:
            attribute = RSZVL_ATTR_NWA_TRACE_EVAL_EPDELAY_STATE;
            break;
            
        case RSZVL_VAL_DISPLAY_RESULTS_MMPT:
            attribute = RSZVL_ATTR_NWA_TRACE_EVAL_MMPT_STATE;
            break;
        
		case  RSZVL_VAL_DISPLAY_RESULTS_MSTD:
            attribute = RSZVL_ATTR_NWA_TRACE_EVAL_MSTD_STATE;
            break;
            
        case RSZVL_VAL_DISPLAY_RESULTS_RMS:
            attribute = RSZVL_ATTR_NWA_TRACE_EVAL_RMS_STATE;
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result Type");
    }
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                attribute, displayResults),
                                                4, "Display Results");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Trace Custom Header
 * Purpose:  This function configures the custom header that can be added to 
 *           an exported trace file.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureTraceCustomHeader (ViSession instrSession,
                                           ViBoolean showCustomHeader,
                                           ViBoolean showInstrumentInfo,
                                           ViBoolean showDateAndTimeInfo,
                                           ViBoolean showText,
                                           ViString customHeaderText,
                                           ViBoolean showCalibrationInfo)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_NWA_CUSTOM_HEADER_STATE, 
												showCustomHeader), 2, "Show Custom Header");
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_NWA_CUSTOM_HEADER_INSTRUMENT_INFO, 
												showInstrumentInfo), 3, "Show Instrument Info");
			
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_NWA_CUSTOM_HEADER_DATE_AND_TIME_INFO,
												showDateAndTimeInfo), 4, "Show Date And Time Info");
			
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_NWA_CUSTOM_HEADER_TEXT_STATE, 
												showText), 5, "Show Text");
			
	viCheckParm( rszvl_SetAttributeViString (instrSession, "", 
	                                            RSZVL_ATTR_NWA_CUSTOM_HEADER_TEXT, 
												customHeaderText), 6, "Custom Header Text");
		
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_NWA_CUSTOM_HEADER_CALIBRATION_INFO, 
												showCalibrationInfo), 7, "Show Calibration Info");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Import Data
 * Purpose:  This function loads trace data from a specified trace file and 
 *           assigns it to a trace with a specified name.
 *           
 *           Note(s):
 *           
 *           Traces are created using the CALCulte<Ch>:PARameter:SDEFine... 
 *           command.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceImportData(
	ViSession	instrSession,
	ViString	traceName,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (viPrintf (instrSession, "MMEM:LOAD:TRAC '%s','%s'", traceName, fileName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Export Data
 * Purpose:  This function stores the trace data of a specified trace to a 
 *           trace file.
 *           
 *           Note(s):
 *           
 *           Traces are created using the CALCulte<Ch>:PARameter:SDEFine... 
 *           command.
 *           
 *           *.s<n>p touchstone files (<n> = 2, 3, 4) are intended for a 
 *           complete set of <n>-port S-parameters. Data export fails if the 
 *           active channel does not contain the full set of <n>^2 traces. If 
 *           the necessary trace are available, '<trc_name>' can be the name of 
 *           any of the traces.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceExportData(
	ViSession	instrSession,
	ViString	traceName,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (viPrintf (instrSession, "MMEM:STOR:TRAC '%s','%s'", traceName, fileName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Export Data With Options
 * Purpose:  This function stores the trace data of a specified trace to a 
 *           trace file.
 *           
 *           Note(s):
 *           
 *           Traces are created using the CALCulte<Ch>:PARameter:SDEFine... 
 *           command.
 *           
 *           *.s<n>p touchstone files (<n> = 2, 3, 4) are intended for a 
 *           complete set of <n>-port S-parameters. Data export fails if the 
 *           active channel does not contain the full set of <n>^2 traces. If 
 *           the necessary trace are available, '<trc_name>' can be the name of 
 *           any of the traces.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceExportDataWithOptions(
	ViSession	instrSession,
	ViString	traceName,
	ViString	fileName,
	ViInt32	exportFormat,
	ViInt32	exportData
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rszvl_invalidViInt32Range (instrSession, exportFormat, RSZVL_VAL_UNFORMATTED, RSZVL_VAL_FORMATTED) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel");

	if (rszvl_invalidViInt32Range (instrSession, exportData, RSZVL_VAL_COMPLEX, RSZVL_VAL_LOGP) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Channel");

    viCheckErr (viPrintf (instrSession, "MMEM:STOR:TRAC '%s','%s',%s,%s", traceName, fileName,
		                                         exportFormatArr[exportFormat],
												 exportDataArr[exportData]));
    

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Trace Export Data With Options Extended
 * Purpose:  This function stores the trace data of a specified trace to a 
 *           trace file.
 *           
 *           Note(s):
 *           
 *           Traces are created using the CALCulte<Ch>:PARameter:SDEFine... 
 *           command.
 *           
 *           *.s<n>p touchstone files (<n> = 2, 3, 4) are intended for a 
 *           complete set of <n>-port S-parameters. Data export fails if the 
 *           active channel does not contain the full set of <n>^2 traces. If 
 *           the necessary trace are available, '<trc_name>' can be the name of 
 *           any of the traces.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_TraceExportDataWithOptionsExtended (ViSession instrSession,
                                                            ViString traceName,
                                                            ViString fileName,
                                                            ViInt32 exportFormat,
                                                            ViInt32 exportData,
                                                            ViInt32 decimalSeparator,
                                                            ViInt32 fieldSeparator)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rszvl_invalidViInt32Range (instrSession, exportFormat, RSZVL_VAL_UNFORMATTED, RSZVL_VAL_FORMATTED) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Channel");

	if (rszvl_invalidViInt32Range (instrSession, exportData, RSZVL_VAL_COMPLEX, RSZVL_VAL_LOGP) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Channel");

    if (rszvl_invalidViInt32Range (instrSession, decimalSeparator, RSZVL_VAL_POIN, RSZVL_VAL_COMM) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Decimal Separator");

	if (rszvl_invalidViInt32Range (instrSession, fieldSeparator, RSZVL_VAL_POIN, RSZVL_VAL_SPAC) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Field Separator");

    viCheckErr (viPrintf (instrSession, "MMEM:STOR:TRAC '%s','%s',%s,%s,%s,%s", traceName, fileName,
		                                         exportFormatArr[exportFormat],
												 exportDataArr[exportData],
												 decimalSeparatorArr[decimalSeparator],
												 fieldSeparatorArr[fieldSeparator]));
    

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Marker State
 * Purpose:  This function creates the marker numbered <Mk> and assigns it to 
 *           trace no. <Ch/Tr>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureMarkerState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViBoolean	markerState
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_STATE, 
                                              markerState), 4, "Marker State");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Marker Stimulus
 * Purpose:  This function defines the stimulus (in Cartesian diagrams: 
 *           x-axis) value of the marker no. <Mk>, which can (but doesn't have 
 *           to) be created using CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureMarkerStimulus(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViReal64	markerStimulus
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_X, 
                                              markerStimulus), 4, "Marker Stimulus");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Reference Marker State
 * Purpose:  This function creates the reference marker and assigns it to 
 *           trace no. <Ch/Tr>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureReferenceMarkerState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViBoolean	referenceMarkerState
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_REF_STATE, 
                                              referenceMarkerState), 4, "Reference Marker State");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Reference Marker Stimulus
 * Purpose:  This function defines the stimulus (in Cartesian diagrams: 
 *           x-axis) value of the reference marker, which can (but doesn't have 
 *           to) be displayed using 
 *           CALCulate<Ch/Tr>:MARKer<Mk>:REFerence[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureReferenceMarkerStimulus(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViReal64	referenceMarkerStimulus
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_REF_X, 
                                              referenceMarkerStimulus), 4, "Reference Marker Stimulus");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Reference Discrete Marker 
 * Purpose:  This function defines the reference marker to continuous or
 *           discrete mode. The marker doesn't have to be created before
 *           (CALCulate<Chn>:MARKer<Mk>:REFerence[:STATe] ON), the mode can
 *           be assigned in advance.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureReferenceDiscreteMarker (ViSession instrSession,
                                                              ViInt32 channel_Trace,
                                                              ViInt32 marker,
                                                              ViInt32 discreteMode)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_REF_MODE, 
                                              discreteMode), 4, "Discrete Mode");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Reference Fixed Marker
 * Purpose:  This function sets the reference to normal or fixed mode. The
 *           marker must be created before using
 *           CALCulate<Chn>:MARKer<Mk>:REFerence[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureReferenceFixedMarker (ViSession instrSession,
                                                       ViInt32 channel_Trace,
                                                       ViInt32 marker,
                                                       ViInt32 fixedMarker)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_REF_FIXED, 
                                              fixedMarker), 4, "Fixed Mode");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Delta Marker State
 * Purpose:  This function switches the delta mode for marker <Mk> on trace 
 *           no. <Ch/Tr> on or off. The marker must be created before using 
 *           CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON. If the active trace 
 *           contains no reference marker, the command also creates a reference 
 *           marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDeltaMarkerState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViBoolean	deltaMarkerState
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DELTA_MARKER_STATE, 
                                              deltaMarkerState), 4, "Delta Marker State");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Coupled Markers
 * Purpose:  This function couples the markers of all traces in the active 
 *           setup to the markers of trace no. <Ch/Tr>, provided that they have 
 *           the same sweep type (SENSe<Ch/Tr>:FUNCtion).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureCoupledMarkers(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean	markerCoupled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_COUPLED, 
                                              markerCoupled), 4, "Marker Coupled");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Discrete Marker
 * Purpose:  This function sets marker no. <Mk> to continuous or discrete 
 *           mode. The marker doesn't have to be created before 
 *           (CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON), the mode can be assigned 
 *           in advance.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDiscreteMarker(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViInt32	discreteMode
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_MODE, 
                                              discreteMode), 4, "Discrete Mode");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Fixed Marker
 * Purpose:  This function sets marker no. <Mk> to normal or fixed mode. The 
 *           marker must be created before using 
 *           CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON (attribute 
 *           RSZVL_ATTR_NWA_MARKER_STATE). 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFixedMarker(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViInt32	fixedMarker
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_TYPE, 
                                              fixedMarker), 4, "Fixed Mode");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Signal Track
 * Purpose:  This function enables or disables the marker tracking mode for 
 *           marker no. <Mk>. Tracking mode causes the active minimum/maximum 
 *           or target search of the active marker to be repeated after each 
 *           sweep. A marker must be created and a search mode must be active 
 *           (CALCulate<Chn>:MARKer<Mk>:FUNCtion:EXECute ...) to use this 
 *           function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSignalTrack(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViBoolean	signalTrackEnabled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_TRACKING, 
                                              signalTrackEnabled), 4, "Signal Track Enabled");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Disable All Markers
 * Purpose:  This function removes all markers from all traces of the active 
 *           setup. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DisableAllMarkers(
	ViSession	instrSession,
	ViInt32	channel_Trace
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_AOFF, 
                                              VI_NULL));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Instrument From Marker
 * Purpose:  This function makes the selected marker frequency to be the 
 *           center frequency or step width of center frequency. It can also 
 *           make the active marker amplitude to be the reference level.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetInstrumentFromMarker(
	ViSession	instrSession,
	ViInt32 channel_Trace,
	ViInt32	marker,
	ViInt32	instrumentSetting
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    if (marker <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
	
    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	switch (instrumentSetting)
    {
        case  RSZVL_VAL_INSTR_SETTING_FREQ_CENTER:
            attribute = RSZVL_ATTR_NWA_MARKER_TO_CENTER;
            break;
            
        case RSZVL_VAL_INSTR_SETTING_FREQ_START:
            attribute = RSZVL_ATTR_NWA_MARKER_TO_START;
            break;
            
        case RSZVL_VAL_INSTR_SETTING_FREQ_STOP:
            attribute = RSZVL_ATTR_NWA_MARKER_TO_STOP;
            break;
        
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Instrument Setting");
    }
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, attribute, VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Marker Format
 * Purpose:  This function defines the output format for the (complex) value 
 *           of marker <Mk> on trace no. <Chn>.
 *           
 *           Note(s):
 *           
 *           The formats of the markers assigned to a trace are independent of 
 *           each other and of the trace format settings; see 
 *           CALCulate<Ch/Tr>:FORMat.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureMarkerFormat(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViInt32	markerFormat
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_FORMAT, 
                                              markerFormat), 4, "Marker Format");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Marker Search
 * Purpose:  This function searches with the active marker for specific points 
 *           on the trace.
 *           
 *           Note(s):
 *           
 *           The marker must be created before using 
 *           CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_MarkerSearch(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViInt32	search
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_MODE, 
                                              search), 4, "Search");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Marker Target Search
 * Purpose:  This function searches with the active marker for target value on 
 *           the trace.
 *           
 *           Note(s):
 *           
 *           The marker must be created before using 
 *           CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_MarkerTargetSearch(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViInt32	search
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_MODE, 
                                              search), 4, "Search");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Marker Bandfilter Search
 * Purpose:  This function activates the search for a bandpass region on the 
 *           active trace. A bandpass region is the tallest peak in the search 
 *           range with a minimum excursion specified by means of the x dB 
 *           Bandwidth parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_MarkerBandfilterSearch(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	bandfilter
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_BFILTER_SEARCH_MODE, 
                                              bandfilter), 3, "Bandfilter");

	sprintf (buffer, "CH%ld,MAR1", channel_Trace);

	viCheckErr( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_MODE, 
                                              RSZVL_VAL_MARK_SEARCH_BFIL));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Marker Search Limits
 * Purpose:  This function configures marker search limits.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureMarkerSearchLimits(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViInt32	searchRange,
	ViReal64	searchRangeStart,
	ViReal64	searchRangeStop
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_RANGE, 
                                              searchRange), 4, "Search Range");
	
	if (searchRange != RSZVL_VAL_MARK_FULL_SPAN)
    {
    	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_RANGE_START, 
                                              searchRangeStart), 5, "Search Range Start");
	
		viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_RANGE_STOP, 
                                              searchRangeStop), 6, "Search Range Stop");
	}

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Marker Target Value
 * Purpose:  This function defines the target value for the target search of 
 *           marker no. <Mk>, which can be activated using 
 *           CALCulate<Ch/Tr>:MARKer<Mk>:FUCTion:EXECute TARGet (attribute 
 *           RSZVL_ATTR_NWA_MARKER_SEARCH_MODE).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureMarkerTargetValue(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViReal64	targetValue
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

    sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_TARGET_VALUE, 
                                              targetValue), 4, "Target Value");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Marker x dB Bandwidth
 * Purpose:  This function sets the bandfilter level for a bandfilter search 
 *           or returns the results. The function is only available after a 
 *           bandfilter search has been executed 
 *           (CALCulate<Chn>:MARKer<Mk>:FUNCtion:EXECute BFILter).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureMarkerxdBBandwidth(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViReal64	xdBBandwidth
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_BANDWIDTH, 
                                              xdBBandwidth), 3, "x dB Bandwidth");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Marker Bandfilter Tracking
 * Purpose:  This function causes the bandfilter search to be repeated after 
 *           each sweep: When tracking mode is active the markers typically 
 *           change their horizontal and their vertical positions as the 
 *           measurement goes on.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_MarkerBandfilterTracking(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean	bandfilterTracking
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld,MAR1", channel_Trace);

	viCheckErr( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_MODE, 
                                              RSZVL_VAL_MARK_SEARCH_BFIL));
	
	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_TRACKING, 
                                              bandfilterTracking), 3, "Bandfilter Tracking");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Marker Search Result State
 * Purpose:  This function shows or hides the bandfilter search results in the 
 *           diagram area.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetMarkerSearchResultState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean	searchResults
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld,MAR1", channel_Trace);

	viCheckErr( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_SEARCH_MODE, 
                                              RSZVL_VAL_MARK_SEARCH_BFIL));
	
	sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_BFILT_SEARCH_RESULTS, 
                                              searchResults), 3, "Search Results");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Marker Search Result State
 * Purpose:  This function returns the bandfilter search results state.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetMarkerSearchResultState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean*	searchResults
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_GetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_BFILT_SEARCH_RESULTS, 
                                              searchResults), 3, "Search Results");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save All Markers
 * Purpose:  This function saves the values of all markers to a ASCII file. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveAllMarkers(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_SAVE_ALL_MARKERS, 
                                              fileName), 2, "File Name");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Limit Line Display
 * Purpose:  This function displays or hides the entire limit line (including 
 *           all segments) associated to the active trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLimitLineDisplay(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean	displayLimitLine
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_DISPLAY_STATE, 
                                              displayLimitLine), 3, "Display Limit Line");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Limit Check
 * Purpose:  This function switches the limit check (or individual upper and 
 *           lower limits) on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLimitCheck(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	limitLine,
	ViBoolean	limitCheck
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);
  
	switch (limitLine)
    {
        case RSZVL_VAL_LIMIT_UPPER:
            attribute = RSZVL_ATTR_NWA_LIMIT_UPPER_STATE;
            break;
            
        case RSZVL_VAL_LIMIT_LOWER:
            attribute = RSZVL_ATTR_NWA_LIMIT_LOWER_STATE;
            break;
            
        case RSZVL_VAL_LIMIT_ALL:
            attribute = RSZVL_ATTR_NWA_LIMIT_STATE;
            break;
        
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit Line");
    }
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, attribute, 
                                              limitCheck), 4, "Limit Check");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Limit Response Domain Complex Units
 * Purpose:  This function deletes the existing limit line and (re-)defines 
 *           the physical units of the response values of the limit line.
 *           
 *           Notes:
 *           
 *           (1) This command is complemented by 
 *           CALCulate<Chn>:LIMit:RDOMain:FORMat and 
 *           CALCulate<Chn>:LIMit:RDOMain:SPACing.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLimitResponseDomainComplexUnits(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	responseUnits
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);
  
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_RDOM_COMPLEX, 
                                              responseUnits), 3, "Response Units");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Limit Response Domain Format Units
 * Purpose:  This function deletes the existing limit line and (re-)defines 
 *           the physical units of the response values of the limit line.
 *           
 *           Notes: 
 *           
 *           (1) This command is complemented by 
 *           CALCulate<Chn>:LIMit:RDOMain:COMPlex and 
 *           CALCulate<Chn>:LIMit:RDOMain:SPACing. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLimitResponseDomainFormatUnits(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	responseUnits
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);
  
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_RDOM_FORMAT, 
                                              responseUnits), 3, "Response Units");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Limit Response Domain Spacing Units
 * Purpose:  This function deletes the existing limit line and (re-)defines 
 *           the physical units of the response values of the limit line.
 *           
 *           Notes: 
 *           
 *           (1) This command is complemented by 
 *           CALCulate<Chn>:LIMit:RDOMain:COMPlex and 
 *           CALCulate<Chn>:LIMit:RDOMain:FORMat.
 *           
 *           (2) The analyzer uses dB units, irrespective of the parameter 
 *           selected.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLimitResponseDomainSpacingUnits(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	responseUnits
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);
  
	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_RDOM_SPACING, 
                                              responseUnits), 3, "Response Units");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Display Line State
 * Purpose:  This function configures state of specified display line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDisplayLineState(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViBoolean	state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DISPLAY_LINE_STATE, 
                                              state), 3, "State");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Display Line Position
 * Purpose:  This function configures position of specified display line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDisplayLinePosition(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViReal64	position
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_DISPLAY_LINE_POSITION, 
                                              position), 3, "Position");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Add Limit Line Segment
 * Purpose:  This function adds a new segment to the limit line list. In each 
 *           segment the limit line is defined as a straight line connecting 
 *           two points.
 *           
 *           Note(s):
 *           
 *           This function does not overwrite existing limit line segments. 
 *           The defined segment is appended to the segment list as new 
 *           segment.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_AddLimitLineSegment(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	segment,
	ViInt32	type,
	ViReal64	startStimulus,
	ViReal64	stopStimulus,
	ViReal64	startResponse,
	ViReal64	stopResponse
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_LIMIT_OFF, RSZVL_VAL_LIMIT_LOWER) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
	
	viCheckErr (viPrintf (instrSession, "CALC%ld:LIM:DATA %ld,%.12Lf,%.12Lf,%.12Lf,%.12Lf",
             channel_Trace,
             type,
             startStimulus,
             stopStimulus,
             startResponse,
             stopResponse));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Edit Limit Line Segment
 * Purpose:  This function edits existing segment of the limit line segments 
 *           list.
 *           
 *           Note(s):
 *           
 *           A segment must be created first to enable this function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_EditLimitLineSegment(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	segment,
	ViInt32	type,
	ViReal64	startStimulus,
	ViReal64	stopStimulus,
	ViReal64	startResponse,
	ViReal64	stopResponse
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel_Trace, segment);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_TYPE, 
                                              type), 4, "Type");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_STIMUL_START, 
                                              startStimulus), 5, "Start Stimulus");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_STIMUL_STOP, 
                                              stopStimulus), 6, "Stop Stimulus");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_RESPONSE_START, 
                                              startResponse), 7, "Start Response");
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_RESPONSE_STOP, 
                                              stopResponse), 8, "Stop Response");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read Limit Line Segment List
 * Purpose:  This function returns entire limit line segment list. The limit 
 *           line segment is calculated as a straight line connecting the two 
 *           points (<Start Stimulus>, <Start Response>) and (<Stop Stimulus>, 
 *           <Stop Response>).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadLimitLineSegmentList(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	listSize,
	ViInt32*	segmentsCount,
	ViInt32 _VI_FAR type[],
	ViReal64 _VI_FAR startStimulus[],
	ViReal64 _VI_FAR stopStimulus[],
	ViReal64 _VI_FAR startResponse[],
	ViReal64 _VI_FAR stopResponse[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     prd_buf = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i, count, j;
	ViUInt32	rd_count;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    viCheckErr (viPrintf (instrSession, "CALC%ld:LIMit:DATA?\n", channel_Trace));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &prd_buf, &rd_count));
    
    *segmentsCount = 0;
    									  
    p2buf = prd_buf;
    do {
        p2buf = strchr (p2buf, ',');
        (*segmentsCount)++;
        if (p2buf)
            p2buf++;
    } while (p2buf);
    
    *segmentsCount /= 5;
    
    count = (listSize < *segmentsCount) ? listSize : *segmentsCount;
    
    p2buf = prd_buf;
    for (i = 0; i < count; i++)
    {
        sscanf (p2buf, "%ld,%Le,%Le,%Le,%Le", &type[i],
                                              &startStimulus[i],
                                              &stopStimulus[i],
                                              &startResponse[i],
                                              &stopResponse[i]);
        for (j = 0; j < 5; j++)
            {
            p2buf = strchr (p2buf, ',');
            if (p2buf)
                p2buf++;
            }
    }
    
    free (prd_buf);
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Write Limit Line Segment List
 * Purpose:  This function defines the stimulus values of the limit line 
 *           and/or creates new limit line segments. It also defines the 
 *           response (y-axis) values of the lower (upper) limit line.
 *           
 *           Note(s):
 *           
 *           (1) This function use a fixed numbering scheme for limit line 
 *           segments: Upper limit line segments are assigned odd numbers (1, 
 *           3, 5,...), lower limit line segments are assigned even numbers (2, 
 *           4, 6,...).
 *           
 *           (2) Rules for creating segments:
 *           The following rules apply to an active trace with n existing 
 *           upper and n existing lower limit line segments:
 *           
 *           An odd number of values is rejected; an error message 
 *           -109,"Missing parameter..." is generated. 
 *           
 *           An even number of 2*k values updates or generates k lower limit 
 *           line segments. 
 *           
 *           For n > k the response values of all existing lower limit line 
 *           segments no. 2, 4, ...,2*k are updated, the existing upper and 
 *           lower limit line segments no. 2*k+1, ..., 2*n are deleted. The 
 *           existing upper limit line segments no. 1, 3, 2*k-1 are not 
 *           affected. 
 *           
 *           For n < k the response values of the lower limit line segments 
 *           no. 2, 4 to 2*n are updated, the lower limit line segments 2*n+2, 
 *           2*n+4,..., 2*k are generated with default stimulus values (see 
 *           CALCulate<Ch/Tr>:LIMit:CONTrol[:DATA] . In addition, the missing 
 *           upper limit line segments 2*n+1, 2*n+3,..., 2*k-1 are generated 
 *           with default stimulus and response values 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_WriteLimitLineSegmentList(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	listSize,
	ViInt32	type,
	ViReal64 _VI_FAR startStimulus[],
	ViReal64 _VI_FAR stopStimulus[],
	ViReal64 _VI_FAR startResponse[],
	ViReal64 _VI_FAR stopResponse[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     pbuffer = NULL;
    ViChar*     p2buf;
    ViInt32     i       = 0,
                index   = 0;
    ViReal64*   tempArr;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    /*

    Upper limit line segments are assigned odd numbers (1, 3, 5,...).
    Lower limit line segments are assigned even numbers (2, 4, 6,...).  

    */  

    p2buf = pbuffer = (ViChar*) malloc (RSZVL_IO_BUFFER_SIZE + listSize * 8 * 4);
    tempArr = malloc (listSize * 4 * sizeof(ViReal64));

    /* --- Response data --- */

    p2buf = pbuffer + sprintf (pbuffer, "CALC%ld:LIM:%sDATA ", channel_Trace, limitCheckArr[type]);
    
    for (i = 0; i < listSize; i++)
        p2buf += sprintf (p2buf, "%.12Lf,%.12Lf,", startResponse[i], stopResponse[i]);
    
    *--p2buf = '\0';
    
    viCheckErr (viPrintf (instrSession, pbuffer));

    if (error)
    {
        free (pbuffer);
        free (tempArr);
        return error;
    }

    /* --- Stimulus data --- */
	
	{
		ViChar*     pbuffer_stimulus = NULL;
	    ViChar*     p2buf_stimulus;
	    ViUInt32    local_count;
	    ViInt32     j = 0;

	    viCheckErr (viPrintf(instrSession, "CALC%ld:LIM:CONT?\n", channel_Trace));
	    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &pbuffer_stimulus, &local_count));
	    p2buf_stimulus = pbuffer_stimulus;

	    p2buf = strtok(pbuffer_stimulus, ",");

	    while(p2buf && (j < listSize * 4))
	        {
	        tempArr[j++] = atof(p2buf_stimulus);
			p2buf_stimulus = strtok(NULL, ",");
	        };

	    checkErr( rszvl_CheckStatus (instrSession));

	    free(pbuffer_stimulus);
	}

    p2buf = pbuffer + sprintf (pbuffer, "CALC%ld:LIM:CONT ", channel_Trace);
    
    index = 0;

    for (i = 0; i < listSize; i++)
        {
        if (type == RSZVL_VAL_LIMIT_LOWER)
            {
            p2buf += sprintf (p2buf, "%.12Lf,%.12Lf,%.12Lf,%.12Lf,", tempArr[index + 0], tempArr[index + 1], startStimulus[i], stopStimulus[i]);
            }
        else
            {
            p2buf += sprintf (p2buf, "%.12Lf,%.12Lf,%.12Lf,%.12Lf,", startStimulus[i], stopStimulus[i], tempArr[index + 2], tempArr[index + 3]);
            }
        
        index += 4;
        }
    
    *--p2buf = '\0';
    
    viCheckErr (viPrintf (instrSession, pbuffer));

    free (pbuffer);
    free (tempArr);
    
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Shift Limit Line Segment List
 * Purpose:  This function shifts the limit line in horizontal direction 
 *           (stimulus values) and also shifts all lower and upper limit line 
 *           segments assigned to the active trace in vertical direction 
 *           (response values).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ShiftLimitLineSegmentList(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	limitLineType,
	ViReal64	stimulusOffset,
	ViReal64	responseOffset
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViAttr		attribute = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_STIMUL_SHIFT, 
                                              stimulusOffset), 4, "Stimulus Offset");
	
	switch (limitLineType)
	{
		case RSZVL_VAL_LIMIT_UPPER:
			attribute = RSZVL_ATTR_NWA_LIMIT_SEGM_UPPER_SHIFT;
			break;
			
		case RSZVL_VAL_LIMIT_LOWER:
			attribute = RSZVL_ATTR_NWA_LIMIT_SEGM_LOWER_SHIFT;
			break;
			
		default:	
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit Line Type"); 
	}

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, attribute, 
                                              responseOffset), 5, "Response Offset");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Limit Line Segment Count
 * Purpose:  This function queries the number of limit line segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryLimitLineSegmentCount (ViSession instrSession,
                                                    ViInt32 channel_Trace,
                                                    ViInt32 *segmentCount)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);
	
	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_SEGM_COUNT, 
                                              segmentCount), 3, "Segment Count");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
	(void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete Limit Line Segments
 * Purpose:  This function deletes all limit line segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteLimitLineSegments(
	ViSession	instrSession,
	ViInt32	channel_Trace
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_DELETE_ALL_SEGM, 
                                              VI_NULL));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Limit Line
 * Purpose:  This function saves the limit lines associated to a specified 
 *           trace to a limit line file. Limit lines are created using the 
 *           CALCulte<Ch/Tr>:LIMit... commands.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveLimitLine(
	ViSession	instrSession,
	ViString	traceName,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
	checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (!traceName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace Name");
	if (!fileName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "File Name");
	
    viCheckErr (viPrintf (instrSession, ":MMEM:STOR:LIM '%s','%s'", traceName, fileName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Limit Line
 * Purpose:  This function loads a limit line definition from a specified file 
 *           and assigns it to a trace with a specified name. Limit lines are 
 *           created using the CALCulte<Ch/Tr>:LIMit... commands.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_RecallLimitLine(
	ViSession	instrSession,
	ViString	traceName,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (!traceName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace Name");
	if (!fileName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "File Name");
	
    viCheckErr (viPrintf (instrSession, ":MMEM:LOAD:LIM '%s','%s'", traceName, fileName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Limit Line With Options
 * Purpose:  This function loads a limit line definition from a specified file 
 *           and assigns it to a trace with a specified name. Limit lines are 
 *           created using the CALCulte<Ch/Tr>:LIMit... commands.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_LoadLimitLineWithOptions(
	ViSession	instrSession,
	ViString	traceName,
	ViString	fileName,
	ViString	sParameter,
	ViReal64	xOffset,
	ViReal64	yOffset,
	ViInt32	type
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (!traceName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace Name");
	if (!fileName)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "File Name");
	if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_LIMIT_OFF, RSZVL_VAL_LIMIT_LOWER) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "type");
	
	if (!sParameter)
	{
		viCheckErr (viPrintf (instrSession, ":MMEM:LOAD:LIM '%s','%s'", traceName, fileName));
	}
	else
	{
    	viCheckErr (viPrintf (instrSession, ":MMEM:LOAD:LIM '%s','%s','%s',%Lf,%Lf,%s", traceName, fileName,
		                                   sParameter, xOffset, yOffset, limitTypeArr[type]));
	}
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Import Trace as Limit Line
 * Purpose:  This function generates an lower (upper) limit line using the 
 *           stimulus values of a data or memory trace and specified offset 
 *           values.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ImportTraceasLimitLine(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	limitLineType,
	ViReal64	stimulusOffset,
	ViReal64	responseOffset,
	ViString	traceName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (rszvl_invalidViInt32Range (instrSession, limitLineType, RSZVL_VAL_LIMIT_UPPER, RSZVL_VAL_LIMIT_LOWER) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit Line Type");
    
	if (rszvl_invalidViReal64Range (instrSession, stimulusOffset, -1000.0e9, 1000.0e9) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Stimulus Offset");
	
	if (rszvl_invalidViReal64Range (instrSession, responseOffset, -10.0e12, 10.0e12) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Response Offset");
	
    viCheckErr (viPrintf (instrSession, "CALC%ld:LIM:%sFEED %.12Lf,%.12Lf,'%s'", channel_Trace,
                                                              limitCheckArr[limitLineType],
                                                              stimulusOffset,
                                                              responseOffset,
                                                              traceName));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Show Ripple Limits
 * Purpose:  This function displays or hides all ripple limit lines (including
 *           all ranges) associated to the active trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ShowRippleLimits (ViSession instrSession,
                                          ViInt32 channel_Trace,
                                          ViBoolean showLimits)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_SHOW_LIMITS, 
                                              showLimits), 3, "Show Limits");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Ripple Limit Check
 * Purpose:  This function switches the global ripple limit check on or off. 
 *           This check covers all traces in the active setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRippleLimitCheck (ViSession instrSession,
                                                   ViInt32 channel_Trace,
                                                   ViBoolean limitCheck)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_CHECK_STATE, 
                                              limitCheck), 3, "Limit Check");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Ripple Global Limit Check Result
 * Purpose:  This function returns a 0 or 1, to indicate whether or not the 
 *           global ripple limit check has failed.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetRippleGlobalLimitCheckResult (ViSession instrSession,
                                                         ViInt32 channel_Trace,
                                                         ViInt32 *limitCheckResult)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_GET_RIPPLE_GLOBAL_LIMIT_CHECK_RESULT, 
                                              limitCheckResult), 3, "Limit Check Result");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Ripple Segment Limit Check
 * Purpose:  This function enables or disables the limit check in the ripple 
 *           limit range no. <Seg>
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRippleSegmentLimitCheck (ViSession instrSession,
                                                          ViInt32 channel_Trace,
                                                          ViInt32 segment,
                                                          ViBoolean limitCheck)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel_Trace, segment);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_RANGE_SEGMENT_LIMIT_CHECK, 
                                              limitCheck), 4, "Limit Check");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Ripple Limit Check Segment Result
 * Purpose:  This function returns the result of the ripple limit check in the
 *           previously defined limit range no. <Seg>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetRippleLimitCheckSegmentResult (ViSession instrSession,
                                                          ViInt32 channel_Trace,
                                                          ViInt32 segment,
                                                          ViInt32 *fail,
                                                          ViReal64 *limitCheckResult)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer [RSZVL_IO_BUFFER_SIZE] = "";
	ViChar      rd_buffer [RSZVL_IO_BUFFER_SIZE] = "";
	ViChar		*p2buf;
	ViInt32		empty_string = 0;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CALC%ld:RIPP:SEGM%ld:RES?", channel_Trace, segment);
	
	viCheckErr (rszvl_QueryViString (instrSession, 
		                              buffer,
									  RSZVL_IO_BUFFER_SIZE,
									  rd_buffer));
	
	empty_string = strcmp (rd_buffer, "");
	
	if (empty_string)
	{
		p2buf = strtok (rd_buffer, ",");
    
		sscanf (p2buf, "%ld", fail);

		p2buf = strtok (VI_NULL, ",");
	
		sscanf (p2buf, "%Le", limitCheckResult);
	}

	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Ripple Number Of Limit Ranges
 * Purpose:  This function queries the number of ripple limit ranges.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetRippleNumberOfLimitRanges (ViSession instrSession,
                                                      ViInt32 channel_Trace,
                                                      ViInt32 segment,
                                                      ViInt32 *numberOfRanges)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel_Trace, segment);

	viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_GET_LIMIT_RANGES, 
                                              numberOfRanges), 4, "Number Of Ranges");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Add Ripple Limit Line Ranges Segment
 * Purpose:  This function adds and enables/disables an arbitrary number of 
 *           ripple limit ranges, assigning the stimulus values and the ripple 
 *           limits.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_AddRippleLimitLineRangesSegment (ViSession instrSession,
                                                         ViInt32 channel_Trace,
                                                         ViInt32 numberOfValues,
                                                         ViInt32 type[],
                                                         ViReal64 startStimulus[],
                                                         ViReal64 stopStimulus[],
                                                         ViReal64 limit[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar*     p2buf = buffer;
    ViInt32     i       = 0;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	p2buf += sprintf (buffer, "CALC%ld:RIPP:DATA", channel_Trace);
	
	for(i=0; i<numberOfValues; i++){
		p2buf += sprintf (p2buf, " %ld,%.12Lf,%.12Lf,%.12Lf,",
             type[i],
             startStimulus[i],
             stopStimulus[i],
             limit[i]);
  
	}
	*p2buf = '\0';
	*--p2buf = '\n';
	
	viCheckErr (viPrintf (instrSession, buffer));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Edit Ripple Limit Line Segment
 * Purpose:  This function edits existing segment of the ripple limit line 
 *           segments list.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_EditRippleLimitLineSegment (ViSession instrSession,
                                                    ViInt32 channel_Trace,
                                                    ViInt32 segment,
                                                    ViReal64 startStimulus,
                                                    ViReal64 stopStimulus,
                                                    ViReal64 limit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (segment <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Segment");
	
    sprintf (buffer, "CH%ld,SEG%ld", channel_Trace, segment);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_RANGE_SEGMENT_START, 
                                              startStimulus), 4, "Start Stimulus");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_RANGE_SEGMENT_STOP, 
                                              stopStimulus), 5, "Stop Stimulus");
	
	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_LIMIT_RANGE, 
                                              limit), 6, "Limit");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete All Ripple Limit Ranges
 * Purpose:  This function deletes all limit line segments.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteAllRippleLimitRanges (ViSession instrSession,
                                                    ViInt32 channel_Trace)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CH%ld", channel_Trace);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_DELETE_ALL_LIMIT_RANGES, 
                                              NULL));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Ripple Limit Physical Units
 * Purpose:  This function deletes the existing ripple limit ranges and 
 *           (re-)defines the physical units of the stimulus values of the 
 *           ripple limit lines. The unit of the ripple limit is defined via
 *           CALCulate<Chn>:RIPPle:RDOMain:FORMat 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRippleLimitPhysicalUnits (ViSession instrSession,
                                                           ViInt32 channel_Trace,
                                                           ViInt32 physicalUnits)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_LIMIT_PHYSICAL_UNITS, 
                                              physicalUnits), 3, "Physical Units");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Ripple Limit Response Domain Format Units
 * Purpose:  This function deletes the existing ripple limit ranges and 
 *           (re-)defines the physical unit of the ripple limit. The units of 
 *           the stimulus values are defined via 
 *           CALCulate<Chn>:RIPPle:CONTrol:DOMain.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRippleLimitResponseDomainFormatUnits (ViSession instrSession,
                                                                       ViInt32 channel_Trace,
                                                                       ViInt32 responseUnits)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_RIPPLE_LIMIT_RESPONSE_DOMAIN_FORMAT_UNITS, 
                                              responseUnits), 3, "Response Units");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Recall Ripple Limit
 * Purpose:  This function saves or recalls the ripple limits associated with 
 *           a specified trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveRecallRippleLimit (ViSession instrSession,
                                               ViInt32 operationToBePerformed,
                                               ViString traceName,
                                               ViString fileName)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer [RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (rszvl_invalidViInt32Range (instrSession, operationToBePerformed, RSZVL_SAVE, RSZVL_RECALL) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation To Be Performed");
    
	switch (operationToBePerformed)
	{
		case RSZVL_SAVE:
			sprintf (buffer, "MMEM:STOR:RIPP '%s','%s'", traceName, fileName); 
		break;
		
		case RSZVL_RECALL:
			sprintf (buffer, "MMEM:LOAD:RIPP '%s','%s'", traceName, fileName);
		break;
		
		default:
			VI_ERROR_PARAMETER2;
	}
	
	viCheckErr (viPrintf (instrSession, buffer));
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Trace Response Data
 * Purpose:  This function reads the current response values of the active 
 *           data trace or memory trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetTraceResponseData(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	dataFormat,
	ViInt32*	noofValues,
	ViReal64 _VI_FAR traceData[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (rszvl_invalidViInt32Range (instrSession, dataFormat, RSZVL_VAL_UNFORMATTED, RSZVL_VAL_UNFORMATTED_MATH) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Data Format");
    
	sprintf (buffer, "CALC%ld:DATA? %s", channel_Trace, traceFormatArr[dataFormat]);

    viCheckErr( rszvl_dataReadTrace (instrSession, buffer, 10*RSZVL_IO_BUFFER_SIZE, traceData, noofValues));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Trace Response Data Error
 * Purpose:  This function reads the current response values of the active 
 *           data trace or memory trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetTraceResponseDataError(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	errorTerm,
	ViInt32*	noofValues,
	ViReal64 _VI_FAR traceData[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	if (rszvl_invalidViInt32Range (instrSession, errorTerm, 1, 27) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Error Term");
    
	sprintf (buffer, "CALC%ld:DATA? SCORr%ld", channel_Trace, errorTerm);

    viCheckErr( rszvl_dataReadTrace (instrSession, buffer, 10*RSZVL_IO_BUFFER_SIZE, traceData, noofValues));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Trace Response Single Sweep Data
 * Purpose:  This function reads the response values of a trace acquired in 
 *           single sweep mode (INITiate<Ch>:CONTinuous OFF).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetTraceResponseSingleSweepData(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	sweepNumber,
	ViInt32*	noofValues,
	ViReal64 _VI_FAR traceData[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CALC%ld:DATA:NSW? SDAT,%ld", channel_Trace, sweepNumber);

    viCheckErr( rszvl_dataReadTrace (instrSession, buffer, 10*RSZVL_IO_BUFFER_SIZE, traceData, noofValues));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Trace Stimulus Data
 * Purpose:  This function reads the stimulus values of the active data or 
 *           memory trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetTraceStimulusData(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32*	noofValues,
	ViReal64 _VI_FAR traceData[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CALC%ld:DATA:STIM?", channel_Trace);

    viCheckErr( rszvl_dataReadTrace (instrSession, buffer, 10*RSZVL_IO_BUFFER_SIZE, traceData, noofValues));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Trace Statistical Evaluation
 * Purpose:  This function selects how to evaluate and display statistical and 
 *           phase information of the entire trace or of a specific evaluation 
 *           range.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetTraceStatisticalEvaluation(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	statisticalParameter,
	ViReal64 _VI_FAR responseValue_s[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count;
    ViInt32     i = 0;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CALC%ld:STAT:RES? %s", channel_Trace, statArr[statisticalParameter]);
    
    if (statisticalParameter == RSZVL_VAL_ALL)
    {
        viCheckErr (viPrintf(instrSession, buffer));
	    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &pbuffer, &local_count));
	    
		p2buf = strtok(pbuffer, ",");

	    while(p2buf && (i < 8))
	    {
	        responseValue_s[i++] = atof(p2buf);
	        p2buf = strtok(NULL, ",");
	    };
    }
    else
    {
        viCheckErr (rszvl_QueryViReal64 (instrSession, buffer, responseValue_s));
    }

    checkErr( rszvl_CheckStatus (instrSession));

Error:
	free(pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Limit Check Result
 * Purpose:  This function returns a 0 or 1, to indicate whether the LIMit 
 *           test has failed or not.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetLimitCheckResult(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32*	limitCheckResult
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CH%ld", channel_Trace);
	
	viCheckParm (rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_LIMIT_CHECK_RESULT, 
                                            limitCheckResult), 3, "Limit Check Result"); 
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Combined Limit Check Result
 * Purpose:  This function queries the result of the limit check of the limit 
 *           line indicated in the selected measurement window. It should be 
 *           noted that a complete sweep must have been performed for obtaining 
 *           a valid result. A synchronization with  *OPC, *OPC? or *WAI should 
 *           therefore be provided.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCombinedLimitCheckResult(
	ViSession	instrSession,
	ViInt32*	state
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_GetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NWA_CLIMIT_CHECK_RESULT, 
                                            state), 2, "State"); 
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Marker Response
 * Purpose:  This function returns the response (in Cartesian diagrams: 
 *           y-axis) value of marker no. <Mk>. The marker must be created 
 *           before using CALCulate<Ch/Tr>:MARKer<Mk>[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetMarkerResponse(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViReal64 _VI_FAR markerResponse[]
)
{
    ViStatus    error   = VI_SUCCESS;
    //ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");

	viCheckErr (viPrintf (instrSession, "CALC%ld:MARK%ld:Y?", channel_Trace, marker));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL));
    
	p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf)
    {
        markerResponse[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
    };

	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Reference Marker Response
 * Purpose:  This function returns the response (in Cartesian diagrams: 
 *           y-axis) value of the reference marker. The reference marker must 
 *           be created before using  
 *           CALCulate<Ch/Tr>:MARKer<Mk>:REFerence[:STATe] ON.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetReferenceMarkerResponse(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViReal64*	referenceMarkerResponse
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CH%ld", channel_Trace);
    
	viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_REFER_MARKER_RESPONSE, 
                referenceMarkerResponse), 3, "Reference Marker Response");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Marker
 * Purpose:  This function returns the horizontal position and the marker 
 *           amplitude level of the selected marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryMarker(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	marker,
	ViReal64*	markerPosition,
	ViReal64*	markerAmplitude
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViInt32Range (instrSession, marker, 1, 10) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");
	
	sprintf (buffer, "CH%ld,MAR%ld", channel_Trace, marker);
    
	viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_POSITION, 
                                             markerPosition), 4, "Marker Position");
	
	viCheckParm (rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_AMPLITUDE, 
                                             markerAmplitude), 5, "Marker Amplitude");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Marker Search Results
 * Purpose:  This function returns the result (stimulus and response value) of 
 *           a search started by means of 
 *           CALCulate<Ch/Tr>:MARKer<Mk>:FUNCtion:EXECute. The search must be 
 *           executed before the command is enabled.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetMarkerSearchResults(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViReal64*	markerStimulus,
	ViReal64 _VI_FAR markerResponse[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;
	ViReal64	result[3];

    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	result[0] = 0.0;
	result[1] = 0.0;
	result[2] = 0.0;

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	viCheckErr (viPrintf (instrSession, "CALC%ld:MARK:FUNC:RES?", channel_Trace));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL));
    
	p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (i < 3))
    {
        result[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
    };

	if (markerStimulus)
        *markerStimulus = result[0];

    if (markerResponse)
        {
        markerResponse[0] = result[1];
        markerResponse[1] = result[2];
        }

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Marker Bandfilter Results
 * Purpose:  This function returns the results of a bandpass or a bandstop 
 *           filter search. Result is only available after a bandfilter search 
 *           has been executed.
 *           
 *           Note(s):
 *           
 *           Units of the result values depends on the sweep type and the 
 *           marker format; see CALCulate<Ch/Tr>:MARKer<Mk>:FORMat.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetMarkerBandfilterResults(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViReal64*	bandwidth,
	ViReal64*	centerStimulus,
	ViReal64*	q,
	ViReal64*	loss,
	ViReal64*	lbe,
	ViReal64*	ube
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	ViInt32 	bandfilter=0;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	
	sprintf (buffer, "CALC%ld:MARK:BWID?", channel_Trace);
    
	
	rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_MARKER_BFILTER_SEARCH_MODE, 
                               &bandfilter);
	if(bandfilter==RSZVL_VAL_MARK_BFIL_SEARCH_BST)
	{
		viCheckErr (viQueryf (instrSession, "%s\n", "%le,%le,%le,%le,%le", buffer, 
                                                               bandwidth, 
                                                               centerStimulus,
                                                               loss,
                                                               lbe,
                                                               ube));
	}
	else
	{
		viCheckErr (viQueryf (instrSession, "%s\n", "%le,%le,%le,%le,%le,%le", buffer, 
                                                               bandwidth, 
                                                               centerStimulus,
                                                               q,
                                                               loss,
                                                               lbe,
                                                               ube));
	}
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Initiate
 * Purpose:  This function initiates an acquisition. After calling this 
 *           function, the spectrum analyzer leaves the idle state and waits 
 *           for a trigger.
 *           
 *           This function does not check the instrument status. Typically, 
 *           the end-user calls this function only in a sequence of calls to 
 *           other low-level driver functions. The sequence performs one 
 *           operation. The end-user uses the low-level functions to optimize 
 *           one or more aspects of interaction with the instrument.
 *           
 *           Notes:
 *           
 *           (1) Call Acquisition Status to determine when the acquisition is 
 *           complete.
 *           
 *           (2) Call the Error Query function at the conclusion of the 
 *           sequence to check the instrument status.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_Initiate(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViUInt32	timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout = -1;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");
	if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Timeout"); 
    
	sprintf (buffer, "CH%ld", channel_Trace);
    
    checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));

    viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_INIT, 
                VI_NULL));
    
	viCheckErr( viPrintf (instrSession, "*OPC\n"));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
	if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Send Software Trigger
 * Purpose:  This function sends a command to trigger the SpecAn.  Call this 
 *           function if the trigger source is set to software trigger.  
 *           
 *           Notes:
 *           
 *           (1) If both the trigger source are not set to software trigger, 
 *           this function returns an error.
 *           
 *           (2) This function does not check the instrument status.   
 *           Typically, you call this function only in a sequence of calls to 
 *           other low-level driver functions.  The sequence performs one 
 *           operation.  You use the low-level functions to optimize one or 
 *           more aspects of interaction with the instrument.  If you want to 
 *           check the instrument status, call the rszvl_error_query function 
 *           at the conclusion of the sequence.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SendSoftwareTrigger(
	ViSession	instrSession,
	ViUInt32	timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout"); 
    
	checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));

    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_INIT_SW_TRIGGER, 
                VI_NULL));
    
	checkErr( rszvl_CheckStatus (instrSession));

Error:
	if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure S-Parameters
 * Purpose:  This function select one of the four elements of the standard 
 *           2-port scattering matrix (S-parameters) or select S-parameters for 
 *           multiport measurements.
 *           
 *           Assigns a measurement parameter to an existing trace or creates a 
 *           trace and assigns a channel number, a name and a measurement 
 *           parameter to it. The trace is not displayed.
 *           
 *           The S-parameters are the basic measured quantities of a network 
 *           analyzer. They describe how the DUT modifies a signal that is 
 *           transmitted or reflected in forward or reverse direction. 
 *           S-parameters are expressed as S<out><in>, where <out> and <in> 
 *           denote the output and input port numbers of the DUT.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSParameters(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName,
	ViInt32	outPort,
	ViInt32	inPort
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [RSZVL_IO_BUFFER_SIZE];
    ViInt32     offset;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	 
    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	viCheckErr (rszvl_traceCommandSelect (instrSession, channel, 
                                        traceName, buffer,
                                        &offset));

    sprintf (buffer + offset, "'%s','S%ld%ld'", traceName, outPort, inPort);
    
    viCheckErr ( viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Impedances
 * Purpose:  This function select the 2-port (or more ports) matched-circuit, 
 *           converted impedances. The parameters describe the impedances of a 
 *           2-port (or more ports) DUT, obtained in forward and reverse 
 *           transmission and reflection measurements:
 *           
 *           Z11 is the input impedance of a 2-port DUT that is terminated at 
 *           its output with the reference impedance Z0 (matched-circuit 
 *           impedance measured in a forward reflection measurement).  
 *           
 *           Z22 is the output impedance of a 2-port DUT that is terminated at 
 *           its input with the reference impedance Z0 (matched-circuit 
 *           impedance measured in a reverse reflection measurement).  
 *           
 *           Z12 and Z21 denote the forward and reverse transfer impedances, 
 *           respectively. 
 *           
 *           The analyzer can also provide matched-circuit impedances for more 
 *           drive ports or balanced port configurations.
 *           
 *           Assigns a measurement parameter to an existing trace or creates a 
 *           trace and assigns a channel number, a name and a measurement 
 *           parameter to it. The trace is not displayed.
 *           
 *           The notation for converted impedance parameters and the 
 *           functionality is analogous to the definition of S-parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureImpedances(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName,
	ViInt32	outPort,
	ViInt32	inPort
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [RSZVL_IO_BUFFER_SIZE];
    ViInt32     offset;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	 
    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	viCheckErr (rszvl_traceCommandSelect (instrSession, channel, 
                                        traceName, buffer,
                                        &offset));

    sprintf (buffer + offset, "'%s','Z-S%ld%ld'", traceName, outPort, inPort);
    
    viCheckErr ( viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Admitances
 * Purpose:  This function select the 2-port (or more ports) converted 
 *           matched-circuit admittance parameters. The parameters describe the 
 *           admittances of a 2-port (or more ports) DUT, obtained in forward 
 *           and reverse transmission and reflection measurements:
 *           
 *           Y11 is the input admittance of a 2-port DUT that is terminated at 
 *           its output with the reference impedance Z0 (matched-circuit 
 *           admittance measured in a forward reflection measurement).
 *           
 *           Y22 is the output admittance of a 2-port DUT that is terminated 
 *           at its input with the reference impedance Z0 (matched-circuit 
 *           admittance measured in a reverse reflection measurement).
 *           
 *           Y12 and Y21 denote the forward and reverse transfer admittances, 
 *           respectively.
 *           
 *           The analyzer can also provide matched-circuit admittances for 
 *           more drive ports or balanced port configurations.
 *           
 *           Assigns a measurement parameter to an existing trace or creates a 
 *           trace and assigns a channel number, a name and a measurement 
 *           parameter to it. The trace is not displayed.
 *           
 *           The notation for converted admittance parameters and the 
 *           functionality is analogous to the definition of S-parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureAdmitances(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName,
	ViInt32	outPort,
	ViInt32	inPort
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [RSZVL_IO_BUFFER_SIZE];
    ViInt32     offset;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	 
    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	
	viCheckErr (rszvl_traceCommandSelect (instrSession, channel, 
                                        traceName, buffer,
                                        &offset));

    sprintf (buffer + offset, "'%s','Y-S%ld%ld'", traceName, outPort, inPort);
    
    viCheckErr ( viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Stability Factors
 * Purpose:  This function select one of the three two port stability factors 
 *           K, u1 or u2.
 *           
 *           Stability factors are calculated as functions of the frequency or 
 *           another stimulus parameter. They provide criteria for linear 
 *           stability of two-ports such as amplifiers. A linear circuit is 
 *           said to be unconditionally stable if no combination of passive 
 *           source or load can cause the circuit to oscillate.
 *           
 *           Assigns a measurement parameter to an existing trace or creates a 
 *           trace and assigns a channel number, a name and a measurement 
 *           parameter to it. The trace is not displayed.
 *           
 *           Stability factors can be calculated for unbalanced ports only. If 
 *           a balanced port configuration is selected, the analyzer generates 
 *           an error message.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureStabilityFactors(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	traceName,
	ViInt32	dutOut,
	ViInt32	dutIn,
	ViInt32	stabilityFactor
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [RSZVL_IO_BUFFER_SIZE];
    ViInt32     offset;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	 
    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (rszvl_invalidViInt32Range (instrSession, stabilityFactor, RSZVL_VAL_SFACTOR_K, RSZVL_VAL_SFACTOR_U2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Stability Factor");
	
	viCheckErr (rszvl_traceCommandSelect (instrSession, channel, 
                                        traceName, buffer,
                                        &offset));

    if (stabilityFactor == RSZVL_VAL_SFACTOR_K)
    {
        sprintf (buffer + offset, "'%s','KFAC%ld%ld'", traceName,
                                                       dutOut,
                                                       dutIn);
    }
    else
    {
        sprintf (buffer + offset, "'%s','MUF%ld%ld%ld'", traceName,
                                                         stabilityFactor,
                                                         dutOut,
                                                         dutIn);
    }
    
    viCheckErr ( viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Parameters Conversion
 * Purpose:  This function converts S-parameters into matched-circuit 
 *           (converted) Y-parameters or Z-parameters and vice versa.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ParametersConversion(
	ViSession	instrSession,
	ViInt32	channel_Trace,
	ViInt32	conversion
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel_Trace <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Trace");

    sprintf (buffer, "CH%ld", channel_Trace);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_PARAMETER_CONVERSION, 
                                            conversion), 3, "Conversion");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault State
 * Purpose:  This function enables or disables the distance-to-fault 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultState(
	ViSession	instrSession,
	ViInt32	channel,
	ViBoolean	state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_STATE, 
                                              state), 3, "State");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault Start Stop
 * Purpose:  This function defines the start and stop distance of the 
 *           distance-to-fault diagram.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultStartStop(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64	startDistance,
	ViReal64	stopDistance
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_START, 
                                             startDistance), 3, "Start Distance");

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_STOP, 
                                             stopDistance), 4, "Stop Distance");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault Center Span
 * Purpose:  This function defines the center distance and span of the 
 *           distance-to-fault diagram.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultCenterSpan(
	ViSession	instrSession,
	ViInt32		channel,
	ViReal64	centerDistance,
	ViReal64	span
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_CENTER, 
                                             centerDistance), 3, "Center Distance");

	viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_SPAN, 
                                             span), 4, "Span");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault Auto Number Of Points
 * Purpose:  This function starts the automatic calculation of the number of 
 *           points.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultAutoNumberOfPoints(
	ViSession	instrSession,
	ViInt32	channel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_POINTS, 
                                            VI_NULL));

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault Add Cable
 * Purpose:  This function adds a new, user-defined cable.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultAddCable(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	cableName,
	ViReal64	permittivity,
	ViReal64	attenuation,
	ViReal64	frequency
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    viCheckErr (viPrintf (instrSession, "CALC%ld:TRAN:DTF:DEF '%s',%Lf,%Lf,%LfHz", channel,
                                                                cableName,
                                                                permittivity,
                                                                attenuation,
                                                                frequency));

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault Select Cable
 * Purpose:  This function selects a standard cable or a previously defined 
 *           cable identified by its name.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultSelectCable(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	cableName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_SELECT_CABLE, 
                                             cableName), 3, "Cable Name");

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Distance To Fault Delete Cable
 * Purpose:  This function deletes a previously defined cable identified by 
 *           its name.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDistanceToFaultDeleteCable(
	ViSession	instrSession,
	ViInt32	channel,
	ViString	cableName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_DFAULT_DELETE_CABLE, 
                                             cableName), 3, "Cable Name");

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Distance To Fault Cable
 * Purpose:  This function copies all pre-defined and user-defined cable 
 *           description files (*.rsc) to a specified directory. The contents 
 *           of the source directories for cable description files
 *           (C:\R_S\Instr\resources\Nwa\CableTypes for predefined,
 *           C:\R_S\Instr\user\Nwa\CableTypes for user-defined cables) are
 *           not affected.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveDistanceToFaultCable (ViSession instrSession,
                                                  ViString targetDirectory)
{
	ViStatus	error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_DFAULT_SAVE_CABLE, 
                                             targetDirectory), 2, "Target Directory");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Distance To Fault Cable
 * Purpose:  This function loads all cable description files (*.rsc) from a 
 *           specified directory. The default source directories for cable
 *           description files (C:\R_S\Instr\resources\Nwa\CableTypes for
 *           predefined, C:\R_S\Instr\user\Nwa\CableTypes for user-defined
 *           cables) are not affected.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_LoadDistanceToFaultCable (ViSession instrSession,
                                                  ViString sourceDirectory)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_DFAULT_LOAD_CABLE, 
                                             sourceDirectory), 2, "Source Directory");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Fault Limit Check
 * Purpose:  This function enables or disables the fault limit check.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFaultLimitCheck (ViSession instrSession,
                                                  ViInt32 channel,
                                                  ViBoolean limitCheck)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm (rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_FAULT_LIMIT_CHECK, 
                                             limitCheck), 3, "Limit Check");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Fault Threshold
 * Purpose:  This function specify a fault limit.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureFaultThreshold (ViSession instrSession,
                                                 ViInt32 channel,
                                                 ViReal64 threshold)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm (rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_NWA_FAULT_THRESHOLD, 
                                             threshold), 3, "Threshold");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Fault Count
 * Purpose:  This function returns the number of peaks on the active distance 
 *           trace which are above the fault limit.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryFaultCount (ViSession instrSession,
                                         ViInt32 channel, ViInt32 *peakCount)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");

    sprintf (buffer, "CH%ld", channel);

	viCheckParm (rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_FAULT_PEAK_COUNT, 
                                             peakCount), 3, "Peak Count");
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Fault Data
 * Purpose:  This function returns the distance and peak response values of 
 *           the detected fault no. <nr>. If the number of detected
 *           faults (rszvl_QueryFaultCount) is smaller than <nr>, an error
 *           message is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryFaultData (ViSession instrSession,
                                        ViInt32 channel, ViInt32 faultNumber,
										ViReal64 *distance, ViReal64 *peakResponseValue)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (channel <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
	if (!distance)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Distance");    
    if (!peakResponseValue)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Peak Response Value");         
   
    viCheckErr( viQueryf(instrSession, "CALC%ld:TRAN:DTF:PEAK:DATA%ld?\n","%Le,%Le", 
                                        channel, faultNumber, distance, peakResponseValue));

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Display
 * Purpose:  This function selects the color scheme for all diagram areas in 
 *           the active setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDisplay(
	ViSession	instrSession,
	ViBoolean	frequencyInfo,
	ViBoolean	markerColor,
	ViBoolean	traceColor
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_FREQ_INFO_STATE, 
                                             frequencyInfo), 2, "Frequency Info");

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_MARKER_COLOR_STATE, 
                                             markerColor), 3, "Marker Color");

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_TRACE_COLOR_STATE, 
                                             traceColor), 4, "Trace Color");

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure RGB Color
 * Purpose:  This function defines the color of all display elements based on 
 *           the Red/Green/Blue color model.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRGBColor(
	ViSession	instrSession,
	ViInt32	element,
	ViReal64	red,
	ViReal64	green,
	ViReal64	blue,
	ViInt32	traceStyle,
	ViInt32	traceWidth
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rszvl_invalidViInt32Range (instrSession,  element, RSZVL_VAL_ELEMENT_BACKGROUND, RSZVL_VAL_ELEMENT_TRACE16))
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Element");
	if (rszvl_invalidViReal64Range (instrSession,  red, 0.0, 1.0))
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "red");
	if (rszvl_invalidViReal64Range (instrSession,  green, 0.0, 1.0))
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "green");
	if (rszvl_invalidViReal64Range (instrSession,  blue, 0.0, 1.0))
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "blue");
	if (rszvl_invalidViInt32Range (instrSession,  traceStyle, RSZVL_VAL_TRACE_STYLE_SOLID, RSZVL_VAL_TRACE_STYLE_DDDOTTED))
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Trace Style");
	if (rszvl_invalidViReal64Range (instrSession,  traceWidth, 1, 20))
		viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Trace Width");
		
	if (element >= RSZVL_VAL_ELEMENT_TRACE1)
    	sprintf (buffer, "DISP:CMAP%ld:RGB %Lf,%Lf,%Lf,%s,%ld", 
		              element, red, green, blue, traceStyleArr[traceStyle], traceWidth);
    else
		sprintf (buffer, "DISP:CMAP%ld:RGB %Lf,%Lf,%Lf", element, red, green, blue);
	
    viCheckErr (viPrintf (instrSession, buffer));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Display
 * Purpose:  This function selects the color scheme for all diagram areas in 
 *           the active setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureColorScheme (ViSession instrSession,
                                              ViInt32 colorScheme)
{
	ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NWA_COLOR_SCHEME, 
                                             colorScheme), 2, "Color Scheme");

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Color Scheme File Manager
 * Purpose:  This function stores a color scheme to a specified NWA color 
 *           scheme file.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ColorSchemeFileManager(
	ViSession	instrSession,
	ViInt32	operation,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (operation)
    {
        case  RSZVL_VAL_CMAP_SAVE:
            attribute = RSZVL_ATTR_NWA_SAVE_COLOR_SCHEME;
            break;
            
        case RSZVL_VAL_CMAP_LOAD:
            attribute = RSZVL_ATTR_NWA_LOAD_COLOR_SCHEME;
            break;
        
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }
    
    viCheckParm( rszvl_SetAttributeViString (instrSession, "", 
                                                attribute, fileName),
                                                3, "File Name");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Display Update
 * Purpose:  This function switches the display on or off while the analyzer 
 *           is in the remote state. This function has no effect while the 
 *           analyzer is in the Local operating state.
 *           
 *           Note(s):
 *           
 *           Switching off the display speeds up the measurement. This 
 *           function may have an impact on the update of trace and channel 
 *           settings; see SYSTem:SETTings:UPDate.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureDisplayUpdate(
	ViSession	instrSession,
	ViBoolean	displayUpdate,
	ViBoolean	errorDisplay
)
{
    ViStatus    error   = VI_SUCCESS;
    
	checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_DISPLAY_UPDATE, 
                                             displayUpdate), 2, "Display Update");

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_ERROR_DISPLAY_STATE, 
                                             errorDisplay), 3, "Error Display");
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Remote Display
 * Purpose:  This function
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureRemoteDisplay(
	ViSession	instrSession,
	ViString	title
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_REMOTE_DISPLAY_TITLE,
                                                title), 2, "Title");
    
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Diagram Area Add
 * Purpose:  This function creates a new diagram area, identified by its area 
 *           number <Wnd>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DiagramAreaAdd(
	ViSession	instrSession,
	ViInt32	diagramArea
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_TRUE));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Diagram Area Delete
 * Purpose:  This function deletes a diagram area, identified by its area 
 *           number <Wnd>.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DiagramAreaDelete(
	ViSession	instrSession,
	ViInt32	diagramArea
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    sprintf (buffer, "DA%ld", diagramArea);

	viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_STATE, 
                                             VI_FALSE));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Diagram Area Maximize
 * Purpose:  This function maximizes all diagram areas in the active setup or 
 *           restores the previous display configuration.
 *           
 *           Note(s):
 *           
 *           Number of the diagram area to become the active diagram area. 
 *           DISPlay:WINDow<Wnd>:MAXimize acts on all diagrams of the current 
 *           setup, however, the digram no. <Wnd> is displayed on top of the 
 *           others.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DiagramAreaMaximize(
	ViSession	instrSession,
	ViInt32	diagramArea,
	ViInt32	diagramAreaOperation
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    sprintf (buffer, "DA%ld", diagramArea);

	viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_MAXIMIZE, 
                                             diagramAreaOperation), 3, "Diagram Area Operation");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Diagram Area Title
 * Purpose:  This function defines a title for diagram area <Wnd>.
 *           
 *           Title provides an input field for the title string. The title may 
 *           comprise a practically unlimited number of characters and is 
 *           centered in a line below the top of the diagram area.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DiagramAreaTitle(
	ViSession	instrSession,
	ViInt32		diagramArea,
	ViBoolean	title,
	ViString	titleString
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (diagramArea <= 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Diagram Area");

    sprintf (buffer, "DA%ld", diagramArea);

	viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_TITLE_STATE, 
                                             title), 3, "Title");

	viCheckParm( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_NWA_DIAGRAM_AREA_TITLE, 
                                             titleString), 4, "Title String");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure New Setup
 * Purpose:  This function creates a new setup <setup_name> using default 
 *           settings for the traces, channels and diagram areas. The created 
 *           setup becomes the active setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureNewSetup(
	ViSession	instrSession,
	ViString	setupName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_CREATE_NEW_SETUP, 
                                             setupName), 2, "Setup Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Select Setup
 * Purpose:  This function selects a setup as the active setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelectSetup(
	ViSession	instrSession,
	ViString	setupName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_SELECT_SETUP, 
                                             setupName), 2, "Setup Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Close Setup
 * Purpose:  This function closes the specified setup.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CloseSetup(
	ViSession	instrSession,
	ViString	setupName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_CLOSE_SETUP, 
                                             setupName), 2, "Setup Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Setup List
 * Purpose:  This function returns the names of all loaded setups.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSetupList(
	ViSession	instrSession,
	ViInt32		bufferSize,
	ViChar _VI_FAR catalog[]
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr( rszvl_QueryViString (instrSession, "MEM:CAT?", bufferSize, catalog));
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Current Directory
 * Purpose:  This function returns the default directory for mass memory 
 *           storage (current directory).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetCurrentDirectory(
	ViSession	instrSession,
	ViChar _VI_FAR currentDirectory[]
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm( rszvl_GetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_EDIT_PATH, 
                                             RSZVL_IO_BUFFER_SIZE, currentDirectory), 2, "CurrentDirectory");
            
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Save Setup
 * Purpose:  This function stores the configuration data of the current setup 
 *           to a specified setup file.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SaveSetup(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_SAVE_SETUP,
                fileName), 2, "File Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Load Setup
 * Purpose:  This function loads configuration data from a specified setup 
 *           file and sets the analyzer to the corresponding instrument state.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_LoadSetup(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_LOAD_SETUP,
                fileName), 2, "File Name");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Presets
 * Purpose:  This function specifies whether a preset (SYSTem:PRESet; *RST) 
 *           affects the active setup only or all open setups.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigurePresets(
	ViSession	instrSession,
	ViInt32	presetScope
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NWA_SYSTEM_PRESET_SCOPE,
                                            presetScope), 2, "Preset Scope");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure User Defined Preset
 * Purpose:  This function selects a factory preset or a user-defined preset.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureUserDefinedPreset(
	ViSession	instrSession,
	ViBoolean	userDefinedPreset,
	ViString	userDefinedPresetFile
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_USER_PRESET_STATE,
                                            userDefinedPreset), 2, "User Defined Preset");

    viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_USER_PRESET_NAME,
                                            userDefinedPresetFile), 3, "User Defined Preset File");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Immediate Settings Update
 * Purpose:  This function initiates an immediate update of the channel or 
 *           trace settings.
 *           
 *           This function has an effect if the analyzer operates in single 
 *           sweep mode (INITiate<Ch>:CONTinuous OFF) and if the display update 
 *           is switched off (SYSTem:DISPlay:UPDate OFF). In this scenario, a 
 *           change of the channel or trace settings is usually not taken into 
 *           account immediately. The analyzer waits until the end of the 
 *           current sweep sequence and changes all settings made during the 
 *           last sweep period when the next single sweep sequence is 
 *           initiated. Several settings can be made en bloc, which generally 
 *           saves time.
 *           
 *           SYSTem:SETtings:UPDate ONCE causes the analyzer to apply the 
 *           settings at once without waiting for the end of the current single 
 *           sweep sequence. This function has no effect in continuous sweep 
 *           mode or if the display update is switched on.
 *           
 *           Note(s):
 *           
 *           The settings are also updated when the continuous sweep mode is 
 *           activated (INITiate<Ch>:CONTinuous ON).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ImmediateSettingsUpdate(
	ViSession	instrSession
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_NWA_SYSTEM_SETTINGS_UPDATE,
                                            VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Sound
 * Purpose:  This function switches alarm sounds on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSound(
	ViSession	instrSession,
	ViBoolean	alarmSounds,
	ViBoolean	statusSounds
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_ALARM_SOUND_STATE,
                                              alarmSounds), 2, "Alarm Sounds");

	viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_NWA_STATUS_SOUND_STATE,
                                              statusSounds), 3, "Status Sounds");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Reference Oscillator
 * Purpose:  This function selects the source of the reference oscillator 
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureReferenceOscillator(
	ViSession	instrSession,
	ViInt32	reference
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_ROSC_SOURCE,
                                            reference), 2, "Reference");

	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Status Register
 * Purpose:  This function sets the Enable, NTransition, and PTransition bits  
 *              of status questionable and operating registers.
 *           
 *           Note:
 *           
 *           (1) For detailed description of Status Reporting System see also 
 *           operating manual.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_setStatusRegister(
	ViSession	instrSession,
	ViInt32	registerOperation,
	ViInt32	questionableRegister,
	ViInt32	enable,
	ViInt32	ptransition,
	ViInt32	ntransition
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));     
    
	switch (registerOperation) {
        case 0:
            viCheckErr (viPrintf (instrSession, ":STAT:PRES\n"));
        break;
        case 1:
            if (rszvl_invalidViInt32Range (instrSession, enable, 0, 65535))
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Enable");
            if (rszvl_invalidViInt32Range (instrSession, ptransition, 0, 65535))
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "PTransition");
            if (rszvl_invalidViInt32Range (instrSession, ntransition, 0, 65535))
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "NTransition");

            viCheckErr (viPrintf (instrSession, ":STAT:QUES%s:ENAB %ld;PTR %ld"
                ";NTR %ld\n", selStatusRegArr[questionableRegister], enable, ptransition,
                ntransition));
        break;
		case 2:
            if (rszvl_invalidViInt32Range (instrSession, enable, 0, 65535))
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Enable");
            if (rszvl_invalidViInt32Range (instrSession, ptransition, 0, 65535))
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "PTransition");
            if (rszvl_invalidViInt32Range (instrSession, ntransition, 0, 65535))
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "NTransition");

            viCheckErr (viPrintf (instrSession, ":STAT:OPER:ENAB %ld;PTR %ld;NTR %ld\n",
                     enable, ptransition, ntransition));
        break;
        default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    }
	
	checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Status Register
 * Purpose:  This function contains the commands for the querying status 
 *           reporting system.
 *           
 *           Note:
 *           
 *           For detailed description of Status Reporting System see also 
 *           operating manual.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_getStatusRegister(
	ViSession	instrSession,
	ViInt32	statusRegistersQuery,
	ViInt32*	registerValue
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rszvl_invalidViInt32Range (instrSession, statusRegistersQuery, 0, 21) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status Register");
    
    
    viCheckErr (viQueryf (instrSession, ":STAT:%s?\n", "%ld", statusRegArr[statusRegistersQuery],
                                                              registerValue));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure Length Unit
 * Purpose:  This function defines the default unit of length. The command 
 *           affects all SCPI commands defining a parameter with a physical 
 *           unit that is related to a length, in particular:
 *
 *           - Length offset parameters ([SENSe:]CORRection:EDELay...)
 *           - Parameters of the distance-to fault measurement
 *           (CALCulate<Chn>:TRANSform:DTFault...)
 *           - Derived units, e.g. the attenuation per unit of length defined
 *           via CALCulate<Ch>:TRANsform:DTFault:DEFine
 *           - Stimulus values for limit lines and ripple limit lines for the
 *           distance-to fault trace (e.g. CALCulate<Chn>:LIMit:DATA)
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureLengthUnit (ViSession instrSession,
                                             ViInt32 lengthUnit)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_NWA_UNIT_OF_LENGTH, 
                lengthUnit), 2, "Length Unit");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Start Stop
 * Purpose:  This function configures the frequency range of the spectrum 
 *           analyzer using start frequency and stop frequency. If start 
 *           frequency is equal to the stop frequency, then the spectrum 
 *           analyzer is in time-domain mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencyStartStop(
    ViSession   instrSession,
    ViReal64    startFrequency,
    ViReal64    stopFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_START, 
                startFrequency), 2, "Start Frequency");

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_STOP, 
                stopFrequency), 3, "Stop Frequency");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Center
 * Purpose:  This function configures the center frequency 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencyCenter(
    ViSession   instrSession,
    ViReal64    centerFrequency
)
{
    ViStatus    error   = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER, 
                centerFrequency), 2, "Center Frequency");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Center Span
 * Purpose:  This function configures the frequency range of the spectrum 
 *           analyzer using the center frequency and the frequency span.  If 
 *           span corresponds to zero hertz, then the spectrum analyzer is in 
 *           time-domain.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencyCenterSpan(
    ViSession   instrSession,
    ViReal64    centerFrequency,
    ViReal64    span
)
{
    ViStatus    error   = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER, 
                centerFrequency), 2, "Center Frequency");

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_FREQUENCY_SPAN, 
                span), 3, "Span");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Step Size
 * Purpose:  This function sets the step width of the center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencyStepSize(
    ViSession   instrSession,
    ViReal64    stepSize
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER_LINK,
                                              RSZVL_VAL_CENT_FREQ_LINK_OFF));
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
            RSZVL_ATTR_FREQUENCY_CENTER_STEP, stepSize), 2, "Step Size");

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Coupling Factor
 * Purpose:  This function couples the step width of the center frequency to 
 *           span (span >0) or to the resolution bandwidth (span = 0) or 
 *           cancels the couplings and sets the coupling factor.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencyCouplingFactor(
    ViSession   instrSession,
    ViInt32 coupling,
    ViInt32 couplingFactor
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER_LINK,
                                              coupling), 2, "Coupling");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_FREQUENCY_CENTER_LINK_FACTOR, 
                                            couplingFactor), 3, "Coupling Factor");

    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Span Full
 * Purpose:  This function sets the frequency span in the analyzer mode to its 
 *           maximum.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencySpanFull(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FREQUENCY_SPAN_FULL,
                                               VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Frequency Offset
 * Purpose:  This function configures the frequency offset attribute of the 
 *           spectrum analyzer. This function affects the setting of the 
 *           spectrum analyzer's absolute frequencies, such as start, stop, 
 *           center, and marker. It does not affect values, which are the 
 *           difference of frequencies, such as span and delta marker. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFrequencyOffset(
    ViSession   instrSession,
    ViReal64    frequencyOffset
)
{
    ViStatus    error   = VI_SUCCESS;
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_FREQUENCY_OFFSET, frequencyOffset),
                                                2, "Frequency Offset");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Sweep Coupling Auto
 * Purpose:  This function activates auto coupling of selected coupling 
 *           method.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSweepCouplingAuto(
    ViSession   instrSession,
    ViInt32 sweepCoupling
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (sweepCoupling)
    {
        case  RSZVL_VAL_COUPLING_RBW:
            attribute = RSZVL_ATTR_RESOLUTION_BANDWIDTH_AUTO;
            break;
            
        case RSZVL_VAL_COUPLING_VBW:
            attribute = RSZVL_ATTR_VIDEO_BANDWIDTH_AUTO;
            break;
            
        case RSZVL_VAL_COUPLING_SWEEP_TIME:
            attribute = RSZVL_ATTR_SWEEP_TIME_AUTO;
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Sweep Coupling");
    }
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                attribute, VI_TRUE),
                                                1, "Dummy");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Sweep Coupling
 * Purpose:  This function configures the coupling values of the spectrum 
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSweepCoupling(
    ViSession   instrSession,
    ViInt32 sweepCoupling,
    ViReal64    couplingValue
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (sweepCoupling)
    {
        case RSZVL_VAL_COUPLING_RBW:
            attribute = RSZVL_ATTR_RESOLUTION_BANDWIDTH;
            break;
            
        case RSZVL_VAL_COUPLING_VBW:
            attribute = RSZVL_ATTR_VIDEO_BANDWIDTH;
            break;
#if 0            
        case RSZVL_VAL_COUPLING_SWEEP_TIME:
            attribute = RSZVL_ATTR_SWEEP_TIME;
            break;
#endif
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Sweep Coupling");
    }
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                            attribute, couplingValue),
                                            3, "Coupling Value");                                  
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Sweep Coupling Advanced
 * Purpose:  This function configures advanced parameters of coupling and 
 *           sweeping.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSweepCouplingAdvanced(
    ViSession   instrSession,
    ViInt32 ratioSelection,
    ViReal64    value
)
{
    ViStatus    error = VI_SUCCESS;
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (ratioSelection)
    {
        case RSZVL_VAL_COUPLING_RATIO_RBW:
            attribute = RSZVL_ATTR_RESOLUTION_BANDWIDTH_RATIO;
            break;
        case RSZVL_VAL_COUPLING_RATIO_VBW:
            attribute = RSZVL_ATTR_VIDEO_BANDWIDTH_RATIO;
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Ratio Selection");
    }
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                attribute, value),
                                                3, "Value");
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Filter Type
 * Purpose:  This function sets the filter type for the resolution bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANFilterType(
    ViSession   instrSession,
    ViInt32 filterType
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_RESOLUTION_BANDWIDTH_FILTER_TYPE,
                                                filterType), 2, "Filter Type");
                                                    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN VBW Mode
 * Purpose:  This function selects the position of the video filter in the 
 *           signal path, provided that the resolution bandwidth is <=100 kHz.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANVBWMode(
    ViSession   instrSession,
    ViInt32 videoFilterType
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_VIDEO_BANDWIDTH_TYPE,
                                                videoFilterType), 2, "Video Filter Type");
                                                    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Averaging
 * Purpose:  This function switches on or off the average calculation for the 
 *           selected trace in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANAveraging(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    }
    sprintf (buffer, "TR%ld", trace);  
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                RSZVL_ATTR_AVG_STATE,
                                                state), 3, "State");
                                                    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Averaging Type
 * Purpose:  This function selects the type of average function. If Video is 
 *           selected, the logarithmic power is averaged and, if Linear is 
 *           selected, the power values are averaged before they are converted 
 *           to logarithmic values.
 *           The type of average calculation is equally set for all traces in 
 *           one measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANAveragingType(
    ViSession   instrSession,
    ViInt32 averagingType
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_AVG_TYPE,
                                                averagingType), 2, "Averaging Type");
                                                    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Acquisition
 * Purpose:  This function configures the acquisition attributes of the 
 *           spectrum analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANAcquisition(
    ViSession   instrSession,
    ViBoolean   sweepModeContinuous,
    ViInt32 numberOfSweeps
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 3, "Sweep Mode Continuous");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_NUMBER_OF_SWEEPS,
                                                numberOfSweeps), 4, "Number Of Sweeps");                                                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Vertical Scale
 * Purpose:  This function configures the vertical scale of analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANVerticalScale(
    ViSession   instrSession,
    ViInt32 verticalScale
)
{
   ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_VERTICAL_SCALE,
                                                verticalScale), 2, "Vertical Scale");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Sweep Points
 * Purpose:  This function defines the number of measurement points for one 
 *           sweep run. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSweepPoints(
    ViSession   instrSession,
    ViInt32 sweepPoints
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_SWEEP_POINTS,
                                                sweepPoints), 2, "Sweep Points"); 
                                                   
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Attenuation
 * Purpose:  This function configures the input attenuation.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANAttenuation(
    ViSession   instrSession,
    ViBoolean   attenuationAuto,
    ViReal64    attenuation
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_ATTENUATION_AUTO,
                                                attenuationAuto), 2, "Attenuation Auto");                                                 
    if (attenuationAuto == VI_FALSE)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_ATTENUATION,
                                                attenuation), 3, "Attenuation");                                                
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Reference Level
 * Purpose:  This function configures the reference level
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANReferenceLevel(
    ViSession   instrSession,
    ViReal64    referenceLevel
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_REFERENCE_LEVEL,
                                                referenceLevel), 2, "Reference Level"); 
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Level
 * Purpose:  This function configures the vertical attributes of the spectrum 
 *           analyzer.  This corresponds to attributes like amplitude units, 
 *           input attenuation, input impedance, reference level, and reference 
 *           level offset.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANLevel(
    ViSession   instrSession,
    ViInt32 amplitudeUnits,
    ViReal64    inputImpedance,
    ViReal64    referenceLevel,
    ViReal64    referenceLevelOffset,
    ViBoolean   attenuationAuto,
    ViReal64    attenuation
)
{
   ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_AMPLITUDE_UNITS,
                                                amplitudeUnits), 2, "Amplitude Units"); 
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_INPUT_IMPEDANCE,
                                                inputImpedance), 3, "Input Impedance"); 
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_REFERENCE_LEVEL,
                                                referenceLevel), 4, "Reference Level"); 
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_REFERENCE_LEVEL_OFFSET,
                                                referenceLevelOffset), 5, "Reference Level Offset"); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_ATTENUATION_AUTO,
                                                attenuationAuto), 6, "Attenuation Auto");                                                 
    if (attenuationAuto == VI_FALSE)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_ATTENUATION,
                                                attenuation), 7, "Attenuation");                                                
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trace
 * Purpose:  This function configures the trace to acquire.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTrace(
    ViSession   instrSession,
    ViInt32 trace,
    ViInt32 traceType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    }
    sprintf (buffer, "TR%ld", trace); 
    switch (traceType){
        case RSZVL_TRAC_MOD_BLANK:
            viCheckErr ( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                        RSZVL_ATTR_TRACE_STATE,
                                                        VI_FALSE));
        break;
        default:
            viCheckErr ( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                        RSZVL_ATTR_TRACE_STATE,
                                                        VI_TRUE));
            viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_TRACE_TYPE,
                                                traceType), 4, "Trace Type"); 
        break;
    }
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trace Detector
 * Purpose:  This function configures the trace detector.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTraceDetector(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   detectorTypeAuto,
    ViInt32 detectorType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    }
    sprintf (buffer, "TR%ld", trace); 
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                RSZVL_ATTR_DETECTOR_TYPE_AUTO,
                                                detectorTypeAuto), 3, "Detector Type Auto");
    if (detectorTypeAuto == VI_FALSE)                                                
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_DETECTOR_TYPE,
                                                detectorType), 4, "Detector Type"); 
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trace Reset Behavior
 * Purpose:  This function specifies whether or not the traces with peak or 
 *           minimum value detection are reset after specific parameter 
 *           changes.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTraceResetBehavior(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   resetatChange
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    }
    sprintf (buffer, "TR%ld", trace); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                RSZVL_ATTR_TRACE_RESET_BEHAVIOR,
                                                resetatChange), 3, "Reset At Change"); 
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Store Trace to File
 * Purpose:  This function stores the selected trace (1 to 6) in the 
 *           measurement window indicated by window in a file with ASCII 
 *           format.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANStoreTraceToFile(
    ViSession   instrSession,
    ViInt32 trace,
    ViString    fileName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    if (fileName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "File Name");

    viCheckErr (viPrintf (instrSession,"MMEM:STOR:TRAC %ld,'%s'\n", trace, fileName));
        
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Trace IQ Set
 * Purpose:  This function defines the settings of the analyzer hardware for 
 *           the measurement of I/Q data. This allows setting the bandwidth of 
 *           the analog filters in front of the A/D converter as well as 
 *           setting the sample rate, trigger conditions and the record length.
 *           
 *           Notes:
 *           
 *           (1) Function Trace I/Q State (confTraceIQState) has to be 
 *           switched ON before using this function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANTraceIQSet(
    ViSession   instrSession,
    ViReal64    samplingRate,
    ViInt32 triggerMode,
    ViInt32 triggerSlope,
    ViInt32 pretriggerSamples,
    ViInt32 no_ofSamples
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViReal64Range (instrSession, samplingRate, 10.0e3, 100.0e6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Sampling Rate");
    if ((rszvl_invalidViInt32Range (instrSession, triggerMode, 0, 3) == VI_TRUE)||(triggerMode==2))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Trigger Mode");
    if (rszvl_invalidViInt32Range (instrSession, triggerSlope, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Trigger Slope");  
    if (rszvl_invalidViInt32Range (instrSession, pretriggerSamples, -16744447, 65023) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Pretrigger Samples");
    if (rszvl_invalidViInt32Range (instrSession, no_ofSamples, 1, 16776704) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "No Of Samples");

    viCheckErr (viPrintf (instrSession, ":TRAC:IQ:SET NORM,0,%.12lG,%s,%s,%ld,%ld\n",
            samplingRate, rszvl_rngTriggerSource.rangeValues[triggerMode].cmdString,
            rszvl_rngPolarity.rangeValues[triggerSlope].cmdString, pretriggerSamples, no_ofSamples));
            
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trace IQ Sample Rate
 * Purpose:  This function sets the sampling rate for the I/Q data acquisition. 
 *           Thus the sample rate can be modified without affecting the other
 *           settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTraceIQSampleRate (ViSession instrSession,
                                                       ViReal64 samplingRate)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_IQ_SAMPLE_RATE, samplingRate),
                                                2, "Sampling Rate");
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trace IQ Data Acquisition
 * Purpose:  This function configures the trace IQ data acquisition.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTraceIQDataAcquisition(
    ViSession   instrSession,
    ViBoolean   traceState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_IQ_DATA_STATE, traceState),
                                                2, "Trace State");
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trace IQ Data Averaging
 * Purpose:  This function configures the trace IQ data averaging.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTraceIQDataAveraging (ViSession instrSession,
                                                          ViBoolean averagingState,
                                                          ViInt32 averagingCount)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_IQ_DATA_AVER_STATE, averagingState),
                                                2, "Averaging State");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_IQ_DATA_AVER_COUNT, averagingCount),
                                                3, "Averaging Count");
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Copy Trace
 * Purpose:  This function copies one trace array to another trace array.  Any 
 *           data in the destination trace is over written. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANCopyTrace(
    ViSession   instrSession,
    ViInt32 destinationTrace,
    ViInt32 sourceTrace
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, destinationTrace, 1, 6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Destination Trace");
    if (rszvl_invalidViInt32Range (instrSession, sourceTrace, 1, 6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Source Trace");        
    
    viCheckErr(viPrintf(instrSession, "TRAC:COPY TRACE%ld,TRACE%ld", 
    destinationTrace, sourceTrace));
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Subtract Traces
 * Purpose:  This function configures subtraction of traces.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSubtractTraces(
    ViSession   instrSession,
    ViReal64    position,
    ViBoolean   traceMathState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_TRACE_MATH_POSITION, 
                    position), 2, "Position");
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_TRACE_MATH_STATE, 
                traceMathState), 3, "Trace Math State");
                
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SANSubtract Traces
 * Purpose:  This function subtracts the array elements of trace defined in 
 *           argument Trace 2 from TRACE1 of the instrument and stores the 
 *           result in the TRACE1.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANSubtractTraces(
    ViSession   instrSession,
    ViInt32 trace2
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, trace2, 2, 6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace 2");
    
    viCheckErr(viPrintf(instrSession, "CALC:MATH (TRACE1-TRACE%ld)",trace2));
                
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker
 * Purpose:  This function enables the active marker on the specified trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarker(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   markerEnabled,
    ViInt32 trace
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");
    
    sprintf (buffer, "M%ld", marker); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_MARKER_ENABLED, 
                markerEnabled), 3, "Marker Enabled");
    if (markerEnabled == VI_TRUE)                
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_MARKER_TRACE, 
                trace), 4, "Trace");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Signal Track
 * Purpose:  This function turns configures signal-tracking. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSignalTrack(
    ViSession   instrSession,
    ViBoolean   signalTrackEnabled,
    ViReal64    signalTrackBandwidth,
    ViReal64    signalTrackThreshold,
    ViInt32 trace
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_SIGNAL_TRACK_ENABLED, 
                signalTrackEnabled), 2, "Signal Track Enabled");
    if (signalTrackEnabled == VI_TRUE)
    {                
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_SIGNAL_TRACK_BWID, 
                    signalTrackBandwidth), 3, "Signal Track Bandwidth");
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_SIGNAL_TRACK_THRESHOLD, 
                    signalTrackThreshold), 4, "Signal Track Threshold");
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_SIGNAL_TRACK_TRACE, 
                    trace), 5, "Trace");
    }          
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Disable All Markers
 * Purpose:  This function turns off all the markers in selected measurement 
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANDisableAllMarkers(
    ViSession   instrSession
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_MARKER_AOFF, 
                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);   
    return error;
}

/*****************************************************************************
 * Function: SAN Marker Search
 * Purpose:  This function specifies the type of marker search and performs 
 *           the search.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANMarkerSearch(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 markerSearch
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker");        
        
    sprintf (buffer, "M%ld", marker); 
    switch (markerSearch)
    {
            case RSZVL_VAL_MARKER_SEARCH_HIGHEST:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_MARKER_SEARCH_PEAK, 
                                     VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_PEAK:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_MARKER_SEARCH_PEAK_NEXT, 
                                     VI_NULL));
            break;

            case RSZVL_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_MARKER_SEARCH_PEAK_LEFT, 
                                     VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_MARKER_SEARCH_PEAK_RIGHT, 
                                     VI_NULL));
                         break;

            case RSZVL_VAL_MARKER_SEARCH_MINIMUM:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_MARKER_SEARCH_MIN, VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_MINIMUM:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_MARKER_SEARCH_MIN_NEXT, VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_MARKER_SEARCH_MIN_LEFT, VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_MARKER_SEARCH_MIN_RIGHT, VI_NULL));
            break;

            default: 
                viCheckParm ( RS_ERROR_INVALID_PARAMETER, 3, "Parameter to set");
            break;
    }
    checkErr( rszvl_CheckStatus (instrSession));   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Marker Peak Auto
 * Purpose:  This function configures an automatic peak search action
 *           for Marker 1 at the end of each particular sweep. This function 
 *           may be used during adjustments of a device under test to keep 
 *           track of the actual peak marker position and level.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANMarkerPeakAuto(
    ViSession   instrSession,
    ViInt32 markerSearch,
    ViBoolean   autoPeak
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (markerSearch){
        case RSZVL_VAL_MARKER_SEARCH_HIGHEST:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_MARKER_SEARCH_PEAK_AUTO, 
                autoPeak), 3, "Auto Peak");
        break;
        case RSZVL_VAL_MARKER_SEARCH_MINIMUM:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_MARKER_SEARCH_MIN_AUTO, 
                autoPeak), 3, "Auto Peak");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Search"); 
    }
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Move Marker
 * Purpose:  This function moves the active marker to the specified horizontal 
 *           position.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANMoveMarker(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64    markerPosition
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");        
        
    sprintf (buffer, "M%ld", marker); 
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_MARKER_POSITION, 
                markerPosition), 3, "Marker Position");
    
    checkErr( rszvl_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set SAN From Marker
 * Purpose:  This function makes the selected marker frequency to be the 
 *           center frequency or step width of center frequency. It can also 
 *           make the active marker amplitude to be the reference level.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetSANFromMarker(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 instrumentSetting
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");        
        
    sprintf (buffer, "M%ld", marker); 
    switch(instrumentSetting)
    {
        case RSZVL_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer,
                RSZVL_ATTR_MARKER_TO_CENTER, VI_NULL));
        break;
        case RSZVL_VAL_INSTRUMENT_SETTING_FREQUENCY_STEP:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer,
                RSZVL_ATTR_MARKER_TO_STEP, VI_NULL));
        break;
        case RSZVL_VAL_INSTRUMENT_SETTING_REFERENCE_LEVEL:
            viCheckErr( rszvl_SetAttributeViString (instrSession, buffer,
                RSZVL_ATTR_MARKER_TO_REFERENCE, VI_NULL));
        break;
        default: 
            viCheckParm ( RS_ERROR_INVALID_PARAMETER, 3, "Parameter to set");
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Zoom
 * Purpose:  This function defines the range to be zoomed around marker 1 in 
 *           the selected measurement window. Marker 1 is activated first, if 
 *           necessary.
 *           The subsequent frequency sweep is stopped at the marker position 
 *           and the frequency of the signal is counted. This frequency becomes 
 *           the new center frequency, and the zoomed span is set. In order to 
 *           recognize the end of the operation the synchronization to the 
 *           sweep end should be activated. This is only possible in single 
 *           sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerZoom(
    ViSession   instrSession,
    ViReal64    markerZoom,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Timeout"); 
        
    checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_MARKER_ZOOM, 
                markerZoom), 2, "Marker Zoom");
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Marker
 * Purpose:  This function returns the horizontal position and the marker 
 *           amplitude level of the selected marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANMarker(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   markerPosition,
    ViReal64*   markerAmplitude
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");        
        
    sprintf (buffer,"M%ld", marker); 
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_MARKER_POSITION, 
                markerPosition), 3, "Marker Position");
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_MARKER_AMPLITUDE, 
                markerAmplitude), 4, "Marker Amplitude");                
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Search Threshold
 * Purpose:  This function configures marker threshold values. The marker 
 *           threshold specifies a lower bound for ALL marker search functions.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerSearchThreshold(
    ViSession   instrSession,
    ViBoolean   thresholdState,
    ViReal64    markerThreshold
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_MARKER_THRESHOLD_STATE, 
                thresholdState), 2, "Threshold State");

    if (thresholdState == VI_TRUE)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_MARKER_THRESHOLD, 
                markerThreshold), 3, "Marker Threshold");
    }
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Search Peak Excursion
 * Purpose:  This function configures the marker peak excursion The marker 
 *           peak excursion specifies the minimum amplitude variation that can 
 *           be recognized as a peak or minimum by the marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerSearchPeakExcursion(
    ViSession   instrSession,
    ViReal64    peakExcursion
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_PEAK_EXCURSION, 
                peakExcursion), 2, "Peak Excursion");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Search Local Oscillator
 * Purpose:  This function configures state of local oscillator supression.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerSearchLocalOscillator(
    ViSession   instrSession,
    ViBoolean   localOscillatorSupression
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_MARKER_LOEX, 
                localOscillatorSupression), 2, "Local Oscillator Supression");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Search Limits
 * Purpose:  This function configures marker search limits.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerSearchLimits(
    ViSession   instrSession,
    ViBoolean   searchLimits,
    ViReal64    searchLimitLeft,
    ViReal64    searchLimitRight
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_MARKER_SEARCH_LIMITS_STATE, 
                searchLimits), 2, "Serach Limit");
    if (searchLimits == VI_TRUE)
    {
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_MARKER_SEARCH_LIMITS_LEFT, 
                    searchLimitLeft), 3, "Search Limit Left");
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_MARKER_SEARCH_LIMITS_RIGHT, 
                    searchLimitRight), 4, "Search Limit Right");
    }
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Demodulation
 * Purpose:  This function configures marker demodulation parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerDemodulation(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   state,
    ViInt32 demodulationType,
    ViReal64    markerStopTime,
    ViBoolean   continuousDemodulation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");    
    
    sprintf (buffer, "M%ld", marker);    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_DEMOD_STATE, state),
                                                3, "State");   
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_DEMOD_TYPE, demodulationType),
                                                4, "Demodulation Type");                                                 
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_DEMOD_HOLDOFF, markerStopTime),
                                                5, "Marker Stop Time"); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MARKER_DEMOD_CONT, continuousDemodulation),
                                                6, "Continuous Demodulation");                                                 

    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Peak List
 * Purpose:  This function searches the selected trace for the indicated 
 *           number of maxima. The number of maxima found depends on the 
 *           waveform and value set for the Peak Excursion parameter, however, 
 *           a maximum number of 50 maxima are determined. Only the signals 
 *           which exceed their surrounding values at least by the value 
 *           indicated by the peak excursion parameter will be recognized as 
 *           maxima. Therefore, the number of maxima found is not automatically 
 *           the same as the number of maxima desired..
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerPeakList(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 peakListCount,
    ViInt32 peakListSort
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");    
    
    sprintf (buffer, "M%ld", marker);    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_PEAK_LIST_COUNT, peakListCount),
                                                3, "Peak List Count");   
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_PEAK_LIST_SORT, peakListSort),
                                                4, "Peak List Sort");                                                 
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Marker Peak List Found
 * Purpose:  This function reads out the number of maxima found during the 
 *           search. If no search for maxima has been performed, 0 is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANMarkerPeakListFound(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32*    peakListFound
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");    
    
    sprintf (buffer, "M%ld", marker);    
    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_MARKER_PEAK_LIST_FOUND, peakListFound),
                                                3, "Peak List Found");
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Marker Peak List
 * Purpose:  This function reads out the list of X or Y values of the maxima 
 *           found.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANMarkerPeakList(
    ViSession   instrSession,
    ViInt32 marker,
    ViInt32 arraySize,
    ViInt32 peakListSelection,
    ViReal64    peakList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff=VI_NULL;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");  
    switch (peakListSelection){
        case RSZVL_VAL_MARKER_SORT_X:
            viCheckErr( viPrintf (instrSession, "CALC:MARK%ld:FUNC:FPE:X?\n", marker));   
        break;
        case RSZVL_VAL_MARKER_SORT_Y:
            viCheckErr( viPrintf (instrSession, "CALC:MARK%ld:FUNC:FPE:Y?\n", marker));   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Peak List Selection"); 
        break;    
    }    
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL));                                             
    pbuff = strtok (buffer, ",");
    i = 0;
    do{
        peakList[i] = atof (pbuff);
        pbuff = strtok (NULL, ",");
        i++;
    }while((pbuff != NULL) && (i<arraySize));   
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Delta Marker
 * Purpose:  This function enables the active marker on the specified trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDeltaMarker(
    ViSession   instrSession,
    ViInt32 deltaMarker,
    ViBoolean   state,
    ViInt32 trace
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");    
    
    sprintf (buffer, "DM%ld", deltaMarker);    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_STATE, state),
                                                3, "State");   
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_TRACE, trace),
                                                4, "Trace");
                
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Delta Marker Position
 * Purpose:  This function configures the delta marker position.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDeltaMarkerPosition(
    ViSession   instrSession,
    ViInt32 deltaMarker,
    ViInt32 mode,
    ViReal64    position
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");    
    
    sprintf (buffer, "DM%ld", deltaMarker);    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_MODE, mode),
                                                3, "Mode");   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_POSITION, position),
                                                4, "Position");
                
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Disable All Delta Markers
 * Purpose:  This function turns off all the delta markers in selected 
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANDisableAllDeltaMarkers(
    ViSession   instrSession
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_REFERENCE_MARKER_AOFF, 
                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);   
    return error;
}

/*****************************************************************************
 * Function: SAN Delta Marker Search
 * Purpose:  This function specifies the type of delta marker search and 
 *           performs the search.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANDeltaMarkerSearch(
    ViSession   instrSession,
    ViInt32 deltaMarker,
    ViInt32 markerSearch
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");        

    sprintf (buffer, "DM%ld", deltaMarker); 
    switch (markerSearch)
    {
            case RSZVL_VAL_MARKER_SEARCH_HIGHEST:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_REFERENCE_MARKER_PEAK, 
                                     VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_PEAK:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_REFERENCE_MARKER_PEAK_NEXT, 
                                     VI_NULL));
            break;

            case RSZVL_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT, 
                                     VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT, 
                                     VI_NULL));
                         break;

            case RSZVL_VAL_MARKER_SEARCH_MINIMUM:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_REFERENCE_MARKER_MIN, 
                                     VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_MINIMUM:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_REFERENCE_MARKER_MIN_NEXT, VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT, VI_NULL));
            break;
            case RSZVL_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
                viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT, VI_NULL));
            break;

            default: 
                viCheckParm ( RS_ERROR_INVALID_PARAMETER, 3, "Parameter to set");
            break;
    }
    checkErr( rszvl_CheckStatus (instrSession));   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Delta Marker
 * Purpose:  This function returns the horizontal position and the marker 
 *           amplitude level of the selected delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANDeltaMarker(
    ViSession   instrSession,
    ViInt32 deltaMarker,
    ViInt32 mode,
    ViReal64*   position,
    ViReal64*   amplitude
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, deltaMarker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Delta Marker");    
    
    sprintf (buffer, "DM%ld", deltaMarker);  
    switch (mode){
        case RSZVL_VAL_ABS:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_POSITION, position),
                                                4, "Position");   
        break;
        case RSZVL_VAL_REL:
            viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_REL_POSITION, position),
                                                4, "Position");   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Mode"); 
    }        
    
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, 
                                                RSZVL_ATTR_REFERENCE_MARKER_AMPLITUDE, amplitude),
                                                5, "Amplitude");
                
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Reference Fixed
 * Purpose:  This function configures marker reference fixed state.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANReferenceFixed(
    ViSession   instrSession,
    ViBoolean   fixedReference
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", 
                RSZVL_ATTR_MARKER_REF_STATE,  fixedReference), 2, "Fixed Reference");  
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Reference Fixed Point
 * Purpose:  This function configures marker reference fixed position.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANReferenceFixedPoint(
    ViSession   instrSession,
    ViReal64    refPointLevel,
    ViReal64    refPointLevelOffset,
    ViReal64    refPointFrequencyTime
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
            RSZVL_ATTR_MARKER_REF_LEVEL,  refPointLevel), 2, "Reference Point Level");       
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
            RSZVL_ATTR_MARKER_REF_LEVEL_OFFSET,  refPointLevelOffset), 
            3, "Reference Point Level Offset");   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
            RSZVL_ATTR_MARKER_REF_FREQ,  refPointFrequencyTime), 
            4, "Reference Point Frequency");                  

    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Reference Fixed Peak Search
 * Purpose:  This function sets the reference point level for all delta 
 *           markers in the selected measurement window for a measurement with 
 *           fixed reference point (CALC:DELT:FUNC:FIX:STAT ON) to the peak of 
 *           the selected trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANReferenceFixedPeakSearch(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", RSZVL_ATTR_MARKER_REF_PEAK, 
                VI_NULL));
   
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set SAN Time And Date
 * Purpose:  This function configures the instrument's internal clock and  
 *           calendar.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SetSANTimeAndDate(
    ViSession   instrSession,
    ViInt32 timeDate,
    ViInt32 hour,
    ViInt32 minute,
    ViInt32 second,
    ViInt32 day,
    ViInt32 month,
    ViInt32 year
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    switch (timeDate)
    {
        case RSZVL_VAL_DATE:
            if (rszvl_invalidViInt32Range (instrSession, day, 1, 31) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Day");
            if (rszvl_invalidViInt32Range (instrSession, month, 1, 12) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Month");
            if (rszvl_invalidViInt32Range (instrSession, year, 1980, 2099) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Year"); 
            viCheckErr (viPrintf(instrSession, ":SYST:DATE %ld,%ld,%ld\n", year, month, day));    
        break;
        case RSZVL_VAL_TIME:
            if (rszvl_invalidViInt32Range (instrSession, hour, 0, 23) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Hour");
            if (rszvl_invalidViInt32Range (instrSession, minute, 0, 59) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Minute");
            if (rszvl_invalidViInt32Range (instrSession, second, 0, 59) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Second"); 
            viCheckErr (viPrintf(instrSession, ":SYST:TIME %ld,%ld,%ld\n", hour, minute, second));   
        break;
        default:
           viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Time/Date"); 
    }       
    checkErr( rszvl_CheckStatus (instrSession));
Error:
     (void) Rs_UnlockSession(instrSession, VI_NULL);     
    return error;
}

/*****************************************************************************
 * Function: Get SAN Time And Date
 * Purpose:  This function gets the instrument's internal clock or calendar.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSANSystemTimeAndDate(
    ViSession   instrSession,
    ViInt32 timeDate,
    ViInt32*    hour,
    ViInt32*    minute,
    ViInt32*    second,
    ViInt32*    day,
    ViInt32*    month,
    ViInt32*    year
)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    switch (timeDate)
    {
        case RSZVL_VAL_DATE:
            if (!day)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Day");
            if (!month)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Month");
            if (!year)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Year"); 
            viCheckErr (viQueryf(instrSession, ":SYST:DATE?\n", "%ld,%ld,%ld", year, month, day));    
        break;
        case RSZVL_VAL_TIME:
            if (!hour)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Hour");
            if (!minute)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Minute");
            if (!second)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Second"); 
            viCheckErr (viQueryf(instrSession, ":SYST:TIME?\n","%ld,%ld,%ld", hour, minute, second));   
        break;
        default:
           viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Time/Date"); 
    }       
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
     (void) Rs_UnlockSession(instrSession, VI_NULL);     
    return error;
}

/*****************************************************************************
 * Function: SAN Speaker Volume
 * Purpose:  This function sets the volume of the built-in loudspeaker for 
 *           demodulated signals.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANSpeakerVolume(
    ViSession   instrSession,
    ViReal64    speakerVolume
)
{
    ViStatus    error = VI_SUCCESS;
    ViReal64    tmp_value;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    tmp_value= speakerVolume/100.0;
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_SYST_SPEAKER, tmp_value),
                                                2, "Speaker Volume");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trigger Source
 * Purpose:  This function specifies the trigger source that causes the 
 *           spectrum analyzer to leave the Wait-for-Trigger state.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTriggerSource (ViSession instrSession,
                                                   ViInt32 channel,
                                                   ViInt32 triggerSource)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	sprintf (rep_cap, "CH%ld", channel);

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSZVL_ATTR_TRIGGER_SOURCE,
                                                triggerSource),
                                                2, "Trigger Source");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function:  Configure SAN Trigger Event Start
 * Purpose:   This function selects the Triggered Meas. Sequence. The
 *            identifier for the sequence is a string variable.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTriggerEventStart (ViSession instrSession,
                                                       ViInt32 channel,
                                                       ViInt32 triggerEventStart)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	sprintf (rep_cap, "CH%ld", channel);
   
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSZVL_ATTR_TRIGGER_EVENT_START,
                                                triggerEventStart),
                                                2, "Trigger Event Start");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN TV Trigger Source
 * Purpose:  This function configures the TV trigger source that causes the 
 *           spectrum analyzer to leave the Wait-for-Trigger state.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTVTriggerSource (ViSession instrSession,
                                                     ViInt32 triggerSource)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_SOURCE_TV,
                                                triggerSource),
                                                2, "Trigger Source");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Trigger
 * Purpose:  This function specifies the delay and polarity for triggering.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTrigger (ViSession instrSession,
                                             ViInt32 channel,
                                             ViReal64 triggerDelay,
                                             ViInt32 triggerPolarity)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	sprintf (rep_cap, "CH%ld", channel);
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSZVL_ATTR_TRIGGER_DELAY,
                                                triggerDelay),
                                                2, "Trigger Delay");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSZVL_ATTR_TRIGGER_SLOPE,
                                                triggerPolarity),
                                                3, "Trigger Polarity");                                                
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Video Trigger
 * Purpose:  This function specifies the video level for triggering.  This is 
 *           applicable when the trigger source is set to video.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANVideoTrigger(
    ViSession   instrSession,
    ViReal64    videoTriggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_VIDEO_TRIGGER_LEVEL,
                                                videoTriggerLevel),
                                                2, "Video Trigger Level");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN IF Power Trigger
 * Purpose:  This function specifies sets the level of the IF power trigger 
 *           source. This is applicable when the trigger source is set to IF 
 *           Power.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANIFPowerTrigger(
    ViSession   instrSession,
    ViReal64    triggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_IFP_LEVEL,
                                                triggerLevel),
                                                2, "Trigger Level");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN IF Power Trigger Parameters
 * Purpose:  This function sets the IF power trigger parameters. This is 
 *           applicable when the trigger source is set to IF Power.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANIFPowerTriggerParameters (ViSession instrSession,
                                                             ViReal64 triggerLevel,
                                                             ViReal64 holdoff,
                                                             ViReal64 hysteresis)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_IFP_LEVEL,
                                                triggerLevel),
                                                2, "Trigger Level");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_IFP_OFFSET,
                                                holdoff),
                                                3, "Holdoff");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_IFP_HYSTERESIS,
                                                hysteresis),
                                                4, "Hysteresis");
    
	checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Time Trigger
 * Purpose:  This function configures the time trigger. This is applicable 
 *           when the trigger source is set to time.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTimeTrigger (ViSession instrSession,
                                                 ViReal64 timeInterval)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_TIME_INTERVAL,
                                                timeInterval),
                                                2, "Time Interval");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN TV Trigger
 * Purpose:  This function configures the TV Trigger parameters. This is 
 *           applicable when the trigger source is set to TV.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTVTrigger (ViSession instrSession,
                                            ViInt32 lineSystem,
                                            ViInt32 synchronization,
                                            ViInt32 horizontalSyncLine,
                                            ViInt32 polarity)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_SOURCE_TV,
                                                RSZVL_VAL_TRG_B6_TV));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_TV_LINE_SYSTEM,
                                                lineSystem),
                                                2, "Line System");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_TV_VERTICAL_SIGNAL,
                                                synchronization),
                                                3, "Synchronization");
    if (synchronization == RSZVL_VAL_TRIG_TV_HORIZONTAL)
    {
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                    RSZVL_ATTR_TRIGGER_TV_HORIZONTAL_SIGNAL,
                                                    horizontalSyncLine),
                                                    4, "Horizontal Sync Line");
    }
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_TV_VIDEO_POLARITY,
                                                polarity),
                                                5, "Polarity");
    
	checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN TV Trigger Free Run State
 * Purpose:  This function configures the TV Trigger Free Run Mode. This is 
 *           applicable when the trigger source is set to TV
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTVTriggerFreeRunState (ViSession instrSession,
                                                  ViBoolean TVFreeRunState)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_TRIGGER_TV_VID_CONT,
                                                TVFreeRunState),
                                                2, "TV Free Run State");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN External Gate
 * Purpose:  This function configures the external gate parameters. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANExternalGate(
    ViSession   instrSession,
    ViBoolean   gating,
    ViInt32 gateSource,
    ViInt32 mode,
    ViInt32 polarity,
    ViReal64    delay,
    ViReal64    length
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (gating == VI_TRUE)
    {
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                    RSZVL_ATTR_EXTERNAL_GATE_SIGNAL_SOURCE,
                                                    gateSource), 3, "Gate Source");   
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                    RSZVL_ATTR_EXTERNAL_GATE_TRIGGER_TYPE,
                                                    mode), 4, "Mode");                                               
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                    RSZVL_ATTR_EXTERNAL_GATE_POLARITY,
                                                    polarity), 5, "Polarity");     
        viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                    RSZVL_ATTR_EXTERNAL_GATE_HOLD,
                                                    delay), 6, "Delay"); 
                                                
        if (mode==RSZVL_VAL_EGAT_TRIG_EDGE)                                                    
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                                                    RSZVL_ATTR_EXTERNAL_GATE_LENGTH,
                                                    length), 7, "Length");                                                     
    }                                                    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_EXTERNAL_GATE,
                                                gating),
                                                2, "Gating");                                                
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Preamplifier State
 * Purpose:  This function switches on the preamplifier for the instrument. 
 *           The switchable gain is fixed to 20 dB.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANPreamplifierState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_AMPL_PREAMPLIFIER,
                                                state), 2, "State");
                                       
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Create SAN Limit Line
 * Purpose:  This function creates new limit or selects existing one and 
 *           configures basic parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CreateSANLimitLine(
    ViSession   instrSession,
    ViInt32 limit,
    ViString    name,
    ViInt32 domain,
    ViString    comment,
    ViInt32 assigntoTrace,
    ViBoolean   deleteExistingLine
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      error_message[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }
    
    if (strlen (name) > 8)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Name");
    }

    if (strlen (comment) > 40)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Comment");
    }

    if (rszvl_invalidViBooleanRange (instrSession, deleteExistingLine) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Delete Existing Line");
    }
    sprintf (buffer, "L%ld", limit);
    
    if (deleteExistingLine==VI_TRUE)
    {
        error = rszvl_SetAttributeViString (instrSession, buffer, RSZVL_LIMIT_DELETE,"");
    
        do
        {
            (void) rszvl_error_query (instrSession, &error, error_message);
        } while (error != 0);
    }
    viCheckParm (rszvl_SetAttributeViString (instrSession, buffer, RSZVL_LIMIT_NAME, name), 3, "Name");
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_LIMIT_CONTROL_DOMAIN,domain), 4, "Domain");
    viCheckParm (rszvl_SetAttributeViString (instrSession, buffer, RSZVL_LIMIT_COMMENT,comment), 5, "Comment");

    sprintf (buffer, "L%ld", limit);
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_LIMIT_ASSIGN_TRACE, assigntoTrace), 6, "Assign to Trace");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Limit Line
 * Purpose:  This function configures specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANLimitLine(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 type,
    ViInt32 units,
    ViInt32 xAxisInterpolation,
    ViInt32 yAxisInterpolation,
    ViInt32 xAxisScaling,
    ViReal64    margin,
    ViReal64    threshold
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViInt32     scaling;
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }
    
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_LIMIT_UPPER, RSZVL_VAL_LIMIT_LOWER) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");
    }
    if (rszvl_invalidViInt32Range (instrSession, units, RSZVL_VAL_UNIT_DBM, RSZVL_VAL_UNIT_UNITLESS) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Units");
    }

    sprintf (buffer, "L%ld", limit);
    
    scaling = (units == RSZVL_VAL_UNIT_DB ) ? RSZVL_VAL_REL : RSZVL_VAL_ABS;
    
    viCheckParm (rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_LIMIT_UNITS, units), 4, "Units");
    viCheckParm (rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_LIMIT_CONTROL_SPACING, xAxisInterpolation), 5, "X Axis Interpolation");
    viCheckParm (rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_LIMIT_CONTROL_SCALING, xAxisScaling), 7, "X Axis Scaling");
	
    switch (type)
    {
        case RSZVL_VAL_LIMIT_LOWER:
            viCheckParm (rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_LIMIT_LOWER_SPACING, yAxisInterpolation), 6, "Y Axis Interpolation");
            viCheckParm (rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_LIMIT_LOWER_MARGIN, margin), 8, "Marging");
            if (scaling == RSZVL_VAL_REL)
            {
                viCheckParm (rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_LIMIT_LOWER_THRESHOLD, threshold), 9, "Threshold");
            }   
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession,buffer,RSZVL_LIMIT_LOWER_SCALING, scaling));
            break;
            
        case RSZVL_VAL_LIMIT_UPPER:
            viCheckParm (rszvl_SetAttributeViInt32(instrSession, buffer, RSZVL_LIMIT_UPPER_SPACING, yAxisInterpolation), 6, "Y Axis Interpolation");
            viCheckParm (rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_LIMIT_UPPER_MARGIN, margin), 8, "Marging");
            if (scaling == RSZVL_VAL_REL)
            {
                viCheckParm (rszvl_SetAttributeViReal64(instrSession, buffer, RSZVL_LIMIT_UPPER_THRESHOLD, threshold), 9, "Threshold");
            }
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession,buffer,RSZVL_LIMIT_UPPER_SCALING, scaling));
            break;
    }

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Define SAN Limit Line
 * Purpose:  This function defines specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DefineSANLimitLine(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 type,
    ViInt32 count,
    ViReal64    xAxis[],
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer= VI_NULL;
    ViChar      *p2buf;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }
    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_LIMIT_UPPER, RSZVL_VAL_LIMIT_LOWER) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");
    }
    if (count < 1)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Count");
    }
    if (xAxis == NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "X Axis");
    }
    if (amplitude == NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Amplitude");
    }

    viCheckAlloc (pbuffer = (ViChar*) malloc ((size_t)(count * 20 + 200)));
    
    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM%ld:CONT ", limit);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", xAxis[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));

    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM%ld:%s ", limit, limTypeArr [type]);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", amplitude[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (pbuffer) free (pbuffer);
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Select SAN Limit Line
 * Purpose:  This function selects the limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelectSANLimitLine(
    ViSession   instrSession,
    ViInt32 limit,
    ViString    name,
    ViBoolean   limitEnabled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }
    if (strlen (name) > 8)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Name");
    }
    
    sprintf (buffer, "L%ld", limit); 
    viCheckParm (rszvl_SetAttributeViString (instrSession, buffer, RSZVL_LIMIT_NAME, name), 3, "Name");
    sprintf (buffer, "L%ld", limit); 
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_LIMIT_STATE, limitEnabled), 4, "Limit Enabled");
    
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Enable SAN Limit Check
 * Purpose:  This function switches on or off the limit check and/or limit 
 *           line for the selected limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_EnableSANLimitCheck(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 type,
    ViBoolean   limitEnabled,
    ViBoolean   checkEnabled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViInt32     attribute = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }

    switch (type)
    {
        case RSZVL_VAL_LIMIT_LOWER:
            attribute = RSZVL_LIMIT_LOWER_STATE;
            break;
        case RSZVL_VAL_LIMIT_UPPER:
            attribute = RSZVL_LIMIT_UPPER_STATE;
            break;
    
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");
    }
    
    sprintf (buffer, "L%ld", limit);

    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_LIMIT_STATE, limitEnabled), 4, "Limit Enabled");
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, buffer, (ViUInt32) attribute, checkEnabled), 5, "Check Enabled");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Move SAN Limit Line
 * Purpose:  This function moves specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_MoveSANLimitLine(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 type,
    ViInt32 method,
    ViReal64    value
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViInt32     attributes[4][2] = {{0,0},
								   {RSZVL_LIMIT_LOWER_OFFSET,RSZVL_LIMIT_LOWER_SHIFT},
                                   {RSZVL_LIMIT_UPPER_OFFSET,RSZVL_LIMIT_UPPER_SHIFT},
                                   {RSZVL_LIMIT_CONTROL_OFFSET,RSZVL_LIMIT_CONTROL_SHIFT}};

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }

    if (rszvl_invalidViInt32Range (instrSession, type, RSZVL_VAL_LIMIT_UPPER, RSZVL_VAL_LIMIT_CONTROL) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");
    }

    if (rszvl_invalidViInt32Range (instrSession, method, RSZVL_VAL_LIMIT_MOVE_OFFSET, RSZVL_VAL_LIMIT_MOVE_SHIFT) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Method");
    }

    sprintf (buffer, "L%ld", limit);

    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, buffer, (ViUInt32)attributes[type][method], value), 5, "Value");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Copy SAN Limit Line
 * Purpose:  This function copies specified limit line into another one.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_CopySANLimitLine(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 copyTo
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }

    sprintf (buffer, "L%ld", limit);

    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_LIMIT_COPY, copyTo), 3, "Copy To");

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Delete SAN Limit Line
 * Purpose:  This function removes specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DeleteSANLimitLine(
    ViSession   instrSession,
    ViInt32 limit
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }

    sprintf (buffer, "L%ld", limit);

    viCheckErr (rszvl_SetAttributeViString (instrSession, buffer, RSZVL_LIMIT_DELETE, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Clear SAN Limit Line Results
 * Purpose:  This function deletes the result of the current limit check.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ClearSANLimitLineResults(
    ViSession   instrSession,
    ViInt32 limit
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Limit Line Number");
    
    sprintf(lim_line_no,"L%ld",limit);

    viCheckErr(rszvl_SetAttributeViString (instrSession, lim_line_no,
                                               RSZVL_LIMIT_CLEAR, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Line State
 * Purpose:  This function configures state of specified display line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayLineState(
    ViSession   instrSession,
    ViInt32 line,
    ViInt32 type,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (type){
        case RSZVL_VAL_LINE_THR:
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                         RSZVL_ATTR_THRLINE_STATE,
                                                         state), 4, "State"); 
        break;
        case RSZVL_VAL_LINE_DISPLAY:
            if (rszvl_invalidViInt32Range (instrSession, line, 1, 2) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
            sprintf (buffer, "L%ld", line);
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer,
                                                         RSZVL_ATTR_DLINE_STATE,
                                                         state), 4, "State"); 
        break;
        case RSZVL_VAL_LINE_FREQ:
            if (rszvl_invalidViInt32Range (instrSession, line, 1, 2) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
            sprintf (buffer, "L%ld", line);
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer,
                                                         RSZVL_ATTR_FLINE_STATE,
                                                         state), 4, "State"); 
        break;
        case RSZVL_VAL_LINE_TIME:
            if (rszvl_invalidViInt32Range (instrSession, line, 1, 2) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
            sprintf (buffer, "L%ld", line);
            viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer,
                                                         RSZVL_ATTR_TLINE_STATE,
                                                         state), 4, "State"); 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");     
    }
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Line Position
 * Purpose:  This function configures position of specified display line.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayLinePosition(
    ViSession   instrSession,
    ViInt32 line,
    ViInt32 type,
    ViReal64    position
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (type){
        case RSZVL_VAL_LINE_THR:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                         RSZVL_ATTR_THRLINE_POSITION,
                                                         position), 4, "Position");                                                           
        break;
        case RSZVL_VAL_LINE_DISPLAY:
            if (rszvl_invalidViInt32Range (instrSession, line, 1, 2) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
            sprintf (buffer, "L%ld", line);
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                         RSZVL_ATTR_DLINE_POSITION,
                                                         position), 4, "Position");                                                           
        break;
        case RSZVL_VAL_LINE_FREQ:
            if (rszvl_invalidViInt32Range (instrSession, line, 1, 2) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
            sprintf (buffer, "L%ld", line);
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                         RSZVL_ATTR_FLINE_POSITION,
                                                         position), 4, "Position");                                                              
        break;
        case RSZVL_VAL_LINE_TIME:
            if (rszvl_invalidViInt32Range (instrSession, line, 1, 2) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
            sprintf (buffer, "L%ld", line);
             viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                         RSZVL_ATTR_TLINE_POSITION,
                                                         position), 4, "Position");                                                          
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");     
    }
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Control
 * Purpose:  This function controls the apperance of display elements.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayControl(
    ViSession   instrSession,
    ViBoolean   frequency,
    ViBoolean   time
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_DISP_FREQ_STATE,
                                                frequency), 2, "Frequency");                                                
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                    RSZVL_ATTR_DISP_TIME_STATE,
                                                    time), 3, "Time");
    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Power Save
 * Purpose:  This function configures power-save mode of the display. With the 
 *           power-save mode activated the display including backlight is 
 *           completely switched off after the elapse of the response time (see 
 *           command DISPlay:PSAVe:HOLDoff).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayPowerSave(
    ViSession   instrSession,
    ViBoolean   state,
    ViInt32 holdoff
)
{
    ViStatus    error = VI_SUCCESS;
  
    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_DISP_PWR_SAVE_STATE,
                                                state), 2, "State"); 
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_DISP_PWR_SAVE_HOLDOFF,
                                                holdoff), 3, "Hold Off");                                                
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Window Size
 * Purpose:  This function switches the measurement window for channel and 
 *           adjacent-channel power measurements to full screen or half screen.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayWindowSize(
    ViSession   instrSession,
    ViInt32 windowSize
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_DISP_WINDOW_SIZE,
                                                windowSize), 2, "Window Size"); 
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Log Range
 * Purpose:  This function defines the display range of the Y axis (level 
 *           axis) in the selected measurement window with logarithmic scaling 
 *           (DISP:TRAC:Y:SPAC LOG).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayLogRange(
    ViSession   instrSession,
    ViReal64    range
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_DISP_LOG_RANGE, 
                range), 2, "Range");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Amplitude Grid Mode
 * Purpose:  This function defines the scale type of the Y axis (absolute or 
 *           relative) in the selected measurement window.
 *           When SYSTem:DISPlay is set to OFF, this command has no immediate 
 *           effect on the screen.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayAmplitudeGridMode(
    ViSession   instrSession,
    ViInt32 yAxisGridMode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                RSZVL_ATTR_DISP_AMPLITUDE_GRID_MODE, yAxisGridMode), 
                2, "Range");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Reference Value
 * Purpose:  If the external generator control option (FSP-B10) is mounted and 
 *           the normalization in the NETWORK mode is activated, this value 
 *           defines the power value assigned to the reference position in the 
 *           selected measurement window. This value corresponds to the 
 *           parameter REFERENCE VALUE in manual operation.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayReferenceValue(
    ViSession   instrSession,
    ViReal64    referenceValue
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                RSZVL_ATTR_DISP_REF_VALUE, referenceValue), 
                2, "Reference Position");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Reference Position
 * Purpose:  This function defines the position of the reference value in the 
 *           selected measurement window.
 *           In operating mode NETWORK (Ext. Generator Option FSP-B10) with 
 *           active normalization, RPOSition defines the reference point for 
 *           the output of the normalized measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayReferencePosition(
    ViSession   instrSession,
    ViReal64    referencePosition
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", 
                RSZVL_ATTR_DISP_REF_POSITION, referencePosition), 
                2, "Reference Value Position");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Single Sweep
 * Purpose:  This function configures the behaviour of the display during a 
 *           single sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplaySingleSweep(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                RSZVL_ATTR_DISP_SINGLE_SWEEP, state), 
                2, "State");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Update
 * Purpose:  This function switches on or off the update of all display 
 *           elements during remote control.
 *           
 *           Note:
 *           
 *           The best performance is obtained when the display output is 
 *           switched off during remote control.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayUpdate(
    ViSession   instrSession,
    ViBoolean   displayInRemote
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                RSZVL_ATTR_DISP_UPDATE, displayInRemote), 
                2, "Display In Remote");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Display Color Default
 * Purpose:  This function resets the screen colors of all display items to 
 *           their default settings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANDisplayColorDefault(
    ViSession   instrSession,
    ViInt32 defaultSetting
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, defaultSetting, 1, 3) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Default Settings");
    }
    sprintf (buffer, "DEF%ld", defaultSetting);  
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_DISP_COL_PRESET, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Color
 * Purpose:  This function defines the color table of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayColor(
    ViSession   instrSession,
    ViInt32 colorMap,
    ViReal64    tint,
    ViReal64    saturation,
    ViReal64    brightness
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, colorMap, 1, 26) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Color Map");
    if (rszvl_invalidViReal64Range (instrSession, tint, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "TINT");
    if (rszvl_invalidViReal64Range (instrSession, saturation, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "SATURATION"); 
    if (rszvl_invalidViReal64Range (instrSession, brightness, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "BRIGHTNESS");        
    
    viCheckErr( viPrintf (instrSession, "DISP:CMAP%ld:HSL %.12f,%.12f,%.12f\n",
                colorMap, tint/100.0, saturation/100.0, brightness/100.0));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Display Color by Predefined
 * Purpose:  This function defines the color table of the instrument using 
 *           predefined colors.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANDisplayColorByPredefined(
    ViSession   instrSession,
    ViInt32 colorMap,
    ViInt32 predefinedColors
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, colorMap, 1, 26) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Color Map");
    }
    sprintf (buffer, "CM%ld", colorMap);  
    
    viCheckParm ( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                RSZVL_ATTR_DISP_COL_PREDEFINED, predefinedColors),
                3, "Predefined Colors");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Hardcopy Device
 * Purpose:  This function selects the printer output medium and the data 
 *           format of the printout.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANHardcopyDevice(
    ViSession   instrSession,
    ViInt32 device,
    ViInt32 destination,
    ViInt32 pageOrientation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, device, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Device");
    if (rszvl_invalidViInt32Range (instrSession, destination, 0, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");
        
    sprintf (buffer, "DE%ld", device);
    switch(destination){
        case  RSZVL_VAL_HCOPY_DEST_BMP:
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,  RSZVL_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSZVL_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer, RSZVL_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, 
                                                    RSZVL_VAL_HCOPY_DEVICE_LANG_BMP));                
        break;
        case  RSZVL_VAL_HCOPY_DEST_WMF:
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,  RSZVL_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSZVL_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                      RSZVL_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSZVL_VAL_HCOPY_DEVICE_LANG_WMF));
        break;
        case  RSZVL_VAL_HCOPY_DEST_EMF:
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,  RSZVL_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSZVL_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                      RSZVL_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSZVL_VAL_HCOPY_DEVICE_LANG_EWMF));
        break;
        case  RSZVL_VAL_HCOPY_DEST_CLP:
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                      RSZVL_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSZVL_VAL_HCOPY_DEVICE_CLP));
        break;
        case  RSZVL_VAL_HCOPY_DEST_PRN:
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                      RSZVL_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSZVL_VAL_HCOPY_DEVICE_PRN));
            viCheckErr (rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                      RSZVL_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSZVL_VAL_HCOPY_DEVICE_LANG_GDI));
            viCheckParm (rszvl_SetAttributeViInt32 (instrSession, buffer,
                                                      RSZVL_ATTR_HCOPY_DEVICE_ORIENTATION,
                                                      pageOrientation), 4, "Page Orientation");                                                   
        break;
    }
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Abort
 * Purpose:  This function aborts a running hardcopy output.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopyAbort(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr ( rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_HCOPY_ABORT, VI_NULL));                
 
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Print
 * Purpose:  This function starts a hardcopy output of the selected items.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopyPrint(
    ViSession   instrSession,
    ViInt32 device,
    ViInt32 items
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, device, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Device");
    if (rszvl_invalidViInt32Range (instrSession, items, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Items");
        
    sprintf (buffer, "DE%ld", device);
    switch (items){
        case RSZVL_VAL_HCOP_ALL:
            viCheckErr ( rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_HCOPY_PRINT_SCREEN, VI_NULL));    
        break;
        case RSZVL_VAL_HCOP_TRACE:
            viCheckErr ( rszvl_SetAttributeViBoolean (instrSession, "", 
                RSZVL_ATTR_HCOPY_PRINT_TRACE, VI_TRUE));
        break;
    }
    viCheckErr ( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_HCOPY_PRINT, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Print Next
 * Purpose:  This function starts a hardcopy output. If the output is printed 
 *           to a file (see HCOPy:DESTination<1|2>), the file name used in 
 *           the last saving process is automatically counted up to the next 
 *           unused name.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopyPrintNext (ViSession instrSession,
                                              ViInt32 device)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	if (rszvl_invalidViInt32Range (instrSession, device, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Device");
    
	sprintf (buffer, "DE%ld", device);
    
	viCheckErr ( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_HCOPY_PRINT_NEXT, VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;        
}

/*****************************************************************************
 * Function: SAN Hardcopy Set File Name
 * Purpose:  This function defines a destination file for the hadcopy output.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopySetFileName(
    ViSession   instrSession,
    ViString    name
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr ( rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_HCOPY_FILE_NAME, name));                
 
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Get Printer List
 * Purpose:  This function queries the list of printers under Windows NT.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopyGetPrinterList(
    ViSession   instrSession,
    ViInt32 bufferSize,
    ViChar  printerList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     count;
    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (bufferSize<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size"); 
    if (printerList == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Printer List");
    
    p2buf=printerList;
    count=0;
    viCheckErr (rszvl_GetAttributeViString (instrSession, "", RSZVL_ATTR_HCOPY_PRINTER_FIRST,
                RSZVL_IO_BUFFER_SIZE, buffer));
    while (sscanf (buffer, "'%[^']'", buffer) == 1)
    {
        count+=(ViInt32)strlen(buffer);  
        count++; //add one position for comma
        if (count<=bufferSize)
            p2buf += sprintf (p2buf, "%s,", buffer);
        viCheckErr (rszvl_GetAttributeViString (instrSession, "", RSZVL_ATTR_HCOPY_PRINTER_NEXT,
                RSZVL_IO_BUFFER_SIZE, buffer));      
    }
    *--p2buf='\0';
    
    if (count>bufferSize)
        error = count;      
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Set Printer
 * Purpose:  This function selects one of the printers installed under Windows 
 *           NT. Prior to use this function get list of available printers 
 *           using function Hardcopy Get Printer List.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopySetPrinter(
    ViSession   instrSession,
    ViInt32 device,
    ViString    printerName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, device, 1, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Device");

    sprintf (buffer, "DE%ld", device);
    viCheckParm (rszvl_SetAttributeViString (instrSession, buffer, RSZVL_ATTR_HCOPY_PRINTER,
                                                printerName), 3, "Printer Name");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Comment
 * Purpose:  This function defines the comment text for measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopyComment(
    ViSession   instrSession,
    ViString    comment
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckErr ( rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_HCOPY_COMM_SCR, comment));    
 
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Hardcopy Color
 * Purpose:  This function resets the colors for a hardcopy to the selected 
 *           default settings. DEFault1(SCREEN COLORS, but background white), 
 *           DEFault2 (OPTIMIZED COLOR SET) and DEFault3 (USER DEFINED). The 
 *           numeric suffix in CMAP is not significant.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANHardcopyColor(
    ViSession   instrSession,
    ViBoolean   color,
    ViInt32 defaultSetting
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "DEF%ld", defaultSetting);  
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_HCOPY_COLOR, color), 
                2, "Color");
    
    if (color == VI_TRUE)
        viCheckParm ( rszvl_SetAttributeViString (instrSession, buffer, 
                RSZVL_ATTR_HCOPY_COLOR_DEF, VI_NULL), 3, "Default Setting");                
                 
    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Hardcopy Color
 * Purpose:  This function defines the color table in USER DEFINED COLORS 
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANHardcopyColor(
    ViSession   instrSession,
    ViInt32 colorMap,
    ViReal64    tint,
    ViReal64    saturation,
    ViReal64    brightness
)
{
   ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, colorMap, 1, 26) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Color Map");
    if (rszvl_invalidViReal64Range (instrSession, tint, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "TINT");
    if (rszvl_invalidViReal64Range (instrSession, saturation, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "SATURATION"); 
    if (rszvl_invalidViReal64Range (instrSession, brightness, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "BRIGHTNESS");        
    
    viCheckErr( viPrintf (instrSession, "HCOP:CMAP%ld:HSL %.12f,%.12f,%.12f\n",
                colorMap, tint/100.0, saturation/100.0, brightness/100.0));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Hardcopy Color by Predefined
 * Purpose:  This function defines the color table in USER DEFINED COLORS 
 *           using predefined color values.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANHardcopyColorByPredefined(
    ViSession   instrSession,
    ViInt32 colorMap,
    ViInt32 predefinedColors
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, colorMap, 1, 26) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Color Map");
    
    sprintf (buffer, "CM%ld", colorMap);  
    
    viCheckParm ( rszvl_SetAttributeViInt32 (instrSession, buffer, 
                RSZVL_ATTR_HCOPY_COLOR_PREDEFINED, predefinedColors),
                3, "Predefined Colors");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN File Decimal Separator
 * Purpose:  This function configures the decimal separator (decimal point or 
 *           comma) for floating-point numerals contained in the file.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANFileDecimalSeparator(
    ViSession   instrSession,
    ViInt32 decimalSeparator
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", RSZVL_ATTR_FILE_DEC_SEPARATOR, 
                 decimalSeparator), 2, "Decimal Separator");
        
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: File Manager Operations
 * Purpose:  The MMEMory (mass memory) subsystem function provides commands 
 *           which allow for access to the storage media of the instrument and 
 *           for storing and loading various instrument settings.
 *           The File to be Printed/Plotted command stores the HCOPy outputs 
 *           in a file. The various drives can be addressed via the mass 
 *           storage unit specifier <msus> using the conventional DOS syntax. 
 *           The internal hard disk is addressed by "C:", the floppy-disk drive 
 *           installed by "A:". The file names <file_name> are indicated as 
 *           string parameters with the commands being enclosed in quotation 
 *           marks. They correspond to the DOS conventions.
 *           DOS file names consist of max. 8 ASCII characters and an 
 *           extension of up to three characters separated from the file name 
 *           by a colon "." Both, the colon and the extension are optional. The 
 *           colon is not part of the file name. DOS file names do not differ 
 *           between uppercase and lowercase notation. All letters and digits 
 *           are permitted as well as the special characters "_", "^", "$", 
 *           "~", "!", "#", "%", "&", "-", "{","}", "(", ")", "@" and "' ". 
 *           Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to 
 *           LPT3, NUL and PRN. The two characters "*" and "?" have the 
 *           function of so-called "wildcards", i.e., they are variables for
 *           selection of several files. The question mark "?" replaces 
 *           exactly one character which may be any, the asterisk means any of 
 *           the remaining characters in the file name. "*.*" thus means all 
 *           files in a directory.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_FileManagerOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, operation, 0, 7) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    if (!source) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Source");
    
    switch(operation){
        case RSZVL_VAL_FILE_NEW:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_MAKE_DIR,
                source));   
        break;
        case RSZVL_VAL_FILE_COPY:
            if (!destination) 
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Destination");
            viCheckErr (viPrintf (instrSession, ":MMEM:COPY '%s','%s'\n", source, destination));
        break;  
        case RSZVL_VAL_FILE_RENAME:
            if (!destination) 
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Destination");
            viCheckErr (viPrintf (instrSession, ":MMEM:MOVE '%s','%s'\n", source, destination));
        break;  
        case RSZVL_VAL_FILE_DELETE:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_DELETE, 
                 source));
        break;           
        case RSZVL_VAL_FILE_RDIR:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_DELETE_DIR, 
                 source));
        break;       
        case RSZVL_VAL_FILE_CDIR:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_EDIT_PATH, 
                 source));
        break;
        case RSZVL_VAL_FILE_CDISC:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_EDIT_PATH_DEVICE, 
                 source));
        break;
    }       
    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read To File From Instrument
 * Purpose:  This function is used to read data from the instrument and write 
 *           it to a user specified file on the host computer.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadToFileFromInstrument(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt32    length = 0;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    ViUInt16 intfc = VI_INTF_GPIB;     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!source)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source");
    if (!destination)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");    
    checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    viCheckErr (viPrintf(instrSession, ":MMEMory:DATA? '%s'\n", source));
    //Read header       
    viCheckErr (viRead(instrSession, (ViPBuf)buffer, 2,  &ret_count));
    if (sscanf (buffer, "#%1ld", &count) != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE); 
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, (ViUInt32) count, &ret_count));
    if (sscanf (buffer, "%lu", &length)  != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);     
    viCheckErr(Rs_ReadToFile2(instrSession, destination, (ViInt32) length, RS_VAL_TRUNCATE));
    //Read \n at the end
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, 2, &ret_count)); 
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rszvl_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/*****************************************************************************
 * Function: Write From File To Instrument
 * Purpose:  This function is used to read data from the host computer and 
 *           write it to a user specified file in the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_WriteFromFileToInstrument(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt32    fileSize            = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!source)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source");
    if (!destination)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");  
        
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
    fileSize = (ViUInt32) ftell (sourceFile);
    (void) fseek (sourceFile, 0, SEEK_SET);

    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#%ld%ld", destination, digits, fileSize));
    
    viCheckErr (Rs_WriteFromFile(instrSession, source, (ViInt32) fileSize, 0, &count));
    if (count!=(ViInt32) fileSize)
        viCheckErr (RS_ERROR_WRITING_FILE);     
        
    checkErr( rszvl_CheckStatus (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/*****************************************************************************
 * Function: Data Set File Operations
 * Purpose:  This function allows to provide data set operation for 
 *           Save/Recall items of memory subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DataSetFileOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViString    path
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, operation, 0, 7) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    
    switch(operation){
        case RSZVL_VAL_MEM_SAVE:
            viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_SAVE,
                path), 2, "Path");  
        break;
        case RSZVL_VAL_MEM_RECALL:
            viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_RECALL,
                path), 2, "Path");  
        break;  
        case RSZVL_VAL_MEM_DELETE:
            viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_DATA_CLEAR,
                path), 2, "Path");  
        break;  
        case RSZVL_VAL_MEM_STARTUP:
            viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_STARTUP_RECALL,
                path), 2, "Path");  
        break;           
    }       
    
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Data Set File Clear All
 * Purpose:  This function deletes all device settings in the selected 
 *           directory.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_DataSetFileClearAll(
    ViSession   instrSession,
    ViString    directory
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_MANAGER_EDIT_PATH,
                directory), 2, "Directory");
    viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_DATA_CLEAR_ALL,
                VI_NULL));              
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/*****************************************************************************
 * Function: SAN Data Set File Select Items
 * Purpose:  This function add/remove settings to/from the list of partial 
 *           datasets of a device setting to be stored/loaded. 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANDataSetFileSelectItems(
    ViSession   instrSession,
    ViInt32 itemSelector,
    ViBoolean   itemState
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, itemSelector, 0, 8) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Item Selector");
    
    switch(itemSelector){
        case RSZVL_VAL_FILE_ITEMS_NONE:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_NONE,
                VI_NULL));  
        break;
        case RSZVL_VAL_FILE_ITEMS_DEFAULT:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_DEFAULT,
                VI_NULL));  
        break;
        case RSZVL_VAL_FILE_ITEMS_ALL:
            viCheckErr (rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_ALL,
                VI_NULL));  
        break;
        case RSZVL_VAL_FILE_ITEMS_HWSETTINGS:
            viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_HWSETTINGS,
                itemState), 3, "Item State");   
        break;
        case RSZVL_VAL_FILE_ITEMS_ALLTRACES:
            viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAC,
                itemState), 3, "Item State");   
        break;
        case RSZVL_VAL_FILE_ITEMS_ALLLINES:
            viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_LINE,
                itemState), 3, "Item State");   
        break;
        case RSZVL_VAL_FILE_ITEMS_SOUR_CAL_DATA:
            viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_SOURCE_CAL_DATA,
                itemState), 3, "Item State");   
        break;
        case RSZVL_VAL_FILE_ITEMS_ALLTRAN:
            viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAN,
                itemState), 3, "Item State");   
        break;
		case RSZVL_VAL_FILE_ITEMS_SPECTROGRAM:
            viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", RSZVL_ATTR_FILE_ITEMS_SAVE_RECAL_SPEM,
                itemState), 3, "Item State");  
        break;
    }   
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/*****************************************************************************
 * Function: SAN Calibration
 * Purpose:  This function initiates the acquisition of system error 
 *           correction data. A "0" is returned if the acquisition was 
 *           successful.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANCalibration(
    ViSession   instrSession,
    ViBoolean   sync,
    ViUInt32    timeout,
    ViInt32*    result
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViBooleanRange (instrSession, sync) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Sync");
    if (sync == VI_TRUE)
    {   
        if (rszvl_invalidViInt32Range (instrSession, timeout, 0, 600000) == VI_TRUE)
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Timeout");     
        viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));   
        viCheckErr( rszvl_SetOPCTimeout (instrSession, timeout));  
        viCheckParm (rszvl_GetAttributeViInt32 (instrSession, "", 
                RSZVL_ATTR_CALIBRATION_CHECK, result), 4, "Result");
    }
    else
    {
        checkErr (rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_CALIBRATION, VI_NULL));
    }   
    
	checkErr( rszvl_CheckStatus (instrSession));
	
Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);  
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Calibration Abort
 * Purpose:  This function aborts the acquisition of correction data and 
 *           restores the last complete correction data set.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANCalibrationAbort(
    ViSession   instrSession
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rszvl_SetAttributeViString (instrSession, "", 
            RSZVL_ATTR_CALIBRATION_ABORT, VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Calibration Result
 * Purpose:  This function reads the results of the correction data 
 *           acquisition. The lines of the result table (see manual section 
 *           "Recording the correction data of FSQ - CAL key") are output as 
 *           string data separated by commas:
 *           "Total Calibration Status: PASSED","Date (dd/mm/yyyy): 
 *           12/07/1999", "Time: 16:24:54","Runtime:00.06"
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANCalibrationResult(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar  result[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer=VI_NULL;
    ViUInt32    ret_val;
    ViInt32     length;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    //viCheckParm (rszvl_GetAttributeViString (instrSession, "", 
    //        RSZVL_ATTR_CALIBRATION_RESULT_QUERY, arraySize, result), 2, "Result");
    viCheckErr (viPrintf (instrSession, "CAL:RES?"));        
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, &ret_val));
    length=(arraySize<=(ViInt32) ret_val)?arraySize-1:(ViInt32)ret_val;
    memcpy (result,pbuffer,length);
    result[length]='\0';
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Calibration State
 * Purpose:  This function determines whether the current calibration data are 
 *           taken into account by the instrument (ON) or not (OFF).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANCalibrationState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", 
            RSZVL_ATTR_CALIBRATION_STATE, state), 2, "State");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Service Input
 * Purpose:  This function toggles between the RF input on the front panel 
 *           and the internal 65.83 MHz reference signal.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANServiceInput (ViSession instrSession,
                                                  ViInt32 input)
{
	ViStatus	error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", 
            RSZVL_ATTR_SERVICE_INPUT_SOURCE, input), 2, "Input");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);

	return error;
}

/*****************************************************************************
 * Function: Configure SAN Service Comb Frequency
 * Purpose:  This function sets the comb generator frequency. This function 
 *           only takes effect, if the internal reference signal is selected 
 *           for calibration (DIAGnostic<1|2>:SERVice:INPut[:SELect]).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANServiceCombFrequency (ViSession instrSession,
                                                          ViInt32 frequency)
{
	ViStatus	error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rszvl_SetAttributeViInt32 (instrSession, "", 
            RSZVL_ATTR_SERVICE_INPUT_COMB_FREQUENCY, frequency), 2, "Frequency");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);

	return error;
}

/*****************************************************************************
 * Function: Configure SAN Service Noise Source
 * Purpose:  This function switches the 28-V supply of the noise source at 
 *           the rear panel on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANServiceNoiseSource (ViSession instrSession,
                                                        ViBoolean noiseSource)
{
	ViStatus	error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", 
            RSZVL_ATTR_SERVICE_NOISE_SOURCE, noiseSource), 2, "Noise Source");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);

	return error;
}

/*****************************************************************************
 * Function: Configure SAN Service HW Info
 * Purpose:  This function queries the contents of the module info table. 
 *           Table lines are output as string data and are separated by commas.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANServiceHWInfo (ViSession instrSession,
                                                   ViInt32 bufferSize,
                                                   ViChar _VI_FAR HWInfo[])
{
	ViStatus	error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rszvl_QueryViString (instrSession, "DIAG:SERV:HWIN?", 
                                      bufferSize, HWInfo), 3, "HW Info");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);

	return error;
}

/*****************************************************************************
 * Function: Read SAN Y Trace
 * Purpose:  This function initiates a signal acquisition based on the present 
 *           instrument configuration.  It then waits for the acquisition to 
 *           complete, and returns the trace as an array of amplitude values. 
 *           The amplitude array returns data that represent the amplitude of 
 *           the signals of the sweep from the start frequency to the stop 
 *           frequency (in frequency domain, in time domain the amplitude array 
 *           is ordered from beginning of sweep to end).  The Amplitude Units 
 *           attribute determines the units of the points in the amplitude 
 *           array.  This function resets the sweep count.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadSANYTrace(
    ViSession   instrSession,
    ViInt32 trace,
    ViUInt32    maximumTime,
    ViInt32 arrayLength,
    ViInt32*    actualPoints,
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
    if (rszvl_invalidViUInt32Range (instrSession, maximumTime, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Maximum Time");         
       
    checkErr( rszvl_SANInitiate (instrSession, maximumTime));
   
    sprintf (buffer, ":TRAC? TRACE%ld", trace);
    viCheckErr (rszvl_dataReadTrace (instrSession, buffer, arrayLength, 
                    amplitude, actualPoints));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Fetch SAN Y Trace
 * Purpose:  This function returns the trace the spectrum analyzer acquires. 
 *           The trace is from a previously initiated acquisition.  You use the 
 *           rszvl_Initiate function to start an acquisition.  You use the 
 *           rszvl_AcquisitionStatus function to determine when the acquisition 
 *           is complete.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_FetchSANYTrace(
    ViSession   instrSession,
    ViInt32 trace,
    ViInt32 arrayLength,
    ViInt32*    actualPoints,
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 3) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Trace");

    sprintf (buffer, ":TRAC? TRACE%ld", trace);
    
    viCheckErr (rszvl_dataReadTrace (instrSession, buffer, arrayLength, 
                    amplitude, actualPoints));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Read SAN Trace IQ Data
 * Purpose:  This function will start a measurement and read I/Q trace data 
 *           results. The result values are scaled linear in unit Volt and 
 *           correspond to the voltage at the RF input of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadSANTraceIQData(
    ViSession   instrSession,
    ViUInt32    timeout,
    ViInt32 bufferSize,
    ViInt32*    noofPoints,
    ViReal64    realPartsI[],
    ViReal64    imaginaryPartsQ[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      cmd [100] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout");  
        
    viCheckErr (rszvl_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               100, cmd));
    
    sprintf (cmd, ":FORM REAL,32;*CLS;:TRAC:IQ:DATA?\n");

    viCheckErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));

    viCheckErr (rszvl_CheckBeforeRead(instrSession, timeout));

    viCheckErr (rszvl_dataReadTraceIQ (instrSession, bufferSize, noofPoints, 
                    realPartsI, imaginaryPartsQ));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Fetch SAN Trace IQ Data
 * Purpose:  This function permits the readout of previously acquired (and 
 *           frequency response corrected) I/Q data from the memory, with 
 *           indication of the offset related to the start of measurement and 
 *           with indication of the number of measurement values. Therefore a 
 *           previously acquired data set can be read out in smaller portions. 
 *           The result values are scaled linear in unit Volt and correspond to 
 *           the voltage at the RF input of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_FetchSANTraceIQData(
    ViSession   instrSession,
    ViInt32 offsetSamples,
    ViInt32 noofSamples,
    ViInt32 bufferSize,
    ViInt32*    noofPoints,
    ViReal64    realPartsI[],
    ViReal64    imaginaryPartsQ[]
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckErr (viPrintf (instrSession, ":FORM REAL,32;*CLS;:TRAC:IQ:DATA:MEM? %ld,%ld\n",
                offsetSamples, noofSamples));
                
    viCheckErr (rszvl_dataReadTraceIQ (instrSession, bufferSize, noofPoints, 
                    realPartsI, imaginaryPartsQ));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read SAN Memory IQ Large Data
 * Purpose:  This function sets the transfer parameters and reads the data 
 *           header.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadSANMemoryIQLargeData(
    ViSession   instrSession,
    ViUInt32    timeout,
    ViInt32 format,
    ViInt32*    no_ofPoints
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    ViUInt32    count = 0;
    ViInt32     lenLength;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout"); 
    if (rszvl_invalidViInt32Range (instrSession, format, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Format");        
        
    switch (format){
        case RSZVL_VAL_FORMAT_ASC:
            viCheckErr (viPrintf (instrSession, ":FORM ASC\n"));
        break;  
        case RSZVL_VAL_FORMAT_REAL:
            viCheckErr (viPrintf (instrSession, ":FORM REAL,32\n"));
        break;
    }
    
    viCheckErr (viPrintf (instrSession, ":TRAC:IQ:DATA?\n"));
    viCheckErr (rszvl_CheckBeforeRead(instrSession, timeout));   
    switch (format){
        case RSZVL_VAL_FORMAT_ASC:
            count=UINT_MAX;
        break;  
        case RSZVL_VAL_FORMAT_REAL:
            checkErr (viRead (instrSession, (ViPBuf)buffer, 2, &count));                
            if (sscanf (buffer, "#%1ld", &lenLength) != 1) 
                viCheckErr (VI_ERROR_INV_RESPONSE);
            checkErr (viRead (instrSession, (ViPBuf)buffer, lenLength, &count));
            buffer[lenLength]='\0';
            if (sscanf (buffer, "%lu", &count) != 1) 
                viCheckErr (VI_ERROR_INV_RESPONSE);
        break;
    }
    if (no_ofPoints) *no_ofPoints = count;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read SAN Memory IQ Large Data Block
 * Purpose:  This function will read I/Q trace data according format defined 
 *           with function rszvl_ReadMemoryIQLargeData.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ReadSANMemoryIQLargeDataBlock(
    ViSession   instrSession,
    ViInt32 bufferSize,
    ViChar  returnedData[],
    ViInt32*    returnedCounts
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    count;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    error = viRead (instrSession, (ViPBuf)returnedData, bufferSize, &count);
    
    if (returnedCounts)
        *returnedCounts=count;

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get SAN Limit Check Result
 * Purpose:  This function queries the result of the limit check of the limit 
 *           line indicated in the selected measurement window. It should be 
 *           noted that a complete sweep must have been performed for obtaining 
 *           a valid result. A synchronization with  *OPC, *OPC? or *WAI should 
 *           therefore be provided.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSANLimitCheckResult(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32*    state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, limit, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit");
    }

    sprintf (buffer, "L%ld", limit);

    viCheckParm (rszvl_GetAttributeViInt32 (instrSession, buffer, RSZVL_LIMIT_CHECK_RESULT, state), 3, "State");
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Initiate
 * Purpose:  This function initiates an acquisition. After calling this 
 *           function, the spectrum analyzer leaves the idle state and waits 
 *           for a trigger.
 *           
 *           This function does not check the instrument status. Typically, 
 *           the end-user calls this function only in a sequence of calls to 
 *           other low-level driver functions. The sequence performs one 
 *           operation. The end-user uses the low-level functions to optimize 
 *           one or more aspects of interaction with the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANInitiate(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Timeout"); 
        
    viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rszvl_SetOPCTimeout (instrSession, timeout));    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_INIT, 
                VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    if (old_timeout > 0) 
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Abort
 * Purpose:  This function aborts a previously initiated measurement and 
 *           returns the spectrum analyzer to the idle state.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANAbort(
    ViSession   instrSession
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_MEAS_ABORT, 
                VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Continue
 * Purpose:  This function continues a stopped measurement at the current 
 *           position in single sweep mode. The function is useful especially 
 *           for trace functions MAXHold, MINHold and AVERage if the previous 
 *           results are not to be cleared with Sweep Count > 0 or Average 
 *           Count > 0 on restarting the measurement (INIT:IMMediate resets the 
 *           previous results on restarting the measurement).
 *           The single-sweep mode is automatically switched on. 
 *           Synchronization to the end of the indicated number of measurements 
 *           can then be performed with the command *OPC, *OPC? or *WAI. In the 
 *           continuous-sweep mode, synchronization to the sweep end is not 
 *           possible since the overall measurement never ends.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANContinue(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout");  
        
    viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rszvl_SetOPCTimeout (instrSession, timeout)); 
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", 
                                                RSZVL_ATTR_INIT_CONMEAS, VI_NULL));   

    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    if (old_timeout > 0) 
        rszvl_SetOPCTimeout (instrSession, old_timeout); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Send Software Trigger
 * Purpose:  This function sends a command to trigger the SpecAn.  Call this 
 *           function if the trigger source is set to software trigger.  
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANSendSoftwareTrigger(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout");  
        
    viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rszvl_SetOPCTimeout (instrSession, timeout)); 
    viCheckErr( rszvl_SetAttributeViString (instrSession, "", RSZVL_ATTR_INIT_SW_TRIGGER, 
                VI_NULL));

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    if (old_timeout > 0) 
        rszvl_SetOPCTimeout (instrSession, old_timeout); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get SAN Sweep Number
 * Purpose:  This function returns the current number of started sweeps. A 
 *           sweep count value should be set and the device should be in single 
 *           sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSANSweepNumber(
    ViSession   instrSession,
    ViInt32*    numberOfSweeps
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_GetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_SWEEP_COUNT_CURRENT,
                                                numberOfSweeps), 2, "Number Of Sweeps"); 
                                                   
    checkErr( rszvl_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Noise Measurement
 * Purpose:  This function switches the noise measurement on or off for all 
 *           markers of the indicated measurement window. The noise power 
 *           density is measured at the position of the markers.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerNoiseMeasurement(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");   
        
    sprintf (buffer, "M%ld", marker);  
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, buffer, RSZVL_ATTR_MARKER_NOISE_STATE, 
                state), 3, "State");
   
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Marker Noise Measurement Result
 * Purpose:  This function queries the result of the noise measurement.
 *           A complete sweep with synchronization to the sweep end must be 
 *           performed between switching on the function and querying the 
 *           measured value in order to obtain a valid query result. This is 
 *           only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANMarkerNoiseMeasurementResult(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   result
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");   
        
    sprintf (buffer, "M%ld", marker);  
    viCheckParm( rszvl_GetAttributeViReal64(instrSession, buffer, RSZVL_ATTR_MARKER_NOISE_RESULT, 
                result), 3, "Result");
   
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Phase Noise Measurement
 * Purpose:  This function configures phase noise marker measurement. To setup 
 *           reference point use function rszvl_ConfigureReferenceFixedPoint.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerPhaseNoiseMeasurement(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean(instrSession, "", 
                RSZVL_ATTR_REFERENCE_MARKER_PNO_STATE,  state), 3, "State");

    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Phase Noise Peak Search
 * Purpose:  This function defines a new reference point level for delta 
 *           marker 2.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANPhaseNoisePeakSearch(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString(instrSession, "", 
                RSZVL_ATTR_MARKER_REF_PEAK,  VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Phase Noise Result
 * Purpose:  This function queries the result of the phase-noise measurement 
 *           in the selected measurement window. The measurement will be 
 *           switched on, if necessary.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANPhaseNoiseResult(
    ViSession   instrSession,
    ViReal64*   phaseNoiseResult
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_GetAttributeViReal64(instrSession, "", 
                RSZVL_ATTR_REFERENCE_MARKER_PNO_RESULT,  phaseNoiseResult), 
                2, "Phase Noise Result");
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Marker Frequency Counter
 * Purpose:  This function sets the marker frequency counter resolution and 
 *           turns the marker frequency counter on/off.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMarkerFrequencyCounter(
    ViSession   instrSession,
    ViInt32 marker,
    ViBoolean   markerFrequencyCounter,
    ViReal64    frequencyCounterResolution
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
     if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");        
    
    sprintf (buffer, "M%ld", marker); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, buffer, RSZVL_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED, 
                markerFrequencyCounter), 3, "Marker Frequency Counter");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "", RSZVL_ATTR_MARKER_FREQUENCY_COUNTER_RESOLUTION, 
                frequencyCounterResolution), 4, "Freqeuncy Counter Resolution");
              
    checkErr( rszvl_CheckStatus (instrSession));  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Marker Frequency Counter Result
 * Purpose:  This function queries the result of the frequency counter for the 
 *           indicated marker in the selected measurement window. Before the 
 *           command, the frequency counter should be switched on and a 
 *           complete measurement performed to obtain a valid count result. 
 *           Therefore, a single sweep with synchronization must be performed 
 *           between switching on the frequency counter and querying the count 
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANMarkerFrequencyCounterResult(
    ViSession   instrSession,
    ViInt32 marker,
    ViReal64*   counterFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
     if (rszvl_invalidViInt32Range (instrSession, marker, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker");        
    
    sprintf (buffer, "M%ld", marker); 
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, buffer, RSZVL_ATTR_MARKER_COUNT, 
                counterFrequency), 3, "Counter Freqeuncy");
              
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN N dB Points
 * Purpose:  This function configures the N dB function.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANNdBPoints(
    ViSession   instrSession,
    ViBoolean   ndBState,
    ViReal64    ndBLevel
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rszvl_SetAttributeViBoolean (instrSession, "", 
                RSZVL_ATTR_MARKER_NDB_STATE,  ndBState), 
                2, "N dB State");
    viCheckParm (rszvl_SetAttributeViReal64 (instrSession, "", 
                RSZVL_ATTR_MARKER_NDB_VAL,  ndBLevel), 
                3, "N dB Level");                
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN N dB Result
 * Purpose:  This function queries the frequency spacing (bandwidth) of the 
 *           N-dB-down markers in the selected measurement window.
 *           A complete sweep with synchronization to sweep end must be 
 *           performed between switching on the function and querying the 
 *           measured value in order to obtain a valid query result. This is 
 *           only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANNdBResult(
    ViSession   instrSession,
    ViReal64*   ndBResult
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_GetAttributeViReal64(instrSession, "", 
                RSZVL_ATTR_MARKER_NDB_RESULT,  ndBResult), 
                2, "N dB Result");
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN N dB Frequencies
 * Purpose:  This function queries the two frequencies of the N-dB-down marker 
 *           in the selected measurement window. The two frequency values are 
 *           output in ascending order.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANNdBFrequencies(
    ViSession   instrSession,
    ViReal64*   ndBFrequencyLower,
    ViReal64*   ndBFrequencyHigher
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!ndBFrequencyLower)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "N dB Frequency Lower");    
    if (!ndBFrequencyHigher)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Frequency Higher");         
   
    viCheckErr( viQueryf(instrSession, "CALC:MARK:FUNC:NDBD:FREQ?\n","%Le,%Le", ndBFrequencyLower, ndBFrequencyHigher ));
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN N dB Times
 * Purpose:  This function queries the two times of the N-dB-down marker in 
 *           the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANNdBTimes (ViSession instrSession,
                                          ViReal64 *n_dBTimeLower,
                                          ViReal64 *n_dBTimeHigher)
{
	ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!n_dBTimeLower)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "N dB Time Lower");    
    if (!n_dBTimeHigher)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Time Higher");         
   
    viCheckErr( viQueryf(instrSession, "CALC:MARK:FUNC:NDBD:TIME?\n","%Le,%Le", n_dBTimeLower, n_dBTimeHigher ));
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN List Power Sequence
 * Purpose:  This function configures the list of settings (max. 100 entries 
 *           for an individual frequency point) for the multiple power 
 *           measurement and starts a measurement sequence. To reduce the 
 *           setting time, all indicated parameters are set up simultaneously 
 *           at each test point.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANListPowerSequence(
    ViSession   instrSession,
    ViInt32 noofListItems,
    ViReal64    analyzerFrequency[],
    ViReal64    referenceLevel[],
    ViReal64    rfInputAttenuation[],
    ViReal64    rfInputElectronicAttn[],
    ViInt32 filterType[],
    ViReal64    resolutionBandwidth[],
    ViReal64    videoBandwidth[],
    ViReal64    measTime[],
    ViReal64    triggerLevel[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pwrite_buffer = NULL;
    ViChar*     pbuffer;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (rszvl_invalidViInt32Range (instrSession, noofListItems, 1, 100) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "No Of List Items");   
    if (analyzerFrequency == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyzer Frequency");  
    if (referenceLevel == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Reference Level");  
    if (rfInputAttenuation == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "RF Input Attenuation");  
    if (rfInputElectronicAttn == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "RF Input Electronic Attn");  
    if (filterType == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Filter Type");  
    if (resolutionBandwidth == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Resolutin Bandwidth");  
    if (videoBandwidth == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Video Bandwidth");  
    if (measTime == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Meas Time");  
    if (triggerLevel == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Trigger Level");  

    /*
     * Alloc buffer. Size is calclulated as:
     * size of header + size required for one entry * number of entries
     */
    pwrite_buffer = (ViChar*) malloc((size_t)(20 + 240 * noofListItems));
    if (pwrite_buffer == NULL)
        return VI_ERROR_ALLOC;

    pbuffer = pwrite_buffer;

    pbuffer += sprintf (pbuffer, ":SENS:LIST:POW:SEQ ");

    for (i = 0; i < noofListItems; i++)
    {
        /* Electronic attenuator is not in the signal path. */
        if (rfInputElectronicAttn[i] < 0)
        {
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,OFF,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i],
                                referenceLevel[i],
                                rfInputAttenuation[i],
                                listFilterTypeArr[filterType[i]],/* RFInputEAttn[i], */
                                resolutionBandwidth[i],
                                videoBandwidth[i],
                                measTime[i],
                                triggerLevel[i]);
        }
        else
        {
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,%.12f,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i],
                                referenceLevel[i],
                                rfInputAttenuation[i],
                                rfInputElectronicAttn[i],
                                listFilterTypeArr[filterType[i]],
                                resolutionBandwidth[i],
                                videoBandwidth[i],
                                measTime[i],
                                triggerLevel[i]);
        }
    }

    *--pbuffer = '\0'; /* Remove remaining comma */

    pbuffer += sprintf (pbuffer, "\n");

    viCheckErr(viWrite (instrSession, (ViBuf) pwrite_buffer, (ViUInt32)strlen (pwrite_buffer), NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    if (pwrite_buffer) free (pwrite_buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN List Power Set
 * Purpose:  This function defines the constant settings for the list during 
 *           multiple power measurement. Parameters Peak Meas, RMS Meas and AVG 
 *           Meas define, which measurements are to be performed at the same 
 *           time at the frequency point. Correspondingly, one, two or three 
 *           results per frequency point are returned for the function Get List 
 *           Power Sequence (actListPwrSeq). If all three parameters are set to 
 *           OFF, the command generates an execution error.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANListPowerSet(
    ViSession   instrSession,
    ViBoolean   peakMeas,
    ViBoolean   RMSMeas,
    ViBoolean   AVGMeas,
    ViInt32 triggerMode,
    ViInt32 triggerSlope,
    ViReal64    triggerOffset,
    ViReal64    gateLength
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViBooleanRange (instrSession, peakMeas) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Peak Meas"); 
    if (rszvl_invalidViBooleanRange (instrSession, RMSMeas) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "RMS Meas");
    if (rszvl_invalidViBooleanRange (instrSession, AVGMeas) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "AVG Meas");
    if (rszvl_invalidViInt32Range (instrSession, triggerMode, 0, 3) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Trigger Mode");   
    if (rszvl_invalidViInt32Range (instrSession, triggerSlope, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Trigger Slope");           
    if (rszvl_invalidViReal64Range (instrSession, triggerOffset, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Trigger Offset");
    if (rszvl_invalidViReal64Range (instrSession, triggerOffset, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Gate Length");
        
    viCheckErr (viPrintf (instrSession, "SENS:LIST:POW:SET %s,%s,%s,%s,%s,%.12f,%.12f\n",
            switchArr[peakMeas], switchArr[RMSMeas], switchArr[AVGMeas],
            rszvl_rngTriggerSource.rangeValues[triggerMode].cmdString, rszvl_rngPolarity.rangeValues[triggerSlope].cmdString,
            triggerOffset, gateLength));
            
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN List Power Measurement Off
 * Purpose:  This function deactivates the list measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANListPowerMeasurementOff(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr ( rszvl_SetAttributeViString (instrSession, "", 
                                                RSZVL_ATTR_LIST_POW_STATE_OFF, ""));
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN List Power Sequence
 * Purpose:  This function returns the list of settings (max. 100 entries for 
 *           an individual frequency point) for the multiple power measurement, 
 *           starts a measurement sequence and returns the measured data. To 
 *           reduce the setting time, all indicated parameters are set up 
 *           simultaneously at each test point.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANListPowerSequence(
    ViSession   instrSession,
    ViInt32 noofListItems,
    ViReal64    analyzerFrequency[],
    ViReal64    referenceLevel[],
    ViReal64    RFInputAttenuation[],
    ViReal64    RFInputElectronicAttn[],
    ViInt32 filterType[],
    ViReal64    resolutionBandwidth[],
    ViReal64    videoBandwidth[],
    ViReal64    measTime[],
    ViReal64    triggerLevel[],
    ViReal64    listPowerResults[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pwrite_buffer = NULL;
    ViChar*     pbuffer;
    ViInt32     i = 0,
                results;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (rszvl_invalidViInt32Range (instrSession, noofListItems, 1, 100) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "No Of List Items");   
    if (analyzerFrequency == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyzer Frequency");  
    if (referenceLevel == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Reference Level");  
    if (RFInputAttenuation == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "RF Input Attenuation");  
    if (RFInputElectronicAttn == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "RF Input Electronic Attn");  
    if (filterType == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Filter Type");  
    if (resolutionBandwidth == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Resolutin Bandwidth");  
    if (videoBandwidth == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Video Bandwidth");  
    if (measTime == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Meas Time");  
    if (triggerLevel == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Trigger Level"); 
    if (listPowerResults == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "List Power Results");         
    
 
    pwrite_buffer = (ViChar*) malloc((size_t)(20 + 240 * noofListItems));
    if (pwrite_buffer == NULL)
        return VI_ERROR_ALLOC;

    pbuffer = pwrite_buffer;

    pbuffer += sprintf (pbuffer, "*CLS;:SENS:LIST:POW:SEQ? ");

    for (i=0; i<noofListItems; i++)
        {
        /* Electronic attenuator is not in the signal path. */
        if (RFInputElectronicAttn[i] < 0)
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,OFF,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i], referenceLevel[i], RFInputAttenuation[i],
                                /* RFInputEAttn[i], */ listFilterTypeArr[filterType[i]], resolutionBandwidth[i],
                                videoBandwidth[i], measTime[i], triggerLevel[i]);
        else
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,%.12f,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i], referenceLevel[i], RFInputAttenuation[i],
                                RFInputElectronicAttn[i], listFilterTypeArr[filterType[i]], resolutionBandwidth[i],
                                videoBandwidth[i], measTime[i], triggerLevel[i]);
        }

    *--pbuffer = '\0'; /* Remove remaining comma */

    pbuffer += sprintf (pbuffer, "\n");

    viCheckErr (viWrite (instrSession, (ViBuf) pwrite_buffer, (ViUInt32) strlen (pwrite_buffer), NULL));

    //viCheckErr (rszvl_WaitForOPC (instrSession, timeout)); 
    /* Read float numbers separated by commas x 3 possibly activated detectors */
    
    
    results = (noofListItems);
    viCheckErr (viScanf (instrSession, "%,#lf", &results, listPowerResults));
    
    checkErr( rszvl_CheckStatus (instrSession)); 
    
Error:
    if (pwrite_buffer) free (pwrite_buffer); 
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN List Power Result
 * Purpose:  This function queries the result of a previous list measurement 
 *           as configured and initiated with List Power Sequence. This 
 *           function may be used to obtain measurement results in an 
 *           asynchronous way, using the service request mechanism for 
 *           synchronization with the end of the measurement. If no measurement 
 *           results are available, the function will return a query error.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANListPowerResult(
    ViSession   instrSession,
    ViInt32 noofResults,
    ViReal64    listPowerResults[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     ptag;
    ViInt32     index;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (noofResults<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "No Of Results");
        
    viCheckErr (viQueryf(instrSession, "SENS:LIST:POW:RES?\n","%s", buffer));
    ptag = strtok(buffer, ",");
    for (index = 0; ptag; index++)
    {
        if (index < noofResults)
            listPowerResults[index] = atof(ptag);
        ptag = strtok(NULL, ",");
    }
    if (returnedValues)
        *returnedValues=index;
    checkErr( rszvl_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Time Domain Power Measurement State
 * Purpose:  This function switches on or off the previously selected time 
 *           domain power measurements. Thus one or several measurements can be 
 *           first selected and then switched on and off together with this 
 *           attribute.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTimeDomainPowerMeasurementState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_STATE,
                                                state), 2, "State"); 
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Time Domain Power Measurement
 * Purpose:  This function configures parameters of time domain power 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTimeDomainPowerMeasurement(
    ViSession   instrSession,
    ViBoolean   peak,
    ViBoolean   RMS,
    ViBoolean   mean,
    ViBoolean   standardDeviation,
    ViBoolean   average,
    ViBoolean   maxHold,
    ViInt32 power
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_PEAK,
                                                peak), 2, "Peak"); 
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_RMS,
                                                RMS), 3, "RMS");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_MEAN,
                                                mean), 4, "Mean");
    if (mean == VI_TRUE)
        viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_SDEV,
                                                standardDeviation), 5, "Standard Deviation");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_AVG,
                                                average), 6, "Average");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_MAX_HOLD,
                                                maxHold), 7, "Max Hold"); 
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_MEAS_TDOM_MODE,
                                                power), 8, "Power");                                                
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Time Domain Power Measurement All Off
 * Purpose:  This function switches off all time domain measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANTimeDomainPowerMeasurementAllOff(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_TDOM_AOFF,
                                                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Time Domain Power Measurement Set Reference
 * Purpose:  With this function the currently measured average value and RMS 
 *           value are declared as reference values for relative measurements 
 *           in the indicated measurement window.
 *           If the function of Averaging or Peak Hold is switched on, the 
 *           current value is the accumulated measurement value at the time 
 *           considered.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANTimeDomainPowerMeasurementSetReference(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_TDOM_SET_REFERENCE,
                                                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Time Domain Power Result
 * Purpose:  This function queries selected time domain power measurement 
 *           result in the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANTDomPowerResult(
    ViSession   instrSession,
    ViInt32 measurement,
    ViInt32 resultType,
    ViReal64*   result
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, measurement, 0, 3) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    if (rszvl_invalidViInt32Range (instrSession, resultType, 0, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result Type");        
    
    switch (measurement){
        case RSZVL_VAL_TDOM_PEAK:
            switch (resultType){
                case RSZVL_VAL_TDOM_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_PEAK_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_AVG_PEAK_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_PEAK_HOLD_RESULT,
                                                                result), 4, "Result");
                break;
            }    
        break;
        case RSZVL_VAL_TDOM_RMS:
            switch (resultType){
                case RSZVL_VAL_TDOM_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_RMS_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_AVG_RMS_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_PEAK_HOLD_RMS_RESULT,
                                                                result), 4, "Result");
                break;
            }    
        break;
        case RSZVL_VAL_TDOM_MEAN:
            switch (resultType){
                case RSZVL_VAL_TDOM_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_MEAN_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_AVG_MEAN_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_PEAK_HOLD_MEAN_RESULT,
                                                                result), 4, "Result");
                break;
            }    
        break;
        case RSZVL_VAL_TDOM_SDEV:
            switch (resultType){
                case RSZVL_VAL_TDOM_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_SDEV_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_AVG_SDEV_RESULT,
                                                                result), 4, "Result");
                break;
                case RSZVL_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                                RSZVL_ATTR_MEAS_TDOM_PEAK_HOLD_SDEV_RESULT,
                                                                result), 4, "Result");
                break;
            }    
        break;
    }    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Time Domain Power Pulse Sequence Result
 * Purpose:  This function is used to determine the power of a sequence of signal 
 *           pulses having the same interval, as are typical for the slots of a GSM
 *           signal, for example. The number of pulses to be measured as well
 *           as the measurement time and the period can be set. To define the
 *           position of the first pulse in the trace, a suitable offset can
 *           be entered.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANTDomPowerPulseSequenceResult (ViSession instrSession,
                                                              ViReal64 timeOffset_ofFirstPulse,
                                                              ViReal64 measurementTime,
                                                              ViReal64 period,
                                                              ViInt32 number_ofPulses,
                                                              ViReal64 *result)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (number_ofPulses<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Number Of Pulses");
    if (result == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Result");        
    
    viCheckErr(viPrintf (instrSession, "CALC:MARK:FUNC:MSUM? %.12f,%.12f,%.12f,%ld\n", 
                 timeOffset_ofFirstPulse, measurementTime, period, number_ofPulses));
    viCheckErr (viScanf (instrSession, "%,#f", &number_ofPulses, result));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Select SAN Power Measurement
 * Purpose:  This function configures the power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelectSANPowerMeasurement(
    ViSession   instrSession,
    ViBoolean   state,
    ViInt32 powerMeasurement
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViBooleanRange (instrSession, state) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "State");
        
    if (state == VI_TRUE)
    {
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                    RSZVL_ATTR_MEAS_POW_SELECT,
                                                    powerMeasurement), 3, "Power Measurement");
    }                                                    
    else
    {
        viCheckErr( rszvl_SetAttributeViString (instrSession, buffer,
                                                RSZVL_ATTR_MEAS_POW_OFF,
                                                VI_NULL));
    }                                                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Power Measurement
 * Purpose:  This function configures the adjacent power channels measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANPowerMeasurement(
    ViSession   instrSession,
    ViInt32 noOfAdjChannels,
    ViInt32 channelMode,
    ViInt32 powerMode,
    ViBoolean   resultMode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_NUM,
                                                noOfAdjChannels), 2, "No Of Adj Channels");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_MODE,
                                                channelMode), 3, "Channel Mode");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_MODE,
                                                powerMode), 4, "Power Mode");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_RESULT_MODE,
                                                resultMode), 5, "Result Mode");                                                
                                                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Power Channel Spacing
 * Purpose:  This function configures the adjacent power channel spacing.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANPowerChannelSpacing(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 channelNumber,
    ViReal64    spacing
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (rszvl_invalidViInt32Range (instrSession, channel, 0, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");        
    
    switch (channel){
        case RSZVL_VAL_ACP_TX:
            if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 1) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
            sprintf (buffer, "CH%ld", channelNumber); 
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                RSZVL_ATTR_MEAS_POW_CHANNEL_SPACING,
                                                spacing), 4, "Spacing");
        break;
        case RSZVL_VAL_ACP_ADJ:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING,
                                                spacing), 4, "Spacing");
        break;
        case RSZVL_VAL_ACP_ALT:
            if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 11) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");  
            sprintf (buffer, "CH%ld", channelNumber); 
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                RSZVL_ATTR_MEAS_POW_ALT_CHANNEL_SPACING,
                                                spacing), 4, "Spacing");
        break;
    }    
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Power Channel Bandwidth
 * Purpose:  This function configures the adjacent power channel bandwidth.
 *           
 *           Note:
 *           (1) This function is only available in the frequency domain (span 
 *           > 0).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANPowerChannelBandwidth(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 channelNumber,
    ViReal64    bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, channel, 0, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");        
    
    switch (channel){
        case RSZVL_VAL_ACP_TX:
            if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 11) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");  
            sprintf (buffer, "CH%ld", channelNumber); 
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                RSZVL_ATTR_MEAS_POW_CHANNEL_BANDWIDTH,
                                                bandwidth), 4, "Bandwidth");
        break;
        case RSZVL_VAL_ACP_ADJ:
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH,
                                                bandwidth), 4, "Bandwidth");
        break;
        case RSZVL_VAL_ACP_ALT:
            if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 11) == VI_TRUE)
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");  
            sprintf (buffer, "CH%ld", channelNumber); 
            viCheckParm( rszvl_SetAttributeViReal64 (instrSession, buffer,
                                                RSZVL_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH,
                                                bandwidth), 4, "Bandwidth");
        break;
    }    
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Channel Power Trace
 * Purpose:  This function assigns the channel/adjacent channel power 
 *           measurement of the indicated trace in the selected measurement 
 *           window. The corresponding trace should be active, ie its states 
 *           should differ from blank.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANChannelPowerTrace(
    ViSession   instrSession,
    ViInt32 traceNumber
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_TRACE,
                                                traceNumber), 2, "Trace Number");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Channel Power Standard
 * Purpose:  This function configures the standard for channel power 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANChannelPowerStandard(
    ViSession   instrSession,
    ViInt32 ACPStandard
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_STANDARD,
                                                ACPStandard), 2, "ACP Standard");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Power Adjust Reference Level
 * Purpose:  This sets the reference value for the relative measurement to the 
 *           currently measured channel power.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANPowerAdjustReferenceLevel(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout");         
    
    checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_REF_VALUE,
                                                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Power Preset Measurement
 * Purpose:  This function adjusts the frequency span, the measurement 
 *           bandwidths and the detector as required for the number of 
 *           channels, the channel bandwidths and the channel spacings selected 
 *           in the active power measurement. If necessary, adjacent-channel 
 *           power measurement is switched on prior to the adjustment.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANPowerPresetMeasurement(
    ViSession   instrSession,
    ViInt32 channelPowerType,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout"); 
    checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_PRESET,
                                                channelPowerType), 3, "Channel Power Type");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Power Results
 * Purpose:  This function returns the power measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANPowerResults(
    ViSession   instrSession,
    ViInt32 powerMeasurement,
    ViInt32 arraySize,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff = VI_NULL; 
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, powerMeasurement, 0, 5) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Power Measurement"); 
    if (arraySize<0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Array Size"); 
    if ((arraySize>0) && (results == NULL))    
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Results"); 
    
    viCheckErr( viPrintf (instrSession, "CALC:MARK:FUNC:POW:RES? %s",
        rszvl_rngMeasPowerSelect.rangeValues[powerMeasurement].cmdString));
    
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL));  
    
    pbuff = strtok (buffer, ",");
    i = 0;
    do{
        if (i<arraySize) results[i] = atof (pbuff);
        pbuff = strtok (NULL, ",");
        i++;
    }while(pbuff != NULL);   
    
    if (returnedValues) *returnedValues = i;
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN ACP Fast ACP State
 * Purpose:  This function switches on or off the high-speed channel/adjacent 
 *           channel power measurement.
 *           The measurement itself is performed in the time domain on the 
 *           center frequencies of the individual channels. The function 
 *           automatically switches to the time domain and back. Depending on 
 *           the selected mobile radio standard, weighting filters with 
 *           sqrt(cos) characteristic or very steep-sided channel filters are 
 *           used for band limitation.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANACPFastACPState(
    ViSession   instrSession,
    ViBoolean   fastACPState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_ACP_HSP,
                                                fastACPState), 3, "Fast ACP State");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN ACP Channel Power Auto Adjust
 * Purpose:  This function adapts the reference level to the measured channel 
 *           power and - if required - switches on previously the adjacent 
 *           channel power measurement. This ensures that the signal path of 
 *           the instrument is not overloaded. Since the measurement bandwidth 
 *           is significantly smaller than the signal bandwidth in channel 
 *           power measurements, the signal path can be overloaded although the 
 *           trace is still significantly below the reference level. If the 
 *           measured channel power equals the reference level, the signal path 
 *           is not overloaded.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANACPChannelPowerAutoAdjust(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1;   
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout");
        
    checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_ACP_PRESET_REF_LEVEL,
                                                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN ACP Limit State
 * Purpose:  This function switches on and off the limit check for adjacent 
 *           channel power measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANACPLimitState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_ACP_LIMIT_STATE,
                                                state), 2, "State");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN ACP Adjacent Channel Limit
 * Purpose:  This function defines the limit value of the adjacent channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANACPAdjacentChannelLimit(
    ViSession   instrSession,
    ViInt32	  	limitCheck,
    ViReal64    limitValue
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (limitCheck)
	{
		case RSZVL_VAL_ACP_LIMIT_CHECK_OFF:
			viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
		                                             RSZVL_ATTR_MEAS_ACP_LIMIT_ABS_STATE,
		                                             VI_FALSE));
			
			viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
		                                             RSZVL_ATTR_MEAS_ACP_LIMIT_REL_STATE,
		                                             VI_FALSE));
		break;
		
		case RSZVL_VAL_ACP_LIMIT_CHECK_ABS:
			if (rszvl_invalidViReal64Range (instrSession, limitValue, 0.0, 100.0) == VI_TRUE)
        		viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit Value");
    	
			viCheckErr (viPrintf (instrSession, "CALC:LIM:ACP:ACH:ABS %.12lf,0.0\n", limitValue));
		    checkErr( rszvl_CheckStatus (instrSession));
		    viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
		                                             RSZVL_ATTR_MEAS_ACP_LIMIT_ABS_STATE,
		                                             VI_TRUE)); 
    
		break;
		
		case RSZVL_VAL_ACP_LIMIT_CHECK_REL:
			if (rszvl_invalidViReal64Range (instrSession, limitValue, -200.0, 200.0) == VI_TRUE)
        		viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit Value");         
    		
			viCheckErr (viPrintf (instrSession, "CALC:LIM:ACP:ACH %.12lf,0.0\n", limitValue));
		    checkErr( rszvl_CheckStatus (instrSession));
		    viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
		                                             RSZVL_ATTR_MEAS_ACP_LIMIT_REL_STATE,
		                                             VI_TRUE));
   
		break;
		
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Limit Check");
	}
	
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN ACP Alternate Channel Limit
 * Purpose:  This function defines the limit value of the alternate adjacent 
 *           channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANACPAlternateChannelLimit(
    ViSession   instrSession,
    ViInt32 	channelNumber,
    ViInt32   	limitCheck,
    ViReal64    limitValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 11) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Number");        
    
    sprintf (buffer, "CH%ld", channelNumber); 
    
	switch (limitCheck)
	{
		case RSZVL_VAL_ACP_LIMIT_CHECK_OFF:
			viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer,
		                                             RSZVL_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE,
		                                             VI_FALSE));
			
			viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer,
		                                             RSZVL_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE,
		                                             VI_FALSE));
		break;
		
		case RSZVL_VAL_ACP_LIMIT_CHECK_ABS:
			if (rszvl_invalidViReal64Range (instrSession, limitValue, 0.0, 100.0) == VI_TRUE)
        		viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Value");
    	
			viCheckErr (viPrintf (instrSession, "CALC:LIM:ACP:ALT%ld:ABS %.12lf,0.0", 
                                                    channelNumber, limitValue));
		    checkErr( rszvl_CheckStatus (instrSession));
		    viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer,
		                                             RSZVL_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE,
		                                             VI_TRUE)); 
    
		break;
		
		case RSZVL_VAL_ACP_LIMIT_CHECK_REL:
			if (rszvl_invalidViReal64Range (instrSession, limitValue, -200.0, 200.0) == VI_TRUE)
        		viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Value");         
    		
			viCheckErr (viPrintf (instrSession, "CALC:LIM:ACP:ALT%ld %.12lf,0.0", 
                            							channelNumber, limitValue));
		    checkErr( rszvl_CheckStatus (instrSession));
		    viCheckErr( rszvl_SetAttributeViBoolean (instrSession, buffer,
		                                             RSZVL_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE,
		                                             VI_TRUE));
   
		break;
		
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Limit Check");
	}
  
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN ACP Adjacent Channel Limit Check Results
 * Purpose:  This function returns the adjacent channel limit results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANACPAdjacentChannelLimitCheckResults(
    ViSession   instrSession,
    ViInt32*    lowerResult,
    ViInt32*    upperResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      buf2[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!lowerResult)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Lower Result");
    if (!upperResult)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Upper Result");        
        
    viCheckErr (viQueryf (instrSession, "CALC:LIM:ACP:ACH:RES?\n","%[^,],%[^\r\n]", 
    buf1, buf2));
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)lowerResult, buf1));     
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)upperResult, buf2));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/*****************************************************************************
 * Function: Query SAN ACP Alternate Channel Limit Check Results
 * Purpose:  This function returns the alternate channel limit results.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANACPAlternateChannelLimitCheckResults(
    ViSession   instrSession,
    ViInt32 channelNumber,
    ViInt32*    lowerResult,
    ViInt32*    upperResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSZVL_IO_BUFFER_SIZE] = "";
    ViChar      buf2[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, channelNumber, 1, 11) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel Number");        
    if (lowerResult == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Lower Result");
    if (upperResult == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Upper Result");        
        
    viCheckErr (viQueryf (instrSession, "CALC:LIM:ACP:ALT%ld:RES?\n","%[^,],%[^\r\n]", 
    channelNumber, buf1, buf2));
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)lowerResult, buf1));     
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)upperResult, buf2));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/*****************************************************************************
 * Function: Configure SAN Multicarrier ACP Power
 * Purpose:  This function configures the adjacent power channels measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANMulticarrierACPPower(
    ViSession   instrSession,
    ViInt32 noOfAdjChannels,
    ViInt32 noOfTXChannels,
    ViInt32 channelMode,
    ViInt32 powerMode,
    ViBoolean   resultMode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_NUM,
                                                noOfAdjChannels), 2, "No Of Adj Channels");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_CARR_SIG_NUM,
                                                noOfTXChannels), 3, "No Of TX Channels");
    if (noOfAdjChannels>0)
        viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_MODE,
                                                channelMode), 4, "Channel Mode");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_MODE,
                                                powerMode), 5, "Power Mode");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_RESULT_MODE,
                                                resultMode), 6, "Result Mode");                                                
                                                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN ACP Reference Channel Auto
 * Purpose:  Activates the automatic selection of a transmission channel to be 
 *           used as a reference channel in relative adjacent-channel power 
 *           measurements. The transmission channel with the highest power, the 
 *           transmission channel with the lowest power, or the transmission 
 *           channel nearest to the adjacent channels can be defined as a 
 *           reference channel.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANACPReferenceChannelAuto(
    ViSession   instrSession,
    ViInt32 reference
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_REF_CHAN_SEL_AUTO,
                                                reference), 2, "Reference");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN ACP Reference Channel Man
 * Purpose:  This function selects a transmission channel to be used as a 
 *           reference channel in relative adjacent-channel power measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANACPReferenceChannelMan(
    ViSession   instrSession,
    ViInt32 channel
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_ADJ_REF_TXCHANNEL,
                                                channel), 2, "Channel");
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Occupied Bandwidth
 * Purpose:  This function defines the percentage of the power with respect to 
 *           the total power. This value is the basis for the occupied 
 *           bandwidth measurement (command: POWer:ACHannel:PRESet OBW).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANOccupiedBandwidth(
    ViSession   instrSession,
    ViReal64    powerBandwidth,
    ViReal64    channelBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_BANDWIDTH,
                                                powerBandwidth), 2, "Power Bandwidth");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_POW_CHANNEL_BANDWIDTH,
                                                channelBandwidth), 3, "Channel Bandwidth");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Signal Statistic Measurement
 * Purpose:  This function configures parameters of signal statistic 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSignalStatisticMeasurement(
    ViSession   instrSession,
    ViInt32 measurement,
    ViInt32 numberofSamples,
    ViReal64    resolutionBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViInt32Range (instrSession, measurement, 0, 2) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
 
    switch (measurement){
        case RSZVL_VAL_SSTAT_OFF:      
            viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_APD_STATE,
                                                VI_FALSE));
            viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_CCDF_STATE,
                                                VI_FALSE));
        break;
        case RSZVL_VAL_SSTAT_APD:      
            viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_CCDF_STATE,
                                                VI_FALSE));
            viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_APD_STATE,
                                                VI_TRUE));                                                
            viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_SAMPLES,
                                                numberofSamples), 4, "Number Of Samples");                                                
        break;
        case RSZVL_VAL_SSTAT_CCDF:      
            viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_APD_STATE,
                                                VI_FALSE));
            viCheckErr( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_CCDF_STATE,
                                                VI_TRUE));
            viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_SAMPLES,
                                                numberofSamples), 3, "Number Of Samples");                                                
        break;
    }
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_RESOLUTION_BANDWIDTH,
                                                resolutionBandwidth), 4, "Resolution Bandwidth"); 
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Signal Statistic Measurement X Axis
 * Purpose:  This function configures parameters of signal statistic 
 *           measurement X-Axis scaling.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSignalStatisticMeasurementXAxis(
    ViSession   instrSession,
    ViReal64    xAxisRefLevel,
    ViReal64    xAxisRange
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_X_REF,
                                                xAxisRefLevel), 2, "X Axis Ref Level");
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_X_RANGE,
                                                xAxisRange), 3, "X Axis Range");                                                
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Signal Statistic Measurement Y Axis
 * Purpose:  This function configures parameters of signal statistic 
 *           measurement Y Axis scaling.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSignalStatisticMeasurementYAxis(
    ViSession   instrSession,
    ViReal64    yAxisMaxValue,
    ViReal64    yAxisMinValue
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_Y_MAX,
                                                yAxisMaxValue), 3, "Y Axis Max Value");  
    viCheckParm( rszvl_SetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_Y_MIN,
                                                yAxisMinValue), 4, "Y Axis Min Value");                                                  
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Signal Statistic Sweep
 * Purpose:  This function configures the acquisition attributes of signal 
 *           statistic.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANSignalStatisticSweep(
    ViSession   instrSession,
    ViBoolean   sweepModeContinuous
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "", 
                                                RSZVL_ATTR_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 2, "Sweep Mode Continuous");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Signal Statistic Measurement Adjust Settings
 * Purpose:  This function optimizes the level setting of the instrument 
 *           depending on the measured peak power, in order to obtain maximum 
 *           instrument sensitivity.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANSignalStatisticMeasurementAdjustSettings(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
     if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout"); 
        
    checkErr (rszvl_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rszvl_SetOPCTimeout (instrSession, timeout));
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_ADJ,
                                                VI_NULL));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (old_timeout >= 0)    
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Signal Statistic Measurement Preset Scaling
 * Purpose:  This function resets the scaling of the X and Y axes in a 
 *           statistical measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANSignalStatisticMeasurementPresetScaling(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_STAT_PRESET,
                                                VI_NULL));
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Signal Statistic Result
 * Purpose:  This function reads out the results of statistical measurements 
 *           of a recorded trace.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANSignalStatisticResult(
    ViSession   instrSession,
    ViInt32 trace,
    ViInt32 resultType,
    ViReal64*   result
)
{
    ViStatus   error = VI_SUCCESS;
    ViString   statisticMeasType[] = {"Mean","Peak","CrestFactor"};  
    ViChar     buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    if (rszvl_invalidViInt32Range (instrSession, trace, 1, 6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trace");
   
    sprintf (buffer, "TR%ld,Stat%s", trace, statisticMeasType[resultType]); 
    
    viCheckErr( rszvl_GetAttributeViReal64 (instrSession, buffer,
                                                RSZVL_ATTR_STAT_RESULT,
                                                result));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Modulation Depth Measurement
 * Purpose:  This function switches on the measurement of the AM modulation 
 *           depth. An AM-modulated carrier is required on the screen for 
 *           correct operation. If necessary, marker 1 is previously activated 
 *           and set to the largest signal available.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANModDepthMeasurement(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_MDEPTH_STATE,
                                                state), 2, "State");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Modulation Depth
 * Purpose:  This function queries the AM modulation depth.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANModulationDepth(
    ViSession   instrSession,
    ViReal64*   modulationDepth
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_MDEPTH_RESULT,
                                                modulationDepth), 2, "Modulation Depth");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Modulation Depth Signal Search
 * Purpose:  This function starts the search of the signals required for 
 *           the modulation depth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANModulationDepthSignalSearch (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_MDEPTH_SEARCH,
                                                NULL));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN TOI Measurement
 * Purpose:  This function initiates the measurement of the third-order 
 *           intercept point.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANTOIMeasurement(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_TOI_STATE,
                                                state), 2, "State");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN TOI
 * Purpose:  This function queries the third-order intercept point measurement 
 *           in the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANTOI(
    ViSession   instrSession,
    ViReal64*   TOIMeasurement
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "",
                                                RSZVL_ATTR_MEAS_TOI_RESULT,
                                                TOIMeasurement), 2, "TOI Measurement");
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Configure SAN Harmonic Distortion Measurement
 * Purpose:  This function configures harmonic distortion measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANHDistMeasurement(
    ViSession   instrSession,
    ViBoolean   state,
    ViInt32 noofHarmonics,
    ViBoolean   harmonicRBWAuto
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_HDIST_STATE,
                                                state), 2, "State");
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "",
                                                RSZVL_ATTR_MEAS_HDIST_NOOFHARM,
                                                noofHarmonics), 3, "No Of Harmonic");
    viCheckParm( rszvl_SetAttributeViBoolean (instrSession, "",
                                                RSZVL_ATTR_MEAS_HDIST_RBWAUTO,
                                                harmonicRBWAuto), 4, "Harmonic RBW Auto");   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Adjust Harmonic Distortion Settings
 * Purpose:  This function optimizes the device settings depending on the mode 
 *           in which the harmonic measurement was started:
 *           If the harmonic measurement was started in the frequency domain 
 *           (span > 0), the frequency and the
 *           level of the first harmonic are calculated, from which the 
 *           measurement list is set up.
 *           If the measurement was started in the time domain (span = 0), the 
 *           frequency of the first harmonic is
 *           not changed. The level, however, is not calculated.
 *           The function is independent of the marker selection.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SANAdjustHDistSettings(
    ViSession   instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Timeout"); 
    viCheckErr( rszvl_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rszvl_SetOPCTimeout (instrSession, timeout)); 
    viCheckErr( rszvl_SetAttributeViString (instrSession, "",
                                                RSZVL_ATTR_MEAS_HDIST_PRESET,
                                                VI_NULL));
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    if (old_timeout > 0) 
        rszvl_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Harmonic Distortion
 * Purpose:  This function queries the results of the total harmonic 
 *           distortion (THD).
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANHDist(
    ViSession   instrSession,
    ViReal64*   harmonicDistortion_in_percent,
    ViReal64*   harmonicDistortion_in_dB
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!harmonicDistortion_in_percent)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Harmonic Distortion in percent");
     if (!harmonicDistortion_in_dB)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Harmonic Distortion in dB");                 
    
    viCheckErr (viQueryf (instrSession, ":CALC:MARK:FUNC:HARM:DIST? TOT\n", "%le,%le", 
                harmonicDistortion_in_percent, harmonicDistortion_in_dB));
                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query SAN Harmonic Distortion Harmonics List
 * Purpose:  This function reads out the list of harmonics. The first value is 
 *           the absolute power of the first harmonic in the unit set via UNIT. 
 *           The other values are relative to the carrier signal and are output 
 *           in dB.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QuerySANHDistHarmonicsList(
    ViSession   instrSession,
    ViReal64    harmonicsList[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (harmonicsList == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Harmonic List");
    
    viCheckErr (viQueryf (instrSession, "CALC:MARK:FUNC:HARM:LIST?\n", "%,255lf", 
                harmonicsList));
                
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: SAN Burst Power Sequence
 * Purpose:  This function configures the instrument setup for multiple burst 
 *           power measurement and starts a measurement sequence. To reduce the 
 *           setting time, the setup is performed simultaneously for all 
 *           selected parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANBurstPowerSequence(
    ViSession   instrSession,
    ViReal64    analyzerFrequencyHz,
    ViReal64    resolutionBandwidthHz,
    ViReal64    measTimes,
    ViInt32 triggerSource,
    ViReal64    triggerLevel,
    ViReal64    triggerOffsets,
    ViInt32 type_ofMeas,
    ViInt32 noofMeasurements
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (analyzerFrequencyHz<0.0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Analyzer Frequency [Hz]");
    if (rszvl_invalidViReal64Range (instrSession, resolutionBandwidthHz, 10.0, 10.0e+6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Resolution Bandwidth [Hz]");
    if (rszvl_invalidViReal64Range (instrSession, measTimes, 1.0e-6, 30.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Meas Time [s]");        
    if ((rszvl_invalidViInt32Range (instrSession, triggerSource, RSZVL_VAL_TRG_EXT, RSZVL_VAL_TRG_EXT) == VI_TRUE)
            && (rszvl_invalidViInt32Range (instrSession, triggerSource, RSZVL_VAL_TRG_VID,RSZVL_VAL_TRG_VID) == VI_TRUE))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Trigger Source");
    if (rszvl_invalidViReal64Range (instrSession, triggerLevel, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Trigger Level");  
    if (rszvl_invalidViReal64Range (instrSession, triggerOffsets, 125.0e-9, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Trigger Offset [s]");  
    if (rszvl_invalidViInt32Range (instrSession, type_ofMeas, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Type Of Meas");
    if (rszvl_invalidViInt32Range (instrSession, noofMeasurements, 1, 501) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "No Of Measurements");
        
    viCheckErr( viPrintf (instrSession, "SENS:MPOW:SEQ %.12lf,%.12lf,%.12lf,%s,%.12lf,%.12lf,%s,%ld;*WAI\n",
    analyzerFrequencyHz,resolutionBandwidthHz,measTimes,rszvl_rngTriggerSource.rangeValues[triggerSource].cmdString,
    triggerLevel,triggerOffsets,measTypeArr[type_ofMeas],noofMeasurements));
   
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Burst Power Filter Type
 * Purpose:  This function configures the filter type for the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ConfigureSANBurstPowerFilterType (ViSession instrSession,
                                                 ViInt32 filterType)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_SetAttributeViInt32 (instrSession, "", 
                                                RSZVL_ATTR_MPOW_FTYPE,
                                                filterType), 2, "Filter Type"); 
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Get SAN Burst Power Sequence
 * Purpose:  This function configures the instrument setup for multiple burst 
 *           power measurement, starts a measurement sequence and reads the 
 *           results. To reduce the setting time, the setup is performed 
 *           simultaneously for all selected parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSANBurstPowerSequence( ViSession instrSession,
                                                  ViReal64 analyzerFrequencyHz,
                                                  ViReal64 resolutionBandwidthHz,
                                                  ViReal64 measTimes,
                                                  ViInt32 triggerSource,
                                                  ViReal64 triggerLevel,
                                                  ViReal64 triggerOffsets,
                                                  ViInt32 type_ofMeas,
                                                  ViInt32 no_ofMeasurements,
                                                  ViUInt32 timeout,
                                                  ViReal64 burstPowerResults[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (analyzerFrequencyHz<0.0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Analyzer Frequency [Hz]");
    if (rszvl_invalidViReal64Range (instrSession, resolutionBandwidthHz, 10.0, 10.0e+6) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Resolution Bandwidth [Hz]");
    if (rszvl_invalidViReal64Range (instrSession, measTimes, 1.0e-6, 30.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Meas Time [s]");        
    if ((rszvl_invalidViInt32Range (instrSession, triggerSource, RSZVL_VAL_TRG_EXT, RSZVL_VAL_TRG_EXT) == VI_TRUE)
            && (rszvl_invalidViInt32Range (instrSession, triggerSource, RSZVL_VAL_TRG_VID,RSZVL_VAL_TRG_VID) == VI_TRUE))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Trigger Source");
    if (rszvl_invalidViReal64Range (instrSession, triggerLevel, 0.0, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Trigger Level");  
    if (rszvl_invalidViReal64Range (instrSession, triggerOffsets, 125.0e-9, 100.0) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Trigger Offset [s]");  
    if (rszvl_invalidViInt32Range (instrSession, type_ofMeas, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Type Of Meas");
    if (rszvl_invalidViInt32Range (instrSession, no_ofMeasurements, 1, 501) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "No Of Measurements");
    if (rszvl_invalidViUInt32Range (instrSession, timeout, 0, 4294967295) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Timeout");        
    if (burstPowerResults == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Burst Power Results");        
        
    viCheckErr( viPrintf (instrSession, "*CLS;:SENS:MPOW:SEQ? %.12lf,%.12lf,%.12lf,%s,%.12lf,%.12lf,%s,%ld;*OPC\n",
    analyzerFrequencyHz,resolutionBandwidthHz,measTimes,rszvl_rngTriggerSource.rangeValues[triggerSource].cmdString,
    triggerLevel,triggerOffsets,measTypeArr[type_ofMeas],no_ofMeasurements));
    viCheckErr( rszvl_WaitForOPC (instrSession, timeout));
    viCheckErr( viScanf (instrSession, "%,#lf", &no_ofMeasurements, burstPowerResults));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get SAN Burst Power Results
 * Purpose:  This function queries the results of a multiple burst power 
 *           measurement as configured and initiated with Get Burst Power 
 *           Result (actBurstPwrResult). The unit used for the return values is 
 *           always dBm. This function may be used to obtain measurement 
 *           results in an asynchronous way using the service request mechanism 
 *           for synchronization with the end of the measurement. If no 
 *           measurement results are available, the function will return a 
 *           query error.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSANBurstPowerResults(
    ViSession   instrSession,
    ViInt32 no_ofResults,
    ViReal64    burstPowerResults[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (no_ofResults<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "No Of Results");
    if (burstPowerResults == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "burstPowerResults");        
        
    viCheckErr( viQueryf (instrSession, "*WAI;SENS:MPOW:RES?\n", "%,#lf",
    &no_ofResults, burstPowerResults));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get SAN Burst Power Result Min
 * Purpose:  This function queries the minimum power value in a multiple burst 
 *           power measurement as configured and initiated with Burst Power 
 *           Sequence (confBurstPwrSeq). The unit used for the return values is 
 *           always dBm. If no measurement result is available, the function 
 *           will return a query error.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetSANBurstPwrResultMin(
    ViSession   instrSession,
    ViReal64*   powerResultMin
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rszvl_GetAttributeViReal64 (instrSession, "", RSZVL_ATTR_MPOW_MIN, 
                powerResultMin), 2, "Power Result Min");

    checkErr( rszvl_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rszvl_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rszvl_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rszvl_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rszvl_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rszvl_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rszvl_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rszvl_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rszvl_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rszvl_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rszvl_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rszvl_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rszvl_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: rszvl_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rszvl_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rszvl_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rszvl_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rszvl_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rszvl_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rszvl_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: Option Checking                                                 
 * Purpose:  This function switches ON/OFF option checking of the instrument 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_setCheckOption (ViSession instrSession,
                                        ViBoolean state)
{
    ViStatus	error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    sessionProperties -> optionChecking = state;

    return error;
}

/*****************************************************************************
 * Function: Range Checking                                                 
 * Purpose:  This function switches ON/OFF range checking of the instrument 
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_setCheckRange (ViSession instrSession,
                                       ViBoolean state)
{
    ViStatus	error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViInt32     idx             = 0;

    checkErr (Rs_GetAttributeIndex (instrSession, RS_ATTR_RANGE_CHECK, &idx));

    *(ViBoolean *)sessionProperties -> attr[idx].p2value = state;

Error:
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rszvl_DefaultInstrSetup (instrSession));
    checkErr( rszvl_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?;*OPC\n"));
        viCheckErr( rszvl_WaitForOPC (instrSession, 600000));  
        viCheckErr( viScanf (instrSession, "%hd", testResult));
        if (*testResult == 0)
            strcpy (testMessage, "Self-Test Passed.");
        else
            strcpy (testMessage, "Self-Test Failed.");
        }
    else
        {
        /* Simulate Self Test */
        *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Self Test Results
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SelfTestResults (ViSession instrSession,
                                            ViInt32 bufferSize,
                                            ViChar results[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[] = "DIAG:SERV:STES:RES?\n";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (viRead (instrSession, (ViBuf) results, (ViUInt32) bufferSize, NULL));

    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSZVL_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSZVL_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which 
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_SystemVersion (ViSession instrSession,
                                          ViInt32 length, 
                                          ViChar systemVersion[])

{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rszvl_GetAttributeViString (instrSession, "", 
                                                RSZVL_ATTR_SYST_VERSION, length,
                                                systemVersion),
                                                2, "System Version");
    checkErr( rszvl_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSZVL_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rszvl_GetAttributeViString( instrSession, "", RSZVL_ATTR_SYST_ERR, RSZVL_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function reads all system messages and returns a list of 
 *           comma separated strings.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ErrorList (ViSession instrSession,
                                      ViInt32 bufferSize, 
                                      ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (bufferSize < 0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size");
    if (errors == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Errors");

    viCheckErr (rszvl_GetAttributeViString( instrSession, "", RSZVL_ATTR_SYST_ERR_LIST, 
        bufferSize, errors));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
        /*=CHANGE:================================================================*
            Insert instrument driver specific error codes here.  Example:

            {rszvl_ERROR_TOO_MANY_SAMPLES,   "Sample Count cannot exceed 512."},
         *=============================================================END=CHANGE=*/
            RSZVL_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Clear Instrument Errors
 * Purpose:  This function deletes all entries in the table SYSTEM MESSAGES.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_ClearInstrumentErrors (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_SYST_ERR_CLEAR, VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_Preset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rszvl_SetAttributeViString (instrSession, "", 
                RSZVL_ATTR_SYST_PRESET, VI_NULL));
    
    checkErr( rszvl_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
    {
        if (sessionProperties == NULL)
        {
            (void) Rs_UnlockSession (instrSession, &locked);
            return 1024;
        }
        else
        {
            checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
            checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
            checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }
    }

    /* Format data */
    if (primary != VI_SUCCESS)
    {
        ViChar msg[256] = "";
        checkErr( rszvl_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
    }

    if (secondary != VI_SUCCESS)
    {
        ViChar msg[256] = "";
        checkErr( rszvl_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
    }

    if (strlen (elaboration) > 0)
    {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
    }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
    {
        error = (ViStatus) actualSize;
    }
    else
    {
        if (bufferSize > 0)
        {
            if (actualSize > bufferSize)
            {
                error = actualSize;
                actualSize = bufferSize;
            }
        }
        if (description == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
        }
      
	    memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
    }

    if (errorCode)
    {
        *errorCode = primary;
    }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rszvl_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rszvl_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rszvl_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rszvl_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rszvl_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rszvl_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/*****************************************************************************
 * Function: rszvl_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rszvl_close or by an RS class driver.
 *
 *           Note:  This function must close the I/O session and set
 *           RS_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC rszvl_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

Error:

    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
		viPrintf (instrSession, "@LOC");
		
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           RSZVL_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rszvl_CheckStatusCallback (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;

    //viCheckErr (viQueryf (instrSession, "*ESR?\n", "%hd", &esr));
    viCheckErr (viQueryf (instrSession, "*STB?\n", "%hd", &esr));

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:

    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rszvl_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    /*

       Change this function to wait for operation complete for your specific
       instrument. This example function is based on the IEEE 488.2 Common
       System Commands. This function assumes that the instrument does not
       generate service requests for other reasons. This example performs
       the following operations:

       - Enables the service request event.
       - Sends a command that forces the instrument to generate a service
         request.
       - Waits for the service request.
       - Cleans up after the service request.

    */
    
#if 0 // --- Select most suitable method ---

    ViUInt16    statusByte;
    ViInt32     opcTimeout = 0;
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viEnableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL));
   
    // wait for SRQ
    viCheckErr( viWaitOnEvent (io, VI_EVENT_SERVICE_REQ, opcTimeout, VI_NULL, VI_NULL));
    viCheckErr( viDisableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE));

    // clean up after SRQ
    viCheckErr( viBufWrite (io, "*CLS", 4, VI_NULL));
    viCheckErr( viReadSTB (io, &statusByte));

#else

    ViUInt16    response    = 0;
    ViInt32     counter     = 0;
    ViUInt16    intfc   = VI_INTF_GPIB;
    ViInt32     opcTimeout = 0;
    //viCheckErr( viPrintf (io, "*OPC\n"));
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    /* intfc specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial intfc */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
            {
#if 1
                error = viReadSTB (instrSession, &response);
#else
                error = viQueryf (instrSession, "*STB?\n","%ld", &response);
#endif

                Rs_Delay (0.001);
                counter++;

                if (counter >= opcTimeout)
                    return VI_ERROR_TMO;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and ESB bits are taken into consideration. */
            
            } while ((response & 0x24) == 0);
            /* (check Error(Event) Queue bit in STB) */
            return ((response & 0x4) != 0) ? RS_ERROR_INSTRUMENT_STATUS : error;
        }

#endif

Error:

#if 0

    viDiscardEvents (io, VI_EVENT_SERVICE_REQ, VI_QUEUE);

#endif

    return error;
}

/*****************************************************************************
 *- Write Callbacks ---------------------------------------------------------*
 *****************************************************************************/
ViStatus rszvl_HcopyColorDef_WriteCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error               = VI_SUCCESS;
    ViInt32     idx                 = 0;
    ViChar      *cmd                = VI_NULL;

    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  
    /* --- If the range table for the attribute exists, pass 'command string' or
       'discreteOrMinValue' instead of attribute value. --- */

    if (sessionProperties -> attr[idx].rangeTable)
    {
        if (sessionProperties -> attr[idx].rangeTable -> type == RS_VAL_DISCRETE)
            {
            ViUInt16    i = 0;
            ViChar      *p2string;
            ViReal64    *p2discrete;
            ViInt32     data_type = sessionProperties -> attr[idx].dataType;
            ViReal64    precision = sessionProperties -> attr[idx].comparePrecision;

            precision = (precision < DBL_MIN) ? DBL_MIN : precision;

            /* Allowed data types are only ViInt32 or ViReal64 */
            switch (data_type)
                {
                case RS_VAL_INT32:
                case RS_VAL_REAL64:
                    /* Valid data types */
                    break;
                case RS_VAL_STRING:
                case RS_VAL_BOOLEAN:
                case RS_VAL_EVENT:
                case RS_VAL_ADDR:
                case RS_VAL_SESSION:
                case RS_VAL_UNKNOWN_TYPE:
                default:
                    viCheckErr (RS_ERROR_INVALID_TYPE);
                }

            /* Go over range table values and find appropriate command string for passed
               attribute value. For floating point values use predefined attribute precision. */
            do
                {
                p2discrete = &sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                p2string = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString;

                if (sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString == RS_RANGE_TABLE_END_STRING)
                    viCheckErr (RS_ERROR_INVALID_VALUE);

                if (data_type == RS_VAL_INT32)
                    if (fabs(*p2discrete - *(ViInt32 *)value) <= DBL_MIN)
                        {
                        //viCheckErr (viPrintf(io, "%s %s\n", cmd, p2string));
                        break;
                        }

                if (data_type == RS_VAL_REAL64)
                    if (fabs(*p2discrete - *(ViReal64 *)value) <= precision)
                        {
                        //viCheckErr (viPrintf(io, "%s %s\n", cmd, p2string));
                        break;
                        }

                i++;

                } while (i != 0);
            /* Check Wait for OPC after writes flag */
            if ((sessionProperties -> attr[idx].flags & RS_VAL_WAIT_FOR_OPC_AFTER_WRITES) != 0)
            {
                viCheckErr (viPrintf(instrSession, "%s%s;*OPC\n", cmd, p2string));
            }   
            else
                viCheckErr (viPrintf(instrSession, "%s%s\n", cmd, p2string));
                
            goto Error; /* Exit the function */
            }
    }

    /* --- Attribute data type is used for VISA I/O --- */
    /* Check Wait for OPC after writes flag */
    if ((sessionProperties -> attr[idx].flags & RS_VAL_WAIT_FOR_OPC_AFTER_WRITES) != 0)
    {               
        switch (sessionProperties -> attr[idx].dataType)
            {
            case RS_VAL_INT32:
                viCheckErr (viPrintf(instrSession, "%s%ld;*OPC\n", cmd, *(ViInt32 *)value));
                break;
            case RS_VAL_REAL64:
            case RS_VAL_STRING:
            case RS_VAL_BOOLEAN:
            case RS_VAL_EVENT:
            case RS_VAL_ADDR:
            case RS_VAL_SESSION:
            case RS_VAL_UNKNOWN_TYPE:
            default:
                viCheckErr (RS_ERROR_INVALID_TYPE);
            }
     }
     else
     {
        switch (sessionProperties -> attr[idx].dataType)
            {
            case RS_VAL_INT32:
                viCheckErr (viPrintf(instrSession, "%s%ld\n", cmd, *(ViInt32 *)value));
                break;
            case RS_VAL_REAL64:
            case RS_VAL_STRING:
            case RS_VAL_BOOLEAN:
            case RS_VAL_EVENT:
            case RS_VAL_ADDR:
            case RS_VAL_SESSION:
            case RS_VAL_UNKNOWN_TYPE:
            default:
                viCheckErr (RS_ERROR_INVALID_TYPE);
            }
     }
       
Error:

    if (cmd)
        free (cmd);

    return error;
}
